
public class FlyingRobot extends Pawn implements ISecuritySystemComponent
{
    public SecuritySystem cSecuritySystem = null;//#\FlyingRobot;Security system controlling of this Robot (if null, Robot is autonomic)
    
    public boolean bEnabledOnStart = true;//#\FlyingRobot;Initial state (Enabled/Disabled) of Robot (if not controlled by a security system)

	// cannon robota:
    FlyingRobotCannon cCannon = null;
	
	// czy leciec do wysokosci gracza
	public boolean bFollowPlayersHeight = false;
    
    // aktualny cel robota
    Pawn cCurrentTarget = null;
    
    // promien w jakim musi byc wrog, zeby robot sie normalnie ruszal
    public float fRelevantRadius = 1000.0f;
    
    // zakres w jakim robot strzela do wroga
    public float fShootingRange = 500.0f;
    
    // zakres w jakim robot szuka wroga
    public float fFindTargetRange = 1500.0f;
    
    
    // mozliwe typy akcji robota:
    final int _ACTION_MOVE = 1;
    final int _ACTION_SWING = 2;
    final int _ACTION_NONE = 0;
    
    
    final float _PI = 3.1415926535897932384626433832795f;

    String sBlowFX = "FXFlyingRobot";
    String sBlowSound = "Data/Sounds/Explosion/RocketExplosionFF";
    
    // bazowa pozycja do hustania sie w powietrzu i przesuniecia
    Vector vBasePosition = null;
    
    // pozycja, od ktorej startowal robot (do kontroli oddalania sie)
    Vector vInitialPosition = null;
    
    // pozycja docelowa
    Vector vTranslation = null;
    
    // numer akcji:
    int nCurrentAction = _ACTION_NONE;
    // numer akcji:
    int nLastAction = _ACTION_NONE;
    
    // czas rozpoczecia akcji:
    float fActionStartTime = 0;
    
    // czas trwania akcji
    float fActionDuration = 0;

	final float _TIME_BTW_LIGHTS = 1f;
	int nLightElementID = -1;	
	String sLightFXName = "RobotSmallLight";
	float fLastLightTime = 0;

	int nBarrelElementID = -1;	
	
	FlyingRobot()
	{
		MeshName = "Data/Meshes/AI/robot_maly.3DA";		
        nTeamID = 10000;
        bIsAITarget = true;
	}
	
	public void FollowPlayersHeight()
	{
		bFollowPlayersHeight = true;
	}
	
	public void OnCreate()
	{
		super.OnCreate();
		
		SetFlag(EFlags._PORTAL_INDEPENDENT);
        SetPhysicsActive(true);
        SetPhysicsType(EPhysics._NEWFLY);
        SetMass(0);
        SetSpeed(new Vector(0, 1, 0));
        SetSpeedValue(0);
        
        SetLighting(true);
        
        vInitialPosition = GetPosition();
        
        cCannon = (FlyingRobotCannon)CreateObjectNoInit("FlyingRobotCannon");
        cCannon.cOwner = this;
        cCannon.InitObject();
        
        CallMethodEveryMinMax( "Update", 0.0005f, 1.0f );
		
		if (nLightElementID == -1)
			nLightElementID = GetElementID("light");
		
		if (nBarrelElementID == -1)
			nBarrelElementID = GetElementID("barrel");
        
        bActiveTrigger = false;
        bIsActiveTarget = true;
        
		// inicjalizacja security system
        if (cSecuritySystem != null)
            cSecuritySystem.RegisterComponent(this);
        else
        {
            if (bEnabledOnStart)
                Enable();
            else
                Disable();
        }
	}
	
	void InitInventory()
	{
	}
	
	void ProcessOnFrame()
	{
	}
    
    public void Bleed(float fDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nDamageType)
    {
	}
	
    public void Die()
    {
        super.Die();
        
        DisableCallMethod("Update");
        CallMethodIn("BlowUp", RandomFloat(0.5f));
    }
    
    void BlowUp()
    {
        // co po smierci? wybuch?
        if( sBlowFX != null)
        {
            ExecuteFX(sBlowFX, GetPosition(), Vector.Up);
            ExecuteParticleSystem(sBlowFX, GetPosition(), Vector.Up);
        }

        if( sBlowSound != null)
        {
            PlaySound(sBlowSound);
        }

        // trzeba robocika zamienic w smiecia i upuscic na ziemie
        CallMethodIn("delete", 0.0f);
    }
    
    Pawn FindTarget()
    {        
                
        ControlObject aPawns[] = GetObjectsInRadiusFromClass(fFindTargetRange, "Pawn");
        
        Pawn cCurrent;
        Pawn cFoundTarget = null;
        
        int nPawns = aPawns.length;
                
        for (int i=0; i<nPawns; i++)
        {
            cCurrent = (Pawn)aPawns[i];
            if ( (!cCurrent.bDead) && IsEnemy(cCurrent) )
            {
                if (CheckTraceAbsolute( cCurrent, cCurrent.GetPointToLookAt(0), GetPosition() ))
                    cFoundTarget = cCurrent;
            }
        }
    
    
        return cFoundTarget;
    }
    
    void Fire()
    {
        if (cCurrentTarget!=null)
        {
            Vector vStart = GetPosition();
            
            Vector vEnd = cCurrentTarget.GetPointToLookAt(0);
            
            Vector vDir = new Vector(vEnd);
            vDir.Sub(vStart);
            
            if (vDir.Len()<fShootingRange)
            {
                vDir.Normalize();
            
                cCannon.Fire(vStart, vDir);
            }
        }
    }
    
    boolean bFiring = false;
    
    float fLastFireTime = 0;
    float fLastStopFireTime = 0;
    float fLastStartFireTime = 0;
    
    
    int MakeDecision()
    {
        //test:
        cCurrentTarget = FindTarget();

        float fCurrTime = GetTime();
        if (bFiring)
        {
            if(fCurrTime - fLastFireTime > 0.2f)
            {
                fLastFireTime = fCurrTime;
                Fire();
            }

            if (fCurrTime - fLastStopFireTime > 1)
            {
                bFiring = false;
                fLastStartFireTime = fCurrTime;
            }
        }
        else
        {
            if (fCurrTime - fLastStartFireTime > 1)
            {
                bFiring = true;
                fLastStopFireTime = fCurrTime;
            }
        }
        
        int nAction;
        
        boolean bIsRelevant = IsInFrustum();
        if (cCurrentTarget!=null)
        {
            Vector vVect = GetPosition();
            vVect.Sub( cCurrentTarget.GetPosition() );
            
            if (vVect.Len() < fRelevantRadius)
                bIsRelevant = true;
        }
        
        // ze wzgledu na optymalizacje:
        if (!bIsRelevant)
            nAction = _ACTION_SWING;
        else
        {
            if (nLastAction==_ACTION_SWING)
                nAction = _ACTION_MOVE;
            else
            {
                if (RandomInt(10)<3)
                    nAction = _ACTION_SWING;
                else
                    nAction = _ACTION_MOVE;
            }
        }
                
        vBasePosition = null;
        vTranslation = null;        
        fActionStartTime = GetTime();
        fActionDuration = _PI/5.0f; //TODO
        
        return nAction;
    }
    
    void RobotSwing(float fTime)
    {
        if (IsInFrustum())
		{         
            //if (vBasePosition==null)
            //    vBasePosition = GetPosition();
                        
            //TODO: wspolczynniki gibania
		    //Vector vPos = new Vector( 0, (float)java.lang.Math.sin((float)fTime*5.0f)*3.0f, 0);            
            //vPos.Add(vBasePosition);
            //SetPositionWorld(vPos);
            
		}   
    }
    
    float GetFactor( float fFactor )
    {
        float fVal = (1.0f + (float)java.lang.Math.sin( (fFactor*_PI) - (_PI/2.0f) ) ) / 2.0f;
        return fVal*fVal;
    }
    
    void RobotMove(float fTime)
    {
        int nVariant = -1;
        
        if (vTranslation==null) // nie ma celu: trzeba ustalic
        {
            
            vBasePosition = GetPosition();
    
            // losowanie punktu:
            int nTrials = 6; // TODO ???
            while ( (nTrials>0) && (vTranslation==null) )
            {
                nVariant = RandomInt(3);
                fActionDuration = 1.0f + (float)nVariant * 0.3f;
                
                float fDurationScale = 0.8f; //= (cCurrentTarget!=null) ? 0.3f : 0.8f;
                
                fActionDuration *= fDurationScale;
                
                Vector vEnemyDir = null;
                
                if (cCurrentTarget!=null)
                {
                    vEnemyDir = cCurrentTarget.GetPosition();
                    vEnemyDir.Sub(GetPosition());
                    
                    float fDistance = vEnemyDir.Len();
                    
                    if (fDistance < 150.0f)
                    {
                        vEnemyDir = null;
                    }
                    else
                    {
                        vEnemyDir.Normalize();
                        vEnemyDir.Mul(200.0f);
                    }
                          
                }
            
				if (bFollowPlayersHeight)
				{
					float fPlayerY = ((ChromeModuleSingle) GetModule()).MainPlayer.GetPointToLookAt(0).fY - 30f;
					
					vTranslation = new Vector( RandomFloat(100.0f) - 50.0f,
					                           RandomFloat(20.0f) - 10.0f + fPlayerY - vBasePosition.fY, 
					                           RandomFloat(100.0f) - 50.0f
					                          );
                    
                    if (vEnemyDir!=null)
                    {
                        vEnemyDir.fY = 0.0f;
                        vTranslation.Add(vEnemyDir);
                    }
				}
				else
				{
					vTranslation = new Vector( RandomFloat(100.0f) - 50.0f,
					                           RandomFloat(20.0f) - 10.0f + vInitialPosition.fY - vBasePosition.fY , 
					                           RandomFloat(100.0f) - 50.0f
					                          );
				
					if (vTranslation.fY > 50f)
						vTranslation.fY = 50f;
					else
					if (vTranslation.fY < -50f)
						vTranslation.fY = -50f;
                    
                    if (vEnemyDir!=null)
                    {                        
                        vTranslation.Add(vEnemyDir);
                    }
				}
				
                vTranslation.Mul( fActionDuration/fDurationScale );
                
                if (vTranslation.Len()==0)
                    vTranslation = null;
                else
                {
                    Vector vStoredPos = GetPosition();
                    if ( !MoveByEllipsoid(vTranslation) )
                        vTranslation = null;
                    SetPositionWorld(vStoredPos);
                }
                nTrials --;
            }
            
            
            Vector vNewDir = null;
            if (cCurrentTarget!=null)
            {
                vNewDir = new Vector(cCurrentTarget.GetPointToLookAt(0));
                vNewDir.Sub( GetPosition() );
                
                if (vTranslation!=null)
                    vNewDir.Sub( vTranslation );
                                
            }
            else
            {
                if (vTranslation!=null)
                    vNewDir = new Vector(vTranslation);
                else
                    vNewDir = null;
            }
            
            if (vNewDir!=null)
            {
                vNewDir.fY = 0.0f;
                vNewDir.Normalize();                
                SetMaxAngSpeed(90);
                SetDestDir(vNewDir);           
            }
        }

        if (vTranslation==null)
            nCurrentAction = _ACTION_NONE;
        else
        {
            switch (nVariant)
            {
               case 0 : PlaySound("Data/Sounds/Ai/FlyingRobot1"); break;
               case 1 : PlaySound("Data/Sounds/Ai/FlyingRobot1_3"); break;
               case 2 : PlaySound("Data/Sounds/Ai/FlyingRobot1_6"); break;
            }
            
            Vector vTrans = new Vector(vTranslation);
            float fFactor = fTime / fActionDuration;
            //vTrans.Mul( (float)java.lang.Math.sqrt( GetFactor(fFactor) ) );
            vTrans.Mul( GetFactor(fFactor) );
        
            vTrans.Add( vBasePosition );
        
            SetPosition( vTrans );
        }
    }
	
	void Update()
	{
        if (bDead)
            return;
        
		float fLightTime = GetTime();
		if (fLightTime - fLastLightTime > _TIME_BTW_LIGHTS)
		{
			ShowLight();
			fLastLightTime = fLightTime;
		}
		
        // czy wiemy co robic?
        if (nCurrentAction==_ACTION_NONE)
        {
            // nie? no to trzeba zdecydowac:            
            nCurrentAction = MakeDecision();
        }
        
        float fTime = GetTime() - fActionStartTime;
        if (fTime>fActionDuration)
            fTime = fActionDuration;
        
        switch (nCurrentAction)
        {
            case _ACTION_SWING: RobotSwing(fTime); break;
            case _ACTION_MOVE: RobotMove(fTime); break;
        }
		
        if (fTime >= fActionDuration)
        {
            nLastAction = nCurrentAction;
            nCurrentAction = _ACTION_NONE;
        }
	}
    
    void ShowLight()
	{
		if (nLightElementID == -1)
			return;
		
		Vector vPos = GetElementLocalPos(nLightElementID);
	    ExecuteFX(this, nLightElementID, sLightFXName, vPos);
	}
	
	public void ActivateRobot()
	{
		ActivateOnModule();
		UnfreezeCallMethods();
	}
	
	public void DeactivateRobot()
	{
		FreezeCallMethods();		
		DeactivateOnModuleAlways();
	}
	
    void ExecuteWeaponBeam(float fDistance, Vector vBeamPos, int nElementID)
    {
        super.ExecuteWeaponBeam(fDistance, vBeamPos, nElementID);
        
        fFXSizeX = 5.0f;
    
        ExecuteFX( this, nBarrelElementID, "WeaponLaser", Vector.Zero);
    }
	
    public void Enable()
    {
        ActivateRobot();
    }
    
    public void Disable()
    {
        DeactivateRobot();
    }
    
    public boolean IsActive(ControlObject Activator)
    {
        return false;
    }
    
    public void Bleed(float fDamage, GameObject Assassin, Vector vHitLocation, Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
    {
        
    }
}
