public class AnimatedObject extends MeshObject implements iact_StartStop
{
    /** Dzwiek zapetlony odgrywany podczas animowania
     */
    public String sLoopSound = null;//#ANIMATED
    
    /** Sekwencja animacji do zapetlonego uruchomienia.
     * Gdy jest null to odpalana jest cala animacja
     */
    public String sSequenceName = null;//#ANIMATED
    
    // czestotliwosc ogrywania animacji
    public float fAnimFreq = 30.f;//#ANIMATED
    
    // wylacza automatyczny start animacji po starcie levelu
    public boolean bDisableAutostart = false;//#ANIMATED
    
    int nAnim = -1;
    int nSound = 0;
    
    void OnCreate()
    {
		bSG = true;
		
        ClearFlag(EFlags._NO_ANIMATION);
        ClearFlag(EFlags._NO_ANIMATION_REFRESH);
        
        if(sSequenceName==null)
        {
            InitAnimSeq("all", 0, (int)GetNumFrames() - 1);
            sSequenceName="all";
        }
        SetAnimFreq(sSequenceName, fAnimFreq);
        
        SetLighting(true);
        
        if(!bDisableAutostart)
            Start();
    }

    public void OnDestroy()
    {
        Stop();
    }
    
    public void Start()
    {
        PlayAnim(sSequenceName, -1);
        if(sLoopSound != null) nSound = AttachSound(sLoopSound);
    }
    
    public void Stop()
    {
        if(nSound != 0)
        {
            DetachSound(nSound);
            nSound = 0;
        }
        StopAnim("");
    }
    
    public void ToggleStartStop()
    {
        if( IsPlaying("") )
            Stop();
        else
            Start();
    }
    
    
    public void OnAudioReinitialize()
    {
        
        if (nSound != 0)
            nSound = AttachSound(sLoopSound);

    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
	boolean bStartSG = false;
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[AnimatedObject.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
		
        if (cFC.GetID() == ESGChunksChrome._CHK_ANIMATED_OBJECT)
        {
			super.SGLoadChunk(cFC);			
			
			bStartSG = cFC.LoadBool();		
        }
		
		cFC.delete();
	}
	
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[AnimatedObject.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_ANIMATED_OBJECT, cFCParent);
		
		super.SGSaveChunk(cFC);

		cFC.SaveBool(IsPlaying(""));
					
		cFC.delete();
	}	    	    	
	
	public void SGUpdate()
	{
		super.SGUpdate();
	
		Stop();						
		
		if (bStartSG)
			Start();			
	}
}
