public class AnimatedOnceObject extends MeshObject
{
    /** Dzwiek odgrywany przy odpalaniu animacji
     */
    public String sSound = "Data/Sounds/Transporter.wav";//#ONCE
    
    /** Opoznienie od momentu zaladowania levelu
     */
    float fTimeDelay = 4.0f;//#ONCE
    
    /** Sekwencja animacji do odpalenia na starcie.
     * Gdy jest null to odpalana jest cala animacja
     */
    String sSequenceName = null;
    
    int nAnim = -1;
    int nSound = 0;
    
    AnimatedOnceObject()
    {
        sSequenceName = "Start";
    }
    
    public void OnCreate()
    {
        super.OnCreate();
        
        bSG = true;
        
        ClearFlag(EFlags._NO_ANIMATION);
        ClearFlag(EFlags._NO_ANIMATION_REFRESH);
        
        if(sSequenceName==null)
        {
            InitAnimSeq("all", 0, (int)GetNumFrames() - 1);
            sSequenceName="all";
        }
        
        SetLighting(true);
        
        ActivateOnModuleGridIndependent();
        Vector vP1 = new Vector(-10000.f, 0, -10000.f);
        Vector vP2 = new Vector(10000.f, 0, 10000.f);
        SetExtents(vP1, vP2);
        
        CallMethodIn("StartAnimOnce", fTimeDelay);
    }

    //void OnAnimEnd(int nAnimID)
    void End()
    {
        if(nSound != 0) StopWave(nSound);
        //CallMethodIn("delete", 0.0f);
        EnableRendering(false);
        DeactivateOnModuleAlways();
    }

    void StartAnimOnce()
    {
        SetAnimFreq(sSequenceName, 15.0f);
        PlayAnim(sSequenceName, 1);
        CallMethodIn("End", GetAnimLength(sSequenceName));
        nSound = PlayWave(sSound);
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************

    //* Metoda wczytujaca zapisane dane w Save gracza
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[AnimatedOnceObject.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_ANIMATED_ONCE)
        {	
			super.SGLoadChunk(cFC);

            if(nSound != 0) StopWave(nSound);
        
            long nWavePos = cFC.LoadLong();
            if(nWavePos != -1)
            {
                nSound = PlayWaveFromPos(sSound, nWavePos);
            }
        }
		
		cFC.delete();
    }
    
    // Metoda zapisujaca dane do Save gracza
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[AnimatedOnceObject.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_ANIMATED_ONCE, cFCParent);
		
		super.SGSaveChunk(cFC);
		
        if(nSound != 0)
        {
            cFC.SaveLong(GetPlayingWavePos(nSound));
        }
        else
            cFC.SaveLong(-1);
        
        cFC.delete();
    }
    
    long l_nWavePos = -1;
    public void OnAudioPreReinitialize()
    {
        if (nSound != 0 && IsPlayingWave(nSound))
        {
            l_nWavePos = GetPlayingWavePos(nSound);
            
        }
    }
    
    public void OnAudioReinitialize()
    {
        
        if (l_nWavePos != -1)
        {
            nSound = PlayWaveFromPos(sSound, l_nWavePos);
        }
        
        l_nWavePos = -1;
    }
    
}
