import java.lang.Math;
import java.lang.Float;


/**
 */
class InterpolatedCameraParameter extends GameObject
{
    final static float INFINITY = 1.e6f;
    final static int nStackSize = 64;
    
    final static int INTERP_SMOOTHSTEP = 1;
    final static int INTERP_LINEAR = 2;
    
    java.util.Vector aParam = new java.util.Vector(nStackSize);
    float fCoeff[] = new float[nStackSize-1];
    float fSpeed[] = new float[nStackSize-1];
    int nInterpolation[] = new int[nStackSize-1];
    int nTop = -1;

    InterpolatedCameraParameter()
    {
        aParam.trimToSize();
    }

	public void ClearParameters()
	{
		nTop = 0;
	}

    public void NewParameter(Object newParam, float fChangeSpeed)
    {
        NewParameter(newParam, fChangeSpeed, INTERP_SMOOTHSTEP);
    }

    public void NewParameter(Object newParam, float fChangeSpeed, int Interp)
    {
        if (nTop > nStackSize-2)
            nTop = nStackSize-2;
        nTop++;
        aParam.insertElementAt(newParam, 0);
        for (int i=nTop-1; i>0; i--)
        {
            fCoeff[i] = fCoeff[i-1];
            fSpeed[i] = fSpeed[i-1];
            nInterpolation[i] = nInterpolation[i-1];
        }
        if (nTop > 0)
        {
            fCoeff[0] = 0.f;
            fSpeed[0] = fChangeSpeed;
            nInterpolation[0] = Interp;
        }
    }

    void DeleteParameter(int iIndex)
    {
		for(int i=0; i<nTop; i++)
		{
			//LogD("coeff: "+fCoeff[i]+", speed:"+fSpeed[i]+"\n");
		}
		
        if (iIndex > nTop) return;
        aParam.removeElementAt(iIndex);
        if (iIndex > 0)
            for (int i=iIndex-1; i<nTop-1; i++)
            {
                fCoeff[i] = fCoeff[i+1];
                fSpeed[i] = fSpeed[i+1];
                nInterpolation[i] = nInterpolation[i+1];
            }
        nTop--;
		//LogD("parameter deleted, nTop:"+nTop+"\n");
    }

    public Object Calculate(float fDt)
    {
        Object cResult = aParam.elementAt(nTop);

        // nie ma wystarczajacej ilosci danych do interpolowania
        if (nTop < 1) 
        {
//            LogD("nTop = "+nTop+", returning "+cResult+"\n");
            return cResult;
        }
        //else
  //          LogD("nTop = "+nTop+"\n");

        int limit = nTop-1;

        //for (int i=nTop-1; i>=0; i--)
        for (int i=limit; i>limit-1; i--)
        {
            if (fSpeed[i] < INFINITY)
                fCoeff[i] += fSpeed[i] * fDt;
            else 
                fCoeff[i] = 1.f;

            switch (nInterpolation[i])
            {
            case INTERP_SMOOTHSTEP :
                {
                    cResult = Mix( aParam.elementAt(i), cResult, 
                                   Smoothstep(0, 1, fCoeff[i]));
                }
                break;
            case INTERP_LINEAR :
                {
                    cResult = Mix( aParam.elementAt(i), cResult, 
                                   Linear(0, 1, fCoeff[i]));
                }
                break;
            }

            if ((fCoeff[i] >= 1.f) && (nTop > 1))
            {
                DeleteParameter(i+1);
            }
        }

        return cResult;
    }

    /** A * f + B * (1-f); */
    Object Mix(Object A, Object B, float f)
    {
        return A;
    }

    static float Smoothstep(float a, float b, float x)
    {
        if (x < a) return 0;
        if (x >= b) return 1;
        x = (x-a)/(b-a);
        return (x*x * (3-2*x));
    }    
    
    static float Linear(float a, float b, float x)
    {
        if (x < a) return 0;
        if (x >= b) return 1;
        return (x-a)/(b-a);
    }
    
    public String toString()
    {
        return "nTop:"+nTop;
    }
}

/**
 */
class CameraParameterTarget extends InterpolatedCameraParameter
{
    Object Mix(Object A, Object B, float f)
    {
        Vector vA = ((Position)A).Get();
        Vector vB = ((Position)B).Get();

        vA.Mul(f);
        vB.Mul(1.f-f);
        vA.Add(vB);

        Position cTarget = new Position();
        cTarget.Set(vA);
        return (Object)cTarget;
    }
	
	void CalculateAbsoluteSpeeds()
	{
		for (int i=nTop-1; i>=0; i--)
		{

			Vector vA = ((Position)aParam.elementAt(i+1)).Get();
			Vector vB = ((Position)aParam.elementAt(i)).Get();
			vA.Sub(vB);
			float dist = vA.Len();

			if (fSpeed[i] < 1.e6f)
			{
				fSpeed[i] = 1.0f/fSpeed[i];
				fSpeed[i] /= dist/1000;
			}
			LogD("nTop="+i+", dist="+dist+", fSpeed[i]"+fSpeed[i]*dist/1000+"\n");
		}
	}
}

/**
 */
class CameraParameterFloat extends InterpolatedCameraParameter
{
    Object Mix(Object A, Object B, float f)
    {
        return new Float( ((Float)A).floatValue() * f + 
                          ((Float)B).floatValue() * (1.f-f));
    }
}

/**
 */
public class CameraMovie extends CameraExtended
{
    float fLastTime = 0;

    //PARAMETRY KAMERY
    CameraParameterTarget pPos = new CameraParameterTarget(); //pozycja
    CameraParameterTarget pTar = new CameraParameterTarget(); //target
    CameraParameterFloat  pFOV = new CameraParameterFloat(); //FieldOfView

    CameraMovie()
    {
        //poczatkowe wartosci parametrow
        pPos.NewParameter( ((Object)new Position()), 0);
        pTar.NewParameter( ((Object)new Position()), 0);
        pFOV.NewParameter( ((Object)new Float(60)), 0);
    }

    public void OnActivate()
    {	
        super.OnActivate();

		SetFOV(60);
        SetBaseFOV(60);
		SetClipNear(30.f);
        
		LogD("camera activated\n");
        fLastTime = GetTime();
        OnPostUpdate();
    }
    
    public void OnDeactivate()
    {
        super.OnDeactivate();
    }

    void NextPosition(Vector vPos, float fTime)
    {
        Position cTarget = new Position();
        cTarget.Set(new Vector(vPos));
        pPos.NewParameter((Object)cTarget, (fTime > 0) ? (1.0f/fTime) : 1.e6f );
    }

    void NextPosition(Vector vPos, float fTime, int nInterp)
    {
        Position cTarget = new Position();
        cTarget.Set(new Vector(vPos));
        pPos.NewParameter( (Object)cTarget,
                           (fTime > 0) ? (1.0f/fTime) : 1.e6f, nInterp );
    }

    void NextPosition(MeshObject cMesh, float fTime)
    {
        Position cTarget = new Position();
        cTarget.Set(cMesh);
        pPos.NewParameter((Object)cTarget, (fTime > 0) ? (1.0f/fTime) : 1.e6f );
    }

    void NextPosition(MeshObject cMesh, int nElem, float fTime)
    {
        Position cTarget = new Position();
        cTarget.Set(cMesh, nElem);
        pPos.NewParameter((Object)cTarget, (fTime > 0) ? (1.0f/fTime) : 1.e6f );
    }

    void NextTarget(Vector vPos, float fTime)
    {
        Position cTarget = new Position();
        cTarget.Set(new Vector(vPos));
        pTar.NewParameter((Object)cTarget, (fTime > 0) ? (1.0f/fTime) : 1.e6f );
/*        LogD("next target:"+vPos+", time:"+fTime);
        Log(", "+pTar.nTop+", "+((CameraTarget)pTar.aParam.elementAt(0)).GetPosition());
        Log("position:"+((CameraTarget)pTar.Calculate(0)).GetPosition()+"\n");
*/        
    }

    void NextTarget(MeshObject cMesh, float fTime)
    {
        Position cTarget = new Position();
        cTarget.Set(cMesh);
        pTar.NewParameter((Object)cTarget, (fTime > 0) ? (1.0f/fTime) : 1.e6f );
    }    
    
    void NextTarget(MeshObject cMesh, int nElem, float fTime)
    {
        Position cTarget = new Position();
        cTarget.Set(cMesh, nElem);
        pTar.NewParameter((Object)cTarget, (fTime > 0) ? (1.0f/fTime) : 1.e6f );
    }    
    
    public void OnPostUpdate()
    {
        float fTime = GetTime();
        float fDt = fTime - fLastTime;
		if (fDt < 0) fDt = 0;

        SetFOV( ((Float)pFOV.Calculate(fDt)).floatValue() );

        // liczymy kierunek do celu
        Vector vPos = ((Position)pPos.Calculate(fDt)).Get();
        Vector vDir = new Vector(vPos);

        Vector vTar = ((Position)pTar.Calculate(fDt)).Get();
        //LogD("camera target:"+vTar+", "+pTar.nTop+"\n");
        vDir.Sub( vTar );
        
        //LogLine("fDt:"+fDt+" vPos:"+vPos+ " vTar:"+vTar+" parameter:"+pPos);        

        FromForwardUpPos(vDir, Vector.Up, vPos);

        fLastTime = fTime;
    }	
}
