public class Chrome3DSceneModule extends ChromeModule
{
	final static String _3DSCENEMAP_FILE = "data/Films/Sterowka/Sterowka.map";
	
    final static String[] aNiuniaAnims = {"Menu_PressButton", "Menu_BodyTurnRight", "Menu_HeadTurnRight", "Menu_LookLeft_OneTime", "Menu_LookLeftUp"};
    final static int[] iNiuniaAnimsFreq = {                4,                    1,                    2,                       5,                 6};
                                    
    final static String[] aLoganAnims = {"Menu_Press","Menu_Talk01",	"Menu_Talk03",};
    final static int[] iLoganAnimsFreq ={          4,             5,              3,};
    
	Movie cMovieLoop = null;
    Movie cMovie = null;
	boolean m_bMovieStopped = false;
	Movie cStartMovie = null;
    
	MeshAnimatedObject m_cNiunia = null;
    MeshAnimatedObject m_cLogan = null;
    
    PlayerProfileModel m_cModel     = null;
    
    ControlObject m_cHoloCenter = null;
    
    public void OnCreate()
    {
        super.OnCreate();
        m_cHoloCenter = (ControlObject)GetObjectByName("HOLO_CENTER");
        m_cHoloCenter.DeactivateOnModuleAlways();
        
        m_cNiunia = (MeshAnimatedObject)GetObjectByName("Carrie01");
        m_cLogan = (MeshAnimatedObject)GetObjectByName("logan01");
        
        m_cNiunia.ClearFlag( EFlags._NO_ANIMATION_REFRESH );
        m_cLogan.ClearFlag( EFlags._NO_ANIMATION_REFRESH );	
        
        TimerStart();
        
		SartPlayingAnims();
        
        ChromeModulePreMission.PumpObjectsExtents(this);
        
        SetDefaultLight(new Vector(0,1,0), new Vector(0,0,0), true);
    }
	
	void SartPlayingAnims()
	{
		RunCarrieAnims();
        RunLoganAnims();
	}
    
    void RunCarrieAnims()
    {
        m_cNiunia.ClearAnimsList();
        for (int i = 0; i < aNiuniaAnims.length; i++)
        {
            m_cNiunia.AddAnimToPlay(aNiuniaAnims[i], iNiuniaAnimsFreq[i]);
        }
        m_cNiunia.StartPlayingAnims("Menu_Breath");
    }
    
    void RunLoganAnims()
    {
        m_cLogan.ClearAnimsList();
        for (int i = 0; i < aLoganAnims.length; i++)
        {
            m_cLogan.AddAnimToPlay(aLoganAnims[i], iLoganAnimsFreq[i]);
        }
        m_cLogan.StartPlayingAnims("Menu_Breath");
    }
    
    void PlayAnimTest()
    {
        m_cLogan.PlayAnim(m_cLogan.GetAnimID( "","Menu_Breath"), -1 );
        m_cNiunia.PlayAnim(m_cNiunia.GetAnimID( "","Menu_BodyTurnRight"), -1 );
    }
    
    void CreateHoloMesh() 
    {
        DisableCallMethod("DeleteHoloMesh");
		if( m_cModel == null)
        {
            m_cModel = (PlayerProfileModel)CreateObject("PlayerProfileModel");
            m_cModel.SetPosition(m_cHoloCenter.GetPosition());
            m_cModel.SetLODCoeff(0);
            m_cModel.ClearFlag(EFlags.USE_TERRAIN_LIGHTMAP_FOR_LIGHTING);
        }
    }
    
    void SetModel(NetPlayerModelInfo cModelInfo)
    {
        if (m_cModel == null)
            CreateHoloMesh();
        m_cModel.SetModel(cModelInfo);
    }
    
    void DeleteLaterHoloMesh()
    {
        CallMethodIn("DeleteHoloMesh",3);
    }
    
    void DeleteHoloMesh()
    {
        if (m_cModel != null)
        {
            m_cModel.delete();
            m_cModel = null;
        }
    }
    
	public void OnActivate()
	{
		//PlayMovieLoop(1);
	}
	
    boolean CanStopMovie()
    {
        //loopowanych filmow nie przerywamy
        return cMovieLoop == null;
    }
    
    public void OnDeactivate()
    {
        
    }
    
    protected void MovieCreateSubtitles()
    {
        
    }
    
    protected void MovieDestroySubtitles()
    {
    }
    
    void OnChangeResolution()
    {   
        super.MovieDestroyView();
        MovieCreateView();
    }
    
	public void PlayMovie(int nMovie, GameObject cObj, String sMethod)
	{
		if( IsMoviePlaying() )
		{
			StopMovie();
		}
		
        if (cMovie != null)
            cMovie.delete();
        
		cMovie = (Movie)CreateObject("Movie");
		cMovie.fWorldFadeIn = 0.0f;
		cMovie.fWorldFadeOut = 0.0f;
		cMovie.fMovieFadeIn = 0.0f;
		cMovie.fMovieFadeOut = 0;
        cMovie.nMovie = nMovie;
        cMovie.Play(cObj, sMethod);
        cMovie.bLeaveLastFrame = true;
		if (cMovieLoop != null)
			cMovieLoop.delete();
		cMovieLoop = null;
	}
	
	public void PlayMovieLoop(int nMovie)
	{
		if( IsMoviePlaying() )
			StopMovie();

        if (cMovieLoop != null)
			cMovieLoop.delete();
        
        if (cMovie != null)
            cMovie.delete();
        
		cMovieLoop = (Movie)CreateObject("Movie");
		cMovieLoop.fWorldFadeIn = 0.0f;
		cMovieLoop.fWorldFadeOut = 0.0f;
		cMovieLoop.fMovieFadeIn = 0.0f;
		cMovieLoop.fMovieFadeOut = 0;
        cMovieLoop.nMovie = nMovie;
        cMovieLoop.Play(this, "LoopMovieEnded");
	}
	
	public void LoopMovieEnded()
	{
		if (cMovieLoop != null)
		{
			cMovieLoop.Play(this, "LoopMovieEnded");
		}
	}
	
    protected void MovieCreateView()
    {
        if (iMovieViewID == 0)
        {
            iMovieViewID = AddView(null, 0.0f, 0.0f, 1.0f, 1.0f);
        }
        ForceClearView(iMovieViewID, true);
        ListenerOnViewActive(iMovieViewID, true);
        SetMovieView(iMovieViewID);
    }
    
    protected void MovieDestroyView()
    {
        //Thread.dumpStack();
    }
        
    
	public void OnEndMovie()
	{
		if (!m_bMovieStopped)
        {
            TimerSetSmoothing(0);
            super.SuperOnEndMovie();            
            bMoviePlaying = false;
            
            // wolamy metode po zakonczeniu filmu
            if(cMoviePlaying!=null)
            {
                cMoviePlaying.OnMovieEnd();
                cMoviePlaying = null;
            }
        }
		else
        {
			m_bMovieStopped = false;
        }
	}
	
	public void StopMovie()
	{
		m_bMovieStopped = true;
		super.StopMovie();
	}
	
	void InitBlackMovieBelts()
    {
        
    }
    
    void ShowBlackMoviesBelts()
    {
        
    }
    
    void HideBlackMoviesBelts()
    {
        
    }
    
    void ReleaseBlackMoviesBelts()
    {
        
    }
}
