/** klasa reprezentujaca skladnik HUD'a
 */
public class HUDComponent extends UserInterfacePyramid
{
    
	/** pozycja na ekranie */
	int nHUDCompPosX = 0;
	int nHUDCompPosY = 0;
	
	/** rozmiary w pixelach */
	int nHUDCompWidth = 0;
	int nHUDCompHeight = 0;
    
    /** kolor fontow wysietlanych na HUD'ach */
    Vector vFontColor = new Vector(0.85f, 0.85f, 0.85f);
	
	/** flaga aktywnosci */
	public boolean bActiveFlag = false;
	
	/** referencja do modulu */
	Module cModule = null;
    /** flaga mowiaca o tym, ze hudcomponent znajduje sie na module sieciowym */
    boolean m_bNet = false;
	
	/** referencja do playera */
	PlayerHUD cPlayer = null;
    
    /** referencja do HUDManagera*/
    HUDManager cHUDManager = null;
	
    /** kursor */
	Cursor cCursor = null;
    /** sciezka do tekstury kursora */
	String sCursorTex = new String("Data/Textures/Cursor.png");
    
    public void OnCreate()
    {
		bSG = false;
        super.OnCreate();
		cModule = GetModule();
        m_bNet  = cModule instanceof ModuleNet;
    }
    
    /**
      * Niektore akcje w hudzie moga sie odbywac tylko
      * na hudzie player ktory jest sterowany przez lokalnego
      * gracza (wyswietlanie, odpalanie dzwiekow)
      * Ta metoda zwraca status gracza:
      */
     //owner, mozna robic wszystko
     final static int _STATUS_OWNER    = 0;
     //notowner, nie mozna wyswietlac huda i generowac dzwiekow
     final static int _STATUS_NOTOWNER = 1;
     //status nieznany, widocznie jeszcze sie gracz nie wyspawnowal
     //czyli bezpiecznie nic nie robic :-)
     final static int _STATUS_UNKNOWN  = 2;
     
     int GetStatus()
     {
         // bramka pilnujaca zeby nie wyswietlac 
         // w sieci huda na notownerze
         if ( m_bDisableInNetGame )
         {
             return _STATUS_NOTOWNER;
         }

         int nStatus = _STATUS_OWNER;
         
         // druga bramka pilnujaca , TODO: w przyszlosci ograniczyc do jednej
         //czy to gra sieciowa
         if ( m_bNet )
         {
             //czy jest juz wyspawnowany gracz?
             if (cPlayer == null)
                 nStatus = _STATUS_UNKNOWN;
             else
             {
                 //jesli to nie wlasciciel to ustaw tryb
                if ( ! cPlayer.bNetIsOwner )
                    nStatus = _STATUS_NOTOWNER;
             }
         }
         return nStatus;
     }
     
     /**
      * Flaga pomocnicza do ustalania statusu
      * ustawiana w hud managerze
      */
     boolean m_bDisableInNetGame = false;
         
    
    boolean Load(Module mod, String sName, boolean bShow)
    {
        return super.Load(mod, sName, false);
    }
	/** metoda aktywujaca skladnik HUD'a
	 */
	public void Activate()
	{
	   bActiveFlag = true;
       Draw();
       MoveTop();
	}
	
	/** metoda aktywujaca skladnik HUD'a wjezdzajac go na ekran
	 */
	public void ActivateSlow()
	{
	   bActiveFlag = true;
       Draw();
	}

	 /** metoda deaktywujaca skladnik HUD'a
	  */
	 public void Deactivate()
	 {
		bActiveFlag = false;   
        DisableCallMethod("Draw");
        Hide();
	 }

     public void ForceDeactivate()
     {
         bActiveFlag = true;
         Deactivate();
     }
     
	 /** metoda deaktywujaca skladnik HUD'a przesuwajac go za ekran
	  */
	 public void DeactivateSlow()
	 {
		bActiveFlag = false;         
	 }
	 
     
     
     
     
	 /** metoda wyswietlajaca skladnik HUD'a na ekranie
	  */
	 public void Draw() 
     {
         int nStatus = GetStatus();
         if ( nStatus == _STATUS_OWNER )
             DrawDo();
         else if ( nStatus == _STATUS_UNKNOWN )
            CallMethodIn( "Draw", 0.1f );
     }
     
     
     
     public void DrawDo() 
     {
         Show();
     }
     
     public void HideAll() 
     {
         DisableCallMethod("Draw");
         Hide();
     }

	 /** destruktor
	  */
     public void OnDestroy()
     {
         Remove();
         GetModule().RemoveSprite(this);
         
         if (cCursor != null)
		    cCursor.delete();
		 cCursor = null;
     }
        
     /** metoda ustawia referencje do player'a
      */
     public void SetParams(PlayerHUD player,HUDManager manager)
     {
		cPlayer = player;   
        cHUDManager = manager;
     }
	 
     /** metoda do przeciazenia w podklasach, sluzaca do pozycjonowania
      * HUDComponentu (parametry oznaczaja dodatkowe przesuniecie - np. w celu
      * wjezdzania na ekran)
      */
	 public void PositionHUDComponent(int relX, int relY)
	 {
		 m_vCurrRelPos.fX = relX;
		 m_vCurrRelPos.fY = relY;
	 }

	float _UPDATE_MOVE_GRAN = 0.05f;
	float m_fStartMoveTime = 0;
	float m_fMoveTime = 0;
	Vector m_vStartPos = null;
	Vector m_vEndPos = null;
	Vector m_vCurrRelPos = new Vector();
						  
	public void MoveComponent(Vector vStartPos, Vector vEndPos, float fTimeToMove)
	{
		DisableCallMethod("UpdateMove");
		
		m_fMoveTime = fTimeToMove;
		m_fStartMoveTime = GetTime();
		m_vStartPos = vStartPos;
		m_vEndPos = vEndPos;

		PositionHUDComponent((int) vStartPos.fX, (int) vStartPos.fY);
		
		CallMethodEvery("UpdateMove", _UPDATE_MOVE_GRAN);
	}
	
	public void ComponentMoved()
	{
		m_vStartPos = null;
		m_vEndPos = null;
	}
	
	public void UpdateMove()
	{
		if (m_vEndPos == null || m_vStartPos == null)
		{
			DisableCallMethod("UpdateMove");
			return;
		}
		
		float fTimePassed = GetTime() - m_fStartMoveTime;
		if (fTimePassed > m_fMoveTime)
		{
			PositionHUDComponent((int) m_vEndPos.fX, (int) m_vEndPos.fY);
			DisableCallMethod("UpdateMove");
			ComponentMoved();
			return;
		}
		
		fTimePassed /= m_fMoveTime;

		Vector vCurrPos = new Vector(m_vEndPos);
		vCurrPos.Sub(m_vStartPos);
		vCurrPos.Mul(InterpolatedCameraParameter.Smoothstep(0, 1, fTimePassed));
		vCurrPos.Add(m_vStartPos);
		
		PositionHUDComponent((int) vCurrPos.fX, (int) vCurrPos.fY);
	}
	
	Vector GetCurrRelPos()
	{
		return m_vCurrRelPos;
	}

    public void MoveTop()
    {
        if (Precache.bPrecaching)
        {
            CallMethodIn("MoveTop", 0.1f);
            return;
        }
        super.MoveTop();
        if (GetModule().cSubtitle != null)
            GetModule().cSubtitle.CallMethodIn("MoveTop", 0);
        if (cPlayer != null && cPlayer.cSubtitle != null)
            cPlayer.cSubtitle.CallMethodIn("MoveTop", 0);
    }
    
	//*********************************************************
	//					SaveGame
    //*********************************************************
	
	boolean bActiveFlagSG = false;
	boolean bLoadSG = false;
    
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[HUDComponent.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
		bLoadSG = true;
		
        if (cFC.GetID() == ESGChunksChrome._CHK_HUD_COMPONENT)
        {
			super.SGLoadChunk(cFC);
			
			bActiveFlagSG = cFC.LoadBool();
			
			m_fStartMoveTime = cFC.LoadFloat();
			m_fMoveTime = cFC.LoadFloat();			
        }
		
		cFC.delete();
		
		FileChunk.Log("[HUDComponent.SGLoadChunk] - END\n");
	}
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[HUDComponent.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_HUD_COMPONENT, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		cFC.SaveBool(bActiveFlag);
		
		cFC.SaveFloat(m_fStartMoveTime);
		cFC.SaveFloat(m_fMoveTime);
					        
		cFC.delete(); 
	}				
	
	public void SGUpdate()
	{
		FileChunk.Log("[HUDComponent.SGUpdate]\n");
		
		super.SGUpdate();
		
		if (bActiveFlag && !bActiveFlagSG)
			Deactivate();
		if (!bActiveFlag && bActiveFlagSG)
			Activate();		

    }	    	    

    public void SetCursorPos(Vector vPos)
    {
        if (cCursor != null)
            cCursor.SetPos(vPos);
    }
    
    Vector GetCursorPos()
    {
        if (cCursor != null)
            return cCursor.GetPointerPos();
        else
            return null;
    }
}

