/** klasa modulu, w ktorym sa pokazywane implanty Logana
 */
public class ImplantModule extends Module
{
	/** czas poruszania sie kamery */
	static float _CAM_MOVE_TIME = 0.5f;

	/** docelowy kat przy implantach */
	static float _CAM_TURN_HORIZ = 30f;
	static float _CAM_TURN_VERT = -30f;

	/** poczatkowe polozenie kamery */
	static Vector _CAM_POS = new Vector(0.0f, 90.0f, 240.0f);
	/** poczatkowy punkt, na ktory patrzy sie kamera */
	static Vector _CAM_LOOK_AT = new Vector(0.0f, _CAM_POS.fY, 0.0f);
	/** minimalna odleglosc od obserwowanego punktu */
	static float _CAM_DIST_FROM_POINT = 20f;
	
	/** identyfikator widoku */
	int m_iViewID = -1;		
	
	MeshObject m_Logan = null;

	MeshObject m_Implants[] = null;
	
	/** kamera, z ktorej patrzymy na Logana */
	Camera m_Camera = null;

	/** docelowy kat, o jaki ma byc zwrocona kamera */
	float m_fDestCamAngleH = _CAM_TURN_HORIZ;
	float m_fDestCamAngleV = _CAM_TURN_VERT;
	float m_fDistFromPoint = _CAM_DIST_FROM_POINT;

	/** docelowy punkt, na ktory ma sie patrzec kamera */
	Vector m_vDestCamPos = new Vector();
	
	/** moment rozpoczecia przesuwania kamery */	
	float m_fStartMoveTime = 0f;
	
	/** biezacy czas poruszania sie - od 0 do 1 */
	float m_fMoveTime = 0f;
	
    final static int _NO_IMPLANT = -1;
	int m_iCurrImplant = _NO_IMPLANT;
	int m_iNextImplant = _NO_IMPLANT;

	String m_AnimName = "animation";
	int m_iNofImplantToStopAnim = -1;

	int m_iCurrAttachedSound = 0;
	
	ControlObject m_SoundSource = null;
    
	ImplantSystem m_cImplantSystem = null;
    
	public void OnCreate()
	{
        TimerStart();
        
		m_Camera = (Camera) CreateObject("Camera");

		FogSetColor(0, 0, 0);
		
		SkySetSunDir(0, -1, -1);

		// ustawiamy kamere w pozycji poczatkowej
		SetCameraPos(0f);
		SetState("StartState");
		
		m_Camera.SetFOV(40.0f);
		m_Camera.SetClipFar(1000.0f);
		m_Camera.SetClipNear(1.0f);
        SetRenderAfterSprites(true);
	
    }
    
    void InitImplantModule(ImplantSystem cSystem)
    {
        m_cImplantSystem = cSystem;
        InitVisibleObjects();
    }
	
	void SetSoundSource(ControlObject source)
	{
		m_SoundSource = source;
	}
	
	/** inicjacja obiektow: Logana i implantow
	 */
	public void InitVisibleObjects()
	{
		// tutaj powstaje bezskory model Logana
		m_Logan = (MeshObject) CreateObjectNoInit("MeshObject");
		m_Logan.MeshName = "Data/Meshes/Implants/kosciec.3da";		
		m_Logan.LoadMesh(m_Logan.MeshName);		
		m_Logan.InitObject();
		//m_Logan.SetLighting(true);
		m_Logan.ActivateOnModule();
		
		// tworzymy implanty
		m_Implants = new MeshObject[ImplantSystem._IMPLANTS_NUM + 1];

        // na razie olejemy druga reke
		//// musimy uwzglednic implanta drugiej reki, stad + 1
        for (int i = 0; i < ImplantSystem._IMPLANTS_NUM; i++)
		{
            Implant cImplant = m_cImplantSystem.GetImplant(i);
            if (cImplant != null)
            {
                //LogR("Creating " + cImplant.m_sMeshName);
                m_Implants[i] = (MeshObject) CreateObjectNoInit("MeshObject");
			    m_Implants[i].MeshName = cImplant.m_sMeshName;
			    m_Implants[i].LoadMesh(cImplant.m_sMeshName);
			    m_Implants[i].InitObject();
			    m_Implants[i].SetLighting(true);
			    m_Implants[i].ActivateOnModule();
			    m_Implants[i].InitAnimSeq(m_AnimName, 0, 100);
			    m_Implants[i].ClearFlag(EFlags._NO_ANIMATION);
			    m_Implants[i].ClearFlag(EFlags._NO_ANIMATION_REFRESH);
			    m_Implants[i].PlayAnim("", m_AnimName, -1); // odgrywamy animacje
			    m_Implants[i].StopAnim("");
                m_Implants[i].SetFlag(EFlags._ALWAYS_ON_TOP);
            }
		}
	}
	
	/** klasa bazowa dla stanow kamery
	 */
	public class CameraState extends State
	{
		// odpowiedz kamery na rozkaz wyswietlenia konkretnego implantu
		public void ViewImplant(int iImplantNo)
		{
		}
		
		// metoda wolana przy poruszaniu kamera
		public void Move()
		{
		}
	}
	
	/** stan poczatkowy - widok sprzed szkieletu
	 */
	public class StartState extends CameraState
	{
		public void OnEnter()
		{
			// jezeli wczesniej ustawilismy sobie jakis implant, do ktorego lecimy
			if (m_iCurrImplant != _NO_IMPLANT)
				ViewImplant(m_iCurrImplant);
		}
		
		public void ViewImplant(int iImplantNo)
		{
			if (iImplantNo == _NO_IMPLANT)
				return;

            Implant cImplant = m_cImplantSystem.GetImplant(iImplantNo);
            Vector vCamParams = cImplant.m_vCamParams;
            
            m_fDistFromPoint = vCamParams.fX;
			m_fDestCamAngleH = vCamParams.fY;
			m_fDestCamAngleV = vCamParams.fZ;
            
			m_iCurrImplant = iImplantNo;
			m_fStartMoveTime = GetTime();
			m_vDestCamPos.Set(cImplant.m_vMeshPos);
			SetState("ToImplantState");
		}
	}
	
	/** stan przemieszczania sie w kierunku implantu
	 */
	public class ToImplantState extends CameraState
	{
		public void OnEnter()
		{
			m_iNextImplant = m_iCurrImplant;
			Move();
		}
		
		public void ViewImplant(int iImplantNo)
		{
			if (iImplantNo == m_iCurrImplant)
				return;
			else
				m_iNextImplant = iImplantNo;
		}
		
		public void Move()
		{
			m_fMoveTime = GetTime() - m_fStartMoveTime;
			m_fMoveTime /= _CAM_MOVE_TIME;
		
			if (m_fMoveTime >= 1f)
			{
				m_fMoveTime = 1f;
				SetState("ImplantState");
			}
			SetCameraPos(GetFunctionValue(m_fMoveTime));
		}
		
		public void OnFrame()
		{
			Move();
		}
	}
	
	/** stan przemieszczania sie w pozycji poczatkowej
	 */
	public class ToStartState extends CameraState
	{
		public void ViewImplant(int iImplantNo)
		{
			if (iImplantNo == m_iCurrImplant)
				return;
			m_iCurrImplant = iImplantNo;
		}
		
		public void Move()
		{
			m_fMoveTime = GetTime() - m_fStartMoveTime;
			m_fMoveTime /= _CAM_MOVE_TIME;
		
			if (m_fMoveTime >= 1f)
			{
				m_fMoveTime = 0f;
				SetState("StartState");
				SetCameraPos(m_fMoveTime);
				return;
			}
			SetCameraPos(1f - GetFunctionValue(m_fMoveTime));
		}
		
		public void OnFrame()
		{
			Move();
		}
	}
	
	/** stan obserwowania implantu
	 */
	public class ImplantState extends CameraState
	{
		public void OnEnter()
		{
			// odpalamy animacje implantu
			if ( (m_iNextImplant == m_iCurrImplant) && (m_iCurrImplant != _NO_IMPLANT) )
			{
				// wyswietlamy opis implantu
				//m_ImplantWindow.DisplayDescriptionText(m_iCurrImplant);
                
                if (m_Implants[m_iCurrImplant] != null)
				    m_Implants[m_iCurrImplant].StartAnim("");
				
				m_iCurrAttachedSound = m_SoundSource.AttachSound(m_cImplantSystem.GetImplant(m_iCurrImplant).m_sMenuSound);
				/*// sprawdzamy, czy trzeba wlaczyc animacje miesniom
				if (m_iCurrImplant == ImplantSystem._IMPLANT_MUSCLE_IMPROVER)
					m_Implants[ImplantSystem._IMPLANTS_NUM].StartAnim(""); // odgrywamy animacje*/
			}
			
			ViewImplant(m_iNextImplant);
		}
		
		public void ViewImplant(int iImplantNo)
		{
			if (iImplantNo == m_iCurrImplant)
				return;

			// zatrzymujemy animacje dla implantu
            if (m_Implants[m_iCurrImplant] != null)
			    m_Implants[m_iCurrImplant].StopAnim("");
			/*// sprawdzamy, czy trzeba wlaczyc animacje miesniom
			if (m_iCurrImplant == ImplantSystem._IMPLANT_MUSCLE_IMPROVER)
				m_Implants[ImplantSystem._IMPLANTS_NUM].StopAnim(""); // odgrywamy animacje
            */

			if (m_iCurrAttachedSound != 0)
			{
				m_SoundSource.DetachSound(m_iCurrAttachedSound);
				m_iCurrAttachedSound = 0;
			}
			
			//m_ImplantWindow.HideDescriptionWindow();
			
			m_iCurrImplant = iImplantNo;
			m_fStartMoveTime = GetTime();
			SetState("ToStartState");
		}
	}
	
	public void ViewImplant(int iImplantNo)
	{
		((CameraState) CurrentState).ViewImplant(iImplantNo);
	}
	
	public void Move()
	{
		((CameraState) CurrentState).Move();
	}

	/** ustawienie kamery tak, by patrzyla na odpowiedni punkt
	 * @param fAngle docelowy kat kamery
	 * @param vDest punkt, na ktory ma sie patrzec kamera
	 * @param fTime czas od 0f do 1f
	 */
	void SetCameraPos(float fTime)
	{
		Vector vNewCamPos = new Vector();
		Vector vNewCamForward = new Vector(_CAM_POS);
		vNewCamForward.Sub(_CAM_LOOK_AT);
		vNewCamForward.Normalize();

		// obliczamy nowy wektor, w kierunku ktorego bedzie sie patrzyla kamera
		Vector vBackward = RotateVector(vNewCamForward, m_fDestCamAngleV * fTime, Vector.Right);
		vBackward = RotateVector(vBackward, m_fDestCamAngleH * fTime, Vector.Up);
		vNewCamForward.Set(vBackward); // nowy forward camery

		Vector vMove = new Vector(vBackward);
		vMove.Mul(m_fDistFromPoint); // ustalamy wektor min odleglosci od punktu docelowego
		
		// obliczamy punkt, z ktorego kamera bedzie sie patrzec
		vNewCamPos.Set(m_vDestCamPos);
		vNewCamPos.Sub(_CAM_LOOK_AT);
		vNewCamPos.Mul(fTime);
		vNewCamPos.Add(_CAM_LOOK_AT);
		vNewCamPos.Add(vMove); // to jest nasz punkt docelowy, na ktory sie patrzymy

		// obliczamy dlugosc wektora odsuniecia
		vBackward.Mul( vNewCamPos.Distance(_CAM_POS) * (1f - fTime) );
		vNewCamPos.Add(vBackward);
		
		m_Camera.FromForwardUpPos(vNewCamForward, Vector.Up, vNewCamPos);
	}
	
	float GetFunctionValue(float fArgument)
	{
		fArgument = fArgument - 1f;
		fArgument *= fArgument;
		return (1f - fArgument);
	}
	
	void Activate()
	{
		super.Activate();
		
		if ( (m_iCurrAttachedSound == 0) && (m_iCurrImplant != _NO_IMPLANT) )
			m_iCurrAttachedSound = m_SoundSource.AttachSound(m_cImplantSystem.GetImplant(m_iCurrImplant).m_sMenuSound);
	}
	
	void Deactivate()
	{
		super.Deactivate();
		
		if (m_iCurrAttachedSound != 0)
		{
			m_SoundSource.DetachSound(m_iCurrAttachedSound);
			m_iCurrAttachedSound = 0;
		}
	}
    
    UIWindow m_ImplantWindow = null;
    
    void SetImplantWindow( UIWindow window )
    {
        m_ImplantWindow = window;
        InitView();
        
    }
    
    void InitView()
    {
        Vector vPos = m_ImplantWindow.GetPos();
        Vector vSize = m_ImplantWindow.GetSize();
		m_iViewID = AddView(m_Camera,
						    (int)vPos.fX,
						    (int)vPos.fY,
						    (int)vSize.fX,
						    (int)vSize.fY);
		ForceClearViewIndependent(m_iViewID, false, true); // widok musi pod soba wyswietlac tlo (w naszym przypadku czarne)
		ListenerOnViewActive(m_iViewID, false); // bron boze nie ma byc zadnych dzwiekow z tego widoku
		InitGrid(2000f, 2000f, -1000f, 1000f, -1000f, 1000f, -1000f, 1000f);
    }
    
    void ForceClearViewIndependent(boolean b1, boolean b2)
    {
        ForceClearViewIndependent(m_iViewID, b1, b2);
    }
    
    void OnInputKey(int nKeyCode, boolean bKeyState, char nKeyAscii)
    {
        if (bKeyState)
        {
            Vector vPos = new Vector(_CAM_POS);
            Tools.MovePointByKeys(_CAM_POS, nKeyCode, 0);
        }
        Tools.ShowInfoTop(_CAM_POS + " " + " fDelta " + Tools.g_fDelta, GetModule());
    }
}
