import java.io.*;

public class Hint implements ISaveGame
{
    /** identyfikatory hinta
     * moga one sie posluzyc do tego, jesli bedziemy chcieli
     * okreslic zaleznosci pomiedzy hintami (np jakis tam hint nie
     * moze sie pojawic wczesniej niz jakis inny, itp)
     */
    final static int _HINT_UNKNOWN               =-1;
    final static int _HINT_MAPAVAILABLE          = 0;
    final static int _HINT_BINOCULARSAVAILABLE   = 1;
    final static int _HINT_JEEPAVAILABLE         = 2;
    final static int _HINT_HEALTHEXNEEDED        = 3;
    final static int _HINT_RELOADNEEDED          = 4;
    final static int _HINT_IMPLANTSAVAILABLE     = 5;
    final static int _HINT_SEARCH_BODY           = 6;
    final static int _HINT_IMPLANT_GENERAL       = 7,
                     _HINT_IMPLANT_HACK          = 8,
                     _HINT_IMPLANT_EYESCOPE      = 9,
                     _HINT_IMPLANT_ACCURACY      = 10,
                     _HINT_IMPLANT_ARMOR         = 11,
                     _HINT_IMPLANT_SPEED         = 12,
                     _HINT_IMPLANT_SLOMO         = 13,
                     _HINT_IMPLANT_STRENGTH      = 14,
                     _HINT_IMPLANT_TERMOVISION   = 15;
    final static int _HINT_BODY_SPOTTED          = 16;
    final static int _HINT_CAMOUFLAGE            = 17;
    final static int _HINT_CLOAKING_DEVICE       = 18;
    final static int _HINT_SNEAKING              = 19;
    
    /**minimalny czas po jakim moze byc nastepny hint*/
    float m_fMinDisplayPause = -1;
	
    static boolean IsImplantHint(int nHint)
    {
        return nHint >= _HINT_IMPLANT_GENERAL && nHint <= _HINT_IMPLANT_TERMOVISION;
    }
    
    /** identyfikator hinta
     */
    protected int m_nId = _HINT_UNKNOWN;
    
    
    /** Flaga mowiaca czy dany hint jest dostepny, 
     * i czy manager moze go uwzgledniac przy wyswietlaniu
     */
    protected boolean m_bEnabled = true;
    
    /** Flaga mowiaca o tym czy czasem hint nie powinien 
     * juz zostac usuniety (przeniesiony do "starych" hintow)
     */
    protected boolean m_bToRemove = false;

    
    /** Flaga mowiaca o tym, czy hint ma byc tylko jednorazowy
     */
    protected boolean m_bDisplayOnce = true;
    
    /** czas gry w ktorym mozna dokonac sprawdzenia warunkow tego hinta
     */
    protected float m_fNextCheckTime = 0.0f;
    
    
    /** czas gry po ktorym mozna wyswietlic hinta
     */
    protected float  m_fMinTimeToDisplay = 10.0f;
    
    
    /** Czas co jaki manager replikacji moze dokonywac sprawdzania tego hinta,
     * nalezy go przedefiniowac jesli sprawdzenie warunkow jest
     * dosyc "kosztowne" , wtedy bedzie sie wykonywalo rzadziej
     */
    protected float  m_fMinCheckTime = 1.0f;
    
    /** Czas co jaki manager replikacji moze dokonywac sprawdzania,
     * jesli wlasnie hint zostal wyswietlony
     * po to, aby nie powtarzaly sie za czasto te same hinty
     */
    protected float  m_fMinCheckTimeIfActivated = 10.0f;
    
        
    /** Tekst do wyswietlenia
     */
    protected String m_sText = null;
    
    /** Plik z tekstem do wyswietlenia, istotne jesli nie poda sie bezposrednio tekstu
     */
    protected String m_sFileName = null;
        
    /** Czas przez jaki tekst po wyswietleniu powinien byc na ekranie
     */
    protected float  m_fDisplayTime = 10.0f;
       

    /** Hint manager w ktorym znajduje sie hint
     */
    HintManager m_cHintManager = null;
    
    Hint(){}
    
    Hint(String sText)
    {
        m_sText = new String(sText);
    }
    
    void SetHintManager(HintManager cHintManager)
    {
         m_cHintManager =  cHintManager;
    }
        
    void SetEnabled()
    {
        m_bEnabled = true;
    }
     
    boolean GetEnabled()
    {
        return m_bEnabled;
    }
    
    void SetToRemove()
    {
        m_bToRemove = true;
    }
    
    boolean GetToRemove()
    {
        return m_bToRemove;
    }
    
    boolean Available()
    {
        return (m_bEnabled && !m_bToRemove);
    }
    
    boolean DisplayOnce()
    {
        return m_bDisplayOnce;
    }
    
    float GetNextCheckTime()
    {
        return m_fNextCheckTime;
    }
    
    void ResolveNextCheckTime(float fTimeNow)
    {
        m_fNextCheckTime = fTimeNow + m_fMinCheckTime;
    }
    
    void ResolveNextCheckTimeAfterActivate(float fTimeNow)
    {
        m_fNextCheckTime = fTimeNow + m_fMinCheckTimeIfActivated;
    }
    
        
    /** Metoda zwracajaca informacje, czy hint powinien sie zaktywowac,
     * Domyslnie hint pojawia sie po jakims czasie,
     * jesli ma byc inaczej ta metode nalezy pokryc!
     */
    boolean ShouldBeActivated(float fNow)
    {
        return (Available() && fNow > m_fMinTimeToDisplay);
    }
        
    
    String GetText()
    {
        // jezeli mamy juz string gotowy, to od razu go zwracamy
        if ( m_sText != null )
            return m_sText;
        
        // nie mamy, wiec sprobujemy odczytac z plyku
        m_sText = "";
            
        if ( m_sFileName != null )
        {
            try
            {
                m_sText = new String( ReadTextFromFile( m_sFileName ) );
            }
            catch (IOException ex)
            {
                GameObject.LogD("Blad w trakcie odczytu pliku "+m_sFileName+" z tekstem \n");
            }
        }
        
        return m_sText;
    }
        
    float GetTimeDuration()
    {
        return m_fDisplayTime;
    }
    
    int GetId()
    {
        return m_nId;
    }
    
    
    /** Callbacki przychodzace z hint managera mowiacy o tym, ze cos sie z
     * z hintem "stalo" mozna na podstawie tego podjaca jakies akcje,
     * np zablokowac hinta (zeby juz sie wiecej nie pojawial)
     */
      
    /* Callback ze wlasnie hint zostal wyswietlony */
    void DisplayStarted()
    {
    }
    
    /** Callback ze wlasnie zakonczylo sie wyswietlanie hinta */
    void DisplayFinished()
    {
    }
    
    /** Callback ze wlasnie zostalo przerwane wyswietlanie hinta 
     *  na skutek jakis "wyszych" okolicznosci */
    void DisplayCanceled()
    {
    }
    
     
    
    /** Komunikat przychodzacy z hint managera
     */
    void IncomingMessage(int nId)
    {
        
    }
    
    /** Odczyt tekstu do wyswietlenia z pliku
     */
    private String ReadTextFromFile(String sTextFile) throws IOException
    {
        GameObject.LogD("(" + sTextFile + ")\n");

        BufferedReader Buff = new BufferedReader(new FileReader(sTextFile));

        StringBuffer sText = new StringBuffer("");
        String sLine = new String("");
        
        while(sLine != null)
        {
            sLine = Buff.readLine();

            if (sLine != null)
                sText.append(sLine);
        }

        Buff.close();
        
        return sText.toString();
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	        
		FileChunk.Log("[Hint.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_CHROMEMODULE_SINGLE)
        {	
			m_fMinDisplayPause = cFC.LoadFloat();
			m_nId = cFC.LoadInt();		
			m_bEnabled = cFC.LoadBool();
			m_bToRemove = cFC.LoadBool();
			m_bDisplayOnce = cFC.LoadBool();
			m_fNextCheckTime = cFC.LoadFloat();
			m_fMinTimeToDisplay = cFC.LoadFloat();
			m_fMinCheckTime = cFC.LoadFloat();			
			m_fMinCheckTimeIfActivated = cFC.LoadFloat();			
			m_sText = cFC.LoadString();
			m_sFileName = cFC.LoadString();
			m_fDisplayTime = cFC.LoadFloat();        			
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Hint.SGSaveChunk]\n");
		
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_CHROMEMODULE_SINGLE, cFCParent);
		
		cFC.SaveFloat(m_fMinDisplayPause);
		cFC.SaveInt(m_nId);		
		cFC.SaveBool(m_bEnabled);
		cFC.SaveBool(m_bToRemove);
        cFC.SaveBool(m_bDisplayOnce);
		cFC.SaveFloat(m_fNextCheckTime);
		cFC.SaveFloat(m_fMinTimeToDisplay);
		cFC.SaveFloat(m_fMinCheckTime);			
        cFC.SaveFloat(m_fMinCheckTimeIfActivated);			
		cFC.SaveString(m_sText);
		cFC.SaveString(m_sFileName);
		cFC.SaveFloat(m_fDisplayTime);        			
        
        cFC.delete();
    }		
	
	public void SGUpdate()
	{
	}	
}
