public class BatteryChargedInvObject extends InvObjectInst
{
    int METHOD_Update = -1;
    
    /**Stan zuzycia baterii w procentach*/
    float m_fPower = 100f;
    /**Tempo zuzycia baterii (w % na sekunde)*/
    float m_fPowerUsage = 10f;
    
    float m_fLastTime = 0;
    
    boolean m_bInUse = false;
    
    UITexture cTextureOn = null;
    
    public void Use()
    {
        if (METHOD_Update == -1)
            METHOD_Update = RegisterCallMethod("UpdateBattery");
        
        if (m_fPower > 0)
            m_bInUse = !m_bInUse;
        else
            m_bInUse = false;
        
        if (m_bInUse)
        {
            m_fLastTime = GetTime();
            CallMethodEvery(METHOD_Update, 0.1f);
        }
        else
        {
            DisableCallMethod(METHOD_Update);
        }
        //ActivateOnModule();
        UpdateHUD();
    }
    
    public boolean IsInUse()
    {
        return m_bInUse;
    }
    
    public void StopUsing()
    {
        m_bInUse = false;
        UpdateHUD();
    }
    
    public void UpdateHUD()
    {
        if (Owner != null)
        {
            if (Owner instanceof PlayerHUD)
			{
                if (((PlayerHUD)Owner).HUD != null &&
                    ((PlayerHUD)Owner).HUD.GetHUDPlayer() != null)
                ((HUDManager)((PlayerHUD)Owner).HUD).GetHUDPlayer().UpdateActualObjUsage();
            }
        }
    }
    
    public void UpdateBattery()
    {
        float fTime = GetTime();
        if (m_fLastTime == 0)
            m_fLastTime = fTime;
        
        if (m_bInUse)
        {
            m_fPower -= (fTime - m_fLastTime) * m_fPowerUsage;
            if (m_fPower < 0)
                m_fPower = 0;
        }
        
        if (m_fPower == 0 && IsInUse())
            StopUsing();
        
        m_fLastTime = fTime;
        
        //jesli obiekt jest daleko od kamery to sie rzadziej updejtuje, wiec go ustawiamy
        //przy graczu
        if (Owner != null)
            SetPosition(Owner.GetPositionVectorVolatile());
    }
    
    public String GetInvName()
    {
        return (super.GetInvName() + " " + (int)m_fPower + "%");
    }
    
    void OnRemoveFromInventory()
    {
        if (IsInUse())
            StopUsing();
    }
    
    boolean _bUsedBeforeMovie = false;
    void InGameMovieStart()
    {
        _bUsedBeforeMovie = m_bInUse;
        StopUsing();
    }
    
    void InGameMovieEnd()
    {
        if (_bUsedBeforeMovie)
            Use();
        _bUsedBeforeMovie = false;
    }
    
    void Acquire(Module mod, Pawn owner)
    {
        super.Acquire(mod, owner);
        StopUsing();
        METHOD_Update = RegisterCallMethod("UpdateBattery");        
    }
    
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[BatteryChargedInvObject.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_BATTERY_CHARGED_INV_OBJECT)
        {	
			super.SGLoadChunk(cFC);

			// Wyzerowanie, aby przy pierwszym OnFrame nie zmniejszy sie w sposob nieobliczalny 
			// stanu zuzycia baterii, poniewaz juz bedzie calkiem inny czas
			m_fLastTime = 0;
			// Czy jest w uyciu
			m_bInUse = cFC.LoadBool();
			// Stan zuzycia baterii w procentach
			m_fPower = cFC.LoadFloat();			
            
            _bUsedBeforeMovie = cFC.LoadBool();
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[BatteryChargedInvObject.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_BATTERY_CHARGED_INV_OBJECT, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		// Czy jest w uyciu
		cFC.SaveBool(m_bInUse);
		// Stan zuzycia baterii w procentach
		cFC.SaveFloat(m_fPower);
		
        cFC.SaveBool(_bUsedBeforeMovie);
        
        cFC.delete();
    }	    								
}
