/** Ladunek wybuchowy */
public class Explosive extends InvObjectInst
{
    /** czas po jakim ekspoduje ladunek od aktywacji */
    public float fDetonationDelay = 20.0f;
    
    /** Zasieg wybuchu obiektu w jakim odbierana jest energia */
    public float fBlastRange = 500.0f;//#Explosive
    
    /** Blast damage odbierany w centrum wybuchu */
    public float fBlastDamage = 300.0f;//#Explosive
    
    /** Nazwa FXow do odpalenia przy wybuchu */
    public String sBlowFX = "FXBlowBarrel";

    /** Nazwa dzwieku odpalanego przy wybuchu*/
    public String sBlowSound = "Data/Sounds/Weapon/GrenadeExplosion.wav";
    
    /** Nazwa dzwieku odgywanego, gdy bombka jest aktywna */
    public String sActiveSound = "Data/Sounds/BombBeep.wav";

	float fMinSoundDist = 200f;
	float fMaxSoundDist = 1000f;
	
	/** Numer dzwieku odgywanego, gdy bombka jest aktywna */
    int nActiveSound_id = 0;
    
    /** odleglosc od obiektu, terenu z jakiej mozna przykleic bombke*/
    float fDistance = 350.0f;

	public boolean bUseDefaultTimer = false;//#Explosive
	public float fDefaultTimer = 15f;//#Explosive
	
    /** Znacznik, ze ma sie attachowac do celu na ktorym sie go mountuje */
    public boolean bAttachToTarget = true;//#Explosive
    
	Collision m_Collision = null;
	Vector m_vEnd = null;
	
	public EnumDamageType cDamageType = new EnumDamageType(EnumDamageType._BLAST);//#\EXPLOSIVE; Damage type of explosive
	
    Explosive()
    {
        super();
        MeshName="Data/Meshes/Equipment/bomba_zdalna.3DA";
        
        nInvIndex = InvObject._E_EXPLOSIVE;
        nInvType = EInvObjType._EQUIP;
        sName = "&E_N_EXPLOSIVES&";
        sTexFileName = "Data/Textures/CH_I_Weapons2.png";
      	nCellsWidth = 2;
	    nCellsHeight = 2;
        
        nOnTexPosX = 160.0f;
        nOnTexPosY = 180.0f;
        
        nTexSizeX = 256.0f;
        nTexSizeY = 256.0f;

        fHealth = 1.0f;
    }
	
    public void OnCreate()
    {
        super.OnCreate();
            
		ClearFlag(EFlags._BLOCK_MOVE);
        bActiveTrigger = true;
        fDamageThreshold = 100000000;
    }
    
    /** metoda powodujaca uzycie przedmiotu - przyklejenie bombki do sciany,
     * terenu, obiektu... */
    public void Use()
    {
        // szukam punktu, w ktorym umieszczamy bombke
        Vector vStart = ((ChromeModuleSingle) GetModule()).MainPlayer.Camera.GetPosition();
        m_vEnd = ((ChromeModuleSingle)GetModule()).MainPlayer.Camera.GetForwardVector();
        m_vEnd.Mul(-fDistance);
        m_vEnd.Add(vStart);
        
		m_Collision = null;
		
		Collision coll = TraceTerrain(vStart, m_vEnd);
		if (coll != null)
			m_Collision = coll;

		coll = TraceTrees(vStart, m_vEnd);
		if (coll != null)
			m_Collision = coll;
		
		coll = TraceObjectsExclude(vStart, m_vEnd, Owner);
		if (coll != null)
			m_Collision = coll;
		
		if (m_Collision != null)
		{
			if ( !(m_Collision.Obstacle instanceof Pawn) )
			{
				RetrieveTimeToExplode();
				return;
			}
		}
		Owner.TextMessage("&BOMB_CANNOT_PLANTED_POSITION&");                                                    
    }
	
	/** ustawienie czasu do wybuchu */
	public void RetrieveTimeToExplode()
	{
		if (bUseDefaultTimer)
		{
			PlantBomb();
			SetTimer(fDefaultTimer);
		}
		else
		{			
			PlayerHUD player = (PlayerHUD) ((ChromeModuleSingle) GetModule()).MainPlayer;
			player.HUD.GetHUDTimer().SetExplosive(this); // ustalamy referencje na explosive
			player.HUD.GetHUDTimer().Activate(); // probujemy aktywowac
			if (player.HUD.GetHUDTimer().bActiveFlag)
				PlantBomb(); // przyczepiamy bombe i czekamy na ustawienie zegara
		}
	}

	public void SetTimer(float fTime)
	{
		fDetonationDelay = fTime;
        Plant();
	}
	
	public void Disarm()
	{
        if (bAttachToTarget)
            DetachFromParentAndKeepWorldPos();
		DeactivateOnModuleAlways();
	}
	
	/** ustawienie bomby na obiekcie i czekanie na ustawienie timera */
    void PlantBomb()
    {
        // teraz trzeba obrocic mesha bombki prostopadle do powierzchni
        Vector vNormal = m_Collision.vNormal;
        vNormal.Normalize();

		if (java.lang.Math.abs(vNormal.Dot(Vector.Up)) > 0.999f)
		{
			Vector vUp = Owner.GetRightVector();
			vUp = vUp.Cross(Vector.Up);
			FromUpForwardPos(vUp, vNormal, m_vEnd);
		}
		else
		{
			SetPosition(m_vEnd);
			SetDir(vNormal);
		}
		
        ActivateOnModule();
        EnableRendering(true);
        if (bAttachToTarget && 
            m_Collision.Obstacle instanceof MeshObject) // to glownie sprawdzenie, czy nie null
        {
            if (m_Collision.ObstacleElement != -1)
                ((MeshObject)m_Collision.Obstacle).AttachChildAndKeepWorldPos(this, m_Collision.ObstacleElement);
            else
                ((MeshObject)m_Collision.Obstacle).AttachChildAndKeepWorldPos(this, 0);//Podlacz do roota
        }
    }
	
    public void Plant()
    {
        Owner.TextMessage("&BOMB_PLANTED&");
        // wywalamy bombe z inventory
        Owner.cInventory.RemoveInvObject(this);
		
        StartCountDown();
        nActiveSound_id = AttachSound(sActiveSound, fMinSoundDist, fMaxSoundDist);
		bActiveTrigger = false;
    }
    
    protected void StartCountDown()
    {
        CallMethodIn("Explode", fDetonationDelay);
    }
	
    void Explode()
    {
        if (!bDead)
        {
            super.Die();   
			
            if ( nActiveSound_id != 0)
			{
				DetachSound(nActiveSound_id);
				nActiveSound_id = 0;
			}
        
            // odpalamy odbieranie energii obiektom w promieniu
            if( (fBlastRange > 0.0f) || (fBlastDamage > 0.0f) )
				Blast();
            // efekt wybuchu
            SpawnFX();
            DetachFromParentAndKeepWorldPos();
            // chowamy po wybuchu mesha bombki
            EnableRendering(false);
            DeactivateOnModuleAlways();
            // nie moze sie kasowac, ze wzgledu na savegame!
            //CallMethodIn("delete", 0.0f);
        }
        
    }
	
    /** Odebranie energii obiektom wokol wybuchu */
    void Blast()
    {
        ControlObject[] aObjects = GetObjectsInRadiusFromClass(fBlastRange, "Actor");
        
        Vector vPos = GetPosition();
        for (int i = 0; i < aObjects.length; i++)
        {
            ControlObject cObj = aObjects[i]; 
			
			float fDistance = vPos.Distance(cObj.GetPosition());
			float fDamage = (1f - fDistance / fBlastRange) * fBlastDamage;
			
            cObj.TakeDamage(fDamage, fDamage, this, null, cObj.GetForwardVector(),
							EBones._NON_BONE, cDamageType.GetValue());
        }
    }
	
   /** Efekt wybuchu */
    public void SpawnFX()
    {
        if( sBlowFX != null)
        {
            ExecuteFX(sBlowFX, GetPosition(), Vector.Up);
            //ExecuteFXTrace(sBlowFX, GetPosition(), Vector.Up);
            ExecuteParticleSystem(sBlowFX, GetPosition(), Vector.Up);
        }

        if( sBlowSound != null)
            PlaySound(sBlowSound);
    } 
	
    /** zeby bombe mozna bylo tez wysadzic przed czasem */
    public void Die() 
    {
        if (!bDead)
            Explode();
    }
	
	// poprawione OnTakeDamage //////////////////////////////////////////////////////////////
	public void OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation,
							 Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
	{
        if(nInvState == EInvObjState._ONGROUND)
            Damage(fDamage, nDamageType);
	}

    boolean bDisappeared = false;
    /////////////////////////////////////////////////////////////////////////////
    // Obsluga akcji act_Appear
    public void Disappear(ControlObject cDestinationObj)
    {
        super.Disappear(cDestinationObj);
        
        if (nActiveSound_id != 0)
			StopSound(nActiveSound_id);
        
        bDisappeared = true;
    }
    
    public void Appear(ControlObject cDestinationObj)
    {
        super.Appear(cDestinationObj);
        
        if ( (nActiveSound_id != 0) && (!IsPlaying(nActiveSound_id)) )
			PlaySound(nActiveSound_id);
        
        bDisappeared = false;
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[Explosive.SGLoadChunk]\n");
		
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_EXPLOSIVE)
        {	
			super.SGLoadChunk(cFC);
            
			fDetonationDelay = cFC.LoadFloat();
			
            if (nActiveSound_id != 0)
			{
				DetachSound(nActiveSound_id);
				nActiveSound_id = 0;
			}

			// jest wlaczony alarm i nie jest obiekt ukryty piskanie on
            if (cFC.LoadBool() && !cFC.LoadBool())                
                nActiveSound_id = AttachSound(sActiveSound, fMinSoundDist, fMaxSoundDist);
        }
		
		cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[Explosive.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_EXPLOSIVE, cFCParent);
		
		super.SGSaveChunk(cFC);

		cFC.SaveFloat(fDetonationDelay);
        cFC.SaveBool( nActiveSound_id > 0 );
        cFC.SaveBool( bDisappeared );
    
        cFC.delete();
    }	    	
	
    void Acquire(Module mod, Pawn owner)
    {
        if (nActiveSound_id != 0)
		{
			DetachSound(nActiveSound_id);
			nActiveSound_id = 0;
		}
		
        super.Acquire(mod, owner);
		m_Collision = null;
		m_vEnd = null;
    }	
    
    public void OnAudioReinitialize()
    {
        
        if(nActiveSound_id != 0) 
            nActiveSound_id = AttachSound(sActiveSound, fMinSoundDist, fMaxSoundDist);

    }
}
