public class StationaryCannonController extends ChromeActor
{
	/** dzialko, ktorym steruje kontroler */
	StationaryCannon m_Cannon = null;
	
	/** kto steruje kontrolerem */
	Pawn m_Owner = null;
	
    public String m_sGunName = "";//#StationaryCannonController
	
	public void OnCreate()
	{
		super.OnCreate();
		
		bSG = true;
		
		// probujemy znalezc dzialko, ktorego nazwa jest wpisana w m_sGunName
		if (m_sGunName.length() == 0)
		{
			CrashLog("StationaryCannonController nie posiada odpowiedniej nazwy StationaryCannon\n");
			return;
		}
		GameObject obj = GetObjectByName(m_sGunName);
		if (obj == null)
		{
			CrashLog("Nie istnieje StationaryCannon o nazwie:" + m_sGunName + "\n");
			return;
		}
		
		try
		{
			m_Cannon = (StationaryCannon) obj;
			m_Cannon.m_Controller = this;
		}
		catch (ClassCastException e)
		{
			CrashLog("Nie istnieje StationaryCannon o nazwie:" + m_sGunName + "\n");
			return;
		}
	}
	
    public boolean IsActive(ControlObject Activator)
    {
		if ( (m_Cannon != null) && (m_Owner == null) )
			return true;
		else
			return false;
	}
	
    String GetTip()
    {
        return "Stationary gun [Use]";
    }
	
	/** rozpoczecie kontrolowania dzialka przez playera
	 */
    void Execute(Actor executor)
    {
		if (m_Cannon == null)
			return;
		
        super.Execute(executor);

		if (executor instanceof Player)
		{
			RegisterKeyRequest();
			m_Owner = (Player) executor;
			m_Owner.UnregisterKeyRequest();
			m_Owner.UnregisterMouseAxisRequest();
			m_Owner.UnregisterMouseButtonRequest();
			m_Cannon.StartControllingCannonByPlayer(m_Owner);
		}
    }
	
	/** skonczenie kontrolowania dzialka przez playera
	 */
	void StopControllingByPlayer()
	{
		if (m_Owner == null)
			return;
		
		m_Cannon.StopControllingCannonByPlayer();
		UnregisterKeyRequest();
		m_Owner.RegisterKeyRequest();
		m_Owner.RegisterMouseAxisRequest();
		m_Owner.RegisterMouseButtonRequest();
		m_Owner = null;
	}
	
    void OnInputKey(int nKeyCode, boolean bKeyState, char nKeyAscii)
    {
		if (bKeyState)
		{
			if (nKeyCode == EGameControl.ACTION)
				StopControllingByPlayer();
		}
	}
    
    public void OnInputMouseButton(int nButton, int nState)
    {
        CallOnInputKey(nButton, nState, '0');
    }
	
	/** kontroler ginie, przestajemy sterowac dzialkiem
	 */
	public void Die()
	{
		StopControllingByPlayer();
		
		super.Die();
	}
	
	/** dzialko ginie, kontroler nie ma czym sterowac
	 */
	void CannonDestroyed()
	{
		StopControllingByPlayer();
		m_Cannon = null;
	}
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[StationaryCannonController.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_STATIONARY_CANNON_CONTROLLER)
        {	
			super.SGLoadChunk(cFC);
			
			// kto steruje kontrolerem
			m_Owner = (Pawn)GameObject.cSG.LoadObjectRef(cFC);
			// dzialko, ktorym steruje kontroler 
			m_Cannon = (StationaryCannon)cFC.LoadObjectRef();			
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[StationaryCannonController.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_STATIONARY_CANNON_CONTROLLER, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		// kto steruje kontrolerem
		GameObject.cSG.SaveObjectRef(cFC,m_Owner);		
		// dzialko, ktorym steruje kontroler 
		cFC.SaveObjectRef(m_Cannon);	
			
        cFC.delete();
    }	    					
}
