public class LiftPlatform extends LiftPlatformBase implements iact_LockUnlock
{
    public ObjectRefList cFloors = null;//#LIFTPLATFORM
    
    /** Tablica pieter do windy */
    public LiftPlatformFloor[] aFloors = null;//#LIFTPLATFORM;Array of lift's floors

    public boolean bLockedForPlayer = false;//#LIFTPLATFORM;When set player won't be able to use lift
    
    int nLastFloor    = 0;
    int nCurrentFloor = 0;

    boolean bLocked = false;
    
    /** dzwiek odpalany przy nacisnieciu guzika jazdy */
    String sButtonSound = "Data/Sounds/LockSystemAccept.wav";
    
    LiftPlatform()
    {
        super();
        ActiveElement = "Button_";
    }
    
    void OnCreate()
    {
        super.OnCreate(); 
                    
        ClearFlag(EFlags._NO_ANIMATION | EFlags._NO_ANIMATION_REFRESH);
        SetFlag(EFlags._PORTAL_INDEPENDENT);
        SetTraceCollType(ECollTraceFlags._HULL);
    
        SetLighting(true);
        
        sMovingSound = "Data/Sounds/Lift/LiftMoving.wav";
        sStartSound  = "Data/Sounds/Lift/LiftStart.wav";
        sStopSound   = "Data/Sounds/Lift/LiftStart.wav";
    }
    
    void Init()
    {
        super.Init();
        
        SetState("Stoped");

        // wycigamy na jakim pietrze jest winda na poczatku
        nCurrentFloor = GetNearestFloor();

        if(nCurrentFloor > -1)
        {
            if(aFloors==null)
            {
                // stary system
                MeshObject cObj = (MeshObject)cFloors.GetObject(nCurrentFloor);
                if( cObj instanceof LiftPlatformFloor )
                    ((LiftPlatformFloor)cObj).LiftEntersFloor(this);
            }
            else
            {
                // nowy system
                aFloors[nCurrentFloor].LiftEntersFloor(this);
            }
        }
    }
    
    
    public LiftPlatformFloor GetCurrentFloor()
    {
        LiftPlatformFloor cCurrFloor = null;
        
        if (bActive) // zatrzymala sie to ok, jak nie to null
        {
            if (aFloors != null)
                cCurrFloor = aFloors[nCurrentFloor];
            else if (cFloors != null && 
                     cFloors.GetObject(nCurrentFloor) instanceof LiftPlatformFloor)
                cCurrFloor = (LiftPlatformFloor)cFloors.GetObject(nCurrentFloor);
        }        
        return cCurrFloor;            
    }

    int GetNearestFloor()
    {
        // wyznaczamy na jakim jestesmy pietrze
        int nMinDist = -1;
        float fMinDist = 1e30f;
        
        if(aFloors==null)
        {
            // stary system
            for(int i=0; i<cFloors.GetObjectsCount(); i++)
            {
                MeshObject cObj = (MeshObject)cFloors.GetObject(i);
                float fDist = GetDistance2To(cObj);
                if(fDist < fMinDist)
                {
                    fMinDist = fDist;
                    nMinDist = i;
                }
            }
        }
        else
        {
            // nowy system
            for(int i=0; i<aFloors.length; i++)
            {
                MeshObject cObj = (MeshObject)aFloors[i];
                float fDist = GetDistance2To(cObj);
                if(fDist < fMinDist)
                {
                    fMinDist = fDist;
                    nMinDist = i;
                }
            }
        }
        
        return nMinDist;
    }
    
        
    /** Zwraca true gdy winda ma jechac do gory.
     * Zwraca false gdy winda ma jechac w dol.
     */
    boolean GetDirection()
    {
        int nFirst = ((Integer)aActiveElements.elementAt(0)).intValue();
        return (nActiveElement == nFirst);
    }
    
    String GetTip()
    {
        return GetDirection() ? "&ELEVATOR_MOVE_UP&" : "&ELEVATOR_MOVE_DOWN&";
    }
    
    public boolean IsActive(ControlObject Activator, int nCheckingElement)
    {
        if (bLockedForPlayer || (m_nState == _STATE_MOVING)) return false;
        
        int nFirst = ((Integer)aActiveElements.elementAt(0)).intValue();
        int nDir = (nCheckingElement == nFirst) ? 1 : -1;
        int nNextFloor = nCurrentFloor + nDir;

        int nFloors = aFloors==null ? (cFloors==null ? 0 : cFloors.GetObjectsCount()) : aFloors.length;
        
        return (cFloors!=null || aFloors!=null) && (nNextFloor >= 0) && (nNextFloor < nFloors) && bActive && !bLocked;
    }
    
    /** Przelaczenie toggle'a
     */
    void Execute( Actor executor )
    {
        if(bLocked) return;
        
        if(executor instanceof Player && bLockedForPlayer) return;
        
        super.Execute(executor);
        
        PlaySound(sButtonSound);
        
        boolean bUp = GetDirection();
        
        SetStateMoving( bUp, GetTime() );
    }
    
    void ActionExecute( Actor executor )
    {
        boolean bUp = GetDirection();
        
        PlaySound(sButtonSound);
                
        if ( Role == ERole._AUTHORITY )
        {
            SetStateMoving( bUp, GetTime() );
            
            MsgSendClientState();
        }
        else
            MsgSendServerLiftRunRequest( bUp );
    }
    
    
    /** Powoduje uruchomienie windy do punktu o podanej wysokosci
     */
    public boolean SetStateMoving(boolean bUp, float fTime)
    {
        int nNextFloor = nCurrentFloor + (bUp ? 1 : -1);
        if (aFloors != null && 
            (nNextFloor >= aFloors.length ||
             nNextFloor < 0))
        {
            Tools.CrashLogError("Winda " + this.GetName() + " chce jechac na pietro " + nNextFloor + " a pieter jest " + aFloors.length + "\n");
            Thread.dumpStack();
            return false;
        }
        else
        {
            MeshObject cDest = aFloors==null ? (MeshObject)cFloors.GetObject(nNextFloor) : (MeshObject)aFloors[nNextFloor];
        
            return SetStateMoving( cDest, fTime );
        }
    }
    
    public boolean SetStateMoving(MeshObject cDest, float fTime)
    {		
        if (bLocked) return false;
        if (!bActive) return false;
     
        // CrashLog("[LiftPlatform.Run] to = <"+cDest.GetName()+">\n");
        
        int nNewFloor = -1;
        
        if(aFloors==null)
        {
            // stary system
            // patrzymy do ktorego pietra kaza nam jechac
            for(int i=0; i<cFloors.GetObjectsCount(); i++)
            {
                MeshObject cObj = (MeshObject)cFloors.GetObject(i);
                if( cObj == cDest )
                {
                    // jest obiekt do ktorego jedziemy
                    nNewFloor = i;
                    break;
                }
            }
        }
        else
        {
            // nowy system
            // patrzymy do ktorego pietra kaza nam jechac
            for(int i=0; i<aFloors.length; i++)
            {
                MeshObject cObj = (MeshObject)aFloors[i];
                if( cObj == cDest )
                {
                    // jest obiekt do ktorego jedziemy
                    nNewFloor = i;
                    break;
                }
            }
        }
		
		// CrashLog("[LiftPlatform.Run] nNewFloor = <" + nNewFloor + ">\n");
            
        if((nNewFloor > -1) && (nCurrentFloor != nNewFloor))
        {
            // ustawiamy parametry ruchu
            fMovingDistance =  cDest.GetPosition().fY - GetPosition().fY;
            fMovingDirection = (fMovingDistance > 0) ? 1.0f : -1.0f;
            fMovingDistance = java.lang.Math.abs(fMovingDistance);
            if(bMidFloor && fMovingDirection<0.f) fMovingDistance/=2.f;

            // CrashLog("[LiftPlatform.Run] going\n");
            
            // informujemy pietro o odjezdzie windy
            nLastFloor = nCurrentFloor;
            
            // wlaczamy pietro docelowe
            nCurrentFloor = nNewFloor;
            
            SetStateMoving( fTime );
            
            return true;
        }
        
        return false;
    }
    
    public void SetStateMoving( float fTime )
    {
        GameObject cLastFloor = null; 
        if ( nLastFloor > -1 )
        {
            cLastFloor =  aFloors != null           ?
                          aFloors[ nLastFloor ]     : 
                          cFloors.GetObject(nLastFloor);
        }
        if (cLastFloor instanceof LiftPlatformFloor)
        {
            //CrashLog("[LiftPlatform.Run] sending info to floor <"+cCurrFloor.GetName()+">\n");
            ((LiftPlatformFloor)cLastFloor).LiftLeavesFloor(this);
            nLastFloor = -1;
        }
                
        super.SetStateMoving( fTime );		
    }
        
	public void InMovingState()
	{
        GameObject cFloor = aFloors==null ? cFloors.GetObject(nCurrentFloor) : aFloors[nCurrentFloor];
        GameObject cCurrentFloor = nCurrentFloor > -1 ? cFloor : null;
		
        if (cCurrentFloor instanceof LiftPlatformFloor)
        {
            //CrashLog("[LiftPlatform.Run] sending info to floor <"+cCurrFloor.GetName()+">\n");
            ((LiftPlatformFloor)cCurrentFloor).LiftLiftRunningToFloor(this);
        }				
	}    
    
    void _Stoped(boolean bSG)
    {
        CrashLog("[LiftPlatform.Stoped] <"+bSG+"> <"+GetName()+">\n");	
                 
        bActive = true;
        
        if (nSoundId!=-1) 
		{
			//CrashLog("[LiftPlatformBase:Stoped] : Detached stop sound : <" + nSoundId + ">\n");
            DetachSound(nSoundId); nSoundId = -1;
		}

        if(aFloors==null)
        {
            //dodatkowe blokowanie wszystkich pieter
            for(int i=0; i<cFloors.GetObjectsCount(); i++)
            {
                MeshObject cObj = (MeshObject)cFloors.GetObject(i);
                if ( cObj instanceof LiftPlatformFloor)
                    ((LiftPlatformFloor)cObj).LiftLeavesFloor(this);
            }
        
            // winda zatrzymana, informujemy o tym pietro na ktorym sie zatrzymala
            GameObject cCurrFloor = cFloors.GetObject(nCurrentFloor);
            if (cCurrFloor instanceof LiftPlatformFloor)
            {
                //CrashLog("[LiftPlatform.Run] sending info to floor <"+cCurrFloor.GetName()+">\n");
				if (bSG == true)
					((LiftPlatformFloor)cCurrFloor).SGLiftEntersFloor(this);
				else
					((LiftPlatformFloor)cCurrFloor).LiftEntersFloor(this);
            }
        }
        else
        {
            // nowy system
            //dodatkowe blokowanie wszystkich pieter
            for(int i=0; i<aFloors.length; i++)
				if (i != nCurrentFloor)
					aFloors[i].LiftLeavesFloor(this);
        
            // winda zatrzymana, informujemy o tym pietro na ktorym sie zatrzymala
			if (bSG == true)
				aFloors[nCurrentFloor].SGLiftEntersFloor(this);
			else
				aFloors[nCurrentFloor].LiftEntersFloor(this);
        }
    }
	
	void Stoped()
	{
		_Stoped(false);
	}
	
    void SGStoped()
    {
		_Stoped(true);
    }

    
    boolean Run(MeshObject cFloor) 
    {
        return SetStateMoving( cFloor, GetTime() );
        //TODO: 
    }

    public void Lock()
    {
        bLockedForPlayer = true;
    }
    
    public void Unlock()
    {
        bLockedForPlayer = false;
    }

    public void ToggleLockUnlock()
    {
        if(bLockedForPlayer)
            Unlock();
        else
            Lock();
    }
    
    public boolean IsLocked()
    {
        return bLockedForPlayer;
    }

    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[LiftPlatform.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_LIFT_PLATFORM)
        {
			super.SGLoadChunk(cFC);
		
			bLocked = cFC.LoadBool();
			nLastFloor = cFC.LoadInt();
			nCurrentFloor = cFC.LoadInt();			
            bLockedForPlayer = cFC.LoadBool();
        }
		
		cFC.delete();
	}
			
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[LiftPlatform.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_LIFT_PLATFORM, cFCParent);
		
		super.SGSaveChunk(cFC);

		cFC.SaveBool(bLocked);
		cFC.SaveInt(nLastFloor);
		cFC.SaveInt(nCurrentFloor);
        cFC.SaveBool(bLockedForPlayer);
        
		cFC.delete(); 
	}		
}
