public class LightSignal extends Destroyable implements IAlarmID
{
    public Alarm cAlarm = null;//#\LIGHTSIGNAL;Alarm connected with this Lightsignal
    
	int m_iAlarmID = AlarmsManager._DEFAULT_ALARM_ID;//#LIGHT_SIGNAL
	
    /** Czas rozruchu koguta jak sie go wlaczy */
    public float fStartDelay = 1.0f;//#LIGHT_SIGNAL

    /** Czas zatrzymania koguta jak sie go wylaczy */
    public float fStopDelay = 1.0f;//#LIGHT_SIGNAL

    /** Kolor koguta */
    public Vector cColor = new Vector(0.3f, 0.0f, 0.0f);//#LIGHT_SIGNAL
    
    LightObject cLight = null;
    
    /** Szybkosc obracania sie koguta */	
    public float fRotationSpeed = 180.f;//#LIGHT_SIGNAL

    /** Nazwa obracajacego sie elementu */	
    public String sRotatingElement = "Lustra";//#LIGHT_SIGNAL
    int nRotatingElementID = -1;
    
    /** Znacznik, ze kogut sie aktualnie obraca */
    boolean bIsRotating = false;
    
    /** Zapamietany czas ostatniego update'u obrotu koguta */
    float fLastRotationTime = 0;

	/** numerek dzwieku podpietego do koguta */
	int nAlarmSoundID = 0;

    LightSignal()
    {
        sBlowFX = null;
        fBlastRange = fBlastDamage = 0.0f;
    }
    
    public void OnCreate()
    {
        super.OnCreate();
        
        nRotatingElementID = GetElementID(sRotatingElement);

        cLight = (LightObject)CreateObject("LightObject");
        cLight.SetPosition(Vector.Zero);
        cLight.SetScale(new Vector(500.f, 20.f, 120.f));
        
        ActivateOnModule();
    }
    
    public void OnPostCreate()
    {
        if (cAlarm != null)
            cAlarm.AddAlarmObject(this);
    }
    
    public void SetAlarmID(int iAlarmID)
    {
        m_iAlarmID = iAlarmID;
    }
    
    public int GetAlarmID()
    {
        return m_iAlarmID;
    }
    
    public void OnDestroy()
    {
    }
    
    /** Wystartowanie koguta
     */
    public void Start()
    {
        if (cLight != null)
        {
            AttachChild(cLight, nRotatingElementID);
            cLight.SetColor(cColor, fStartDelay);
        }

        bIsRotating = true;
        fLastRotationTime = 0;
        
    }
    
    /** Zatrzymanie koguta
     */
    public void Stop()
    {
        if (cLight != null)
        {
            cLight.SetColor(new Vector(0.0f,0.0f,0.0f), fStopDelay);
        }
        CallMethodIn("StopRotating", fStopDelay);
    }
    
    void StopRotating()
    {
        bIsRotating = false;
        if(cLight!=null)
        {
            cLight.DetachFromParent();
        }
    }

    public void OnFrame()
    {
        if (bIsRotating)
        {
            float fTime = GetTime();
            if (fLastRotationTime > 0)
            {
                RotateElementWithChildren(nRotatingElementID, Vector.Up, fRotationSpeed * (fTime - fLastRotationTime));
            }
            fLastRotationTime = fTime;
        }
    }
    
    public void Die()
    {
        super.Die();

        // ktos rozwalil koguta, trzeba wylaczyc swiatlo
        StopRotating();
        
        LightDestroyed();
    }
	
    void LightDestroyed()
    {
        DetachAlarmSound();
        
        AlarmsManager cAlarmsMan = ((ChromeModuleSingle) GetModule()).m_AlarmsManager;
        
        // notyfikacja do systemu alarmow, ze sie obiekt skasowal
        AlarmGroup group = cAlarmsMan!=null ? cAlarmsMan.GetAlarmGroupByID(m_iAlarmID) : null;
        
		if (group != null)
			group.LightDestoyred(this);
        
        if (cLight != null)
        {
            cLight.DetachFromParent();
            cLight.delete();
            cLight = null;
        }
    }
    
	public void AttachAlarmSound()
	{
        if (nAlarmSoundID == 0)
            nAlarmSoundID = AttachSound(AlarmsManager._ALARM_SOUND_NAME);
	}
	
	public void DetachAlarmSound()
	{
        if (nAlarmSoundID != 0)
        {
            DetachSound(nAlarmSoundID);
            nAlarmSoundID = 0;
        }
	}
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
	boolean bIsSoundSG = false;
	boolean bIsRotatingSG = false;
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {
		FileChunk.Log("[LightSignal.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);
        
        if (cFC.GetID() == ESGChunksChrome._CHK_LIGHT_SIGNAL)
        {
			super.SGLoadChunk(cFC);

			m_iAlarmID = cFC.LoadInt();
			// czy dzwiek podpiety do koguta
			bIsSoundSG = cFC.LoadBool();
			// Znacznik, ze kogut sie aktualnie obraca 
			bIsRotatingSG = cFC.LoadBool();
			
			cLight = (LightObject)cFC.LoadObject(cLight,this);
        }
		cFC.delete();
	}
	
    /** Metoda zapisujaca dane do Save gracza
     */		
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[LightSignal.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_LIGHT_SIGNAL, cFCParent);
		
		super.SGSaveChunk(cFC);

		cFC.SaveInt(m_iAlarmID);
		// czy dzwiek podpiety do koguta
		cFC.SaveBool(nAlarmSoundID!=0);
		// Znacznik, ze kogut sie aktualnie obraca 
		cFC.SaveBool(bIsRotating);
		
		cFC.SaveObject(cLight);
						
		cFC.delete();
	}				
	
	public void SGUpdate()
	{
		FileChunk.Log("[LightSignal.SGUpdate]\n");
		
		super.SGUpdate();

		fLastRotationTime = 0;
		
		DetachAlarmSound();
								
		if (bIsRotating)
		{
			if (cLight != null)
            cLight.SetColor(new Vector(0.0f,0.0f,0.0f), fStopDelay);
			StopRotating();
			// Stop();		
		}
							
		if (bIsRotatingSG)
			Start();
		else
			Stop();	
				
		if (bIsSoundSG)
			AttachAlarmSound();		
	}

    public void OnAudioReinitialize()
    {
        if (nAlarmSoundID != 0)
        {
            nAlarmSoundID = AttachSound(AlarmsManager._ALARM_SOUND_NAME);
        }
    }


}
