/** Klasa automatycznego dzialka systemow bezpieczenstwa.
 */
public class SecurityCannon extends ChromeActor implements ISecuritySystemComponent, ICannonMesh
{
    public SecuritySystem cSecuritySystem = null;//#\SecurityCannon;Security system controlling this cannon (if null, cannon is autonomic)
    
    public boolean bEnabledOnStart = true;//#\SecurityCannon;Initial state (Enabled/Disabled) of cannon (if not controlled by security system)
	
	/** ilosc strzalow w serii */
    public int nRoundShots = 30;//#SecurityCannon
    /** czestotliwosc, ilosc strzalow na minute */
    public float fRateOfFire = 600f;//#SecurityCannon
    /** czas pomiedzy seriami */
    public float fLongDelay = 0.5f;//#SecurityCannon
    /** zasieg strzalu */
    public float fFireRange = 2000f;//#SecurityCannon
    /** sila razenia */
    public float fFireDamage = 60f;//#SecurityCannon

    public float m_fMaxHAngle = 180.f;//#SecurityCannon
    public float m_fMinVAngle = -90f;//#SecurityCannon
    public float m_fMaxVAngle = 30f;//#SecurityCannon

    public float m_fRotationSpeedVert = 500f;//#SecurityCannon
    public float m_fRotationSpeedHoriz = 500f;//#SecurityCannon
    
	/** czas od zauwazenia przez dzialko goscia do strzalu */
	public float m_fShootVisibleDelay = 0.5f;//#SecurityCannon
    
    /** Strzelajace dzialko */
    protected ICannonController m_Cannon = null;
    
    protected boolean m_bIsEnabled = false;

    
    public SecurityCannon()
    {
        fDamageThreshold = 100.f;
        fHealth = 10000;
        bIsAITarget = true;
    }
    
	void OnCreate()
    {
        super.OnCreate();
		
		bSG = true;

        InitCannon();

        m_iRevolvingBarrel = GetElementID("lufy");
        if (m_iRevolvingBarrel != -1)
        {
            SetElementAnimType(m_iRevolvingBarrel, (byte)(EAnimType._SEQUENCE | EAnimType._TRANSFORM), false);
            SetAnimMorphTime("lufy", 0f);
        }
        
		SetTraceCollType(ECollTraceFlags._HULL);
        SetCollType(ECollFlags.OBJCOL_TYPE_PERFACEOBJECT);
        ClearFlag(EFlags._NO_ANIMATION | EFlags._NO_ANIMATION_REFRESH |
                  EFlags._RECEIVE_SHADOWS | EFlags._CAST_SHADOWS);
        SetFlag(EFlags._BLOCK_MOVE | EFlags._BLOCK_TRACE);
        SetLighting(true);
        if (cSecuritySystem != null)
            cSecuritySystem.RegisterComponent(this);
        else
        {
            if (bEnabledOnStart)
                Enable();
            else
                Disable();
        }
	}

    
    public void Enable()
    {
        m_bIsEnabled = true;
        
        m_Cannon.StartAutonomicHuntingByUser(this);
    }
    
    public void Disable()
    {
        m_bIsEnabled = false;
        m_Cannon.StopAutonomicHunting();
    }
    
    public void ChangeConflictSide(int nNewConflictSide)
    {
        super.ChangeConflictSide(nNewConflictSide);

        if (m_bIsEnabled) 
            Enable(); // zawolaj jeszcze raz, zeby odswiezyc cel
    }
    
    public void LogState()
    {
        CrashLog(GetName() + " enabled: " + m_bIsEnabled + 
                 " side: " + cConflictSide.GetValue() + "\n");
    }
        
    
	void InitCannon()
    {
        // inicjalizujemy guna
        m_Cannon = (ICannonController) CreateObject("CannonController");//TEST
        m_Cannon.InitController(this);
        
		m_Cannon.SetHorizAngSpeed(m_fRotationSpeedHoriz);
		m_Cannon.SetVertAngSpeed(m_fRotationSpeedVert);
        
        m_Cannon.SetFireParams( nRoundShots, fRateOfFire, fLongDelay, 
							    fFireRange, fFireDamage );
		
		// ustalenie rozrzutu broni
        m_Cannon.SetAccuracy(1.5f, 1.5f);

		// ustalenie katow dla security cannon
		m_Cannon.SetTargetFindingAngle(m_fMaxHAngle);
		m_Cannon.SetMaxHorizAngle(m_fMaxHAngle);
		m_Cannon.SetMinVertAngle(m_fMinVAngle);
		m_Cannon.SetMaxVertAngle(m_fMaxVAngle);

        m_Cannon.SetInitialFireDelay(m_fShootVisibleDelay);
    }
	    
                              
	public void Die()
	{
		super.Die();
		
        ExecuteFX("FXBlowBarrel", GetPosition());
        PlaySound("Data/Sounds/Weapon/GrenadeExplosion");
        
        DetachFromParentAndKeepWorldPos();
        
        Disable();
        
        // HACK, bo wypierdala sie w fx'ach 
        EnableRendering(false);
        DeactivateOnModuleAlways();
        FreezeCallMethods();
        
        m_Cannon.Die();
        
        DetachCannonSound();
        
		// nie moe by to robione z powodu SaveGame
        //CallMethodIn("delete", 10f);
	}
    
	public void OnTakeDamage(float fDamage, float fPhSimDamage, GameObject Assassin, Vector vHitLocation,
							 Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
	{
        // powiadom przeciwnika, ze trafil
        if (Assassin instanceof ChromeActor)
            ((ChromeActor)Assassin).NotifyHit(this, nDamageType);

        Damage(fDamage, nDamageType);
	}
	
	void TakeDamage(float fDamage)
	{
		OnTakeDamage(fDamage, fDamage, null, null, null, 0, EnumDamageType._BULLET);
	}
    
    public void SetFXSizeX(float fSizeX)
    {
        fFXSizeX = fSizeX;
    }
    
    public void SetFXSizeY(float fSizeY)
    {
        fFXSizeY = fSizeY;
    }
    
    public void SetFXZero(float fZero)
    {
        fFXZero = fZero;
    }
    
    public void SetFXPosY(float fPosY)
    {
        fFXPosY = fPosY;
    }

    public int GetRole()
    {
        return Role;
    }
    
    int m_iRevolvingBarrel = -1;
    float _MAX_ANGLE_SPEED = 1080f;
    float _ANGLE_ACCEL = 2880f;
    float m_fCurrAngle = 0f;
    float m_fCurrAngleAccel = 0f;
    float m_fCurrAngleSpeed = 0f;
    float m_fLastUpdateTime = 0f;
    boolean m_bFiring = false;
    byte m_iMode = 0;
    
    public void FireStarted(byte iMode)
    {
        m_iMode = iMode;
        m_bFiring = false;
        m_fCurrAngleAccel = _ANGLE_ACCEL;
        m_fLastUpdateTime = GetTime();
    }
    
    public void FireStopped()
    {
        m_bFiring = false;
        m_fCurrAngleAccel = -_ANGLE_ACCEL;
    }
    
    public void UpdateRotationBarrel()
    {
        if (!bDead)
			UpdateCannonSounds(m_fCurrAngleSpeed);
        
        if ( (m_fCurrAngleAccel != 0f) && (m_iRevolvingBarrel != -1) )
        {
            float fT = GetTime();

            m_fCurrAngleSpeed += (fT - m_fLastUpdateTime) * m_fCurrAngleAccel;
            
            if (m_fCurrAngleSpeed <= 0)
            {
                m_fCurrAngleAccel = 0f;
                m_fCurrAngleSpeed = 0f;
            }
            if (m_fCurrAngleSpeed > _MAX_ANGLE_SPEED)
            {
                if (!m_bFiring)
                {
                    m_bFiring = true;
                    m_Cannon.SuperStartFire(m_iMode);
                    m_fCurrAngleAccel = _ANGLE_ACCEL;

                }
                m_fCurrAngleSpeed = _MAX_ANGLE_SPEED;
            }
            
            m_fCurrAngle += (fT - m_fLastUpdateTime) * m_fCurrAngleSpeed;

            RotateElementWithAnim(m_iRevolvingBarrel, Vector.Forward, m_fCurrAngle);
            
            m_fLastUpdateTime = fT;
        }
    }
   
    String m_CannonSoundName = "Data/Sounds/Weapon/JeepCannonSpin";
    int m_iSoundLoopId = 0;
    float m_fMinCannonFreq = 0.7f;
    float m_fMaxCannonFreq = 1.6f;
    
    void AttachCannonSound()
    {
        if ((m_CannonSoundName != null ) && 
            (m_iSoundLoopId == 0) )
        {
             m_iSoundLoopId = AttachSound(m_CannonSoundName, 1000f, 5000f);
        }
    }
    
    void DetachCannonSound()
    {
        if (m_iSoundLoopId != 0)
        {
            DetachSound(m_iSoundLoopId);            
            m_iSoundLoopId = 0;
        }
    }
    
    void UpdateCannonSounds(float fSpeed)
    {
        if (fSpeed == 0f && m_iSoundLoopId == 0)
            return;
        if (fSpeed == 0f)
        {
            DetachCannonSound();
            return;
        }
        if (m_iSoundLoopId == 0)
            AttachCannonSound();
            
        if (m_iSoundLoopId != 0)
        {
            // zmieniamy czestotliwosc
            float fFreq = m_fMinCannonFreq +
                          (m_fMaxCannonFreq - m_fMinCannonFreq) * fSpeed / _MAX_ANGLE_SPEED;
            SetSoundFreq(m_iSoundLoopId, fFreq);
        }
    }

    public void OnFrame()
    {
        UpdateRotationBarrel();
    }

    // implementacja metody z ICannonMesh
    public void SetFireParams(int nShots, float fRateOfFire, float fLongDelay, float fRange, float fDamage)
    {
        if (m_Cannon == null)
            return;
        m_Cannon.SetFireParams( nShots, fRateOfFire, fLongDelay, fRange, fDamage );
    }
    
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[SecurityCannon.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_SECURITY_CANNON)
        {	
			super.SGLoadChunk(cFC);

			// atrybuty edytorowe. zachowywane, bo na obiekcie woane destroy, poza OnDestroy()
	
			// Security system controlling this cannon (if null, cannon is autonomic)
			cSecuritySystem = (SecuritySystem)cFC.LoadObjectRef();
			// Initial state (Enabled/Disabled) of cannon (if not controlled by security system)
			bEnabledOnStart = cFC.LoadBool();			
			// ilosc strzalow w serii
			nRoundShots = cFC.LoadInt();
			// czestotliwosc, ilosc strzalow na minute
			fRateOfFire = cFC.LoadFloat();
			// czas pomiedzy seriami
			fLongDelay = cFC.LoadFloat();		
			// zasieg strzalu
			fFireRange = cFC.LoadFloat();
			// sila razenia
			fFireDamage = cFC.LoadFloat();

			m_fMaxHAngle = cFC.LoadFloat();
			m_fMinVAngle = cFC.LoadFloat();
			m_fMaxVAngle = cFC.LoadFloat();

			// czas od zauwazenia przez dzialko goscia do strzalu
			m_fShootVisibleDelay = cFC.LoadFloat();

			// pozostae atrybuty
		
			// Strzelajace dzialko
			m_Cannon = (ICannonController)cFC.LoadObject(m_Cannon, this);
    
			m_bIsEnabled = cFC.LoadBool();			
			
			m_fCurrAngle = cFC.LoadFloat();
			m_fCurrAngleAccel = cFC.LoadFloat();
			m_fCurrAngleSpeed = cFC.LoadFloat();
			m_fLastUpdateTime = cFC.LoadFloat();
			m_bFiring = cFC.LoadBool();
			m_iMode = cFC.LoadByte();					
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[SecurityCannon.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_SECURITY_CANNON, cFCParent);
		
		super.SGSaveChunk(cFC);
		
		// atrybuty edytorowe. zachowywane, bo na obiekcie woane destroy, poza OnDestroy()
		
		// Security system controlling this cannon (if null, cannon is autonomic)
		cFC.SaveObjectRef(cSecuritySystem);
		// Initial state (Enabled/Disabled) of cannon (if not controlled by security system)
		cFC.SaveBool(bEnabledOnStart);			
		// ilosc strzalow w serii
		cFC.SaveInt(nRoundShots);
		// czestotliwosc, ilosc strzalow na minute
		cFC.SaveFloat(fRateOfFire);
		// czas pomiedzy seriami
		cFC.SaveFloat(fLongDelay);		
		// zasieg strzalu
		cFC.SaveFloat(fFireRange);
		// sila razenia
		cFC.SaveFloat(fFireDamage);

		cFC.SaveFloat(m_fMaxHAngle);
		cFC.SaveFloat(m_fMinVAngle);
		cFC.SaveFloat(m_fMaxVAngle);

		// czas od zauwazenia przez dzialko goscia do strzalu
		cFC.SaveFloat(m_fShootVisibleDelay);

		// pozostae atrybuty
		
		// Strzelajace dzialko
		cFC.SaveObject((GameObject)m_Cannon);
    
		cFC.SaveBool(m_bIsEnabled);
	
		cFC.SaveFloat(m_fCurrAngle);
		cFC.SaveFloat(m_fCurrAngleAccel);
		cFC.SaveFloat(m_fCurrAngleSpeed);
		cFC.SaveFloat(m_fLastUpdateTime);
		cFC.SaveBool(m_bFiring);
		cFC.SaveByte(m_iMode);		
		
        cFC.delete();
    }	    			

	public void SGUpdate()
	{
		super.SGUpdate();
		
		DetachCannonSound();
		
		if (m_bIsEnabled)
			Enable();
		else
			Disable();
		
        m_iSoundLoopId = 0;
	}
    
    public void OnAudioReinitialize()
    {
        if (m_iSoundLoopId != 0)
        {
            m_iSoundLoopId = 0;
            AttachCannonSound();
        }
    }
}
