/**
 * Zmiana podanego parametru mapy
 */
public class CommandMapSetParam extends ConsoleCommandParams
{
    CommandMapSetParam()
    {
        super();
        m_nRequiredAdminRights  = EAdminRights._STANDARD;
        m_nMinParams            = 
        m_nMaxParams            = 2;
        m_bResultsToAll         = true;
    }
    
    
    boolean   m_bBoolParam = false;
    
    void Clean()
    {
        super.Clean();
        m_bBoolParam = false;
    }
   
    final static int _PARAM_SETTING_NAME  = 0;
    final static int _PARAM_SETTING_PARAM = 1;
   
    String GetSParam(int nParam)
    { 
        String sParam = super.GetSParam( nParam );
    
        //drugi parametr (jesli jest stringowy) zawsze 
        //dotyczy przelacznikow "on" "off"
        //to mozemy wykryc czy tak jest
        if ( nParam == _PARAM_SETTING_PARAM )
        {
            if ( (sParam != null) &&
                 ((sParam.equalsIgnoreCase("yes")||
                   sParam.equalsIgnoreCase("no")||
                   sParam.equalsIgnoreCase("on")||
                   sParam.equalsIgnoreCase("off"))) )
            {
                m_bBoolParam = true;
                SetParam( 1, sParam.toUpperCase() );
            }
            else
            {
                SetErrorId( EConsoleCommandError._WRONG_PARAM );
            }
        }
        return sParam; 
    }
    
    int GetIParam(int nParam)
    { 
        String sParam = super.GetParam( nParam );
    
        if ( nParam == _PARAM_SETTING_PARAM )
        {
            if ( (sParam != null) &&
                 (sParam.equalsIgnoreCase("no")||
                  sParam.equalsIgnoreCase("off")) )
            {
                m_bBoolParam = true;
                SetParam( 1, sParam.toUpperCase() );
                return -1;
            }
        }
        
        return super.GetIParam( nParam );
    }

    float GetFParam(int nParam)
    { 
        String sParam = super.GetParam( nParam );
    
        if ( nParam == _PARAM_SETTING_PARAM )
        {
            if ( (sParam != null) &&
                 (sParam.equalsIgnoreCase("no")||
                  sParam.equalsIgnoreCase("off")) )
            {
                m_bBoolParam = true;
                SetParam( 1, sParam.toUpperCase() );
                return -1.0f;
            }
        }
        
        return super.GetFParam( nParam );
    }
    
        
    /**
     * Callback z GameSettings podczas wyszukiwania parametru
     * do zmiany.
     * Tu mozemy stwierdzic czy cos poszlo nie tak,
     * jesli byly jakies problemy to akcja zastosowania
     * parametru nie odbedzie sie.
     */
    boolean VerifyModifier(ModifierNetGameSettings cModifier)
    {
        if ( cModifier == null )
        {
            //nie znalazl zadnego modyfikatora, czyli wpisalismy
            //zla nazwe
            SetErrorId( EConsoleCommandError._MAP_PARAMETER_UNKNOWN );
            return false;    
        }
        
        if ( !PerformedSuccessfully() )
        {
            //cos po drodze bylo nie tak
            return false;
        }

        if ( ! EModifiersNetGameSettings.CanBeUseByConsoleCommand( cModifier.GetId() ) )
        {
            //tego parametru nie mozna modyfikowac z consoli
            SetErrorId( EConsoleCommandError._MAP_PARAMETER_CHANGE_NOT_ALLOWED );
            return false;    
        }
        
        return true;
    }
    
    /**
     * Callback z GameSettings podczas wyszukiwania parametru
     * do zmianu.
     * Informuje ze wlasnie znaleziono (ignorecase) odpowiednia
     * komende i tu przekazywana jest jej poprawna postac
     * (do wyswietlenia w odpowiedzi)
     */
    void FoundSettingName( String sName )
    {
        SetParam( 0, sName );
    }
    
    
    
    void Perform()
    {
        boolean bApplied = m_cGameInfo.GetGameSettings().ParseGameSettingsCommand( this );
        if ( PerformedSuccessfully() ) 
        {
            if ( bApplied )
                m_cGameInfo.EventGameSettingsChanged();
            else
                SetErrorId( EConsoleCommandError._MAP_PARAMETER_NOT_CHANGED );
        }
    }
    
    
    void PackResults(GameObject cMessager)
    {
        cMessager.MessagePackStringBoolTable( GetParams(), 2 );
        cMessager.MessagePackBoolean( m_bBoolParam );
    }
    
    void UnPackResults(GameObject cMessager)
    {
        SetParams( cMessager.MessageUnPackStringBoolTable( "" ) );
        m_bBoolParam = cMessager.MessageUnPackBoolean();
    }
    
    void ShowResults()
    {
        String sResults1 = NetGameTools.MixString(
                                Text.Get("&ConsoleCommandAccepted&"),
                                GetName() );
        String sResults2  = null;
        
        if ( m_bBoolParam )
            sResults2 = GetParam(0)+" "+GetParam(1);
        else
            sResults2 = NetGameTools.MixString(
                        Text.Get("&ConsoleCommandParameterSetTo&"),
                        GetParam(0),
                        GetParam(1) );

        WriteResults( sResults1 + " " + sResults2 );
    }
    

    /*String GetTextCommandName()
    {
        String sName = super.GetTextCommandName();
        if ( PerformFailed() )
            return + " " + GetParams( 0 );
    }*/
    
    void PackErrors(GameObject cMessager)
    {
        super.PackErrors( cMessager );
        cMessager.MessagePackStringBoolTable( GetParams(), 2 );
    }
    
    void UnPackErrors(GameObject cMessager)
    {
        super.UnPackErrors( cMessager );
        SetParams( cMessager.MessageUnPackStringBoolTable( "" ) );
    }
    
    void ShowErrors()
    {
        String sError = GetTextCommandNotAccepted();
        switch ( GetErrorId() )
        {
        case EConsoleCommandError._MAP_PARAMETER_UNKNOWN:            
            sError = sError + " " + Text.Get("&ConsoleCommandParameterUnknown&");
            break;
            
        case EConsoleCommandError._MAP_PARAMETER_CHANGE_NOT_ALLOWED:            
            sError = sError + " " + Text.Get("&ConsoleCommandParameterChangeNotAllowed&");
            break;
            
        case EConsoleCommandError._MAP_PARAMETER_NOT_CHANGED:            
            sError = sError + " " + Text.Get("&ConsoleCommandParameterNotChanged&");
            break;            
            
        default:
            sError = null;
            super.ShowErrors();
        }
        if ( sError != null )
            WriteResults( sError );
    }
    

}
