import java.io.*;
public class ConsoleCommandBaseMap extends ConsoleCommandParams
{
    ConsoleCommandBaseMap()
    {
        super();
        m_nRequiredAdminRights  = EAdminRights._STANDARD;
        m_nMinParams            = 1;
        m_nMaxParams            = 2;
        m_bResultsToAll         = true;
    }
    
    void Clean()
    {
        super.Clean();
        m_sMapName      = null;
        m_nGameType     = ENetGameType.UNKNOWN;
        m_aNetMapInfo   = null;
    }
    
    String          m_sMapName      = null;
    int             m_nGameType     = ENetGameType.UNKNOWN;
    NetMapInfo[]    m_aNetMapInfo   = null;
        
    
    final static int _PARAM_MAP_NAME  = 0;
    final static int _PARAM_GAME_TYPE = 1;
    
    void InitPerform()
    {
        super.InitPerform();
        
        if ( PerformFailed() )
            return;
        
        if ( m_cGameInfo.IsGameOver() )
        {
            SetErrorId( EConsoleCommandError._UNKNOWN );
            return;
        }
        
        m_sMapName  = GetSParam( _PARAM_MAP_NAME );
        m_nGameType = ENetGameType.UNKNOWN;
        if ( GetParamsCnt() > 1 )
        {
            String sGameType = GetSParam( _PARAM_GAME_TYPE );
            m_nGameType      = ENetGameType.GetGameTypeIdByShortName( sGameType );
            if ( m_nGameType != ENetGameType.UNKNOWN )
            {
                sGameType = ENetGameType.GetGameNameShort( m_nGameType );
                SetParam( _PARAM_GAME_TYPE, sGameType );
            }
            else
            {
                SetErrorId( EConsoleCommandError._UNKNOWN_GAMEPLAY_MODE );
            }
        }
        
        if ( PerformFailed() )
            return;
        
        m_aNetMapInfo = FindNetMapInfo();
        if ( m_aNetMapInfo == null )
        {
            SetErrorId( EConsoleCommandError._UNKNOWN );
            return;
        }
    }
    
    void Perform()
    {
    }
    
    final static String  sm_sMapDir = "Data/LevelsNet";
    
    private NetMapInfo[] FindNetMapInfo()
    {
        NetMapInfo[] aMapInfo = null;
        
        File cMapDir = new File( sm_sMapDir );
        if (!cMapDir.exists())
        {
            SetErrorId( EConsoleCommandError._WRONG_MAP_NAME );
            return null;
        }
        
        //tu "wychwicimy" nazwe mapy z uwzglednieniem wielkich liter
        String []aList = cMapDir.list();
		for ( int i = 0 ; i < aList.length ; i++ )
		{
            if ( m_sMapName.equalsIgnoreCase( aList[ i ] ) )
            {
			    String         sDirToCheck = sm_sMapDir + "/" + aList[ i ];
			    java.io.File   cDirToCheck = new java.io.File( sDirToCheck );
			    if ( cDirToCheck.isDirectory() )
                {
                    m_sMapName = aList[ i ];
                    SetParam( _PARAM_MAP_NAME, m_sMapName );
                }
             }
        }
                

        String sDir     = sm_sMapDir + "/" + m_sMapName;
        String sMapFile = sDir + "/"+ m_sMapName + ".map";
        File cFile = new File( sMapFile );
        if ( ! cFile.exists() )
        {
            SetErrorId( EConsoleCommandError._WRONG_MAP_NAME );
            return null;
        }
        
        if ( m_nGameType != ENetGameType.UNKNOWN )
        {
            NetMapInfo cMapInfo = new NetMapInfo( sMapFile );
            if (cMapInfo.SetGameType( m_nGameType ))
            {
                aMapInfo    = new NetMapInfo[1];
                aMapInfo[0] = cMapInfo;
                return aMapInfo;
            }
            SetErrorId( EConsoleCommandError._CANT_RUN_THIS_MAP_ON_GIVEN_GAMEPLAY_MODE );
            return null;
        }
            
        ObjectsList cList = new ObjectsList();
        for ( int iGameType = 0; iGameType < ENetGameType.TYPES_COUNT; iGameType++)
        {
            NetMapInfo cMapInfo = new NetMapInfo( sMapFile );
            if (cMapInfo.SetGameType( iGameType ))
                cList.Add( (Object)cMapInfo );
        }
        if ( cList.GetSize() > 0 )
        {
            aMapInfo = new NetMapInfo[ cList.GetSize() ];
            for ( int i=0; i<cList.GetSize(); ++i )
            {
                aMapInfo[ i ] = (NetMapInfo)cList.GetObjectAt( i );
            }
            cList.Destroy();
            return aMapInfo;
        }
        SetErrorId( EConsoleCommandError._NO_GAMEPLAY_MODE_AVAILABLE_FOR_GIVEN_MAP );
        return null;
    }

    
   
    void PackResults(GameObject cMessager)
    {
        cMessager.MessagePackStringBoolTable( GetParams() );
    }
    
    void UnPackResults(GameObject cMessager)
    {
        SetParams( cMessager.MessageUnPackStringBoolTable("") );
    }

    
    String GetTextCommandName()
    {
        String sName = super.GetTextCommandName();
        if ( GetParamsCnt() > _PARAM_MAP_NAME )
        {
            sName = sName + " " + GetParam(_PARAM_MAP_NAME);
            if ( GetParamsCnt() > _PARAM_GAME_TYPE )
                sName = sName + " " + GetParam(_PARAM_GAME_TYPE);
        }
        return sName;
    }
    
    void ShowErrors()
    {
        String sError = GetTextCommandNotAccepted();
        switch ( GetErrorId() )
        {
        case EConsoleCommandError._WRONG_MAP_NAME:            
            sError = sError + " " + Text.Get("&ConsoleCommandWrongMapName&");
            break;
        case EConsoleCommandError._UNKNOWN_GAMEPLAY_MODE:
            sError = sError + " " + Text.Get("&ConsoleCommandUnknownGameplayMode&");
            break;
        case EConsoleCommandError._CANT_RUN_THIS_MAP_ON_GIVEN_GAMEPLAY_MODE:
            sError = sError + " " + Text.Get("&ConsoleCommandYouCantRunThisMapInGivenGameplayMode&");
            break;
        case EConsoleCommandError._NO_GAMEPLAY_MODE_AVAILABLE_FOR_GIVEN_MAP:
            sError = sError + " " + Text.Get("&ConsoleCommandNoGameplayModeAvailableForGivenMap&");
            break;            
        case EConsoleCommandError._CANNOT_REMOVE_THIS_MAP_FROM_THE_MAP_LIST:
            sError = sError + " " + Text.Get("&ConsoleCommandCannotRemoveThisMapFromTheMapList&");
            break;                        
        default:
            sError = null;
            super.ShowErrors();
        }
        if ( sError != null )
            WriteResults( sError );
    }
}
