public class NetPlayerInfoList extends NetCompetitorInfoList
{
    /**
     * Flaga blokujaca na pewien czas mozliwosc sortowania
     * listy graczy.
     * Robione jest to po to, aby przy kolejnych
     * iteracjach listy graczy miec pewnosc ze nie
     * zmieni sie kolejnosc jej elementow (co mogloby 
     * skutkowac pominieciem ktoregos z nich)
     */
    boolean m_bSortingLocked = false;
                      
    NetPlayerInfoList(NetGameInfoBase cGameInfo)
    {
        super( cGameInfo );
    }

    /** Pobranie NetPlayerInfo z NetCompetitorInfo
     */
    private NetPlayerInfo GetPlayerInfo( NetCompetitorInfo cInfo )
    {
        return cInfo instanceof NetPlayerInfo ? (NetPlayerInfo)cInfo : null;
    }
    
    /** Pobranie NetPlayerInfo po indeksie
     */
    NetPlayerInfo GetPlayerByIndex(int nIndex)
    {
        NetCompetitorInfo cInfo = GetInfoByIndex( nIndex );
        return GetPlayerInfo( cInfo );
    }
       
    /** Pobranie NetPlayerInfo po nicku
     */
    NetPlayerInfo GetPlayerByNick(String sNick)
    {
        NetCompetitorInfo cInfo = GetInfoByName( sNick );
        return GetPlayerInfo( cInfo );
    }
    
    /** Pobranie NetPlayerInfo po idendyfikatorze PID
     */
    NetPlayerInfo GetPlayerByPID(int nPID)
    {
        //TODO: zaimplementowac PIDy
        NetCompetitorInfo cInfo = GetPlayerByHostId( nPID );
        return GetPlayerInfo( cInfo );
    }
    
    /** Pobranie NetPlayerInfo po nicku lub PIDzie
     * Metoda sama musi rozpoznac czy ma do czynienia
     * z nickiem czy z PIDem
     */
    NetPlayerInfo GetPlayerByNickOrPID(String sNickOrPID)
    {
        String sNick = sNickOrPID;
        
        String sFirstLetter = sNickOrPID.substring(0,1);
        if ( ( sFirstLetter.compareTo("0") >= 0 )&&
             ( sFirstLetter.compareTo("9") <= 0 ) )
        {
            int nPID = -1;
            try
            {
                nPID = Integer.parseInt( sNickOrPID );
            }
            catch ( Exception e ){}
            
            return GetPlayerByPID( nPID );;
        }
        else
            return GetPlayerByNick( sNick );
    }
    
    
    
    /** Pobranie NetPlayerInfo ktory ma bNetIsOwner = true,
     * czyli jego wlascicielem jest lokalny gracz
     */
    NetPlayerInfo GetMyPlayer()
    {
        NetCompetitorInfo cInfo = GetMyInfo();
        return GetPlayerInfo( cInfo );
    }
    
    /** Pobranie NetPlayerInfo po identyfikatorze hosta wlasciciela
     */
    NetPlayerInfo GetPlayerByHostId(int nHostId)
    {
        for ( int i=0; i<GetSize(); i++ )
        {
            NetPlayerInfo cInfo = GetPlayerByIndex( i );
            
            if ( cInfo == null )
                continue;
            
            if ( cInfo.GetClientOwnerId() == nHostId )
			{
				return cInfo;
			}
        }
        return null;
    }
    
    /** Pobranie NetPlayerInfo po identyfikatorze sieciowym obiekty
     */
    NetPlayerInfo GetPlayerByNetId(int nNetId)
    {
        NetCompetitorInfo cInfo = GetInfoByNetId( nNetId );
        return GetPlayerInfo( cInfo );
    }
        
    /** Ustawienie wszystkim gracza stanu GameOver
     */    
    void ForcePlayersDieOnGameOver()
    {
        boolean bSortingLocked = m_bSortingLocked;
        m_bSortingLocked = true;
        
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo info = GetPlayerByIndex( i );
            
            if ( info == null )
                continue;
            
			if (info.GetPlayer()==null)
				continue;
            
            info.GetPlayer().ForceDieOnGameOver();
        }
        
        m_bSortingLocked = bSortingLocked;
    }
    
    /** Ustawienie wszystkim gracza stanu GameRestart
     */    
    void ForcePlayersDieOnGameRestart()
    {
        boolean bSortingLocked = m_bSortingLocked;
        m_bSortingLocked = true;
        
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo info = GetPlayerByIndex( i );

            if ( info == null )
                continue;
            
			if (info.GetPlayer()==null)
				continue;
                        
            info.GetPlayer().ForceDieOnGameRestart();
        }
        
        m_bSortingLocked = bSortingLocked;
    }
    
    /**
     * Zdeaktywowanie wszystkich trupkow
     */
    void ForceDeactivateDeadBodyOnGameRestart()
    {
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo info = GetPlayerByIndex( i );
            
            if ( info == null )
                continue;
            
			if (info.GetPlayer()==null)
				continue;
                        
            DeadBody cDeadBody = info.GetPlayer().GetDeadBody();
            if ( cDeadBody != null )
                cDeadBody.ServerDeactivate();
        }
    }
    
    /** Sprawdzenie czy nie ma juz na liscie graczy o
     * podanym nicku, jesli jest, to zostaje on zmodyfikowany
     * Zwracany jest zaakceptowany(zmodyfikowany) nick
     */
    String VerifyPlayerNick( String sRequestedNick )
    {
        //na poczatku to w ogole sprawdzmy czy on chce cos konkretnego
        //jak przyslal smieci to nadajemy mu domyslnego nicka
        if ( ! NetGameTools.IsStringValid( sRequestedNick ) )
            sRequestedNick = "PlayerName";

        //rozpoczynamy prace nad nickiem
        
        //najpierw oczyscmy nicka ze spacji
        int nSafety = 100;
        do {
            int nBegin = sRequestedNick.indexOf( " " );
        
            if ( nBegin < 0 || nSafety-- < 0 )
                break;
            sRequestedNick = sRequestedNick.substring( 0, nBegin ) + sRequestedNick.substring( nBegin+1 );
        } while (true);
        
        //teraz zabezpieczemy sie zeby z przodu nie bylo cyferki
        String sFirstLetter = sRequestedNick.substring(0,1);
        if ( ( sFirstLetter.compareTo("0") >= 0 )&&
             ( sFirstLetter.compareTo("9") <= 0 ) )
        {
            //jest, zatem dodajemy przyrostek
            sRequestedNick = "_" + sRequestedNick;
        }
        
        String sNick = null;
        //teraz bedziemy sprawdzali czy nicki sie nie powtarzaja
        int i = 0;
        do {
            sNick = sRequestedNick;
            //przy pierwszym przejsciu nic nie dopiszemy, 
            //przy kolejnym (znaczy sie znaleziono juz takiego) 
            //dopisujemy przyrostki - numerki 2, 3, 4...
            if ( ++i > 1 )
                sNick = sNick + String.valueOf( i );    
        } while ( GetPlayerByNick( sNick ) != null );
        
        return sNick;
    }
    
    /** Sprawdzenie czy ktos z graczy juz skonczyl gre
     */
    boolean CheckPlayerWon()
    {
        //domyslnie sprawdzamy czy jakis gracz skonczyl...
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
            
			if (cPlayerInfo == null)
				continue;
			
			if ( cPlayerInfo.CheckWon() )
			{
				// The winner is... :-)
				return true;
			}
		}
        return false;
    }
    
    //zwraca ilosc punktow lidera
    int GetLeaderPoints() 
    { 
        int nPoints = Integer.MIN_VALUE;
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
			if (cPlayerInfo == null)
				continue;
			if ( nPoints < cPlayerInfo.GetPoints() )
                nPoints = cPlayerInfo.GetPoints();
		}
        return nPoints;
    }
    //zwraca ilosc fragow lidera
    int GetLeaderFrags() 
    { 
        int nFrags = Integer.MIN_VALUE;
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
			if (cPlayerInfo == null)
				continue;
			if ( nFrags < cPlayerInfo.GetFrags() )
                nFrags = cPlayerInfo.GetFrags();
		}
        return nFrags;
    }
    
    /**
     * Procka zwraca trzy liczby, ktore obrazuja aktualny wynik gry
     * [0] to punkty gracza lokalnego
     * [1] to :
     * - +1 jesli jestesmy liderem i bedziemy podawali nasza 
     * przewage nad drugim zawodnikiem
     * - -1 jesli ktos inny jest liderem i bedziemy podawali nasza
     * strate do niego
     * [2] to _wartosc bezwzgledna_:
     * - przewaga nad vicelideram (jesli my prowadzimy)
     * - strata do lidera (jesli gracz lokalny nie jest liderem)
     * Liczby te beda interpretowane i wyswietlone w hudzie Gamestatus
     */
    int[] GetGameScoreStatus()
    {
        Sort();
        
        NetPlayerInfo cMyPlayer = GetMyPlayer();
        if ( cMyPlayer == null )
            return null;
        
        
        int nMyScore = cMyPlayer.GetScore();
        
        int[] aStatus = new int[ 3 ];
        
        //wlasne punkty ida na poczatek
        aStatus [ 0 ] = nMyScore;
        
        if ( IndexOf( cMyPlayer ) == 0 )
        {
            //jestesmy liderem, zatem
            //zaznaczamy ze ma byc plus przed kolejna wartoscia
            aStatus [ 1 ] = +1;
            
            //znajdzmy kto jest po nas i wpiszmy roznice punktow
            NetPlayerInfo cPlayer = GetPlayerByIndex( 1 );
            if ( cPlayer != null )
                 aStatus [ 2 ] = nMyScore - cPlayer.GetScore();
         }
        else
        {
            //nie jestesmy liderem, wiec
            //zaznaczamy ze ma byc minus przed kolejna wartoscia
            aStatus [ 1 ] = -1;
            
            //pobiezmy lidera i sprawdzmy ile nam brakuje do niego
            NetPlayerInfo cPlayer = GetPlayerByIndex( 0 );
            if ( cPlayer != null )
                aStatus [ 2 ] = cPlayer.GetScore() - nMyScore;
        }
        return aStatus;
    }

    /** Sprawdzenie czy gracze z podanej druzyny juz przegrali
     * TODO: cos mi tu nie gra, sprawdzic to
     */
    boolean CheckPlayersTeamFailed(int nTeamId, float fMinPlayersInGame)
    {
        //domyslnie liczymy graczy, w grach teamowych
        //wezmiemy pod uwage druzyny poszczegolne druzyny
        int nPlayersExcluded = 0;
        int nPlayersInGame   = 0;
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
			if (cPlayerInfo == null)
				continue;
            
            if ( nTeamId != NetTeamInfo.NONE && 
                 nTeamId != cPlayerInfo.GetTeamId() )
                continue;
            
            //ten juz wczssniej odpadl
            if (cPlayerInfo.IsExcluded())
            {
                nPlayersExcluded ++;
                continue;
            }
			
			if ( cPlayerInfo.CheckFailed() )
			{
                //ten juz sie wykluczyl
                cPlayerInfo.SetExcluded( true );
                nPlayersExcluded ++;
			}
            else
            {
                nPlayersInGame++;
            }
		}
        if ( nTeamId == NetTeamInfo.NONE )
		    return (nPlayersExcluded > 0 && nPlayersInGame < fMinPlayersInGame );
        else
            return (nPlayersExcluded > 0 && nPlayersInGame < 1 );
    }
    
    /** Obliczenie aktualne limitu respawnow dla druzyny
     */
    int CalculateTeamRespawnLimitCnt( NetTeamInfo cTeamInfo )
    {
        int nLimit = 0;
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
			if ( cPlayerInfo == null )
				continue;
                
            if ( cPlayerInfo.GetTeamId() != cTeamInfo.GetTeamId() )
                continue;
                
            nLimit += cPlayerInfo.GetRespawnLimitCnt();
        }
        return nLimit;
    }
    
    int m_nLastSize = 0 ;
    boolean Sort()
    {
        //sa momenty w ktorych nie mozna sortowac
        if ( m_bSortingLocked )
            return false;
        
        boolean bOrderChanged = super.Sort();
        if ( m_nLastSize != GetSize() || bOrderChanged )
        {
            m_nLastSize = GetSize();
            //TODO: "pozaznaczac" ktorych ewentualnie nie wyswietlac na HUDzie
            //najpierw zakladamy ze wszyscy widoczni
            for ( int i=0; i<GetSize(); i++ )
	        {
	        	NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
	        	if ( cPlayerInfo == null )
	        		continue;
                cPlayerInfo.SetVisibleOnHudNet( true );
            }
            //ustalic czy to gra teamowa czy indywidualna
            //m_cGameInfo
        }
        return bOrderChanged;
    }
    
    /** Czy nalezy zmienic kolejnosc na liscie?
     */
    boolean WrongOrder(NetCompetitorInfo cCurrent,NetCompetitorInfo cNext)
    {
        //najpierw sortujemy wg druzyn (malejaca zeby NONE wypadly na koniec)
        if ( cCurrent.GetTeamId() < cNext.GetTeamId() )
            return true;
        
        return super.WrongOrder( cCurrent, cNext );
    }
    

    /**
     * Metoda pakuje do komunikatu wysylanego przez cGameInfo
     * aktualne teamy wszystkich graczy na serwerze
     */
    void MessagePackPlayersTeam()
    {
        m_cGameInfo.MessagePackInt( PlayerNetBase.GetNextGlobalChangeSkinCounter() );
        m_cGameInfo.MessagePackByte( (byte)GetSize() );
        for ( int i = 0; i < GetSize(); i++ )
        {
            NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
            
            m_cGameInfo.MessagePackInt( cPlayerInfo.GetNetId() );
            m_cGameInfo.MessagePackByte( (byte)cPlayerInfo.GetTeamId() );
        }
    }

    /**
     * Metoda odpakowuje z komunikatu odebranego przez cGameInfo
     * teamy wszystkich graczy z serwera
     */
    void MessageUnPackPlayersTeam()
    {
        int nChangeSkinCounter = m_cGameInfo.MessageUnPackInt();
        int nSize              = m_cGameInfo.MessageUnPackByte();
        for ( int i = 0; i < nSize; i++ )
        {
            int  nPlayerInfoId = m_cGameInfo.MessageUnPackInt();
            byte nTeamId       = m_cGameInfo.MessageUnPackByte();
            
            m_aReceivedPlayersTeam.addElement(
                              new ReceivedPlayerTeam( nPlayerInfoId,
                                                      nChangeSkinCounter,
                                                      nTeamId )
                                             );
        }
    }
    
    java.util.Vector m_aReceivedPlayersTeam = new java.util.Vector();

    void VerifyPlayersTeam()
    {
        for ( int i = 0; i < m_aReceivedPlayersTeam.size(); i++ )
        {
            if (!( m_aReceivedPlayersTeam.elementAt(i) instanceof ReceivedPlayerTeam ))
            {
                m_aReceivedPlayersTeam.removeElementAt(i);
                i--;
                continue;
            }
                
            ReceivedPlayerTeam cRecvInfo = (ReceivedPlayerTeam)m_aReceivedPlayersTeam.elementAt(i);
            
            NetPlayerInfo cPlayerInfo = GetPlayerByNetId( cRecvInfo.m_nPlayerInfoId );
            if ( (cPlayerInfo != null) && (cPlayerInfo.GetPlayer()!=null) )
            {
                PlayerNet cPlayer = cPlayerInfo.GetPlayer();
                
                if ( cRecvInfo.m_nChangeSkinCounter >
                     cPlayer.GetChangeSkinCounter() )
                {
                    cPlayer.ReloadTeamSkins( cRecvInfo.m_nTeamId,
                                             cRecvInfo.m_nChangeSkinCounter );
                }
                
                m_aReceivedPlayersTeam.removeElementAt(i);
                i--;
            }
        }
        
        if ( m_aReceivedPlayersTeam.size() <= 0 )
            m_cGameInfo.PlayersTeamVerified();
    }
    
    /**
     * Metoda pakuje do komunikatu wysylanego przez cGameInfo
     * stan wszystkich graczy na serwerze
     */
    void MessagePackPlayersState(float fTime)
    {
        m_cGameInfo.MessagePackFloat( fTime );
        m_cGameInfo.MessagePackInt( GetSize() );
        for ( int i = 0; i < GetSize(); i++ )
        {
            NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
            PlayerNet     cPlayer     = cPlayerInfo.GetPlayer();
            
            m_cGameInfo.MessagePackInt( cPlayerInfo.GetNetId() );
            m_cGameInfo.MessagePackByte( cPlayer.GetStateFromServer() );
        }
    }

    /**
     * Metoda odpakowuje z komunikatu odebranego przez cGameInfo
     * stan wszystkich graczy z serwera
     * Zapamietuje ten stan w NetlayerInfo aby po pewnym
     * czasie zweryfikowac ten stan
     */
    void MessageUnPackPlayersState()
    {
        float fTime = m_cGameInfo.MessageUnPackFloat();
        int   nSize = m_cGameInfo.MessageUnPackInt();
        for ( int i = 0; i < nSize; i++ )
        {
            int  nPlayerInfoId = m_cGameInfo.MessageUnPackInt();
            byte nPlayerState  = m_cGameInfo.MessageUnPackByte();
            
            NetPlayerInfo cPlayerInfo = GetPlayerByNetId( nPlayerInfoId );
            if ( cPlayerInfo != null )
            {
                cPlayerInfo.SetReceivedPlayerState( nPlayerState, fTime );
            }
        }
    }

    
    /**
     * Metoda pakuje do komunikatu wysylanego przez cGameInfo
     * pingi wszystkich graczy na serwerze
     */
    void MessagePackPlayersPing()
    {
        m_cGameInfo.MessagePackByte( (byte)GetSize() );
        for ( int i = 0; i < GetSize(); i++ )
        {
            NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
            
            m_cGameInfo.MessagePackInt( cPlayerInfo.GetNetId() );
            m_cGameInfo.MessagePackInt( cPlayerInfo.UpdateAndGetPing() );
            m_cGameInfo.MessagePackInt( cPlayerInfo.UpdateAndGetPacketLoss() );
        }
    }

    /**
     * Metoda odpakowuje z komunikatu odebranego przez cGameInfo
     * pingi wszystkich graczy z serwera
     */
    void MessageUnPackPlayersPing()
    {
        int   nSize = m_cGameInfo.MessageUnPackByte();
        for ( int i = 0; i < nSize; i++ )
        {
            int nPlayerInfoId = m_cGameInfo.MessageUnPackInt();
            int nPing         = m_cGameInfo.MessageUnPackInt();
            int nPacketLoss   = m_cGameInfo.MessageUnPackInt();
            
            NetPlayerInfo cPlayerInfo = GetPlayerByNetId( nPlayerInfoId );
            if ( cPlayerInfo != null )
            {
                cPlayerInfo.SetPing( nPing );
                cPlayerInfo.SetPlayerPacketLoss( nPacketLoss );
                if ( cPlayerInfo.bNetIsOwner )
                {
                    m_cGameInfo.GetConnectionStatusMngr().UpdateLatency( nPing );
                    m_cGameInfo.GetConnectionStatusMngr().UpdatePacketLoss( nPacketLoss );
                }
            }
        }
    }
    
    /**
     * Weryfikacja ostatanio odebranego stanu poszczegolnych
     * graczy.
     * Kazdemu z Playerow "pokazywany" jest jego aktualny stan
     * na serwerze i on sam jesli stwierdzi ze cos sie "rozjechalo"
     * to prosi serwer o szczegolowe dane na swoj temat
     */
    void VerifyPlayersState()
    {
        for ( int i = 0; i < GetSize(); i++ )
        {
            NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
            if ( cPlayerInfo == null )
                continue;
            PlayerNet     cPlayer     = cPlayerInfo.GetPlayer();
            if ( cPlayer == null )
                continue;
                     
            cPlayer.CheckStateFromServer( cPlayerInfo.GetReceivedPlayerState(),
                                          cPlayerInfo.GetReceivedPlayerStateTime() );
        }
    }
 
    /**
     * TODO: tymczasowo
     */    
    void Cheat()
    {
        NetPlayerInfo cMyPlayer = GetMyPlayer();
        if ( cMyPlayer == null )
            return;
        
        for ( int i=0; i<GetSize(); i++ )
		{
			NetPlayerInfo cPlayerInfo = GetPlayerByIndex( i );
			if (cPlayerInfo == null || cPlayerInfo == cMyPlayer)
				continue;
            cPlayerInfo.IncFrags();
        }
    }
}

class ReceivedPlayerTeam
{
    int m_nPlayerInfoId = -1;
    int m_nChangeSkinCounter = 0;
    int m_nTeamId  = 0;
    
    ReceivedPlayerTeam(int nPlayerInfoId,int nChangeSkinCounter,int nTeamId)
    {
        m_nPlayerInfoId         = nPlayerInfoId;
        m_nChangeSkinCounter    = nChangeSkinCounter;
        m_nTeamId               = nTeamId;
    }
}