/** Bazowa klasa dla NetGameInfo,
 * tu beda niektore metody, celem odchudzenia NetGameInfo
 */ 
public class NetGameInfoBase extends GameObject
{
    /** Modul sieciowy, na ktorym wszystko sie kreci. 
     *  Ustawiane w OnCreate automatycznie.
     */
    ModuleNet m_cModule = null;
    
    ////////////////////////////////
    // atrybuty informacji o grze //
    ////////////////////////////////
    
    private int     m_nGameType = ENetGameType.DEFAULT;
    void SetGameType(int nType)   { m_nGameType = nType;   }
    int  GetGameType()            { return m_nGameType;    }
        
    private NetGameSettings m_cGameSettings = null;
    void SetGameSettings( NetGameSettings cSettings ){ m_cGameSettings = cSettings; }
    NetGameSettings GetGameSettings(){ return m_cGameSettings; }
 
    // czas gry, liczony w sekundach, zeby sie za czesto nie zmienial
    private float   m_fGameTime       = 0.0f;
    void    SetGameTime(float fTime){ m_fGameTime = fTime; }
    float   GetGameTime()    { return m_fGameTime;         }
    
    //stan gry, jesli sie skonczyla to dlaczego
    private int     m_nGameResults    = ENetGameResults._GAME_IS_READY_TO_START;
    int     GetGameResults() { return m_nGameResults;      }
    void    SetGameResults(int nResults){ m_nGameResults = nResults; }
    void    AddGameResults(int nFlag){ m_nGameResults |= nFlag; }
    
    boolean IsGameStarted()   { return ((m_nGameResults&ENetGameResults._GAME_IS_GOING)!=0); }   
    boolean IsGameOver()      { return ((m_nGameResults&ENetGameResults._GAME_OVER)!=0); }
    boolean IsGameGoing()     { return ( IsGameStarted() && !IsGameOver() ); }
    boolean IsRoundOver()     { return ((m_nGameResults&ENetGameResults._ROUND_OVER)!=0); }
    boolean IsScoringEnabled(){ return ( IsGameGoing() && !IsRoundOver() ); }
       
    // lista informacji o graczach
    private NetPlayerInfoList  m_cPlayerInfoList = null;
    NetPlayerInfoList  GetPlayerInfoList(){ return m_cPlayerInfoList; }
    // lista informacji o team'ach 
    private NetTeamInfoList    m_cTeamInfoList   = null;
    NetTeamInfoList    GetTeamInfoList()  { return m_cTeamInfoList;   }
    
    private AdminsManager      m_cAdminsManager  = null;
    AdminsManager      GetAdminsManager(){ return m_cAdminsManager; }
    
    NetTimeCompensator m_cTimeCompensator = null;
    
    private ConnectionStatusManager m_cConnectionStatusManager = null;
    ConnectionStatusManager GetConnectionStatusMngr(){ return m_cConnectionStatusManager; }

    //czy bedziemy odpalac gracza czy mamy serwer dedykowany
    private boolean m_bPlayerOnServer = true;
    boolean IsPlayerOnServer(){ return m_bPlayerOnServer; }
    void    SetPlayerOnServer(boolean bVal){ m_bPlayerOnServer = bVal; }
    
    int METHOD_ServerUpdateGame         = 0;
    int METHOD_ServerForceGameOver      = 0;
    int METHOD_MsgSendClientTimeInfo    = 0;
    
    static String PROF_RESTARTROUND_TAB = "NetRestartRound";

    int PROFILE_RestartGame     = 0;
    int PROFILE_CleanGame       = 0;
    int PROFILE_StopGame        = 0;
    int PROFILE_ForcePlayersDie = 0;
    int PROFILE_StartGame       = 0;
    
    void OnCreate()
    {
        LogD(" "+this+"\n");
        
        METHOD_ServerUpdateGame         = RegisterCallMethod("ServerUpdateGame");
        METHOD_ServerForceGameOver      = RegisterCallMethod("ServerForceGameOver");
        METHOD_MsgSendClientTimeInfo    = RegisterCallMethod("MsgSendClientTimeInfo");
    
        PROFILE_RestartGame     = ProfileAdd(PROF_RESTARTROUND_TAB, "RestartGame");
        PROFILE_CleanGame       = ProfileAdd(PROF_RESTARTROUND_TAB, " CleanGame");
        PROFILE_StopGame        = ProfileAdd(PROF_RESTARTROUND_TAB, " StopGame");
        PROFILE_ForcePlayersDie = ProfileAdd(PROF_RESTARTROUND_TAB, "  ForcePlayersDie");
        PROFILE_StartGame       = ProfileAdd(PROF_RESTARTROUND_TAB, " StartGame");
    
        // inicjalizacja obiektu i na serwerze i na kliencie
        if ( GetModule() instanceof ModuleNet )
        {
            m_cModule = (ModuleNet)GetModule();
            
            //to czy jest gracz na serwerze, to zalezy of tego czy serwer
            //ma ustawiona opcje dedicated czy nie
            SetPlayerOnServer( ! m_cModule.GetGameInitInfo().GetServerInfo().GetDedicated() );
        }
        else
        {
            CrashLog(" Obiekt "+this+" powinien istniec tylko na module sieciowym!\n");
            return;
        }
            
        // tworzymy liste informacji o graczach 
        m_cPlayerInfoList = new NetPlayerInfoList( this );
        // i druzynach
        m_cTeamInfoList   = new NetTeamInfoList( this );
        
        m_cAdminsManager = (AdminsManager)CreateObject("AdminsManager");
        
        m_cConnectionStatusManager = (ConnectionStatusManager)CreateObject("ConnectionStatusManager");
        m_cConnectionStatusManager.SetGameInfo( this );

        //teraz trzeba zadbac o GameSettings
        //stworzmy obiekt
        NetGameSettings cGameSettings = cGameSettings = ENetGameType.NewGameSettings( m_nGameType );
        //na serwerze inicjujemy go skryptami
        if ( Role == ERole._AUTHORITY )
        {
            cGameSettings.Init( m_cModule.GetGameInitInfo().cLevels.GetCurrent() );
            
            //zapamietujemy tez jakie modele bedziemy obslugiwali w grze
            m_aAvailableModels = NetPlayerProfile.FindNetModels();
        }
            
        if ( cGameSettings != null  )
        {
            SetGameSettings( cGameSettings );
        }
        else
        {
            CrashLog(" Obiekt ustawien GameSettings "+
                     " dla gry "+this+" nie zostal stworzony poprawnie!\n");
            return;
        }
        
        //Ladujemy potrzebne materialy
        //Uwaga: Ich Zwalnianie bedzie w ModuleNet.OnDestroy()
        LoadUsingMaterials();
    }
    
    public void OnDestroy()
    {
        ShutdownGame();

        if ( m_cTimeCompensator != null )
        {
            m_cTimeCompensator.Stop();
            m_cTimeCompensator.delete();
            m_cTimeCompensator = null;
        }
        
        if ( m_cAdminsManager != null )
        {
            m_cAdminsManager.delete();
            m_cAdminsManager = null;
        }
        
        if ( m_cConnectionStatusManager != null )
        {
            m_cConnectionStatusManager.delete();
            m_cConnectionStatusManager = null;
        }
        
        m_cPlayerInfoList.Destroy();
        m_cTeamInfoList.Destroy();
        m_cTargetInfo = null;
        m_cModule = null;
    }
    
    void UpdateEnableObjectivesIfOwner(int nPlayerInfo)
    {
        NetPlayerInfo cPlayerInfo = GetPlayerInfoByNetId( nPlayerInfo );
        if ( cPlayerInfo == null )
            LogD(" cPlayerInfo == null\n");
        if ( cPlayerInfo != null && cPlayerInfo.bNetIsOwner )
            UpdateEnableObjectives( cPlayerInfo );
    }
    
    void UpdateEnableObjectives(NetPlayerInfo cPlayerInfo)
    {
    }
    
    void InitTimeCompensator()
    {
        if ( m_cTimeCompensator != null )
        {
            m_cTimeCompensator.delete();
            m_cTimeCompensator = null;
        }
        m_cTimeCompensator = (NetTimeCompensator)CreateObject("NetTimeCompensator");
        m_cTimeCompensator.SetGameInfo( this );
    }
    
    void StartTimeCompensator()
    {
        if ( m_cTimeCompensator != null )
             m_cTimeCompensator.Start();
    }
    
    void OnSetGameSettings()
    { 
        /**
         * Na kliencie dopiero po odebraniu GameSettings z serwera 
         * mozna zaktywowac pojazdy, bo dopiero wtedy wiemy czy
         * sa one przewidziane w grze
         */
        if ( Role < ERole._AUTHORITY )
            m_cModule.EnableOrDisableVehicles(); 
    }
    
    
    /**
     * Start i restart gry!!
     */
    void ServerInitGame()
    {
        DisableCallMethod( METHOD_MsgSendClientTimeInfo ); 
        CallMethodEvery( METHOD_MsgSendClientTimeInfo, 1.0f );
    }
    
    /**
     * W tej metodzie serwer powinien skasowac wszystkie obiekty
     * ktore nie zostaly zainicjowane podczas InitGame i ktorych
     * nie zyczymy sobie w danej rozgrywce.
     * Bedzie to przydatne na mapach na ktorych bedzie mozna
     * prowadzic wiele trybow rozgrywek
     */
    void ServerPostInitGame()
    {
        DeleteNotActiveObjectFromClass( "ASSAULTDestroyable" );
        DeleteNotActiveObjectFromClass( "CTFBase" );
        DeleteNotActiveObjectFromClass( "DominationBase" );
    }
    
    void DeleteNotActiveObjectFromClass( String sClassName )
    {
        GameObject []aObjs = GetObjectsFromClass( sClassName );
        if ( aObjs != null )
            for ( int i = 0;  i <aObjs.length; ++i )
            {
                if ( aObjs[ i ] instanceof INetGameObject )
                    ((INetGameObject)aObjs[ i ]).ServerDeleteIfNotActive();
            }
    }
    
    void ServerCleanGame()
    {
        ServerCleanGame( true );
    }
    void ServerCleanGame(boolean bCleanPlayersResults)
    {
        SetGameResults( ENetGameResults._GAME_IS_READY_TO_START );
        if (bCleanPlayersResults)
        {
            m_cPlayerInfoList.CleanResults();
            m_cTeamInfoList.CleanResults();
        }
    }
    void CleanGameTime()
    {
        m_fGameTime = 0.0f;
    }
    boolean CheckGameShouldStarted()    {       return true;    }
        
    void StartGameIfNeeded()
    {
        if ( CheckGameShouldStarted() )
            ServerRestartGame( true, true/*false*/ );
    }
    //flaga mowiaca czy to pierwszy restart, docelowo zamienic
    //na int'a i zliczac rundy
    boolean m_bFirstRestart = true;
    int m_nRestartMode = _RESTART_MODE_UNKNOWN;
    final static int _RESTART_MODE_UNKNOWN              = 0;
    final static int _RESTART_MODE_NEXT_ROUND           = 1;
    final static int _RESTART_MODE_NEXT_REVENGE_ROUND   = 2;
    final static int _RESTART_MODE_REVENGE_MATCH        = 3;
        
    void ServerRestartGame()
    {
        ServerRestartGame( true, true );
    }
    //TODO: zrobic restart gry (respawny, czyszczenie danych)
    void ServerRestartGame(boolean bCleanPlayersResults, boolean bStopGame)
    {
        ProfileBegin( PROFILE_RestartGame );
        
        //TODO: jezeli nie zatrzymujemy gry a czyscimy
        //wyniki gracza, to Ci aktualnie wyspawnowani maja
        //jeden spawn mniej na koncie, !!
                
        ProfileBegin( PROFILE_CleanGame );
        ServerCleanGame( bCleanPlayersResults );
        //zeby mozna bylo dokonywac zmian w wynikach
        AddGameResults( ENetGameResults._GAME_IS_GOING );
        NetLog.Profile("CleanGame "+ProfileEnd(PROFILE_CleanGame));
        
        if (bStopGame)
        {
            ProfileBegin( PROFILE_StopGame );
            //zablokowac rozsylanie komunikatow
            DisableMessages();
            ServerStopGame();
            //przewrocic rozsylanie komunikatow
            EnableMessages();
            NetLog.Profile("StopGame "+ProfileEnd(PROFILE_StopGame));
        }
        
        ProfileBegin( PROFILE_StartGame );
        ServerStartGame();
        NetLog.Profile("StartGame "+ProfileEnd(PROFILE_StartGame));
        
        NetLog.Profile("RestartGame (calosc) "+ProfileEnd(PROFILE_RestartGame));
    }
    
    //Ta metoda powinna zapewnic aby:
    //wszyscy gracze byli w trybie DieState
    //a w klasach potomnych nalezy wylaczyc wszystkich graczy
    //zeby "pozbyli" sie wszelkich cech swiadczacych o tym ze sa w grze
    //np niesienie flagi, kontrolowanie bazy, itp.
    void ServerStopGame()
    {
        ProfileBegin( PROFILE_ForcePlayersDie );
        m_cPlayerInfoList.ForcePlayersDieOnGameRestart();
        NetLog.Profile("-ForcePlayersDie "+ProfileEnd(PROFILE_ForcePlayersDie));
                
        //wyczyscic mape!
        //TODO: przeniesc to gdzies to startu!
        m_cModule.ServerRestartLevel();
    }
    
    void ServerStartGame()
    {
        // inicjalizacja zliczania czasu i sprawdzania
        // warunkow zakonczenia gry
        DisableCallMethod( METHOD_ServerUpdateGame );
        CallMethodEvery( METHOD_ServerUpdateGame , 1.0f );
        //w czasie trwania gry message bedzie wolany "recznie" po zmianie czasu
        DisableCallMethod( METHOD_MsgSendClientTimeInfo );
        
        MsgSendClientGameRestart();    
        SetGameRestart();
        
        m_bFirstRestart = false;
        m_nRestartMode  = _RESTART_MODE_UNKNOWN;
    }
       
    
    void ShutdownGame()
    {
        DisableMessages();
        ModuleNet.GameIsGoing( false );
        //ServerStopGame();
    }

    /////////////////////////////////////////////////////
    ////////////////////////////////////////////////////
    
    /**SERVER updejt licznika czasu i sprawdzanie scenatiusza gry
     */
    void ServerUpdateGame()
    {
        if (Role < ERole._AUTHORITY) 
			return;
		
        if ( ! ServerProcessIsGameOver() )
        {
             //TODO: liczyc dokladnie czas
			m_fGameTime += 1.0f;
            
            MsgSendClientTimeInfo();
        }
        //m_cPlayerInfoList.UpdatePings();
    }
    
    boolean ServerProcessIsGameOver()
    {
        if ( ServerCheckGameOver() )
        {
            ServerGameOver();
            return true;
        }
        return false;
    }
    
    void ServerForceGameOver()
    {
        if ( ! IsGameOver() )
            ServerGameOver();
    }
    void ServerGameOver()
    {
        if (Role < ERole._AUTHORITY) 
			return;
        
        if (m_cModule != null)
        {
            //mapa nie ma znaczenia
            m_cModule.SetServerState(EServerState.GAME_OVER, null);
        }
        
        AddGameResults( ENetGameResults._GAME_OVER );
        
        DisableMessages();
        m_cPlayerInfoList.ForcePlayersDieOnGameOver();
        EnableMessages();
        
        MsgSendClientGameOver();
        SetGameOver();
        
        DisableCallMethod( METHOD_ServerUpdateGame );
        //jak sie nie updateuje czas to chociaz niech pingi pojda w sieci
        DisableCallMethod( METHOD_MsgSendClientTimeInfo );
        //CallMethodEvery( METHOD_MsgSendClientTimeInfo, 1.0f );
        
        // send empty msg just to know that clients live
        // send interval is half of disconnection timeout
        CallMethodEvery( "MsgSendClientGameOverPing", GameObject.GetDisconnectTime()/2.f );
    }
    
    void SetGameOver()
    {
        ENetGameResults.Log( GetGameResults() );
        if ( m_cTimeCompensator != null )
             m_cTimeCompensator.Stop();
    }
    
    
    
    void SetGameRestart()
    {
        ENetGameResults.Log( GetGameResults() );
        
        int nMsg = NetIncomingMsg._GAME_STARTED;
        if ( !m_bFirstRestart )
        {
            switch ( m_nRestartMode )
            {
            case _RESTART_MODE_NEXT_ROUND:
                nMsg = NetIncomingMsg._GAME_RESTARTED_NEXT_ROUND;
                break;
            case _RESTART_MODE_NEXT_REVENGE_ROUND:
                nMsg = NetIncomingMsg._GAME_RESTARTED_NEXT_REVENGE_ROUND;
                break;
            case _RESTART_MODE_REVENGE_MATCH:
                nMsg = NetIncomingMsg._GAME_RESTARTED_REVENGE_MATCH;
                break;
            default:            
                nMsg = NetIncomingMsg._GAME_RESTARTED;
            }
        }
                
        AddIncomingMessage( nMsg );
        OnScoreChange();
    }
            
    boolean ServerCheckGameOver()
    {
        //1. czas gry
        if ( CheckGameTimeOut() )
        {
            m_fGameTime = GetGameSettings().GetGameTimeLimit();
            
            AddGameResults( ENetGameResults._GAME_OVER_REASON_TIME_OUT  );
            return true;
        }
        
        //2. osiagniecie przez ktoregos z graczy (druzyne) celu
        if ( CheckGameWon() )
        {
            AddGameResults( ENetGameResults._GAME_OVER_RESON_SCORED_MAX );
            return true;
        }
            
        //3. wyczerpnanie sie limitow (np. respawnow)
        if ( CheckGameFailed() )
        {
            AddGameResults( ENetGameResults._GAME_OVER_REASON_RESPAWN_LIMIT_OUT  );
            return true;
        }
        
        return false;
    }
    
    //czy juz skonczyl sie czas gry
    boolean CheckGameTimeOut()
    { 
        return GetGameSettings().GetGameTimeLimit() >  -1.0 && 
               GetGameSettings().GetGameTimeLimit() <= m_fGameTime; 
    }
    //czy ktos juz skonczyl gre?, do pokrycia
    boolean CheckGameWon()    { return false;  }
    int GetLeaderPoints()     { return 0; }
    int GetLeaderFrags()      { return 0; }
    int[]   GetGameScoreStatus(){ return null; }
    //czy wszyscy juz sie "wytlukli"?, do pokrycia
    boolean CheckGameFailed() { return false;  }
        
    boolean CheckPlayersTeamFailed(int nTeamId)
    {
        //uwaga: sprawdzamy to na liscie graczy!!!!
        return GetPlayerInfoList().CheckPlayersTeamFailed( nTeamId, 
                                                           GetGameSettings().GetMinNumberActivePlayersInGame() );
    }
   
    ////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////////
    //Metody narzedziowe - wyszukujace
    
    NetPlayerInfo GetPlayerInfoByNetId(int nNetId)
    {
        GameObject object = GetObjectByNetId( nNetId );
        if ( object instanceof NetPlayerInfo ) 
            return (NetPlayerInfo) object;
        else
        {
            if ( object != null )
                LogD(": Object "+object+" (id "+String.valueOf(nNetId)+") should have been inherited from NetPlayerInfo\n");
        }
        return null;
    }
    
    NetTeamInfo GetTeamInfoByNetId(int nNetId)
    {
        GameObject object = GetObjectByNetId( nNetId );
        if ( object instanceof NetTeamInfo )
            return (NetTeamInfo) object;
        else
        {
            if ( object != null )
                LogD(": Object "+object+" (id "+String.valueOf(nNetId)+") should have been inherited from NetTeamInfo\n");
        }
        return null;
    }

    NetTeamInfo GetTeamInfoById( int nId )
    {
        return m_cTeamInfoList.GetTeamById( nId );
    }
    
    NetTargetInfo m_cTargetInfo = new NetTargetInfo( "", 100.0f );  
        
    NetTargetInfo GetTargetInfo( ChromeActor cTargetActor, NetPlayerInfo cObserver )
    {
        if ( cTargetActor == null || cObserver == null )
            return null;
        
        int nObserverTeam = cObserver.GetTeamId();
        
        if ( cTargetActor instanceof PlayerNet )
        {
            NetPlayerInfo cTarget   = ((PlayerNet)cTargetActor).GetPlayerInfo();
            if (( cTarget == null ) || (cTarget.GetPlayer() == null))
                return null;
            
            int nTargetTeam   = cTarget.GetTeamId();
            if ( ! IsTeamAllowedInTargetInfo( nObserverTeam, nTargetTeam ) )
                return null;
        
            float fHealth     = cTarget.GetPlayer().fHealth;
            float fMaxHealth  = cTarget.GetPlayer().fMaxHealth;
            m_cTargetInfo.Set( cTarget.GetNick(), 
                               100.0f * fHealth / fMaxHealth,
                               nTargetTeam );
            return m_cTargetInfo;
        }
        else if ( cTargetActor instanceof Vehicle )
        {
            Vehicle cVehicle = (Vehicle)cTargetActor;
            
            Pawn[] aPawns = cVehicle.GetAllOccupants();
            if ( (aPawns == null) || (aPawns.length<=0)) 
                return null;
            
            if ( !(aPawns[ 0 ] instanceof PlayerNet) )
                return null;
            
            PlayerNet cPlayer = (PlayerNet)aPawns[ 0 ];
            if ( cPlayer.GetPlayerInfo() == null )
                return null;
            
            int nTargetTeam = cPlayer.GetPlayerInfo().GetTeamId();
            if ( ! IsTeamAllowedInTargetInfo( nObserverTeam, nTargetTeam ) )
                return null;
            
            PlayerNet cLocalPlayer = cObserver.GetPlayer();
            String[] aPassengers = new String[aPawns.length];
            int nPassengerIndex = 0;
            for ( int i = 0; i < aPawns.length; ++i )
            {
                if (aPawns[ i ] == cLocalPlayer)
                    return null;
                
                if (aPawns[ i ] != null )
                    aPassengers[ nPassengerIndex++ ] = aPawns[ i ].GetName();
            }
            
            m_cTargetInfo.Set( aPassengers, nTargetTeam );
            return m_cTargetInfo;
        }
        
        return null;
    }
    
    boolean IsTeamAllowedInTargetInfo( int nObserverTeam, int nTargetTeam )
    {
        return ((GetGameSettings()!=null)&&
                (GetGameSettings().IsValid())&&
                (GetGameSettings().GetIdentifyEnemyPlayers()));
    }

    
    /** Ta metoda powinna zostac pokryta w potomnych klasach
     * i zwracac punkty: fragi, smierci itp
     * nParam oznacza ktora wartosc potrzebujemy (do sortowania wielopoziomowego)
     * Jesli chcemy aby dane wartosc byly sortowane malejaco
     * nalezy podac wartosc z minusem
     * Domyslnie sortuje po fragach i smierciach
     */
    int GetPointsFromInfoToCompare( NetCompetitorInfo info, int nParam )
    {
        switch ( nParam )
        {
            case 0: return info.GetPoints();
            case 1: return info.GetFrags();
        }
        return 0;
    }
    
    /////////////////////////////////////////////////////
    /////////////////////////////////////////////////////
    /////////////////////////////////////////////////////
    //Metody narzedziowe - tworzenie / kasowanie graczy
    
    /** SERVER 
     *  Stworzenie lokalnego gracza na serwerze
     */
    void ServerCreateMyPlayer()
    {
        //ale najpierw bociki
        while ( nPlayersCntToAdd > 0 )
            CheckAdditionalPlayersToCreate();
        
        // serwer ma id = 0 !
        ServerCreatePlayer( NetPlayerProfile.GetCurrent(), 0/* nClientID*/ );
    }
    
    int ServerCreatePlayer( NetPlayerProfile cProfile, int nClientId )
    {
        // sprawdzamy zgodnosc danych playera
        VerifyPlayerProfile( cProfile );
        
        //ustalmy klase gracza
        String sPlayerClassName = ENetGameType.GetPlayerClassName( GetGameType() );
        
        
        // prosimy modul o stworzenie gracza
        return m_cModule.CreateNewPlayer( cProfile, 
                                          nClientId, 
                                          sPlayerClassName );
    }
    
    /**Dodanie informacji o grze nowego gracza.
     */
    NetPlayerInfo ServerAddPlayerInfo( PlayerNet cNewPlayer,int nClientId )
    {
        // tworzymy nowy obiekt informacji o graczu, 
        // automatycznie dodaje sie on w OnCreate do tego obiektu
        //TODO: zmienic na NoInit, po czym ustawic parametry do definiowania replikacji
        //ewentualnie dorobie klasy potomne NetPlayerInfo
        NetPlayerInfo cPlayerInfo = (NetPlayerInfo)CreateObjectNet("NetPlayerInfo",nClientId);
        
        // powiazanie na serwerze playerinfo z playerem
        cPlayerInfo.ServerInit( cNewPlayer, nClientId );
        // i odwrotnie		        		        
        cNewPlayer.SetPlayerInfo( cPlayerInfo );
        //cNewPlayer.nNetPlayerInfo = cPlayerInfo.GetNetId();
        // na kliencie zrobi sie to poprzez replikacje
				
	    int  nTeamId = FindTeamIdForNewPlayer( cPlayerInfo );
        if ( nTeamId != NetTeamInfo.NONE )
            cPlayerInfo.AssignTeam( nTeamId );
        
        RefreshStatistics();
        
        return cPlayerInfo;
    }
    
    
    /**Usuniecie informacji o grze gracza.
     */
    void ServerDeletePlayerInfo(PlayerNet cPlayer)
    {
        // usuwam obiekt informacji o grze gracza
        NetPlayerInfo cPlayerInfo = cPlayer.GetPlayerInfo();
        cPlayerInfo.RemoveFromGameInfo();
        cPlayerInfo.DeleteInFewSeconds();
        
        RefreshStatistics();
    }   
        
    void AddPlayerInfo(NetPlayerInfo cPlayerInfo)
    {
        if ( cPlayerInfo != null )
        {
            m_cPlayerInfoList.Add( cPlayerInfo );
            cPlayerInfo.SetGameInfo( this );
        }
    }
    
    void RemovePlayerInfo(NetPlayerInfo cPlayerInfo)
    {
        if ( cPlayerInfo != null )
        {
            m_cPlayerInfoList.Remove( cPlayerInfo );
            cPlayerInfo.SetGameInfo( null );
        }
    }
                            
    void AddTeamInfo(NetTeamInfo cTeamInfo)
    {
        if ( cTeamInfo != null )
        {
            m_cTeamInfoList.Add( cTeamInfo );    
            cTeamInfo.SetGameInfo( this );
        }
    }
    
    void RemoveTeamInfo(NetTeamInfo cTeamInfo)
    {
        if ( cTeamInfo != null )
        {
            m_cTeamInfoList.Remove( cTeamInfo );    
            cTeamInfo.SetGameInfo( null );
        }
    }
    
    /////////////////////////////////////////////////////
    /////////////////////////////////////////////////////
    /////////////////////////////////////////////////////
    //Metody narzedziowe - zliczanie wynikow        
    
    float GetTimeToRespawn( PlayerNet cPlayer )
    {
        NetPlayerInfo cPlayerInfo = cPlayer != null ? cPlayer.GetPlayerInfo() : null;
                
        if ( cPlayerInfo == null )
        {
            LogD(" "+cPlayer+"cPlayerInfo == null\n");
            return 0.0f;
        }
        
        return ( cPlayerInfo.GetNextRespawnTime() - GetTime() );
    }

    int GetReasonIfRespawnDisabled(PlayerNetBase player)
    {
        EventPlayerTryToRespawn( player );
        
        //sprawdzmy czy w ogole ktos moze sie wyspawnowac
        if ( !IsGameGoing() )
        {
            if ( IsGameOver() )
                return ERespawnRequestResults._DISABLED_GAME_OVER;
            else //TODO: co z tym zrobic?
                return ERespawnRequestResults._DISABLED_GAME_NOTSTARTED;
        }
        
        if ( player == null || player.GetPlayerInfo() == null )
            return ERespawnRequestResults._DISABLED_NO_REASON;
        
        //sprawdzmy gostkowi limity respawnow
        if ( ! player.GetPlayerInfo().RespawnLimitAvailable() )
            return ERespawnRequestResults._DISABLED_RESPAWN_LIMIT_OUT;
        
        return ERespawnRequestResults._ENABLED;
    }
    
    //Do nadpisania w grze druzynowej, zeby sprawdzic czy
    //gosc ma druzyne ustalona
    void EventPlayerTryToRespawn(PlayerNetBase  player)
    {
    }
        
    void UpdateTeamsRespawnLimitCnt()
    {
        for ( int i=0; i<m_cTeamInfoList.GetSize(); i++ )
        {
            NetTeamInfo cTeamInfo = m_cTeamInfoList.GetTeamByIndex( i );
            
            if ( cTeamInfo == null )
                continue;
            
            UpdateTeamRespawnLimitCnt( cTeamInfo );
        }
    }
    void UpdateTeamRespawnLimitCnt( int nTeamId )
    {
        NetTeamInfo cTeamInfo = GetTeamInfoById( nTeamId );
        UpdateTeamRespawnLimitCnt( cTeamInfo );
    }
    void UpdateTeamRespawnLimitCnt( NetTeamInfo cTeamInfo )
    {
        if ( cTeamInfo != null && GetGameSettings().GetRespawnCntLimit() > 0 )
        {
            int nLimit = m_cPlayerInfoList.CalculateTeamRespawnLimitCnt( cTeamInfo );
            cTeamInfo.SetRespawnLimitCnt( nLimit );
        }
    }
    
    void ChangePlayerTeamSkins(int nPlayerInfo,int nSkin,int nChangeSkinCounter)
    {
        NetPlayerInfo cPlayerInfo = GetPlayerInfoByNetId( nPlayerInfo );
        if ( cPlayerInfo != null && cPlayerInfo.GetPlayer() != null )
            cPlayerInfo.GetPlayer().ReloadTeamSkins( nSkin, nChangeSkinCounter );
    }
    
    
    java.util.Vector m_aAvailableModels = null;
    
    /** Server, weryfikacja poprawnosci danych w NetPlayerProfile
     * w chwili obecnej sprawdzane jest tylko czy nick jest unikalny,
     * jesli nie, to dodawany jest na koncu numerek
     */
    void VerifyPlayerProfile( NetPlayerProfile cProfile )
    {
        //nejpierw weryfikujemy nicka
        String sVerifiedName = 
            m_cPlayerInfoList.VerifyPlayerNick( cProfile.GetPlayerName() );
        cProfile.SetPlayerName( sVerifiedName );
        
        //teraz model gracza
        boolean bModelOk = false;
        String  sDefaultModel = null;
        for ( int i=0; i<m_aAvailableModels.size(); i++ )
        {
            if ( m_aAvailableModels.elementAt( i ) instanceof String )
            {
                String sModelName = NetPlayerProfile.sm_sModelDir +
                                    (String)m_aAvailableModels.elementAt( i );
                if ( sModelName.compareToIgnoreCase( cProfile.GetMeshName() ) == 0 )
                {
                    bModelOk = true;
                    break;
                }
                //jesli bysmy stwierdzili ze wybrany przez gracza mesh nie
                //jest na liscie, to przydzielimy mu pierwszego lepszego z listy
                if ( sDefaultModel == null )
                     sDefaultModel = sModelName;
            }
        }        
        //jesli mesh nie jest prawidlowy, to podmieniamy
        if ( ! bModelOk )
            cProfile.SetMeshName( sDefaultModel );
    }
    
    //do nadpisania w trybie druzynowym
    boolean VerifySpawnPoint( NetSpawnPoint cSpawnPoint, NetPlayerInfo cPlayerInfo )
    {
        return true;
    }
    //do nadpisania w trybie druzynowym
    int FindTeamIdForNewPlayer(NetPlayerInfo cInfo)
    {
        return NetTeamInfo.NONE;
    }
    //do nadpisania w trybie druzynowym
    boolean TryToChangeTeam( NetPlayerInfo cInfo, int nNewTeamId ) 
    { 
        return false; 
    }
    
    /** Serwer, 
     * Trafienie gracza, zwraca mnoznik obrazen, 
     * dla wlaczonego trybu FriendlyFire w grze teamowej
     * domyslnie 1.0f
     * zwracalo wartosc 0..1.0 dla trafien od swojej druzyny
     */ 
    float GetDamageMultiplier( PlayerNetBaseMortal victim, PlayerNetBaseMortal assassin )
    {
        // upewniamy sie ze sie nie zawola na cliecie
        if (Role < ERole._AUTHORITY) 
			return 0.0f;
        //tu mozemy sobie wpisac inne wartosc,jesli chcemy aby w 
        //grze sieciowej obrazenia byly mniejsze/wieksze
        return 1.0f;
    }
    
    /**
     * 
     * Dane wysylane z serwera na klienta w momencie inicjalizacji
     * 
     */
    
    /**
     * GameSettings - czyli wszystkie ustawienia gry:
     */
    //wyslanie z serwera
    void MessagePackGameSettings()
    {
        //pobieramy obiekt z gry
        NetGameSettings  cGameSettings     = GetGameSettings();    
        //konwertujemy jego istotne dane do kontenera
        NetDataContainer cGameSettingsData = cGameSettings.ToDataContainer();
        //kontener wrzucamu do komunikatu
        cGameSettingsData.ToMessage( this );
        //na koncu wrzucamy do komunikatu licznik
        MessagePackInt( cGameSettings.GetNextGlobalCounter() );
    }
    //odebranie na kliencie            
    boolean MessageUnPackGameSettings()
    {
        //teraz tworzymy obiekt kontenera
        NetDataContainer cGameSettingsData = new NetDataContainer();
        //wypelniamy go danymi z przychodzacego komunikatu
        cGameSettingsData.FromMessage( this );
        //odczytujemy tez licznik z serwera
        int nCounter = MessageUnPackInt();
        //teraz pobieramy obiekt game settings
        NetGameSettings  cGameSettings     = GetGameSettings();    
        //sprawdzamy czy odebrany licznik jest zweryfikowany
        boolean bVerified = cGameSettings.VerifyCounter( nCounter );
        if ( bVerified )
        {
            //i jesli tak, wpisujemy do niego dane z konteneta
            cGameSettings.FromDataContainer( cGameSettingsData );
        }
        //zwracamy informacje czy faktycznie zaktualizowalismy
        //obiekt GameSettings
        return bVerified;
    }
    
    void MessagePackObjectives()
    {
        //teraz objectivy, ktore w zasadzie tez mozna zaciagnac z 
        //level content, ale znajduja sie tez osobno w objectivemanagerze
        //pobranie obiektu z gry
        ObjectivesManager  cObjectivesManager = m_cModule.GetObjectivesManager();
        //wrzucenie do kontenera
        NetDataContainer   cObjectivesData = cObjectivesManager.ToDataContainer();
        //kontener do message'a
        cObjectivesData.ToMessage( this );
    }

    //odebranie na kliencie
    void MessageUnPackObjectives()
    {
        //teraz to samo z Objectivami
        //otwieramy kontener
        NetDataContainer cObjectivesData = new NetDataContainer();
        //odczytujemy do niego dane z message'a
        cObjectivesData.FromMessage( this );
        //pobieramy obiekt objectivemanagera z gry
        ObjectivesManager  cObjectivesManager = m_cModule.GetObjectivesManager();
        //i wpisujemy do niego dane
        cObjectivesManager.FromDataContainer( cObjectivesData );
        //po czym starujemy go w trybie klienta
        cObjectivesManager.StartOnClient();
    }
       
    
    void MessagePackCurrentGameStatus()
    {
        //TODO
    }
    
    void MessageUnPackCurrentGameStatus()
    {
        //TODO
    }
    
    //do nadpisania w trybie Team'owym
    void MessagePackPlayersTeamSkins(){}
    void MessageUnPackPlayersTeamSkins(){}
        
    //do nadpisania w trybie Assault
    void MessagePackTeamRole(){}
    void MessageUnPackTeamRole(){}
    void PlayersTeamVerified(){}
    
    ///////////////////////////////////////////////////////////////
    //////////////////////////////////////////////////////////////
    
    boolean SetPhysicsGranularity(float fPhysicsGranularity)
    {
        if ( m_cModule                          != null && 
             PlayerNet.sm_fPhysicsGranularity   != fPhysicsGranularity )
        {
            PlayerNet.sm_fPhysicsGranularity  = fPhysicsGranularity;
            m_cModule.SetPhUpdateGranularity( PlayerNet.sm_fPhysicsGranularity );
            return true;
        }
        return false;
    }
    
    boolean SetMaxResendMoveTime( float fMaxResendMoveTime )
    {
        if ( PlayerNet._MAX_RESEND_MOVE_TIME != fMaxResendMoveTime )
        {
            PlayerNet._MAX_RESEND_MOVE_TIME = fMaxResendMoveTime;
            return true;
        }
        return false;
    }

    boolean SetAutoPlayersFire( boolean bFire )
    {
        if ( PlayerNetTestBot.sm_bCanUseWeapon != bFire )
        {
            PlayerNetTestBot.sm_bCanUseWeapon = bFire;
            return true;
        }
        return false;
    }

    
    boolean m_bClientGameShutdown = false;
        
    boolean SetClientShutdown()
    {
        if ( ! m_bClientGameShutdown )
        {
            m_bClientGameShutdown = true;
            if ( Role < ERole._AUTHORITY )
                CallMethodIn("ForceGameShutdown",2.0f);
            return true;
        }
        return false;
    }
    
    void ForceGameShutdown()
    {
        ReplicationManagerMsg( EReplicationManagerMsg._MSG_GAME_SHUT_DOWN );
    }
    
        
    //wysylanie komunikatow do nadpisania
    void MsgSendClientGameOver()    {}
    void MsgSendClientGameRestart() {}
    void MsgSendServerPing()        {}    
    void MsgSendClientGameStatusMsgUpdated( NetDataContainer cContainer ) {}
    
    ///////////////////////////////////////////////////////////
    /** Do testow
     */
    static int sm_nBotsCnt = 0;
    static void SetBotsCnt(int nCnt){ sm_nBotsCnt = nCnt; }
    int nPlayersCntToAdd    = 0;
    int nAddedPlayers       = 0;
    int nLastAddedPlayerId  = ModuleNet.sm_nBotsIdMin;
    
    void CheckAdditionalPlayersToCreate()
    {
        if ( nPlayersCntToAdd > 0 )
        {
            if ( CreateAdditionalPlayer( nLastAddedPlayerId ) )
            {
                nAddedPlayers ++;
                nPlayersCntToAdd --;
                nLastAddedPlayerId ++;
            }
        }
    }
    /*
     * Tworzenie dodatkowych playerow - bocikow
     */
    boolean CreateAdditionalPlayer(int nId)
    {
        NetPlayerProfile cProfile = new NetPlayerProfile();
        cProfile.Load();
                
        cProfile.SetPlayerName( "Bocik" );
        cProfile.SetBot( true );
        
        // sprawdzamy zgodnosc danych playera
        VerifyPlayerProfile( cProfile );
        
        //ustalmy klase gracza
        String sPlayerClassName = ENetGameType.GetPlayerClassName( GetGameType() );
        
        // prosimy modul o stworzenie dodatkowego gracza
        int nResults = m_cModule.CreateNewPlayer( cProfile, 
                                                  nId, 
                                                  sPlayerClassName );
        
        return (nResults == ECreatePlayerResults._CREATED_SUCCESFULL);
    }
    
    void LogNetObjects()
    {
        LogD(" "+GetTime()+"\n");
        ReplicationManagerMsg( 2 );
    }

    void LoadUsingMaterials()
    {
        
    }
    
    void LoadUsingMaterials(String[] aMaterials)
    {
        if ( aMaterials != null )
            for ( int i=0; i<aMaterials.length; i++ )
                MaterialLoad( aMaterials[i] );
    }
    
    
    void EventServerIsGoingShutdown()
    {
        DisableCallMethod( "EventServerIsGoingShutdownDo" );
        CallMethodIn( "EventServerIsGoingShutdownDo", 3.0f );
    }
    
    void EventServerIsGoingShutdownDo()
    {
        DisableCallMethod( "EventServerIsGoingShutdownDo" );
        ReplicationManagerMsg( EReplicationManagerMsg._MSG_GAME_SHUT_DOWN );    
    }
    
    /**
     * Callback przychodzacy z TimeCompensatora mowiacy o tym
     * ze serwer przestal odpowiadac czyli mamy klopoty z siecia
     */
    void EventServerNotResponding()
    {
        
    }
    
    /**
     * Callback przychodzacy z TimeCompensatora mowiacy o tym
     * ze serwer odpowiedzial wiec chyba klopoty z siecia sie konczyly
     */
    void EventServerResponding()
    {
    }
        
    void EventDisconnectFromServer( int nReason )
    {
        if ( m_cTimeCompensator != null )
             m_cTimeCompensator.Stop();
    }
    
    void EventConnectionToServerLost()
    {
        if ( m_cTimeCompensator != null )
             m_cTimeCompensator.Stop();
    }
    
    void EventDisconnectByConnectionStatusManager()
    {
        if ( m_cTimeCompensator != null )
             m_cTimeCompensator.Stop();
    }
    
    /**
     * Callback przychodzacy z danych gry, ze ulegla zmianie punktacja
     */
    void OnScoreChange()
    {
    }

    
    ///////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////
    ////// HUD sieciowy
    ///////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////////
    
    //do nadpisania
    void    AddIncomingMessage( int nId )  {}
    void    AddIncomingMessage( int nId, String sText )  {}
    void    EnableMessages()               {}
    void    DisableMessages()              {}
    void    RefreshStatistics()            {} 
    HUDNetManager GetHUDNet(){ return null; }
    void    SystemMessageHide(){} 
    
    void OnTeamChange(){}
    boolean IsChangeTeamMode(){ return false; }
    void    LocalEventRequestChangeTeam(){};
        
    void MsgSendClientTimeInfo(){}
    void EventPlayerToKick( int nClientId, String sReason ){}
    void EventPlayerBanned( int nClientId, String sReason ){}
    void EventPlayerChangedNick(NetPlayerInfo cPlayerInfo,String sOldNick){}
    
    
    /**
     * Informacja od lokalnego gracza, ze wlasnie zaczyna tracewac 
     * otoczenie w celu wykrycia aktywnego triggera
     * Bedzie to przydatne w CTFie i trybach Domination aby
     * wymusic odpowiednie flagi do "lapania" traceow na obiektach, 
     * ktore zwykle maja te flagi wylaczone (no flagi hologramowe) 
     */
    void OnGetActiveTrigger(){}
    
    /**
     * Informacja od lokalnego gracza, ze wlasnie skonczyl tracewac 
     * otoczenie w celu wykrycia aktywnego triggera
     * Mozna tu obsluzyc przywrocenie flag obiektow ze stanu 
     * z przed zawolania metody OnGetActiveTrigger()
     * Uzywane w CTFie i trybach Domination
     */
    void OnPostGetActiveTrigger(){}
    
    /** Czy jest przewidzany lokalny gracz */
    //TODO: zmienic, jesli bedzie tryb "spectator only"
    boolean IsLocalPlayer(){ return Role < ERole._AUTHORITY || m_bPlayerOnServer; }
    /** Czy jest wyspawnowany lokalny gracz */
    boolean IsLocalPlayerPresent(){ return GetPlayerInfoList().GetMyPlayer() != null; }
    /** Czy lokalny gracz zyje */
    boolean IsLocalPlayerAlive(){   return IsLocalPlayerPresent() && GetPlayerInfoList().GetMyPlayer().IsAlive(); }
    /** Czy lokalny gracz jest przypisany do jakiejs druzyny? do nadpisania w Teamowych*/
    boolean IsLocalPlayerAssignedToTeam(){ return false; }
    /** Czy lokalny gracz zadal juz spawnu - czy wcisnal FIRE */
    boolean WasLocalPlayerPressedFireToRespawn(){ return IsLocalPlayerPresent() && GetPlayerInfoList().GetMyPlayer().WasPressedFireToRespawn(); }
    
    NetPlayerInfo GetMyPlayerInfo(){ return GetPlayerInfoList().GetMyPlayer(); }
    PlayerNet     GetMyPlayerNet(){ return (GetMyPlayerInfo()!=null)?GetMyPlayerInfo().GetPlayer():null; }
       
    /** Czy jest dostepny InventoryRoom */
    boolean IsInventoryRoomAvailable(){ return m_cGameSettings.IsValid() && m_cGameSettings.GetAvailableInventoryRoom(); }
 
    //TODO FIXIT tylko do testow
    void TestUpdateLatency( int nLatency )
    {
        GetConnectionStatusMngr().UpdateLatency( nLatency );
    }
}

