/** Ta klasa powstala aby "trzymac" wszystkie wspolne rzeczy
 * dla trybow Domination: (SMD,TD), ktore musza dziedziczyc
 * po, odpiwiednio NetGameInfoIndividual i NetGameInfoTeam
 * Mozna tu wpisywac wszystkie inne rzeczy specyficzne dla
 * roznych trybow gier (takie ktore sie powtarzaja w wiecej
 * niz jednym trybie) a ktorych nie chcemy wrzucac do NetGameInfo.
 */
public class NetGameInfoCommon extends NetGameInfo
{
    //maksymalna ilosc baz jaka moze byc obslugiwana (tu, w hudmap, itp)
    final static int  sm_nMaxDominationBases = 8;
    
    private boolean m_bModeDominationTotal = false;
    void SetModeDominationTotal(){ m_bModeDominationTotal = true; }
    boolean IsModeDominationTotal(){ return m_bModeDominationTotal; }
    
    private float m_fTimeToNextDominationScore = -1;
    public  float GetTimeToNextDominationScore(){ return m_fTimeToNextDominationScore; };
    public  void  SetTimeToNextDominationScore(float fTime){ m_fTimeToNextDominationScore = fTime; };
    
    protected java.util.Vector  m_aDominationBases = new java.util.Vector();
    int  GetDominationBaseCnt(){ return m_aDominationBases.size(); }
    DominationBase GetDominationBase(int i)
    { 
        if ( i >= 0 && i < m_aDominationBases.size() )
            return (DominationBase)m_aDominationBases.elementAt( i );
        return null;
    }
    
    void ServerInitDominationGame()
    {
        GameObject[] aBases = GetObjectsFromClass("DominationBase");
        
        if ( aBases == null || aBases.length < 1 )
        {
            CrashLog("Brak bazy\n");
            return;
        }
        
        for ( int i=0; i<aBases.length; i++ )
            InitDominationBase( (DominationBase)aBases[ i ] );
    }
     
    void InitDominationBase(DominationBase cBase) 
    {
        if ( cBase == null )
            return;
        
        if ( m_aDominationBases.size() < sm_nMaxDominationBases )
        {
            m_aDominationBases.addElement( cBase );
            int nIndex = m_aDominationBases.indexOf( cBase );
            cBase.Init( this, nIndex );
        }
    }

    
    float GetDominationScoreUpdateTime()
    {
        return 1.0f;
    }
    
    
    boolean HasAllBasesTheSameOwner(){ return false; }
    
    void UpdateTimeToNextDominationScore()
    {
        if (m_bModeDominationTotal)
            UpdateTimeToNextDominationScoreModeTotal();
        else
            UpdateTimeToNextDominationScoreModeNormal();
    }
            
    void UpdateTimeToNextDominationScoreModeNormal()
    {
        float fTimeToNextScore = -1.0f;
        
        //przegladamy wszystkie bazy
        for ( int i=0; i<m_aDominationBases.size(); i++ )
        {
            DominationBase cBase = (DominationBase)m_aDominationBases.elementAt( i ) ;
                
            if ( cBase == null )
                continue;
            
            if ( HasBaseOwner( cBase ) )
            {
                //jesli ma wlasciciela, ale nie ma zaplanowanego jeszcze
                //czasu, to go zaplanujmy na "od teraz"
                if ( cBase.GetTimeToNextScore() <= -1.0f )
                {
                    float fTime = GetGameTime() + 
                                  GetDominationScoreUpdateTime();
                    cBase.SetTimeToNextScore( fTime );
                }
                         
                //szukamy, najblizszego (najmniejszego) czasu do zliczania
                if ( fTimeToNextScore > cBase.GetTimeToNextScore() ||
                     fTimeToNextScore <= -1.0f )
                {
                    fTimeToNextScore = cBase.GetTimeToNextScore();
                }
            }
            else
                cBase.SetTimeToNextScore( -1.0f );
        }
        
        SetTimeToNextDominationScore( fTimeToNextScore );
    }

    void UpdateTimeToNextDominationScoreModeTotal()
    {
        if ( HasAllBasesTheSameOwner() )
        {
            //wszyscy maja tego samego wlasciciela
            //ustalmy zatem "najpozniejszy" czas, ktory bedzie 
            //obowiazywal przy zliczaniu punktow
            for ( int i=0; i<m_aDominationBases.size(); i++ )
            {
                DominationBase cBase = (DominationBase)m_aDominationBases.elementAt( i ) ;
                
                if ( cBase == null )
                    continue;
                
                //jesli nie ma zaplanowanego jeszcze
                //czasu, to go zaplanujmy na "od teraz"
                if ( cBase.GetTimeToNextScore() <= -1.0f )
                {
                    float fTime = GetGameTime() + 
                                  GetDominationScoreUpdateTime();
                    cBase.SetTimeToNextScore( fTime );
                }
                         
                //szukamy, najpozniejszego (najwiekszego) czasu do zliczania
                if ( GetTimeToNextDominationScore() < cBase.GetTimeToNextScore() )
                     SetTimeToNextDominationScore( cBase.GetTimeToNextScore() );
            }
        }
        else
        {
            //ilosc tych samych wlascicieli jest mniejsza niz ilosc baz, 
            //zatem nie mozna jeszcze zaczac zliczac punktow
            SetTimeToNextDominationScore( -1.0f );
        }
                
        //wpiszmy ten czas wszystkim bazom na liscie
        for ( int i=0; i<m_aDominationBases.size(); i++ )
        {
            DominationBase cBase = (DominationBase)m_aDominationBases.elementAt( i ) ;
            
            if ( cBase == null )
                continue;
            
            cBase.SetTimeToNextScore( GetTimeToNextDominationScore() );
        }
    }

    boolean HasBaseOwner(DominationBase cBase){ return false; }
        
    void SetBaseOwner( DominationBase cBase, NetPlayerInfo cPlayerInfo )
    {
        if ( cBase == null )
            return;
        
        if ( cBase.GetPlayerOwner() != null )
             cBase.GetPlayerOwner().CleanDominationBaseOwner( cBase.GetIndex() );
    
        cBase.SetPlayerOwner( cPlayerInfo );
            
        if ( cBase.GetPlayerOwner() != null )
            cBase.GetPlayerOwner().SetDominationBaseOwner( cBase.GetIndex() );
    }
    
    void SetBaseOwner( DominationBase cBase, NetTeamInfo cTeamInfo )
    {
        if ( cBase == null )
            return;
        
        if ( cBase.GetTeamOwner() != null )
             cBase.GetTeamOwner().CleanDominationBaseOwner( cBase.GetIndex() );
    
        cBase.SetTeamOwner( cTeamInfo );
            
        if ( cBase.GetTeamOwner() != null )
            cBase.GetTeamOwner().SetDominationBaseOwner( cBase.GetIndex() );
    }
    
    void LoadDominationUsingMaterials()
    {
        LoadUsingMaterials( DominationBaseComputer.sm_aUsingMaterials );
        LoadUsingMaterials( DominationBaseFlag.sm_aUsingMaterials );
    }
    
    /**
     * Odnajduje wszystkie obiekty DominationBase na levelu,
     * i kazdemu z nich wymusza flagi do umozliwienia
     * traceowania tych obiektow. Oryginalne flagi sa zapamietywane
     * celem pozniejszego ich przywrocenia
     */
    void StoreFlagsAndEnableGetActiveTriggerForDominationObjs()
    {
        for ( int i=0; i<m_aDominationBases.size(); i++ )
        {
            DominationBase cBase = (DominationBase)m_aDominationBases.elementAt( i ) ;
            cBase.StoreFlagsAndEnableGetActiveTrigger();
        }
    }
    
    /**
     * Przywrocenie wszystkim obiektom DominationBase na 
     * levelu flag zapamietanych podczas wywolania metody
     * StoreFlagsAndEnableGetActiveTriggerForDominationObjs()
     */
    void RestoreFlagsForDominationObjs()
    {
        for ( int i=0; i<m_aDominationBases.size(); i++ )
        {
            DominationBase cBase = (DominationBase)m_aDominationBases.elementAt( i ) ;
            cBase.RestoreFlags();
        }
    }    
   
}
