public class NetGameInfoRound extends NetGameInfoTeam
{
    int METHOD_ServerRestartMatch   = 0;
    int METHOD_ServerRestartRound   = 0;
    int METHOD_EventObjectiveTimeOut= 0;
    
    int PROFILE_RespawnDestroyable = 0;
    int PROFILE_RespawnGlass       = 0;
        
    void OnCreate()
    {
        METHOD_ServerRestartMatch   = RegisterCallMethod("ServerRestartMatch");
        METHOD_ServerRestartRound   = RegisterCallMethod("ServerRestartRound");
        METHOD_EventObjectiveTimeOut= RegisterCallMethod("EventObjectiveTimeOut");
        
        PROFILE_RespawnDestroyable = ProfileAdd(PROF_RESTARTROUND_TAB, "RespawnDestroyable");
        PROFILE_RespawnGlass       = ProfileAdd(PROF_RESTARTROUND_TAB, "RespawnGlass");
        
        super.OnCreate();
    }
        
    NetGameSettingsRound m_cGameSettingsRound = null;
    NetGameSettingsRound GetGameSettingsRound(){ return m_cGameSettingsRound; }
    void SetGameSettings( NetGameSettings cGameSettings )
    {
        super.SetGameSettings( cGameSettings );
        if ( cGameSettings instanceof NetGameSettingsRound )
             m_cGameSettingsRound = (NetGameSettingsRound)GetGameSettings(); 
        else
            CrashLog(" Niepoprawny obiekt GameSettings "+cGameSettings+
                     " dla NetGameInfoRound: "+this+"\n");
    }
    
    
    /**
     * Start i restart gry!!
     */
    void ServerInitGame()
    {
        super.ServerInitGame();
        
        ServerInitObjects();
        ServerRespawnObjects();
    }
    
    boolean m_bDeactivateDeadBodyOnRespawn = false;
    
    void ServerStopGame()
    {
        super.ServerStopGame();
        ServerRespawnObjects();
        m_bDeactivateDeadBodyOnRespawn = true;
    }

    
    //lista obiektow destroyable
    protected java.util.Vector  m_aDestroyableObjects    = new java.util.Vector();
    //lista szyb
    protected java.util.Vector  m_aGlassObjects          = new java.util.Vector();
    
    void ServerInitObjects()
    {
        ServerInitDestroyableObjects();
        ServerInitGlassObjects();
    }
    
    void ServerRespawnObjects()
    {
        ServerRespawnDestroyableObjects();
        ServerRespawnGlassObjects();
    }

    
    void ServerInitDestroyableObjects()
    {
        //pobieramy wszystkie obiektu _pochodne_ od DestroyableNet
        GameObject []aObjects = GetObjectsFromClass("DestroyableNet");
        if ( aObjects == null )
        {
            CrashLog("Brak obiektow destroyable\n");
            return;
        }
        
        for (int i=0; i<aObjects.length; i++)
        {
            DestroyableNet cObj  = (DestroyableNet)aObjects[ i ];
            m_aDestroyableObjects.addElement( cObj );
        }
    }
    
    void ServerInitGlassObjects()
    {
        //pobieramy wszystkie obiektu _pochodne_ od DestroyableNet
        GameObject []aObjects = GetObjectsFromClass("GlassNet");
        if ( aObjects == null )
        {
            CrashLog("Brak obiektow GlassNet\n");
            return;
        }
        
        for (int i=0; i<aObjects.length; i++)
        {
            GlassNet cObj  = (GlassNet)aObjects[ i ];
            m_aGlassObjects.addElement( cObj );
        }
    }
    
    void ServerRespawnDestroyableObjects()
    {
        for ( int i=0; i<m_aDestroyableObjects.size(); i++ )
        {
            if ( m_aDestroyableObjects.elementAt( i ) instanceof DestroyableNet )
            {
                DestroyableNet cObj = (DestroyableNet)m_aDestroyableObjects.elementAt( i );
                cObj.ServerRespawn();    
            }
        }
    }
    
    void ServerRespawnGlassObjects()
    {
        for ( int i=0; i<m_aGlassObjects.size(); i++ )
        {
            if ( m_aGlassObjects.elementAt( i ) instanceof GlassNet )
            {
                GlassNet cObj = (GlassNet)m_aGlassObjects.elementAt( i );
                cObj.ServerRespawn();    
            }
        }
    }
    
    

    boolean m_bObjectiveTimeOut = false;
    
    boolean CheckGameTimeOut()
    {
        if ( !m_bObjectiveTimeOut && super.CheckGameTimeOut() )
        {
            m_bObjectiveTimeOut = true;
            CallMethodIn( METHOD_EventObjectiveTimeOut, 0.0f );
        }
        //zwracamy false, zeby nie zawolalo sie GameOver
        return false;
    }
    
    void EventPlayerSpawned( PlayerNet cPlayer )
    {
        super.EventPlayerSpawned( cPlayer );
        
        if ( m_bDeactivateDeadBodyOnRespawn )
        {
            m_bDeactivateDeadBodyOnRespawn = false;
            if ( GetPlayerInfoList() != null )
                 GetPlayerInfoList().ForceDeactivateDeadBodyOnGameRestart();
        }
    }
    void EventObjectiveTimeOut()
    {
        ServerRoundOver();
    }
    
    //czy to nie powinno byc w trybie NetGameInfoTeam?
    
    //czy aktualnie mamy tryb rewanzowy
    boolean m_bRevengeRound = false;
    
    void ServerRoundOver()
    {
        AddGameResults( ENetGameResults._ROUND_OVER );
        //TODO: a moze juz teraz pokazac statystyki? ale to trzeba
        //tez "powiedziec" klientowi
        //ActivateStatisticsOnPleaseWait();
        
        if ( ServerProcessIsGameOver() )
        {
            if ( CanStartRevengeMatch() )    
            {
                DisableCallMethod( METHOD_EventAfterGameOver );
                DisableCallMethod( METHOD_LoadNextLevel );
                DisableCallMethod( METHOD_QuitGameOnGameOver );
                CallMethodIn( METHOD_ServerRestartMatch, 10.0f );
            }
        }
        else
            CallMethodIn( METHOD_ServerRestartRound, 5.0f );
        
    }
    
    //czy byl juz mecz rewanzowy?
    boolean m_bRevengeMatch = false;
    
    boolean CanStartRevengeMatch()
    {
        return (GetGameSettingsRound().GetRevengeMatch() && (!m_bRevengeMatch));
    }
    
    void ServerRestartRound()
    {
        if (GetGameSettingsRound().GetChangeSideOnNextRound())
        {
            //TODO: "obrocic" wszystkie obiekty druzynowe
            m_bRevengeRound = !m_bRevengeRound;
            
            m_nRestartMode = _RESTART_MODE_NEXT_REVENGE_ROUND;
        }
        else
            m_nRestartMode = _RESTART_MODE_NEXT_ROUND;
        
        m_bObjectiveTimeOut = false; 
        ServerRestartGame( false, true );
    }
        
    void ServerRestartMatch()
    {
        //zawsze startujemy od rewanzowej rundy
        if ( ! m_bRevengeRound )
        {
            //TODO: "obrocic" wszystkie obiekty druzynowe    
            m_bRevengeRound = true;
        }
        m_nRestartMode = _RESTART_MODE_REVENGE_MATCH;
                
        m_bObjectiveTimeOut = false; 
        m_bRevengeMatch = true;
        ServerRestartGame( true, true );
    }
    
    boolean VerifySpawnPoint( NetSpawnPoint cSpawnPoint, NetPlayerInfo cPlayerInfo )
    {
        if ( cSpawnPoint == null )
        {
            LogD(" failed\n");
            return false;
        }
        
        return cSpawnPoint.AllowTeam( cPlayerInfo.GetTeamId(), m_bRevengeRound );
    }
}
