public class NetGameSettingsBase extends NetObjectContainer
{
    private int  m_nGameType   = 0;
    
    void SetGameType(int nType) {  m_nGameType = nType; }
    int  GetGameType()          {  return m_nGameType;  }
    
    private String  m_sLevelName = "";
    
    void    SetLevelName(String sName) { m_sLevelName = sName;    }
    String  GetLevelName()             { return m_sLevelName;     }
        
    private String  m_sDefaultScriptFileName = null;
    private String  m_sCustomScriptFileName  = null;
    
    private java.util.Vector m_aDefaultScriptModifiers = null;
    private java.util.Vector m_aCustomScriptModifiers = null;
    
    
    void Init(LevelContent cLevelContent)
    {
        if ( cLevelContent != null )
            Init( cLevelContent.GetMapPath(), cLevelContent.GetLevelName() );
    }
    
    void Init(String sPathName,String sLevelName) 
    {
        m_sLevelName = sLevelName;
        
        m_sDefaultScriptFileName = sPathName +"/"+ ENetGameType.GetScriptFileName( m_nGameType );
        //m_sCustomScriptFileName  = sPathName +"/"+ ENetGameType.GetCustomScriptFileName( m_nGameType );
        m_sCustomScriptFileName  = NetPlayerProfile.GetCurrentGameSettingsFile( sLevelName, m_nGameType );
        
        LoadDefaultModifiers();
        boolean bFileExists = LoadCustomModifiers();
        
        Init();
        //dodalem! ostatnio
        ApplyScriptModifiers( m_aDefaultScriptModifiers );
        //
        ApplyScriptModifiers( m_aCustomScriptModifiers );
        
        if ( !bFileExists )
            SaveCustomSettings();
        
        SetValid( true );
    }
    
    void Init()
    {
        //do nadpisania
    }
            
    private void LoadDefaultModifiers()
    {
        if ( m_sDefaultScriptFileName == null )
        {
            GameObject.LogD(" ERROR "+this+"\n");
            return;
        }
        
        m_aDefaultScriptModifiers =
            ParseScriptFile( m_sDefaultScriptFileName );
    }
    
    private boolean LoadCustomModifiers()
    {
        if ( m_sCustomScriptFileName == null )
        {
            GameObject.LogD(" ERROR "+this+"\n");
            return false;
        }
        java.io.File cFile  = new java.io.File( m_sCustomScriptFileName );
        boolean bFileExists = cFile.exists();
        if ( bFileExists )
            m_aCustomScriptModifiers =
                ParseScriptFile( m_sCustomScriptFileName );
        else
            m_aCustomScriptModifiers =
                ParseScriptFile( m_sDefaultScriptFileName );
        
        return bFileExists;
    }
    
    void SetDefaultSettings()
    {
        GameObject.LogD(" "+this+"\n");
        Init();
        ApplyScriptModifiers( m_aDefaultScriptModifiers );
    }
    
    void SetCustomSettings()
    {
        GameObject.LogD(" "+this+"\n");
        LoadCustomModifiers();
        Init();
        ApplyScriptModifiers( m_aDefaultScriptModifiers );
        ApplyScriptModifiers( m_aCustomScriptModifiers );
    }
    
    
    private java.util.Vector ParseScriptFile(String sFileName) 
    {
        java.util.Vector aScriptModifiers = new java.util.Vector();
            
        ParseKeyword cRoot = GameObject.ParseFile( sFileName );
        if (cRoot != null)
        {
            for (ParseKeyword cKey = cRoot; cKey != null; cKey = cKey.GetNext())
            {
                if (cKey.Equals("GameSettings"))
                {
                    ParseGameSettingsCommand( cKey.GetChild(), 
                                              aScriptModifiers );
                }
            }
            cRoot.delete();
        }
        else
            GameObject.CrashLog("No file "+ sFileName +"!\n");

        return aScriptModifiers;
    }

    final static int _PARSE_MODE_NONE   = 0,
                     _PARSE_MODE_FILE   = 1,
                     _PARSE_MODE_STRING = 2;
    
    int m_nParseMode = _PARSE_MODE_NONE;
    
    
    ParseKeyword m_cSubKey = null;
    
    private void ParseGameSettingsCommand(ParseKeyword cSubKey,
                                  java.util.Vector aScriptModifiers)
    {
        m_nParseMode = _PARSE_MODE_FILE;
        
        
        for(; cSubKey!=null; cSubKey = cSubKey.GetNext() )
        {
            m_cSubKey = cSubKey;
            ModifierNetGameSettings cMod = ParseGameSettingsCommand();
            if ( cMod != null )
                aScriptModifiers.addElement( cMod );
        }
        
        m_cSubKey = null;
    }
        
    
    CommandMapSetParam m_cCommand = null;
    
    boolean ParseGameSettingsCommand(CommandMapSetParam cCommand)
    {
        m_nParseMode = _PARSE_MODE_STRING;
        m_cCommand   = cCommand;
        
        ModifierNetGameSettings cMod = ParseGameSettingsCommand();
        boolean bVerified = cCommand.VerifyModifier( cMod );
            
        m_cCommand   = null;
        if ( bVerified )            
            return ApplyModifier( cMod );
        return false;
    }
    
    private boolean GameSettingNameIs( String sName )
    {
        switch (m_nParseMode)
        {
            case _PARSE_MODE_FILE:
            {
                if ( m_cSubKey == null )
                    return false;
                return m_cSubKey.Equals( sName );
            }
            case _PARSE_MODE_STRING:            
            {
                String sParam = m_cCommand.GetSParam( 0 );
                if ( ! NetGameTools.IsStringValid( sParam ) )
                    return false;
                boolean bFound = sParam.equalsIgnoreCase( sName ) ;
                if ( bFound )
                {
                    //szukamy ignorujac wielkosc liter, ale jak 
                    //juz znajdziemy to chcemy miec zapamietana
                    //poprawna forme (z uwzglednieniem wielkich liter)
                    m_cCommand.FoundSettingName( sName );
                }
                return bFound;
            }
        }
        return false;
    }
    
    private String GetSParam(int nParam)
    {
        switch (m_nParseMode)
        {
        case _PARSE_MODE_FILE:
            return m_cSubKey.GetSParam(nParam);
        case _PARSE_MODE_STRING:            
            // + 1 bo pierwszym parametrem jest sama nazwa GameSettings
            return m_cCommand.GetSParam( nParam + 1 );
        }
        return null;
    }
    
    private int GetIParam(int nParam)
    {
        switch (m_nParseMode)
        {
        case _PARSE_MODE_FILE:
            return m_cSubKey.GetIParam( nParam );
        case _PARSE_MODE_STRING:            
            // + 1 bo pierwszym parametrem jest sama nazwa GameSettings
            return m_cCommand.GetIParam( nParam + 1 );
        }
        return 0;
    }
    
    private float GetFParam(int nParam)
    {
        switch (m_nParseMode)
        {
        case _PARSE_MODE_FILE:
            return m_cSubKey.GetFParam( nParam );
        case _PARSE_MODE_STRING:            
            // + 1 bo pierwszym parametrem jest sama nazwa GameSettings
            return m_cCommand.GetFParam( nParam + 1 );
        }
        return 0.0f;
    }
    
    private ModifierNetGameSettings ParseGameSettingsCommand()
    {
        ModifierNetGameSettings cMod = null;
            
        if (GameSettingNameIs("GameTime"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._MAX_GAME_TIME,
         			    GetIParam(0) );
        } else
        if (GameSettingNameIs("MaxPoints"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._MAX_POINTS,
         			    GetIParam(0) );
        } else
        if (GameSettingNameIs("MaxFrags"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._MAX_FRAGS,
         			    GetIParam(0) );
        } else
        if (GameSettingNameIs("RespawnCntLimit"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._RESPAWN_CNT_LIMIT,
         			    GetIParam(0) );
        } else
        if (GameSettingNameIs("MinRespawnTime"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._MIN_RESPAWN_TIME,
         			    GetFParam(0) );
        } else
        if (GameSettingNameIs("VehicleRespawnCntLimit"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._VEHICLE_RESPAWN_CNT_LIMIT,
         			    GetIParam(0) );
        } else
        if (GameSettingNameIs("VehicleRespawnTime"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._VEHICLE_MIN_RESPAWN_TIME,
         			    GetFParam(0) );
        } else
        if (GameSettingNameIs("VehicleAutoReturnTime"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._VEHICLE_AUTO_RETURN_TIME,
         			    GetFParam(0) );
        } else            
        if (GameSettingNameIs("TeamBalance"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._TEAM_BALANCE,
         			    GetSParam(0) );
        } else
        if (GameSettingNameIs("FriendlyFire"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._FRIENDLY_FIRE,
         			    GetSParam(0) );
        } else
        if (GameSettingNameIs("FriendlyFireMul"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._FRIENDLY_FIRE_MUL,
         			    GetFParam(0) );
        } else
        if (GameSettingNameIs("MinTeamsToStartGame"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._MIN_TEAMS_TO_START_GAME,
         			    GetIParam(0) );
        } else                
        if (GameSettingNameIs("ChangeSideOnNextRound"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._CHANGE_SIDE_ON_NEXT_ROUND,
         			    GetSParam(0) );
        } else
        if (GameSettingNameIs("RevengeMatch"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._REVENGE_MATCH,
         			    GetSParam(0) );
        } else
            if (GameSettingNameIs("IdentifyEnemyPlayers"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._IDENTIFY_ENEMY_PLAYERS,
         			    GetSParam(0) );
        } else            
        if (GameSettingNameIs("EnemiesVisibleOnHudMap"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._ENEMIES_VISIBLE_ON_HUD_MAP,
         			    GetSParam(0) );
        } else
        if (GameSettingNameIs("FriendsVisibleOnHudMap"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._FRIENDS_VISIBLE_ON_HUD_MAP,
         			    GetSParam(0) );
        } else
        if (GameSettingNameIs("VehiclesVisibleOnHudMap"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._VEHICLES_VISIBLE_ON_HUD_MAP,
         			    GetSParam(0) );
        } else            
        if (GameSettingNameIs("FlagsVisibleOnHudMap"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._FLAGS_VISIBLE_ON_HUD_MAP,
         			    GetSParam(0) );
        }
        else
        if (GameSettingNameIs("FlagAutoReturnTime"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._FLAG_AUTO_RETURN_TIME,
         			    GetFParam(0) );
        }
        else    
        if (GameSettingNameIs("FlagsVisibleOnHudMap"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._FLAG_AUTO_RETURN_TIME,
         			    GetSParam(0) );
        }
        else
        if (GameSettingNameIs("BasesVisibleOnHudMap"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._BASES_VISIBLE_ON_HUD_MAP,
         			    GetSParam(0) );
        }
        else
                
        if (GameSettingNameIs("ScoreUpdateTime"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._DOMINATION_SCORE_UPDATE_TIME,
         			    GetIParam(0) );
        }
        else
        if (GameSettingNameIs("AllowDominateByDeadPlayer"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._ALLOW_DOMINATE_BY_DEAD_PLAYER,
         			    GetSParam(0) );
        } else
        if (GameSettingNameIs("AvailableInventoryRoom"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._AVAILABLE_INVENTORY_ROOM,
         			    GetSParam(0) );
        } else
        if (GameSettingNameIs("AvailableVehicles"))
        {
		    cMod = new ModifierNetGameSettings(
                        EModifiersNetGameSettings._AVAILABLE_VEHICLES,
         			    GetSParam(0) );
        } 
        return cMod;
    }
    
    
    void ApplyScriptModifiers(java.util.Vector aScriptModifiers)
    {
        if ( aScriptModifiers == null )
            return;
        for ( int i=0; i<aScriptModifiers.size(); i++ )
        {
            if ( aScriptModifiers.elementAt( i ) instanceof ModifierNetGameSettings )
            {
                ModifierNetGameSettings cMod = (ModifierNetGameSettings)aScriptModifiers.elementAt( i );
                ApplyModifier( cMod );
            }
        }
    }
    
    boolean ApplyModifier(ModifierNetGameSettings cModifier)
    {
        return false;
    }
        
    
    void SaveCustomSettings()
    {
        if ( m_sCustomScriptFileName == null )
        {
            GameObject.LogD(" ERROR "+this+"\n");
            return;
        }
        
        SaveSettings( m_sCustomScriptFileName );
    }
    
    ScriptWriter m_cScriptWriter = new ScriptWriter();
    
    void SaveSettings(String sFileName)
    {
        m_cScriptWriter.Open( sFileName );
        m_cScriptWriter.Write("!include(\"../../LevelsNet/LevelNet.def\")");
        m_cScriptWriter.NewLine();
        m_cScriptWriter.Write("GameSettings()");
        m_cScriptWriter.Write("{");
        m_cScriptWriter.IncIndent();
        
        SaveSettings();
        
        m_cScriptWriter.DecIndent();
        m_cScriptWriter.Write("}");
        m_cScriptWriter.Close();
    }
    
    void SaveSettings()
    {
        //do nadpisania
    }
    
    
    
    
    /**
     * Globalny licznik zmian stanow obiektow
     * Bedzie on wykorzystywany na klientach do rozstrzygania czy dana
     * akcja (ktora zleca nam serwer) ma sie wykonac czy nie (bo jest stara)
     * TODO: dac szerszy opis
     */
    private static 
    ChangeStateCounter sm_cGlobalCounter = new ChangeStateCounter();
    static void CleanGlobalCounter() { sm_cGlobalCounter.Clean(); }
    static int  GetNextGlobalCounter(){ return sm_cGlobalCounter.GetNext(); }
    /**
     * Indywidualny licznik zapamietujacy ostatnio
     * odebrana i wykonana akcje na tym obiekcie
     */
    private 
    ChangeStateCounter m_cCounter = new ChangeStateCounter(this.getClass().getName());
    int     GetCounter(){ return m_cCounter.Get(); }
    boolean VerifyCounter( int nCounter ){ return m_cCounter.Verify( nCounter ); }
    
    
    
    
    //TODO: w przyszlosci wywalic klase NetGameRule!
    NetGameRule GetGameRule()
    {
        //do nadpisania
        GameObject.LogD(" nie nadpisano metody!\n");
        return null;
    }
    
    String GetTextGameRule()
    {
        NetGameRule cGameRule = GetGameRule();
        return cGameRule != null ? cGameRule.GetGameRuleString() : "";
    }
    
        
    
    //dodatkowe narzedzia

    String Bool2String(boolean bVal)
    {
        return bVal ? "\"yes\"" : "\"no\"";
    }

}
