import java.io.*;
public class NetPlayerModelInfo
{
    static int GetSkinsPartialCnt(){ return SkinPartial._SKINS_CNT_; };
            
    private String      m_sMeshName = null;
    SkinPartialList[]   m_aSkins    = null;
    
    NetPlayerModelInfo(String sMeshName)
    {
        m_sMeshName = sMeshName;
        
        m_aSkins = new SkinPartialList[ GetSkinsPartialCnt() ];
        for ( int i=0; i<m_aSkins.length; i++ )
            m_aSkins[ i ] = new SkinPartialList();
        
        //parsujemy plik ze skinami
        String sSkinFile = m_sMeshName.substring(0, m_sMeshName.length() - 3) + "skn";
        ParseScriptFile( sSkinFile );
        
        //pozmieniac inne (zalezne)
        RefreshCurrentSkins();
        //i okreslic dostepnosc zmian                   
        SetChangeAvailability();
    }
    
    NetPlayerModelInfo(String sMeshName,int [] aPartialSkinsIndex)
    {
        this( sMeshName );
        
        if ( aPartialSkinsIndex != null )
            for ( int i=0; i<aPartialSkinsIndex.length; i++ )
                ForcePartialSkin( i , aPartialSkinsIndex[ i ] );
    }
    
    NetPlayerModelInfo(NetPlayerProfile cProfile)
    {
        this( cProfile.GetMeshName(), cProfile.GetSkinsIndex() );
    }
    
    
    String GetMeshName(){ return m_sMeshName; }
    int []GetCurrentPartialSkins()
    {
        int[] aSkins = new int[ m_aSkins.length ];
        for ( int i=0; i<m_aSkins.length; i++ )
            aSkins[ i ] = m_aSkins[ i ].GetCurrentIndex();
        return aSkins;
    }
    
    void SetNextPartialSkin( int nElement )
    {
        ChangePartialSkin( nElement, true );
    }
    void SetPreviousPartialSkin( int nElement )
    {
        ChangePartialSkin( nElement, false );
    }
    void SetPartialSkin( int nElement, int nIndex )
    {
        ChangePartialSkin( nElement, true );
    }
    

    boolean []GetChangeAvailability()
    {
        boolean[] aChangeAvailability = new boolean[ m_aSkins.length ];
        for ( int i=0; i<aChangeAvailability.length; i++ )
            aChangeAvailability[ i ] = m_aSkins[ i ].IsChangeAvailable();
        return aChangeAvailability;
    }
    //TODO: jak to sie powinno nazywac
    boolean []GetCurrentActivity()
    {
        boolean[] aActivity = new boolean[ m_aSkins.length ];
        for ( int i=0; i<aActivity.length; i++ )
        {
            boolean bActive = true;
            switch ( i )
            {
            case SkinPartial._HELMET:
                bActive = HasHelmet();
                break;
            case SkinPartial._EYEGLASS:
                bActive = HasEyeGlass();
                break;
            case SkinPartial._IROKEZ:
                bActive = HasIrokez();
                break;
            default:
                bActive = true;
            }
            aActivity[ i ] = bActive;
        }
        return aActivity;
    }
    
    void ApplyPartialSkins( MeshObject cObject )
    {
        cObject.SetSkin("Default");
        for ( int i=0; i<m_aSkins.length; i++ )
            if ( m_aSkins[ i ].GetCurrentSkinName() != null )
                cObject.SetSkinLeavePrevious( m_aSkins[ i ].GetCurrentSkinName() );
    }
        
    String sParseDef = "!Skin(s)\r\n!Replace(s, s)\r\n!Hide(s)\r\n!NoHelmet()\r\n!Bald()\r\n!NoEyeGlass()\r\n!NoIrokez()\r\n";
    
    private void ParseScriptFile(String sFile)
    {
        File cFile = new File( sFile );
        if (!cFile.exists())
        {
            GameObject.LogD("File " + sFile + " not exist !\n");
            return;
        }
                
        //ParseKeyword cRoot = GameObject.ParseFileDef( sFile, sParseDef );
        ParseKeyword cRoot = GameObject.ParseFile( sFile );
            
        if (cRoot == null)
        {
            GameObject.LogD("Error while read/parse " + sFile + " file !\n");
            return;
        }
        
        ParseKeyword cKey;
        for ( cKey = cRoot; cKey != null; cKey = cKey.GetNext() )
        {
            if ( cKey.Equals( "Skin" ) ) 
            {
                String sName = cKey.GetSParam( 0 );
                if ( sName != null && sName.startsWith( "@" ) )
                {
                    NewSkinPartial( sName, cKey.GetChild() );
                }
            }
        }
        cRoot.delete();
    }

    private void NewSkinPartial( String sName, ParseKeyword cKey )
    {
        int         nElement = SkinPartial.GetElementIdBySkinName( sName );
        SkinPartial cSkin    = new SkinPartial( sName );
        
        if ( nElement >= 0 && nElement < m_aSkins.length )
            m_aSkins[ nElement ].Add( cSkin );
        else
            return;
        
        //TODO: wsadzic to w poszczegolne klasy
        for ( ; cKey!=null; cKey=cKey.GetNext() )
        {
            if (nElement == SkinPartial._HELMET && cKey.Equals("NoHelmet")) 
                cSkin.SetNoHelmet();

            if (nElement == SkinPartial._HEAD && cKey.Equals("Bald")) 
                cSkin.SetBald();
            
            if (nElement == SkinPartial._EYEGLASS && cKey.Equals("NoEyeGlass")) 
                cSkin.SetNoEyeGlass();
            
            if (nElement == SkinPartial._IROKEZ && cKey.Equals("NoIrokez")) 
                cSkin.SetNoIrokez();
        }
        
        GameObject.LogD("Dodano "+cSkin.GetInfo()+"\n");
    }   
    
    
    private void ChangePartialSkin( int nElement, boolean bNext )
    {
        if ( nElement >= 0 && nElement < m_aSkins.length )
        {
            SkinPartialList cSkinList = m_aSkins[ nElement ];
            if ( cSkinList != null && cSkinList.IsChangeAvailable() )
            {
                int         nSecure   = cSkinList.GetSize()+1;
                SkinPartial cOldSkin  = cSkinList.GetCurrent();
                do {
                    if (bNext)
                        cSkinList.SetNext();
                    else
                        cSkinList.SetPrevious();
                } while ( ! VerifyChange( nElement ) && nSecure-- > 0 );
                
                if ( nSecure <= 0 )
                {
                    GameObject.LogD("Change "+nElement+" "+bNext+
                                    " not verified \nOldSkin:"+cOldSkin.GetInfo()+"\n");
                }
                
                //pozmieniac inne (zalezne)
                RefreshCurrentSkins();
                //i okreslic dostepnosc zmian                   
                SetChangeAvailability();
            }
        }
    }

    private void ForcePartialSkin( int nElement, int nIndex )
    {
        if ( nElement >= 0 && nElement < m_aSkins.length )
        {
            SkinPartialList cSkinList = m_aSkins[ nElement ];
            if ( cSkinList != null )
            {
                cSkinList.SetCurrentByIndex( nIndex );
                //pozmieniac inne (zalezne)
                RefreshCurrentSkins();
                //i okreslic dostepnosc zmian                   
                SetChangeAvailability();
            }
        }
    }
    
    private boolean VerifyChange( int nElement )
    {
        boolean bVerified = true;
        switch ( nElement )
        {
        //takich sytuacji chyba nie bedzie bo powinno
        //byc po jednym skinie "okulary bez okularow"                            
        //oraz "irokez bez irokeza" ale...    
        case SkinPartial._EYEGLASS:
            bVerified = HasHelmet() ? !HasEyeGlass() : true ;
            break;
        case SkinPartial._IROKEZ:
            bVerified = HasHelmet() ? !HasIrokez() : true ;
            break;
        default:
            bVerified = true;
        }
        return bVerified;
    }
    
    void RefreshCurrentSkins()
    {
        for ( int i=0; i<m_aSkins.length; i++ )
        {
            switch ( i )
            {
            //jak mamy kask to okularow ma nie byc
            case SkinPartial._EYEGLASS:
                if ( HasHelmet() )
                    m_aSkins[ i ].SetCurrentByFlag( SkinPartial._FLAG_NOEYEGLASS );
                break;
            //jak mamy kask lub wlosy to okularow ma nie byc                
            case SkinPartial._IROKEZ:
                if ( HasHelmet() || !IsBald() )
                    m_aSkins[ i ].SetCurrentByFlag( SkinPartial._FLAG_NOIROKEZ );
                break;
            }
        }
    }
    
    //ustawia mozliwosci zmiany poszczegolnych elementow
    private void SetChangeAvailability()
    {
        for ( int i=0; i<m_aSkins.length; i++ )
        {
            boolean bAvailable = true;
            
            switch ( i )
            {
            //okulary mozna wybierac tylko
            //jak nie mamy kasku
            case SkinPartial._EYEGLASS:
                bAvailable = !HasHelmet();
                break;
            //irokezy mozna wybierac tylko
            //jak nie mamy kasku oraz mamy lysa glowe
            case SkinPartial._IROKEZ:
                bAvailable = !HasHelmet() && IsBald();
                break;
            }
            m_aSkins[ i ].SetChangeAvailable( bAvailable );
        }
    }
    
    
    
    private boolean HasHelmet() 
    { 
        SkinPartialList cSkinList = m_aSkins[ SkinPartial._HELMET ];
        SkinPartial     cSkin     = cSkinList.GetCurrent();
        return cSkin != null && cSkin.HasHelmet();
    }
    
    private boolean IsBald() 
    { 
        SkinPartialList cSkinList = m_aSkins[ SkinPartial._HEAD ];
        SkinPartial     cSkin     = cSkinList.GetCurrent();
        return cSkin != null && cSkin.IsBald();
    }
    
    private boolean HasIrokez() 
    { 
        SkinPartialList cSkinList = m_aSkins[ SkinPartial._IROKEZ ];
        SkinPartial     cSkin     = cSkinList.GetCurrent();
        return cSkin != null && cSkin.HasIrokez();
    }
    
    private boolean HasEyeGlass() 
    { 
        SkinPartialList cSkinList = m_aSkins[ SkinPartial._EYEGLASS ];
        SkinPartial     cSkin     = cSkinList.GetCurrent();
        return cSkin != null && cSkin.HasEyeGlass();
    }
}
