/** Klasa przechowujaca liste obiektow NetInfo
 */
public class NetInfoList
{
    /** NetGameInfo w ktorym umieszczona jest ta lista
     */
    NetGameInfoBase m_cGameInfo          = null;
    
    /** Wektor obiektow
     */
    private java.util.Vector m_aList = null;
        
    
    NetInfoList(NetGameInfoBase cGameInfo)
    {
        m_cGameInfo = cGameInfo;
        m_aList     = new java.util.Vector();
    }
    
    /** Dodaj element do listy
     */
    void Add(NetInfo cInfo)
    {
        m_aList.addElement( cInfo );
        if ( m_cGameInfo != null )
             m_cGameInfo.OnScoreChange();
    }
    
    /** Usun element do listy
     */
    void Remove(NetInfo cInfo)
    {
        m_aList.removeElement( cInfo );
        if ( m_cGameInfo != null && ModuleNet.IsGameGoing() )
             m_cGameInfo.OnScoreChange();
    }
    
    /** pobranie dlugosci listy
     */
    int GetSize()
    {
        return m_aList.size();
    }
    
    /** pobranie elementu po indeksie
     */
    NetInfo GetInfoByIndex(int nIndex)
    {
        if ( nIndex >= 0 && nIndex < m_aList.size() )
            return (NetInfo)m_aList.elementAt( nIndex );
        return null;
    }
    
    /**
     * Sprawdzenie indexu (czyli de facto aktualnej pozycji) 
     */
    int IndexOf(NetInfo cInfo)
    {
        return m_aList.indexOf( cInfo );
    }
    
    /** pobranie elementu po identyfikatorze teamu
     */
    NetInfo GetInfoByTeamId(int nTeamId)
    {
        for ( int i=0; i<m_aList.size(); i++ )
        {
            if ( !(m_aList.elementAt( i ) instanceof NetInfo) )
                continue;
            
            NetInfo cInfo = (NetInfo)m_aList.elementAt( i );
            
            if ( cInfo.GetTeamId() == nTeamId )
                return cInfo;
        }
        
        return null;
    }

    /** pobranie elementu po identyfikatorze sieciowych
     */
    NetInfo GetInfoByNetId(int nNetId)
    {
        for ( int i=0; i<m_aList.size(); i++ )
        {
            if ( !(m_aList.elementAt( i ) instanceof NetInfo) )
                continue;
            
            NetInfo cInfo = (NetInfo)m_aList.elementAt( i );
            
            if ( cInfo.GetNetId() == nNetId )
                return cInfo;
        }
        
        return null;
    }

    
    /** pobranie elementu ktory ma bNetIsOwner = true,
     * czyli jego wlascicielem jest lokalny gracz
     */
    NetInfo GetMyInfo()
    {
        for ( int i=0; i<m_aList.size(); i++ )
        {
            if ( !(m_aList.elementAt( i ) instanceof NetInfo) )
                continue;
            
            NetInfo cInfo = (NetInfo)m_aList.elementAt( i );
            
            if ( cInfo.bNetIsOwner )
                return cInfo;
        }
        
        return null;
    }
    
    NetInfo GetInfoByName(String sName)
    {
        if ( sName == null )
            return null;
            
        for ( int i=0; i<m_aList.size(); i++ )
        {
            if ( !(m_aList.elementAt( i ) instanceof NetInfo) )
                continue;
            
            NetInfo cInfo = (NetInfo)m_aList.elementAt( i );
            
            if ( sName.equalsIgnoreCase( cInfo.GetName() ) )
			{
				return cInfo;
			}
        }
	    return null;
    }
    
    /** Posortowanie listy 
     * Sortowanie odbywa sie kolejno wg nastepujacych kryteriow
     * - id druzyny (malejaco)
     * - bioracy udzial w grze / wykluczony
     * - punktacja (kilka poziomow "wyciaganych" z GameInfo")
     */
    boolean Sort()
    {
        //LogList("START");
        boolean bOrderChanged = false;
        int     nSafety  = m_aList.size();
        do {
            boolean bEnd = true;
            for (int i=0; i<m_aList.size()-1; i++ )
            {
                if ( ! (m_aList.elementAt( i ) instanceof NetInfo) )
                    continue;
                if ( ! (m_aList.elementAt( i+1 ) instanceof NetInfo) )
                    continue;
                                
                NetInfo cCurrent = (NetInfo)m_aList.elementAt( i );
                NetInfo cNext    = (NetInfo)m_aList.elementAt( i+1 );
                
                if ( WrongOrder( cCurrent, cNext ) )
                {
                    m_aList.removeElement( cNext );
                    m_aList.insertElementAt( cNext, i );
                    
                    //LogList("CHANGE "+i+" <-> "+(i+1)+"\n");
                                        
                    bEnd          = false;
                    bOrderChanged = true;
                }
            }
            if ( bEnd )
                break;
            //LogList("END for");
        } while ( nSafety-- > 0 );
        
        //LogList("END bOrderChanged"+bOrderChanged+"");
        return bOrderChanged;
    }
    
    /** Czy nalezy zmienic kolejnosc na liscie?
     */
    boolean WrongOrder(NetInfo cCurrent,NetInfo cNext)
    {
        //jesli pierwszy jest wykluczony a drugi nie, to znaczy ze tak
        if ( cCurrent.IsExcluded() && !cNext.IsExcluded() )
            return true;
        //jesli pierwszy nie jest wykluczony a drugi tak, to znaczy ze nie
        if ( !cCurrent.IsExcluded() && cNext.IsExcluded() )
            return false;
        
        //teraz kryteria na kolejnych poziomach
        for ( int nLevel=0; nLevel < GetSortLevelsCnt(); nLevel++ )
        {
            int nCurrent= m_cGameInfo.GetPointsFromInfoToCompare( cCurrent, nLevel );
            int nNext   = m_cGameInfo.GetPointsFromInfoToCompare( cNext, nLevel );
            
            //jezeli moje punkty sa MNIEJSZE niz nastepnego to TAK (zmiana kolejnosci)
            if ( nCurrent < nNext )
                return true;
            //jezeli moje punkty sa WIEKSZE niz nastepnego to NIE trzeba nic zmieniac
            if ( nCurrent > nNext )
                return false;
            
            // jezeli dotarl tu, tzn ze  nCurrent == nNext
            // w takim przypadku rozpatrywane bedzie kolejne kryterium
        }
        return false;
    }
    
    private int GetSortLevelsCnt()
    {
        return 2;
    }
    
    void CleanResults()
    {
        for ( int i=0; i<m_aList.size(); i++ )
        {
            if ( !(m_aList.elementAt( i ) instanceof NetInfo) )
                continue;
            
            NetInfo cInfo = (NetInfo)m_aList.elementAt( i );
            
            cInfo.CleanResults();
        }
    }
    
    void Destroy()
    {
        //TODO:
        //GameObject.CrashLog("TODO: Dorobic usuwanie z pamieci obiektow z listy "+this+"\n");
    }
    
    int GetLeaderInfoId()
    {
        Sort();
        int nId = -1;
        if ( m_aList.size() > 0 && m_aList.elementAt( 0 ) instanceof NetInfo )
        {
            NetInfo cInfo = (NetInfo) m_aList.elementAt( 0 );
            nId = cInfo.GetNetId();
        }
        return nId;
    }

    /**
     * Metoda pomocnicza do testow sortowania
     */
    void LogList(String sText)
    {
        GameObject.LogR("-List state "+sText+"\n");
        
        for (int i=0; i<m_aList.size(); i++ )
        {
            if ( ! (m_aList.elementAt( i ) instanceof NetInfo) )
                continue;
            NetInfo cInfo = (NetInfo)m_aList.elementAt( i );
            GameObject.LogR(""+i+" "+cInfo.GetFrags()+" "+cInfo.GetDeaths()+" "+cInfo.IsExcluded()+"\n");
        }
        GameObject.LogR("----------\n");
    }
    
    

}
