import java.io.*;
/** Klasa zawiera informacje o graczu 
 * Kazdy profil bedzie przechowywany w osobnym katalogu:
 * na Data/Profiles/Zdzicho" dla profilu o nazwie Zdzicho
 * Tam znajda sie pliki :
 * - Profile.scr - informacje o nicku, meshu, skinach
 * - UserControls.scr - informacje o zdefiniowanych controlsach
 * Dane te beda wysylane do serwera i wykorzystane przy 
 * tworzeniu gracza.
 * W przyszlosci w katalogu z profilem beda mogly byc tez przechowywane
 * dodatkowe pliki np ze skinami.
 * Uwaga: powyzsze nazwy katalogow/plikow moga ulec zmianie, w zaleznosci
 * od zmiennych statycznych zdefiniowanych w klasie (patrz nizej)
 */
public class NetPlayerProfile
{
    /**
     * Parametry profili (katalogi, nazwy plikow)
     */
    /** Katalog w ktorym beda znajdowaly sie profile graczy */
    private 
    static String sm_sProfileDir            = "Data/Profiles/";
    /** Plik (w powyzszym katalogu) w ktorym bedzie nazwa biezacego profilu*/
    private
    static String sm_sCurrentProfileFile    = "CurrentProfile.scr";
    /** Nazwa domyslnego profilu (jak gracz nie utworzy i/lub wybierze swojego)*/
    private
    static String sm_sDefaultProfile        = "DefaultProfile";
    /** Nazwa pliku w ktorym bedziemy przechowywali dane o profilu(nick,mesh,skiny)*/
    private
    static String sm_sProfileFile           = "Profile.scr";
    /** Nazwa pliku w ktorym bedziemy przechowywali dane o controlsach gracza*/
    private
    static String sm_sControlsFile          = "UserControls.scr";
    /** Nazwa pliku w ktorym bedziemy przechowywali dane o filtrach gracza*/
    private
    static String sm_sFiltersFile           = "UserFilters.scr";
    /** Nazwa pliku w ktorym bedziemy przechowywali dane o ulubionych serwerach gracza*/
    private
    static String sm_sFavoritesServersFile  = "FavoritesServers.scr";
    /** Nazwa pliku w ktorym bedziemy przechowywali dane o ulubionych mapach*/
    private
    static String sm_sFavoritesMapsFile     = "FavoritesMaps.scr";
    /** Nazwa pliku w ktorym bedziemy przechowywali dane o ustawieniach servera*/
    private
    static String sm_sServerSettingsFile    = "ServerSettings.scr";
    /** Prefix nazwa pliku w ktorych bedziemy przechowywali dane ustawieniach map*/
    private
    static String sm_sGameSettingsPrefixFile= "GameSettings";
    
    /**
     * Modele dostepne przy wyborze profilu (katalogi, nazwy plikow)
     */
    /** Plik w ktorym jest dostepna lista dostepnych meshy*/
    static String sm_sModelsScript          = "Data/ModelsNet.scr";
    /** Katalog w ktorym powinny znajdowac sie powyzsze meshe*/
    //TODO: docelowo katalog /AI
    static String sm_sModelDir              = "Data/Meshes/Ai/"; 
    /** Rozszerzenie plikow z meshami*/
    static String sm_sPostfixMeshFile       = ".3da";    
    /** Rozszerzenie plikow ze skinami*/
    static String sm_sPostfixSkinFile       = ".skn";
    
    
    /** nazwa profilu */
    private 
    String  m_sProfileName = "";
    String  GetProfileName(){ return m_sProfileName; }
    void    SetProfileName(String sName){ m_sProfileName = sName; }
    
    /** Imie (nick) gracza */
    private 
    String  m_sPlayerName = "";
    String  GetPlayerName(){ return m_sPlayerName; }
    void    SetPlayerName(String sName){ m_sPlayerName = sName; }
    
    /** Nazwa mesha */
    private 
    String  m_sMeshName   = "";
    String  GetMeshName(){ return m_sMeshName; }
    void    SetMeshName(String sName){ m_sMeshName = sName; }
    
    /** Lista wybranych skinow "czastkowych" */
    private 
    int[]   m_aSkins = new int[ NetPlayerModelInfo.GetSkinsPartialCnt() ];
    int[]   GetSkinsIndex(){ return m_aSkins; }
    void    SetSkins(int[] aSkins){ m_aSkins = aSkins; }

    /**
     * Informacje czy profil dotyczy bota
     * Zmienna wykorzystywana tylko do testow
     */  
    private
    boolean m_bBot = false;
    boolean IsBot(){ return m_bBot;}
    void    SetBot(boolean bVal){ m_bBot = bVal;}
    
    /**
     * Informacje czy profil dotyczy autoplayera (klienta podlaczonego
     * do serwera i sterujacego graczem ale automatycznie - taki bot
     * z klienta)
     * Zmienna wykorzystywana tylko do testow
     */  
    private
    boolean m_bAutoPlayer = false;
    boolean IsAutoPlayer(){ return m_bAutoPlayer;}
    void    SetAutoPlayer(boolean bVal){ m_bAutoPlayer = bVal;}

    
    /** Obiekt pomocny przy odczycie/zapisie profilu z/do skryptu*/
    ScriptWriter m_cScriptWriter = new ScriptWriter();
        
        
    NetPlayerProfile()
    {
    }
    
    NetPlayerProfile(String sProfileName)
    {
        New( sProfileName );
    }

    /** Stworzenie nowego profilu o podanej nazwie
     */
    void New(String sProfileName)
    {
        //ladujemy domyslny profil
        Load( sm_sDefaultProfile );
        //nazywamy go tak jak sobie zyczyl user
        SetProfileName( sProfileName );
        //TODO: a moze nicka tez domyslnie dac takiego jak nazwa profilu?
    }
    
    /** Zaladowanie aktualnie wybranego profuilu
     */
    void Load()
    {
        // ladujemy podany profil
        Load( FindCurrentProfileName() );
    }
           
    /**wczytujemy profil o podanej nazwie*/
    void Load(String sProfileName)
    {   
        //skladamy pelna nazwe pliku
        String sFile = sm_sProfileDir + sProfileName + "/" + sm_sProfileFile;
                
        //parsujemy plik
        ParseKeyword cRoot = GameObject.ParseFile( sFile );
        if ( cRoot != null )
        {
            ParseKeyword cKey, cSubKey;
            for ( cKey=cRoot; cKey!=null; cKey=cKey.GetNext() )
            {
                if (cKey.Equals("Nick"))
                    m_sPlayerName = cKey.GetSParam(0);
                if (cKey.Equals("Model"))
                    m_sMeshName = sm_sModelDir + "" + cKey.GetSParam(0);
                if (cKey.Equals("Skin"))
                {
                    int nElement = cKey.GetIParam(0);
                    int nIndex   = cKey.GetIParam(1);
                    if ( nElement >= 0 && nElement < m_aSkins.length )
                        m_aSkins[ nElement ] = nIndex;
                }
            }
            cRoot.delete();
            //nazwa profilu nie jest pamietana w pliku wiec ustawiamy
            //ja "z palca"
            SetProfileName( sProfileName );
        }
        else
        {
            GameObject.LogR("Read or parse error in file " + sFile + " !\n");
        }
    }
    
    /**
     * Ustawienie parametrow modelu (mesh, skiny czesciowe) 
     * na podstawie podanych informacji
     */
    void SetModel(NetPlayerModelInfo cModelInfo)
    {
        if ( cModelInfo == null )
            return;
        
        String sMeshName   = cModelInfo.GetMeshName();
        int    nSlashIndex = sMeshName.lastIndexOf('/');
        m_sMeshName = sMeshName.substring( nSlashIndex + 1 );
        m_aSkins    = cModelInfo.GetCurrentPartialSkins();
    }
    
    /**
     * Zapis aktualnego profilu
     */
    void Save()
    {
        //sprawdzamy czy jest katalog z profilami, jesli nie to go tworzymy
        File cDir = new File( sm_sProfileDir );
        if (!cDir.exists())
            cDir.mkdir();
        
        //sprawdzamy czy jest katalog z biezacym profilem, jesli nie to go tworzymy
        String sProfileDir = sm_sProfileDir + m_sProfileName;
        cDir = new File( sProfileDir );
        if (!cDir.exists())
            cDir.mkdir();

        GameObject.LogD("Save profile file for Player: " + 
                        m_sPlayerName + "\n");
        
        //otwieramy plik oraz zapisujemy do niego co trzeba
        //najpierw definicje, pozniej prawdziwe dane
        m_cScriptWriter.Open( sProfileDir +"/"+ sm_sProfileFile );
        
        m_cScriptWriter.Write("!Nick(s)");
        m_cScriptWriter.Write("!Model(s)");            
        m_cScriptWriter.Write("!Skin(i,i)");            
        m_cScriptWriter.NewLine();
        
        m_cScriptWriter.Write("Nick(\"" + m_sPlayerName + "\")");            
        m_cScriptWriter.Write("Model(\"" + m_sMeshName   + "\")");                   
        for ( int i=0; i<m_aSkins.length; i++ )
            m_cScriptWriter.Write("Skin(" + i +"," + m_aSkins[i] + ")");
        
        m_cScriptWriter.Close();
        
        //musimy jeszcze zapamietac ze ten profil byl ostatnio "obrabiany",
        //czyli staje sie aktualnie wybranym
        m_cScriptWriter.Open( sm_sProfileDir + sm_sCurrentProfileFile );
        m_cScriptWriter.Write( m_sProfileName );
        m_cScriptWriter.Close();
    }
    
    /**
     * Szereg metod statycznych do wykorzystania glownie w ekranach menu
     */
    
    /**
     * Pobranie obiektu biezacego profilu
     */    
    static NetPlayerProfile GetCurrent()
    {
        NetPlayerProfile cProfile = new NetPlayerProfile();
        cProfile.Load();
        return cProfile;
    }

    /**
     * Pobranie nazwy biezacego profilu
     */    
    static String GetCurrentProfileName()
    {
        NetPlayerProfile cProfile = GetCurrent();
        return cProfile.GetProfileName();
    }
    
    
    
    /**
     * Pobranie pliku ze sciazka do kontrolsow biezacego profilu
     */    
    static String GetCurrentControlsFileName()
    {
        NetPlayerProfile cProfile = GetCurrent();
        return GetControlsFileName( cProfile.GetProfileName() );
    }
    
    static String GetControlsFileName( String sProfileName )
    {
        return sm_sProfileDir + sProfileName +"/" + sm_sControlsFile;
    }
    
    /**
     * Pobranie pliku ze sciazka do kontrolsow biezacego profilu
     */    
    static String GetCurrentFiltersFileName()
    {
        NetPlayerProfile cProfile = GetCurrent();
        return GetFiltersFileName( cProfile.GetProfileName() );
    }
    
    static String GetFiltersFileName( String sProfileName )
    {
        return sm_sProfileDir + sProfileName +"/" + sm_sFiltersFile;
    }
    
    /**
     * Pobranie pliku ze sciazka do ulubionych serwerow biezacego profilu
     */    
    static String GetCurrentFavoritesServersFileName()
    {
        NetPlayerProfile cProfile = GetCurrent();
        return GetFavoritesServersFileName( cProfile.GetProfileName() );
    }
    
    static String GetFavoritesServersFileName( String sProfileName )
    {
        return sm_sProfileDir + sProfileName +"/" + sm_sFavoritesServersFile;
    }

    /**
     * Pobranie pliku ze sciazka do ulubionych serwerow biezacego profilu
     */    
    static String GetCurrentFavoritesMapsFileName()
    {
        NetPlayerProfile cProfile = GetCurrent();
        return GetFavoritesMapsFileName( cProfile.GetProfileName() );
    }
    
    static String GetFavoritesMapsFileName( String sProfileName )
    {
        return sm_sProfileDir + sProfileName +"/" + sm_sFavoritesMapsFile;
    }

    /**
     * Pobranie pliku z informacja o ustawieniach servera dla biezacego profilu
     */
    static String GetCurrentServerSettingsFile()
    {
        NetPlayerProfile cProfile = GetCurrent();
        return GetServerSettingsFile( cProfile.GetProfileName() );
    }
    
    static String GetServerSettingsFile( String sProfileName )
    {
        return sm_sProfileDir + sProfileName +"/" + sm_sServerSettingsFile;
    }
    
    /**
     * Pobranie pliku z informacja o ustawieniach podanej mapy w podanym trybie
     */
    static String GetCurrentGameSettingsFile( String sLevelName, int nGameType )
    {
        NetPlayerProfile cProfile = GetCurrent();
        return GetGameSettingsFile( cProfile.GetProfileName(), sLevelName, nGameType );
    }
    
    static String GetGameSettingsFile( String sProfileName, String sLevelName, int nGameType )
    {
        return sm_sProfileDir + sProfileName +"/" + 
               sm_sGameSettingsPrefixFile + "_" + sLevelName + "_" +
               ENetGameType.GetCustomScriptFileName( nGameType );
    }
    
    
  	/** 
  	 * Skasowanie aktualnie wybranego profilu
  	 * Zwraca info czy operacja sie powiodla
	 */
	static boolean DeleteCurrentProfile()
	{
		NetPlayerProfile cProfile = GetCurrent();
        return DeleteProfile( cProfile.GetProfileName() );
	}

    final static int _MAX_LEVEL_DIRECTORY = 10;
    
    /**
     * Metoda kasuje profil o podanej nazwie
     * Zwraca info czy operacja sie powiodla
     */
    static boolean DeleteProfile( String sProfileName )
	{
        if ( (! FindProfile( sProfileName )) || IsCurrentDefaultProfile() )
            return false;
        
        if ( IsCurrentDefaultProfile() )
            return false;
       
        //skasowac profil
        String sDir = sm_sProfileDir + sProfileName;
        return NetGameTools.DeleteDir( sDir, _MAX_LEVEL_DIRECTORY );
	}
    
    
    /**
     * Sprawdza czy obecnie jest wybrany profil domyslny
     */
    static boolean IsCurrentDefaultProfile()
    {
        NetPlayerProfile cProfile = GetCurrent();
        return IsDefaultProfileName( cProfile.GetProfileName() );
    }
    
    /**
     * Sprawdza czy podana nazwa profilu to nazwa profilu domyslnego
     */
    static boolean IsDefaultProfileName(String sProfileName)
    {
        return sProfileName != null && 
               sProfileName.equals( sm_sDefaultProfile );
    }
    
    /**
     * Pobranie listy profili na dysku
     * Pszeszukujemy katalog z profilami i nazwy profili wpisujemy do listy
     * Wykorzystywane w menu do wypelnienia combobox'a
     */
    static java.util.Vector FindProfiles()
    {
        java.util.Vector aList = new java.util.Vector();
        
        //sprawdzamy czy istnieje katalog z profilami
        File cProfileDir = new File( sm_sProfileDir );
        if ( !cProfileDir.exists() )
        {
            cProfileDir.mkdir();
            GameObject.LogD(" Katalogu "+sm_sProfileDir+
                            " nie ma, tworzymy go i wychodzimy");
            return aList;
        }
        
        GameObject.LogD(" sprawdzamy zawartosc katalogu");
        String []aFileNames = cProfileDir.list();
        for ( int i = 0; i < aFileNames.length; i++ )
        {
            String sProfileName = aFileNames[ i ];
            GameObject.LogD(" kolejny profil: "+sProfileName);
            if ( FindProfile( sProfileName ) )
                aList.addElement( sProfileName );
        }
        return aList;
    }
    
    /**
     * Ustalenie biezacego profilu 
     * 1. odczyt pliku Data/Profiles/CurrentProfile.scr
     * 2. zawolanie FindProfile() dla odczytanej nazwy
     * Jesli funkcja zwroci true, to zwraca ta nazwe
     * Jesli nie to zwraca defaultProfile
     */
    static String FindCurrentProfileName()
    {
        String sProfileName = null;
        
        // sprawdzamy czy istnieje  plik z aktualnym profilem 
        File cFile = new File( sm_sProfileDir + sm_sCurrentProfileFile );
        if (cFile.exists())
        {
            //jesli tak to do sprobujmy odczytac
            String sReadProfileName = null;
            try
            {
                BufferedReader cBuff = new BufferedReader(new FileReader(sm_sProfileDir + sm_sCurrentProfileFile));
                //odczytujemy nazwe aktualnego profilu
                sReadProfileName = cBuff.readLine();
                cBuff.close();
            }
            catch (IOException e)
            { 
                GameObject.LogD("ERROR " + e.getMessage()); 
            }
            
            //jesli faktycznie istnieje taki profil, to zapamietujemy to
            if ( FindProfile( sReadProfileName ) )
                sProfileName = sReadProfileName;
        }
        
        //jesli nie znaleziono aktualnego profilu, to biezemy domyslny
        if ( sProfileName == null )
             sProfileName = sm_sDefaultProfile;
        return sProfileName;
    }

    /**
     * Sprawdzenie czy istnieje profil o podanej nazwie
     * 1. sprawdzenie czy taki profil istnieje (odczyt pliku Profile.scr
     * 2. sprawdzenie czy chociaz mozna tam znalezc nicka gracza)
     * Jesli wszystko sie powiedzie to zwraca ta true
     * Jesli nie to zwraca false
     */
    static boolean FindProfile(String sProfileName)
    {
        boolean bFound = false;
        
        String sProfile = sm_sProfileDir +  sProfileName +
                          "/" + sm_sProfileFile;
                
        ParseKeyword cRoot = GameObject.ParseFile( sProfile );
        if (cRoot==null)
        {
            GameObject.LogD(" Read or parse error in file " + sProfile + " !\n");
        }
        else
        {
            ParseKeyword cKey, cSubKey;
            for ( cKey = cRoot; cKey != null; cKey = cKey.GetNext() )
            {
                //sprawdzmy czy chociaz jest zdefiniowany nick
                if ( cKey.Equals("Nick") )
                {
                    bFound = true;
                    break;
                }
            }
            cRoot.delete();
        }
        return bFound;
    }
       
    /**
     * Pobranie listy meshy dostepnych na dyzku do gry sieciowej
     * Parsujemy plik ModelsNet.scr i odczywujemy z niego nazwy meshy
     * Kazdego z nich weryfikujemy, tzn sprawdzamy czy taki plik
     * istnieje oraz czy istnieje odpowiedni plik ze skinami
     * //TODO: dorobic sprawdzenie istnienia pliku .scr
     * Wykorzystywane w menu do wypelnienia combobox'a
     */
    static java.util.Vector FindNetModels()
    {
        java.util.Vector aList = new java.util.Vector();
                
        ParseKeyword cRoot = GameObject.ParseFile( sm_sModelsScript );
        if ( cRoot != null )
        {
            ParseKeyword cKey;
            for ( cKey = cRoot; cKey != null; cKey = cKey.GetNext() )
            {
                if ( cKey.Equals("Mesh") )
                {
                    String sFileName = cKey.GetSParam( 0 );
                    GameObject.Log(" "+sFileName+" ");
                    
                    //sprawdzmy czy jest taki plik
                    java.io.File cFile  = new java.io.File( sm_sModelDir + sFileName );
                    //jesli nie ma to papa...
                    if ( ! cFile.exists() )
                        continue;
            
                    //jesli nazwa pliku za krotka, to papa
                    if ( sFileName.length() < sm_sPostfixMeshFile.length() )
                        continue;
                    
                    String sExt = sFileName.substring( sFileName.length() - 
                                                       sm_sPostfixMeshFile.length() );
            
                    //jesli plik nie ma odpowiedniego rozszerzenia to bye
                    if (!sExt.equalsIgnoreCase( sm_sPostfixMeshFile ))
                        continue;
                    
                    //no to sprawdzmy czy istnieje plik ze skinami
                    int nDot = sFileName.lastIndexOf(".");
                    String sSkinFile = sFileName.substring( 0, nDot ) + sm_sPostfixSkinFile;
                    cFile  = new java.io.File( sm_sModelDir + sSkinFile );
                    //jesli nie ma to papa...
                    if ( ! cFile.exists() )
                        continue;
                    
                    aList.addElement( sFileName );
                    GameObject.Log("Add mesh " + sm_sModelDir + "" + sFileName + " \n");
                }
            }
            cRoot.delete();
        }
        else
        {
            GameObject.LogD("Read or parse error in file " + sm_sModelsScript + " !\n");
        }
        
        return aList;        
    }
    
    
    final static int _NEW_PROFILE_NAME_OK    = 0;
    final static int _NEW_PROFILE_NAME_EXIST = 1;
    final static int _NEW_PROFILE_NAME_WRONG = 2;
	final static int _NO_PROFILE_DIR		 = 3;

    /**
     * Weryfikacja poprawnosci nazwy dla nowo tworzonego 
     * profilu,
     * Jesli wszystko ok to zwraca
     * _NEW_PROFILE_NAME_OK
     * Jesli niepowodzenie to
     * - _NEW_PROFILE_NAME_EXIST - profil o podanej nazwie juz istnieje 
     * - _NEW_PROFILE_NAME_WRONG - podana nazwa nie jest prawidlowa (zawiera "zle" znaki
     **/
    static int VerifyProfileName( String sProfileName )
    {
        if ( FindProfile( sProfileName ) )
            return _NEW_PROFILE_NAME_EXIST;
        
        //sprawdzic czy czasem nie podano niepoprawnej nazwy.
        File cDir = new File( sm_sProfileDir );
        if (!cDir.exists())
		{
			//fdf
			return _NO_PROFILE_DIR;
		}

        //sprawdzamy czy da sie stworzyc katalog o podanej nazwie
        String sProfileDir = sm_sProfileDir + sProfileName;
        cDir = new File( sProfileDir );
        if (!cDir.exists())
            cDir.mkdir();
        if ( !cDir.isDirectory()|| !cDir.exists())
            return _NEW_PROFILE_NAME_WRONG;
        cDir.delete();
                
        return _NEW_PROFILE_NAME_OK;
    }
	String[] aT = {"*", "?"};
	
    /**
     * Zaladowanie i zastosowanie 
     * kontrolsow dla biezacego profilu
     */
    static void LoadCurrentControlsAndApply()
    {
        //jaka jest nazwa pliku dla biezacego profilu?
        String sName = GetCurrentControlsFileName();
        //czy plik istnieje
        File cFile = new File( sName );
        if ( ! cFile.exists() )
        {
            //jesli nie to sprawdzmy czy istnieje plik UsersControls
            //jesli tak to go ladujemy, jesli nie to ladujemy domyslne
            cFile = new File( ControlLoadSave.sm_sUserControlsFile );
            if ( cFile.exists() )
                ControlLoadSave.LoadKeySettings( ControlLoadSave.sm_sUserControlsFile );
            else                
                ControlLoadSave.LoadDefaultKeySettings();
            //i po tym wszystkim zapisujemy pod nasza nazwa
            ControlLoadSave.SaveControls( sName );
        }    
        //a teraz zaladowujemy ten plik i zastosowujemy kontrolsy
        ControlLoadSave.LoadSettingsAndApply( sName );
    }


    
    
        ;

        
        
    
}
