/**
 * Klasa sluzaca do przechowywania i wyswietlania informacji
 * o serwerze na ekranie ServerFullInfo
 */
public class NetServerFullInfo extends NetServerInitInfoBase
{
    //wersja zrodel ktorymi aktualnie dysponujemy
    private
    int     m_nGameVerion = 0;
    int     GetGameVerion() { return m_nGameVerion; }
    
    String  GetNetGameVersionNumber() { return ENetGameVersion.GetNetNumber( m_nGameVerion ); }
        
    //aktualna lista graczy
    private java.util.Vector m_aPlayers = new java.util.Vector();
    int     GetPlayersCnt() { return m_aPlayers.size(); }
    String  GetPlayer(int i) { return (m_aPlayers.elementAt(i) instanceof String)?
                                       (String)m_aPlayers.elementAt(i) : null ; }
    //lista map
    private java.util.Vector m_aMaps = new java.util.Vector();
    int     GetMapsCnt() { return m_aMaps.size(); }
    MapsListItem GetMap(int i) { return (m_aMaps.elementAt(i) instanceof MapsListItem)?
                                       (MapsListItem)m_aMaps.elementAt(i) : null ; }
    //numer aktualnej gry (z listy)
    private 
    int m_nCurrentGame = 0;
    int GetCurrentGame(){ return m_nCurrentGame; }
    
    //typ aktualnej gry
    private 
    int m_nCurrentGameType     = ENetGameType.DEFAULT; 
    int GetCurrentGameType(){ return m_nCurrentGameType; }
    
    //ustawienia aktualnej gry
    private 
    NetGameSettings m_cCurrentGameSettings = null;
    NetGameSettings GetCurrentGameSettings(){ return m_cCurrentGameSettings; }
    
    private
    String m_sAddress = " ";          
    String GetAddress(){ return m_sAddress; }
    void   SetAddress(String sAddress){ m_sAddress = sAddress; }
    
    NetServerFullInfo(NetDataContainer cContainer)
    {
        super();
        FromDataContainer( cContainer );
    }
    
    NetServerFullInfo(ModuleNet cModule)
    {
        super( cModule.GetGameInitInfo().GetServerInfo() );

        //lista graczy
        m_aPlayers.removeAllElements();
        NetPlayerInfoList cList = cModule.GetGameInfo().GetPlayerInfoList();
        for ( int i=0; i<cList.GetSize(); i++ )
        {
            NetPlayerInfo cInfo = cList.GetPlayerByIndex( i );
            if ( cInfo == null )
                continue;
            
            m_aPlayers.addElement( cInfo.GetNick() );
        }
                
        //lista map
        m_aMaps.removeAllElements();
        LevelsManager cLevelsMan = cModule.GetGameInitInfo().cLevels;
        for ( int i=0; i<cLevelsMan.GetLevelsCnt(); i++ )
        {
            LevelContent cLevel = cLevelsMan.GetLevel( i );
            MapsListItem cItem = new MapsListItem( cLevel );
            m_aMaps.addElement( cItem );
        }
                
        //stan aktualnej gry
        //numer gry
        m_nCurrentGame         = cLevelsMan.GetCurrentLevelPos();
        //typ
        m_nCurrentGameType     = cModule.GetGameInitInfo().nGameType;
        //game settings
        m_cCurrentGameSettings = cModule.GetGameInfo().GetGameSettings();
        //wersja gry
        m_nGameVerion          = cModule.GetGameInfo().GetGameVersion();
        //adres serwera        
        m_sAddress             = cModule.GetLocalHostIP();
    }
    
    void CopyDataToContainer()
    {
        //Uwaga! Wersja gry idzie ZAWSZE jako pierwsza
        //wersja gry
        m_cContainer.PackInt( m_nGameVerion );
        //adres serwera
        m_cContainer.PackString( m_sAddress );
        
        //teraz dane z NetServerInitInfoBase        
        super.CopyDataToContainer();
                                          
        //lista graczy
        int nPlayersCnt = 0;
        for ( int i=0; i<m_aPlayers.size(); i++ )
        {
            if ( m_aPlayers.elementAt( i ) instanceof String )
            {
                m_cContainer.PackString( (String)m_aPlayers.elementAt( i ) );
                nPlayersCnt++;
            }
        }
        //mozna zapisac na koncu, bo int jest innym typem niz String
        m_cContainer.PackInt( nPlayersCnt );
        
        //lista map
        int nMapsCnt = m_aMaps.size();
        //trzeba zapisac na poczatku, bo int tez bedzie zapisywany
        m_cContainer.PackInt( nMapsCnt );
        for ( int i=0; i<nMapsCnt; i++ )
        {
            MapsListItem cItem = new MapsListItem();
            if ( m_aMaps.elementAt( i ) instanceof MapsListItem )
                cItem = (MapsListItem) m_aMaps.elementAt( i );
            m_cContainer.PackString( cItem.m_sName );
            m_cContainer.PackInt( cItem.m_nType );
        }
        //stan aktualnej gry
        //aktualna mapa
        m_cContainer.PackInt( m_nCurrentGame );
        //typ gry
        m_cContainer.PackInt( m_nCurrentGameType );
        //game settings
        m_cCurrentGameSettings.ToDataContainer( m_cContainer );
    }
    
    void CopyDataFromContainer()
    {
        //wersja gry
        m_nGameVerion = m_cContainer.UnPackInt();
        //adres serwera
        m_sAddress = m_cContainer.UnPackString();

        //teraz dane z NetServerInitInfoBase        
        super.CopyDataFromContainer();
        
        //lista graczy
        int nPlayersCnt = m_cContainer.UnPackInt();
        m_aPlayers.removeAllElements();
        for ( int i=0; i<nPlayersCnt; i++ )
            m_aPlayers.addElement( m_cContainer.UnPackString() );
        
        //lista map
        int nMapsCnt = m_cContainer.UnPackInt(); 
        m_aMaps.removeAllElements();
        for ( int i=0; i<nMapsCnt; i++ )
        {
            MapsListItem cItem = new MapsListItem();
                       
            cItem.m_sName = m_cContainer.UnPackString();
            cItem.m_nType = m_cContainer.UnPackInt();
            
            m_aMaps.addElement( cItem );
        }
        //stan aktualnej gry
        //aktualna mapa
        m_nCurrentGame = m_cContainer.UnPackInt();
        //typ gry
        m_nCurrentGameType = m_cContainer.UnPackInt();
        //game settings
        m_cCurrentGameSettings = ENetGameType.NewGameSettings( m_nCurrentGameType );
        m_cCurrentGameSettings.FromDataContainer( m_cContainer );
    }
}

class MapsListItem
{
    String m_sName = "";
    int    m_nType = ENetGameType.DEFAULT;
    
    MapsListItem()
    {
        m_sName = "";
        m_nType = ENetGameType.DEFAULT;
    }
    
    MapsListItem(LevelContent cLevel)
    {
        m_sName = cLevel.GetLevelName();
        m_nType = cLevel.GetNetGameType();
    }
}
