import java.io.*;
public class ServerInitInfoNet
{
    final static int _MAX_PLAYERS = 16;
    
    static final String sm_sDefaultScriptFileName = "Data/ServerSettingsDefault.scr";
    static final String sm_sScriptFileName        = "Data/ServerSettings.scr";
    
    // adres serwera, moze byc ip lub hostname
    private String m_sServerAddr = "localhost";
    
    // nazwa serwera
    private String m_sServerName = "Chrome BioServer";
    
    // port na ktorym dziala serwer
    private int m_nGamePort = 2176;
    
    // port komunikacyjny na ktorym serwer sie zglasza
    private int m_nCommPort = 2177;
    
    // maksymalna liczba graczy
    private int m_nMaxPlayers = 16;
    
    // gdy true to levele z listy sa loopovane w nieskonczonosc
    private boolean m_bLoopLevels = false;
    
    //czy server dedykowany czy z graczem
    private boolean m_bDedicated = false;
    
    
    
    void SetServerAddr(String sAddr){ m_sServerAddr = sAddr; }
    String GetServerAddr(){ return m_sServerAddr; }
    
    void SetServerName(String sName){ m_sServerName = sName; }
    String GetServerName(){ return m_sServerName; }
    
    void SetGamePort(int nPort){ m_nGamePort = nPort; }
    int GetGamePort(){ return m_nGamePort; }
    
    void SetCommPort(int nPort){ m_nCommPort = nPort; }
    int GetCommPort(){ return m_nCommPort; }
    
    void SetMaxPlayers(int nMax){ m_nMaxPlayers = nMax; }
    int GetMaxPlayers(){ return m_nMaxPlayers; }
    
    void SetLoopLevels(boolean bLoop){ m_bLoopLevels = bLoop; }
    boolean GetLoopLevels(){ return m_bLoopLevels; }
    
    void SetDedicated(boolean bDedicated){ m_bDedicated = bDedicated; }
    boolean GetDedicated(){ return m_bDedicated; }
    
    
    void SetMaxPlayers(String sMax)
    {
        m_nMaxPlayers = Integer.parseInt( sMax );
        if (m_nMaxPlayers <= 0 )
            m_nMaxPlayers = 1;
        else if (m_nMaxPlayers > _MAX_PLAYERS )
            m_nMaxPlayers = _MAX_PLAYERS;
    }
    
    void ParseDefaultScriptFile()
    {
        ParseScriptFile( sm_sDefaultScriptFileName );
    }
    
    void ParseScriptFile()
    {
        ParseScriptFile( sm_sScriptFileName );
    }
    
    void ParseScriptFile(String sFile)
    {
        File cFile = new File( sFile );
        if (!cFile.exists())
        {
            GameObject.LogD("File " + sFile + " not exist !\n");
        }
        else
        {
            ParseKeyword cRoot = GameObject.ParseFile( sFile );
            
            if (cRoot == null)
            {
                GameObject.LogD("Error while read/parse " + sFile + " file !\n");
            }
            else
            {
                ParseKeyword cKey, cSubKey;
                for(cKey=cRoot; cKey!=null; cKey=cKey.GetNext())
                {
                    if (cKey.Equals("ServerName")) 
                        m_sServerName = new String(cKey.GetSParam(0));
                    if (cKey.Equals("MaxPlayer")) 
                        m_nMaxPlayers = cKey.GetIParam(0);
                    if (cKey.Equals("LoopLevels")) 
                        m_bLoopLevels = true;
                    if (cKey.Equals("Dedicated")) 
                        m_bDedicated = true;
                }
                cRoot.delete();
            }
        }
    }
    
    ScriptWriter m_cScriptWriter = new ScriptWriter();
    
    void SaveSettings()
    {
        m_cScriptWriter.Open( sm_sScriptFileName );
                        
        m_cScriptWriter.Write("!ServerName(s)");
        m_cScriptWriter.Write("!MaxPlayer(i)");
        m_cScriptWriter.Write("!LoopLevels()");
        m_cScriptWriter.Write("!Dedicated()");
        m_cScriptWriter.NewLine();
            
        m_cScriptWriter.Write("ServerName(\"" + m_sServerName + "\")");
        m_cScriptWriter.Write("MaxPlayer(" + String.valueOf( m_nMaxPlayers ) + ")");
            
        if ( m_bLoopLevels )
            m_cScriptWriter.Write("LoopLevels()");
        
        if ( m_bDedicated )
            m_cScriptWriter.Write("Dedicated()");

        m_cScriptWriter.Close();
    }
}
