public class NetTeamInfoList extends NetInfoList
{
    NetTeamInfoList(NetGameInfoBase cGameInfo)
    {
        super( cGameInfo );
    }
    
    
    /** Pobranie NetTeamInfo z NetInfo
     */
    private NetTeamInfo GetTeamInfo( NetInfo cInfo )
    {
        return cInfo instanceof NetTeamInfo ? (NetTeamInfo)cInfo : null;
    }
    
    /** Pobranie NetTeamInfo po indeksie
     */
    NetTeamInfo GetTeamByIndex(int nIndex)
    {
        NetInfo cInfo = GetInfoByIndex( nIndex );
        return GetTeamInfo( cInfo );
    }
    
    /** Pobranie NetPlayerInfo po identyfikatorze teamu
     */
    NetTeamInfo GetTeamById(int nTeamId)
    {
        NetInfo cInfo = GetInfoByTeamId( nTeamId );
        return GetTeamInfo( cInfo );
    }

    /** Pobranie NetTeamInfo wg kolejnosci wyswietlania w hudach
     * Podczas gry kolejnosc jest stala: BLUE na grze, RED na dole
     * Po GameOver na gorze jest druzyna ktora zwycizyla
     */
    NetTeamInfo GetTeamByHudDisplayOrder(int nIndex)
    {
        if ( m_cGameInfo != null && m_cGameInfo.IsGameOver() )
            return GetTeamByIndex( nIndex );
        NetInfo cInfo = GetInfoByTeamId( NetTeamInfo._FIRST + nIndex );
        return GetTeamInfo( cInfo );
    }
    
    /** Ustalenie identyfikatora druzyny do ktorej moze
     * podlaczyc sie nowy gracz (na postawie balansu)
     */
    int FindTeamIdByTeamBalance()
    {
        GameObject.LogD(" ");
        
        //zaszla zmiana w wyborze druzyny
        //zakladamy ze nie mamy wybranej zadnej druzyny
        int nRequestedTeamId = NetTeamInfo.NONE;
        
        //Uwaga: pozizszy kod, tak naprawde juz nie ma prawa sie
        //wykonac poniewaz na wejsciu do gry sa tworzone dwie druzyny
        //ktore na poczatku poprostu nie maja graczy
        //na razie jednak tego nie wywalam tego, bo nie wiem czy czasem
        //mi sie to jeszcze nie przyda.
        //TODO: fixit
        /*
        // jak lista jest pusta to trafiasz do druzyny 
        // druzyny ktora sobie wybrales, albo domyslnie do 1 czyli BLUE
        if ( GetSize() == 0 )
        {
            int nTeam = nRequestedTeamId == NetTeamInfo.NONE ? 
                        NetTeamInfo.BLUE : 
                        nRequestedTeamId;
            GameObject.LogLine("first->"+nTeam);
            return nTeam;
        }
        
        // jak na liscie jest tylko jedna osoba to trafiasz
        // do druzyny ktorej jeszcze nie ma
        if ( GetSize() == 1 )
        {
            //zobaczmy jaki numer ma druzyna ktora juz istnieje
            NetTeamInfo cFirstTeamInfo = GetTeamByIndex( 0 );
            
            if ( cFirstTeamInfo == null )
            {
                GameObject.LogD("ERROR: na liscie obiekt 0 "+
                                " nie jest druzyna!!!\n");
                return NetTeamInfo.BLUE;
            }
                
            //jesli ta druzyna jest BLUE, to my bedziemy RED, w przeciwnym razie BLUE
            int nTeam = cFirstTeamInfo.GetTeamId() == NetTeamInfo.BLUE ?
                        NetTeamInfo.RED:
                        NetTeamInfo.BLUE;
            GameObject.LogLine("second->"+nTeam);
            return nTeam;
        }
        */
            
        int nMinTeamPlayers = (int)1e10;
        int nMinTeamIndex   = 0;
        for ( int i=0; i<GetSize(); i++ )
        {
            NetTeamInfo cTeamInfo = GetTeamByIndex( i );
            
            if ( cTeamInfo == null )
                continue;
            
            GameObject.Log(" team "+i+":"+cTeamInfo.GetNumPlayers()+" ");
            //celowo daje znak >= bo teamy sa posotrowane wg
            //wynikow (od najlepszej do najgorszej) zatem jesli 
            //beda druzyny mialy taka sama liczbe graczy, to nowy 
            //gracz dojdzie do tej slabszej)
            if ( nMinTeamPlayers > cTeamInfo.GetNumPlayers() )
            {
                GameObject.LogLine("(min) ");
                nMinTeamPlayers = cTeamInfo.GetNumPlayers();
                nMinTeamIndex   = i;
            }
        }

        NetTeamInfo cFoundTeam = GetTeamByIndex( nMinTeamIndex );
        if ( cFoundTeam == null )
        {
            GameObject.LogD("ERROR: na liscie obiekt "+
                            nMinTeamIndex +
                            " nie jest druzyna!!!\n");
            return NetTeamInfo.BLUE;
        }                        
                
        GameObject.LogLine("->"+nMinTeamIndex+
                "("+cFoundTeam.GetTeamId()+
                " "+NetTeamInfo._TEAM_NAMES[cFoundTeam.GetTeamId()]+")");
        return cFoundTeam.GetTeamId();
    }
    
    /** Zlicza druzyny w ktorej sa jacys gracze
     */
    int CalculateNotEmptyTeamsCnt()
    {
        int nCnt = 0;
        for ( int i=0; i<GetSize(); i++ )
        {
            NetTeamInfo cTeamInfo = GetTeamByIndex( i );
            
            if ( cTeamInfo == null )
                continue;
            
            if ( cTeamInfo.GetNumPlayers() > 0 ) 
                nCnt ++;
        }
        
        return nCnt;
    }
    
    
    /** Sprawdzamy na liscie druzyn, czy ktoras z nich czasem
     * juz nie wygrala
     */
    boolean CheckTeamWon()
	{
        for ( int i=0; i<GetSize(); i++ )
        {
            NetTeamInfo cTeamInfo = GetTeamByIndex( i );
            
            if ( cTeamInfo == null )
                continue;
            
            if ( cTeamInfo.CheckWon() )
            {
                // The winner is... :-)
				return true;
            }
        }
            
        return false;
	}
    
    //zwraca ilosc punktow lidera
    int GetLeaderPoints() 
    { 
        int nPoints = Integer.MIN_VALUE;
        for ( int i=0; i<GetSize(); i++ )
		{
			NetTeamInfo cTeamInfo = GetTeamByIndex( i );
            if ( cTeamInfo == null )
                continue;
			if ( nPoints < cTeamInfo.GetPoints() )
                nPoints = cTeamInfo.GetPoints();
		}
        return nPoints;
    }
    //zwraca ilosc fragow lidera
    int GetLeaderFrags() 
    { 
        int nFrags = Integer.MIN_VALUE;
        for ( int i=0; i<GetSize(); i++ )
		{
			NetTeamInfo cTeamInfo = GetTeamByIndex( i );
            if ( cTeamInfo == null )
                continue;
			if ( nFrags < cTeamInfo.GetFrags() )
                nFrags = cTeamInfo.GetFrags();
		}
        return nFrags;
    }
    /**
     * Procka zwraca trzy liczby, ktore obrazuja aktualny wynik gry
     * [0] to identyfikator druzyny na prowadzeniu
     * [1] to punkty prowadzacej druzyny
     * [2] to punkty przegrywajacej druzyny
     * Liczby te beda interpretowane i wyswietlone w hudzie Gamestatus
     */
    int[] GetGameScoreStatus()
    {
        Sort();
        
        int[] aStatus = new int [3];
            
        NetTeamInfo cTeam1 = GetTeamByIndex( 0 );
        NetTeamInfo cTeam2 = GetTeamByIndex( 1 );
                
        if ( cTeam1 != null )
            aStatus [ 0 ] = cTeam1.GetTeamId();
        if ( cTeam1 != null )
            aStatus [ 1 ] = cTeam1.GetScore();
        if ( cTeam2 != null )
            aStatus [ 2 ] = cTeam2.GetScore();
        
        return aStatus;
    }
    
    /** Sprawdzamy na liscie druzyn, czy minimum 
     * 2 druzyny moga jeszcze grac
     */
    boolean CheckTeamFailed( float fMinTeamsInGame )
	{
        int nTeamsExcluded = 0;
        int nTeamsInGame = 0;
        
        for ( int i=0; i<GetSize(); i++ )
        {
            NetTeamInfo cTeamInfo = GetTeamByIndex( i );
            
            if ( cTeamInfo == null )
                continue;
            
            //tej druzyny juz nie ma w grze
            if ( cTeamInfo.IsExcluded() )
            {
                nTeamsExcluded++;
                continue;
            }
            
            if ( m_cGameInfo.CheckPlayersTeamFailed( cTeamInfo.GetTeamId() ) )
            {
                //wlasnie zostaja wykluczeni
                cTeamInfo.SetExcluded( true );
                nTeamsExcluded++;
            }
            else
            {
                nTeamsInGame++;
            }
        }
            
        return ( nTeamsExcluded > 0 && nTeamsInGame < fMinTeamsInGame );
	}
    
    
    
}
