public class HUDNetStatisticsCTF extends HUDNetStatistics
{
    final static String _COL_TEXTS[] = {"&NetHudStatsCH_Nick&",
                                        "&NetHudStatsCH_Points&",
                                        "&NetHudStatsCH_Frags&",
                                        "&NetHudStatsCH_FlagPossess&",
                                        "&NetHudStatsCH_Ping&"};
    final static int    _COL_WIDTH[] = { 50,    50 ,     50,     80,    50   };
    final static int    _COL_POS  [] = { 50,    170,     250,    330,   440  };
        
    {
        m_sScriptName = "data/hudnet/HUDNetStatisticsCTF.scr";
    }
    
    void InitializeStatistics()
    {
        super.InitializeStatistics();
        
        SetTitle("&NetGameNameCTF&");
    }
   
    void UpdateStatistics()
    {
        if ( m_cNetGameInfo == null || 
             m_tRowsTeam1 == null ||
             m_tRowsTeam2 == null)
            return;
                
        NetTeamInfoList   cTeamInfoList   = m_cNetGameInfo.GetTeamInfoList();
        NetPlayerInfoList cPlayerInfoList = m_cNetGameInfo.GetPlayerInfoList();
        
        if ( cTeamInfoList == null || cPlayerInfoList == null )
            return;
        
        cTeamInfoList.Sort();
        cPlayerInfoList.Sort();
        
        m_nSpaceRowCount = 0;
        
        int nRows = 0;
        
        for (int i = 0; i < cTeamInfoList.GetSize(); i++)
        {
            //teraz wyswietlamy teamy wg scisle okreslonej kolejnosci
            NetTeamInfo cTeam = cTeamInfoList.GetTeamByHudDisplayOrder( i );
            
            if ( cTeam == null )
                continue;

            java.util.Vector vPlayers = new java.util.Vector();
            for (int j = 0; j < cPlayerInfoList.GetSize(); j++)
            {
                NetPlayerInfo cPlayer = cPlayerInfoList.GetPlayerByIndex( j );
                
                if ( cPlayer == null )
                    continue;
                
                if (cPlayer.GetTeamId() == cTeam.GetTeamId())
                    vPlayers.addElement(cPlayer);
            }
            
            UpdateTeam(i, cTeam, vPlayers);
        } 
    }
    
    void SetTeamRow(UIControlObject cRow, NetTeamInfo cTeam)
    {
        Vector vColor = HUDNetManager.GetColorForTeamInfo(cTeam);
        cRow.Show();
        SetRow(cRow, vColor, 1, fTeamRowFontScale, "",
                   Text.Get("&HUDNET_TEAM&") + cTeam.GetTeamName(), 
                   Integer.toString(cTeam.GetPoints()),
                   Integer.toString(cTeam.GetFrags()),
                   "",
                   -1);
    }
    
    void SetRow(int nPlace, UIControlObject cRow, float fAlpha, NetPlayerInfo cPlayer)
    {
        cRow.Show();
        if (cPlayer != null)
        {
            int nFlag = -1;
            if (cPlayer.IsCTFFlagOwner())
                nFlag = cPlayer.GetTeamId();
                    
            Vector vColor = HUDNetManager.GetColorForPlayerInfo( cPlayer );
            SetRow(cRow, vColor, fAlpha, 1, 
                   String.valueOf(nPlace), 
                   cPlayer,
                   String.valueOf( cPlayer.GetPoints() ),
                   String.valueOf( cPlayer.GetFrags() ),
                   String.valueOf( cPlayer.GetLatencyAndLoss()) ,
                   nFlag);
        }
        else
        {
            Vector vColor = new Vector(1,1,1);
            SetRow(cRow, vColor, fAlpha, 1, "...", "...", "...", "...", "...", -1);
        }
    }
    
    void SetRow(UIControlObject cRow, Vector vColor, float fAlpha, float fFontScale, String sPlace, NetPlayerInfo cPlayer, String sPoints, String sFrags, String sPing, int nFlag)
    {
        SetElemText((UIStatic)cRow.GetElem("_PLACE"), sPlace, vColor, fAlpha, fFontScale);
        UIStatic cNick = (UIStatic)cRow.GetElem("_NICK");
        SetElemText(cNick, cPlayer.GetNickClamped(cNick), vColor, fAlpha, fFontScale);
        SetElemText((UIStatic)cRow.GetElem("_POINTS"), sPoints, vColor, fAlpha, fFontScale);        
        SetElemText((UIStatic)cRow.GetElem("_FRAGS"), sFrags, vColor, fAlpha, fFontScale);
        
        switch (nFlag)
        {
        case NetTeamInfo.RED:
            cRow.GetElem("_FLAG_PIC_RED").Show();
            cRow.GetElem("_FLAG_PIC_BLUE").Hide();            
            break;
        case NetTeamInfo.BLUE:
            cRow.GetElem("_FLAG_PIC_BLUE").Show();
            cRow.GetElem("_FLAG_PIC_RED").Hide();
            break;
        default:
            cRow.GetElem("_FLAG_PIC_BLUE").Hide();
            cRow.GetElem("_FLAG_PIC_RED").Hide();
        }
        
        SetElemText((UIStatic)cRow.GetElem("_PING"), sPing, vColor, fAlpha, fFontScale);
    }
    
    void SetRow(UIControlObject cRow, Vector vColor, float fAlpha, float fFontScale, String sPlace, String sNick, String sPoints, String sFrags, String sPing, int nFlag)
    {
        SetElemText((UIStatic)cRow.GetElem("_PLACE"), sPlace, vColor, fAlpha, fFontScale);
        SetElemText((UIStatic)cRow.GetElem("_NICK"), sNick, vColor, fAlpha, fFontScale);
        SetElemText((UIStatic)cRow.GetElem("_POINTS"), sPoints, vColor, fAlpha, fFontScale);        
        SetElemText((UIStatic)cRow.GetElem("_FRAGS"), sFrags, vColor, fAlpha, fFontScale);
        
        cRow.GetElem("_FLAG_PIC_BLUE").Hide();
        cRow.GetElem("_FLAG_PIC_RED").Hide();
        
        SetElemText((UIStatic)cRow.GetElem("_PING"), sPing, vColor, fAlpha, fFontScale);
    }
    
    void InitColumns()
    {
        m_tColumns = new UIStatic[6];
        m_tColumns[0] = (UIStatic)GetElem("ID_COLUMN_PLACE");
        m_tColumns[1] = (UIStatic)GetElem("ID_COLUMN_NICK");
        m_tColumns[2] = (UIStatic)GetElem("ID_COLUMN_POINTS");
        m_tColumns[3] = (UIStatic)GetElem("ID_COLUMN_FRAGS");
        m_tColumns[4] = (UIStatic)GetElem("ID_COLUMN_FLAG");
        m_tColumns[5] = (UIStatic)GetElem("ID_COLUMN_PING");
    }
    
  
    void InitializePlayersInfo()
    {
        if ( m_cNetGameInfo == null)
            return;
                
        NetTeamInfoList   cTeamInfoList   = m_cNetGameInfo.GetTeamInfoList();
        NetPlayerInfoList cPlayerInfoList = m_cNetGameInfo.GetPlayerInfoList();
        
        if ( cTeamInfoList == null || cPlayerInfoList == null )
            return;
        
        cTeamInfoList.Sort();
        cPlayerInfoList.Sort();
        
        m_nSpaceRowCount = 0;
        
        for (int i = 0; i < cTeamInfoList.GetSize(); i++)
        {
            /*NetTeamInfo cTeam = cTeamInfoList.GetTeamByIndex( i );
            
            if ( cTeam == null )
                continue;
                                        
            String aTexts[] = new String[ GetColumnCnt() ];
            aTexts[0] = "Team: " + cTeam.GetTeamName();
            aTexts[1] = Integer.toString(cTeam.GetPoints());
            aTexts[2] = Integer.toString(cTeam.GetFrags());
            aTexts[3] = "";
            aTexts[4] = "";
            
            //TODO pobierac kolor
            //Vector vColor = HUDNetManager.GetColorForTeamInfo(cTeam);
            Vector vColor = new Vector(1,1,1);
            
            AddRow( aTexts, vColor );
                        
            for (int j = 0; j < cPlayerInfoList.GetSize(); j++)
            {
                NetPlayerInfo cPlayer = cPlayerInfoList.GetPlayerByIndex( j );
                
                if ( cPlayer == null )
                    continue;
                
                if (cPlayer.GetTeamId() == cTeam.GetTeamId())
                {
                    if ( ! cPlayer.IsVisibleOnHudNet() )
                    {
                        AddEmptyRowIfPossible();
                        continue;
                    }
                    
                    aTexts = new String[ GetColumnCnt() ];
                    aTexts[0] = "  " + cPlayer.GetNick();
                    aTexts[1] = Integer.toString( cPlayer.GetPoints() );
                    aTexts[2] = Integer.toString( cPlayer.GetFrags() );
                    aTexts[3] = cPlayer.IsCTFFlagOwner()?"YES!":""; 
                    aTexts[4] = Integer.toString( cPlayer.GetLatencyAndLoss() );
                        
                    vColor = HUDNetManager.GetColorForPlayerInfo( cPlayer );
                    
                    AddRow( aTexts, vColor );
                }
            } 
            m_nSpaceRowCount++;*/
        }
    }    
}
