/** System komunikatow przychodzacych oczekujacych 
 * na ogsluzenie
 * Niektore komunikaty przychodzace z sieci musza
 * zaczekac na wyswietlenie do czasu az beda komplente,
 * tzn znany bedzie na kliencie nick gracza "o ktorym mowa" itp.
 */
public class IncomingMsgManager extends GameObject
{
    /** Czas co jaki bedziemy dokonywac walidacji
     */
    static float _VALIDATE_MSG_TIME = 0.1f;
        
    /** NetGameInfo w ktorym umieszczony jest ten manager
     */
    protected NetGameInfoBase m_cGameInfo   = null;
    void SetGameInfo(NetGameInfoBase cGameInfo) { m_cGameInfo = cGameInfo; }
    
    /** Lista obiektow
     */
    private java.util.Vector m_aList = null;
    
    /** HudManager gracza, przez ktory bedziemy (dodatkowo)
     * wysylali komunikaty do okna logow
     */
    protected HUDManager m_cPlayerHud       = null;
            
    IncomingMsgManager()
    {
        m_aList     = new java.util.Vector();
    }
    
    int METHOD_Validate = 0;
    
    public void OnCreate()
    {
        METHOD_Validate = RegisterCallMethod("Validate");
    }
    
    public void OnDestroy()
    {
        m_cPlayerHud = null;
        m_cGameInfo  = null;
    }
    
    /** Dodaj element do systemu
     */
    void Add(NetIncomingMsg cMsg)
    {
        if ( cMsg.Validate() )
        {
            AddToHUD( cMsg );
        }
        else
        {
            // dopisac do listy, zeby komunikat przebywal tam
            // do czasu az nie zostanie poprawnie skompletowany
            m_aList.addElement( cMsg );
                
            // i zaplanowac probe skladania za jakis czas
            CallMethodIn( METHOD_Validate, _VALIDATE_MSG_TIME );
        }
            
        /*
        //ta wersje nalezy zastosowac, jesli chcialoby sie
        //modyfikowac "recznie" jakies parametry komunikatu
        //takie jak kolor czy dzwiek, wtedy AddIncomingMsg(... musi
        //zwracac komunikat (teraz jest void) a tu nalezy wstawiac go _zawsze_
        //do listy, zeby obsluzyc (wyswietlic) go z maly opoznieniem
            
        // dopisac do listy, zeby komunikat przebywal tam
        // do czasu az nie zostanie poprawnie skompletowany
        m_aList.addElement( cMsg );
            
        // i zaplanowac probe skladania za jakis czas
        DisableCallMethod( METHOD_Validate );
        CallMethodIn( METHOD_Validate, 0.0f );
        */
        
    }
        
    /** pobranie dlugosci listy komunikatow w systemie
     */
    int GetSize()
    {
        return m_aList.size();
    }
    
    /**
     * Metoda probujaca wyslac komunikaty na huda
     */
    void Validate()
    {
        for ( int i=0; i<m_aList.size(); i++ )
		{
			NetIncomingMsg cMsg = (NetIncomingMsg)m_aList.elementAt(i);
            
            if ( cMsg == null )
            {
                GameObject.LogD(" Message "+i+" == null !\n");
                continue;
            }
                
            // sprawdzmy czy komunikat juz jest gotowy
            if ( cMsg.Validate() )
            {
                // jesli tak, to dodajemy go do huda
                AddToHUD( cMsg );
                
                // i usuwamy z listy oczekujacych
                m_aList.removeElementAt( i );
                // po usunieciu wszystkie elementy "z tylu" zostana przesuniete
                // o 1 do przodu, zatem zeby nikogo nie pominac:
                i--;
            }
        }
        
        if ( m_aList.size() > 0 )
        {
            CallMethodIn( METHOD_Validate, _VALIDATE_MSG_TIME );
        }
    }
       
    /**
     * Dodanie gotowego komunikatu do huda
     */
    void AddToHUD(NetIncomingMsg cMsg)
    {
        if ( cMsg == null )
        {
            CrashLog("ERROR: "+this+" AddToHUD cMsg == null\n");
            return;
        }
        
        String sText = cMsg.GetMessage();
        if ( sText == null )
        {
            CrashLog("ERROR: "+this+" AddToHUD "+cMsg+" sText == null\n");
            return;
        }
        
        Vector vColor = cMsg.GetColor();
        
        if ( m_cPlayerHud == null )
        {
            if ( m_cGameInfo                                               != null &&
                 m_cGameInfo.GetPlayerInfoList()                           != null &&
                 m_cGameInfo.GetPlayerInfoList().GetMyPlayer()             != null &&
                 m_cGameInfo.GetPlayerInfoList().GetMyPlayer().GetPlayer() != null )
            {
                m_cPlayerHud = m_cGameInfo.GetPlayerInfoList().GetMyPlayer().GetPlayer().HUD;
            }
        }
        
        //TODO: czy wszystkie komunikaty maja trafiac do okienka
        //logow, czy tylko niektore?
        if ( m_cPlayerHud != null )
            m_cPlayerHud.AddLogsText( sText, vColor );
                
        if ( m_cGameInfo            != null &&
             m_cGameInfo.GetHUDNet()!= null )
        {
            HUDNetManager cHUDNet = m_cGameInfo.GetHUDNet();
            
            switch ( cMsg.GetTargetHudId() )
            {
            case NetMsg._TARGETHUD_INCOMING:
                cHUDNet.AddIncomingMessage( sText, vColor );    
                break;
                
            case NetMsg._TARGETHUD_OBJECTIVE:
                //cHUDNet.AddIncomingMessage( sText );    
                break;
                
            default:
                cHUDNet.AddIncomingMessage( sText, vColor );    
            }
        }    
        
        String sWave = cMsg.GetSound();
        if ( sWave != null )
            PlayWave( sWave + ".wav" );
        
        cMsg.OnMessageAppearsOnHud();
    }
}
