/**
 * Szyby sieciowe
 * Zbijanie szyb w grze sieciowej odbywa sie niezaleznie na serwerze
 * oraz na klientach
 * Serwer po zbiciu szyby wysyla potwierdzenie tego faktu na klienta.
 * Klient jesli zbije u siebie szybe to czeka na to potwierdzenie
 * od serwera, jesli nie przyjdzie przez odpowiedni czas to
 * respawnuje ja (zakladajac ze zbil ja przypadkiem)
 * Jesli przyjdzie do klienta informacja od serwera o zbiciu szyby
 * a na kliencie z jakis przyczyn szyba sie nie zbila, to zostaje
 * ona zbita "recznie" przez klienta (wg parametrow z otrzymanego komunikatu)
 * 
 * uwaga: klasa GlasssNet dziedziczy po MeshObject'cie mimo ze singlowa 
 * klasa Glass po ChromeActorze!!!!!
 * 
 */
public class GlassNet extends MeshObject
{
    /**
     * Lista rozbitych szyb (elementow) na serwerze
     * Na serwerze lista ta jest uaktualniana wg kolejnych zdarzen
     * zbijania szyb
     * Na kliencie:
     * - po wejsciu do gry dostajemy od serwera uproszczone informacje
     * z samymi numerkami meshelementow ktora szyba jest rozbita i wtedy
     * rozbijamy te szyby u siebie "po cichu"
     * - podczas gry otrzymujemy od serwera szczegolowe dane na temat
     * rozbicia szyby, po to, zeby ewentualnie rozbic to szybe u siebie
     * lokalnie (jak nie zostala jeszcze rozbita)
     */
    java.util.Vector m_aBrokenGlassesOnServer = new java.util.Vector();

    /**
     * _Tymczasowa_ lista rozbitych szyb (elementow) na kliencie
     * Przechowujemy informacje o numerkach meshelementow rozbitych
     * szyb, wraz z czasami kiedy nastapilo rozbicie,
     * Jesli serwer nie potwierdzi zbicia szyby to zostanie odpowiednia
     * szyba zostanie zrespawnowana
     * Jak juz wczesniej podano jest to lista tymczasowa, bowiem 
     * po potwierdzeniu z serwera (lub gdy ten nie potwierdzi zbicia 
     * w okreslonym czasie) info o stluczonej szybie jest usuwane
     */
    java.util.Vector m_aBrokenGlassesOnClient = new java.util.Vector();

    
    /**
     * Flaga mowiaca o tym czy ignorowac przychodzenie
     * handlera FXBrokenGlass_OnBreakGlass
     * Ustawiany na true na kliencie an czas "recznego"
     * zbijania szyb (wg parametrow serwera)
     */
    private boolean m_bIgnoreOnBreakGlassHandler = false;
    
    int METHOD_ClientCheckBrokenGlass = 0;
    
    void OnCreate()
    {
        METHOD_ClientCheckBrokenGlass = RegisterCallMethod( "ClientCheckBrokenGlass" );
        super.OnCreate();
    }
    
    /**
     * Handler przychodzacy z enginu informujacy o zbitciu szyby
     */
    void FXBrokenGlass_OnBreakGlass( int nElement, boolean bBullet, 
                                     Vector vPosition, Vector vDirection, 
                                     float fVelocity, float fRadius )
    {
        LogD(" FXBrokenGlass_OnBreakGlass "+(m_bIgnoreOnBreakGlassHandler?"ignored":"")+"\n");
        
        if ( m_bIgnoreOnBreakGlassHandler )
            return;
        
        if ( Role == ERole._AUTHORITY )
        {
            BrokenGlassServerInfo cInfo = new BrokenGlassServerInfo( 
                                                nElement, bBullet, 
                                                vPosition, vDirection, 
                                                fVelocity, fRadius );
            
            m_aBrokenGlassesOnServer.addElement( cInfo );
            MsgSendClientBrokenGlass( cInfo );

            //dodatkowo wylaczam kolizje z elementem zbitej szyby, 
            //poniewaz na serwerze dedykowanym nie dziala system FX'ow
            //i szyba sie tak na prawde nie zbijala "do konca"
            //Uwaga! Jezeli szyba bedzie tylko _czescia_ jakiegos
            //mesh elementu, to moze to byc przyczyna bledow, bowiem
            //kolizja wylaczana jest z caly elementem!
            EnableCollisionsElement( nElement, false );
        }
        else
        {
            BrokenGlassClientInfo cInfo = new BrokenGlassClientInfo( 
                                                nElement, GetTime() );
            m_aBrokenGlassesOnClient.addElement( cInfo );
            
            if ( m_aBrokenGlassesOnClient.size() > 0 )
            {
                DisableCallMethod( METHOD_ClientCheckBrokenGlass );
                CallMethodEvery( METHOD_ClientCheckBrokenGlass, 1.0f );
            }
        }
    }

    /**
     * Respawn wolany na serwerze przy restarcie gry/rundy
     */        
    void ServerRespawn()
    {
        Respawn();
        
        if ( Role == ERole._AUTHORITY )
            MsgSendClientRespawn();
    }

    /**
     * Metoda wolana na kliencie, sprawdza czy wszystkie zbite 
     * szyby na kliencie (lokalnie) juz sa potwierdzone przez serwer
     * Jezeli przez jakis czas nie bedziemy mieli takiego 
     * potwierdzenia to respawnujemy ta szybe i zapominamy o
     * naszym zbiciu
     */
    void ClientCheckBrokenGlass()
    {
        for ( int nC = 0; nC < m_aBrokenGlassesOnClient.size() ; nC++ )
        {
            BrokenGlassClientInfo cClientInfo = (BrokenGlassClientInfo)m_aBrokenGlassesOnClient.get( nC );
            
            for ( int nS = 0; nS < m_aBrokenGlassesOnServer.size() ; nS++ )
            {
                BrokenGlassServerInfo cServerInfo = (BrokenGlassServerInfo)m_aBrokenGlassesOnServer.get( nS );
                
                if ( cServerInfo.m_nElement == cClientInfo.GetElement() )
                {
                    //jest tez na liscie serwerowej, no to 
                    //przestajemy go sprawdzac
                    m_aBrokenGlassesOnClient.removeElement( cClientInfo );        
                    nC --;
                    cClientInfo = null;
                    break;
                }
            }
            
            if ( cClientInfo != null )
            {
                //nie znaleziony na liscie serwerowej
                
                if ( cClientInfo.IsTimeOut( GetTime() ) ) 
                {
                    //juz minelo zbyt wiele czasu na potwierdzenie,
                    //chyba trzeba ta szybe u nas przywrocic
                    FXBrokenGlass_RespawnGlassAtElement( cClientInfo.GetElement() );
                    //i zapominamy o tym                    
                    m_aBrokenGlassesOnClient.removeElement( cClientInfo );        
                    nC --;
                }
            }
        }
        
        if ( m_aBrokenGlassesOnClient.size() < 0 )
            DisableCallMethod( METHOD_ClientCheckBrokenGlass );
    }
    
    /**
     * Metoda wolana na kliencie, zbija lokalnie szybe wg parametrow 
     * przyslanych z serwera
     * Wolana w przypadku, gdy z jakis przyczyn na kliencie
     * zbicie szyby nie doszlo do skutku o serwer nas o takim
     * fakcie powiadomil
     */
    void ClientBreakGlass( BrokenGlassServerInfo cServerInfo )
    {
        //zapamietajmy stan flagi
        boolean bIgnoreOnBreakGlassHandler = m_bIgnoreOnBreakGlassHandler;
        //ustawmy flage na true
        m_bIgnoreOnBreakGlassHandler = true;
        
        //dokonajmy odpowiedniego zbicia szyby
        if ( cServerInfo.m_bBullet )
            FXBrokenGlass_ShotABulletToElement( cServerInfo.m_nElement, 
                                                cServerInfo.m_vPosition, cServerInfo.m_vDirection, 
                                                cServerInfo.m_fVelocity, cServerInfo.m_fRadius );
        else                                                
            FXBrokenGlass_CauseAnExplosionToElement( cServerInfo.m_nElement, 
                                                     cServerInfo.m_vPosition, 
                                                     cServerInfo.m_fVelocity, cServerInfo.m_fRadius );
        
        //przywrocmy stan handlera
        m_bIgnoreOnBreakGlassHandler = bIgnoreOnBreakGlassHandler;
    }
    
    /**
     * Metoda wolana zarowno na serwerze jak i kliencie
     * Respawn szyb w obiekcie,
     */
    void Respawn()
    {
        FXBrokenGlass_RespawnAll();
        
        m_aBrokenGlassesOnServer.removeAllElements();
        m_aBrokenGlassesOnClient.removeAllElements();
    }
    
    
    // to bedzie obiekt sieciowy wiec go zainicjujmy
    void ReplicationInit()
    {
    }
    
    
    void OnReplicateServer()
    {
        if ( bNetInit )
        {
            MsgSendClientState();
        }
    }
    
    
    final static int _MSG_CLIENT_STATE          = 0;
    final static int _MSG_CLIENT_BROKEN_GLASS   = 1;
    final static int _MSG_CLIENT_RESPAWN        = 2;
    
    public void OnMessage()
    {
        switch ( MessageGetId() )
        {
        case _MSG_CLIENT_STATE:    
            LogD(" _MSG_CLIENT_STATE\n");
            MsgRecvClientState();
            break;
            
        case _MSG_CLIENT_BROKEN_GLASS:
            LogD(" _MSG_CLIENT_BROKEN_GLASS\n");
            MsgRecvClientBrokenGlass();
            break;
            
        case _MSG_CLIENT_RESPAWN:
            LogD(" _MSG_CLIENT_RESPAWN\n");
            MsgRecvClientRespawn();
            break;
            
        default:
            MessageCancel();
        }
    }
    
    void MsgSendClientState()
    {
        LogD("\n");
        MessageNew( _MSG_CLIENT_STATE );
        MessagePackInt( m_aBrokenGlassesOnServer.size() );
        for ( int i=0; i<m_aBrokenGlassesOnServer.size(); i++ )
        {
            BrokenGlassServerInfo cServerInfo = (BrokenGlassServerInfo)m_aBrokenGlassesOnServer.get( i );
            MessagePackInt( cServerInfo.m_nElement );
        }
        MessageSend();
    }
    
    void MsgRecvClientState()
    {
        m_aBrokenGlassesOnServer.removeAllElements();
        m_aBrokenGlassesOnClient.removeAllElements();
        
        int nSize = MessageUnPackInt();
        for ( int i=0; i<nSize; i++ )
        {
            BrokenGlassServerInfo cServerInfo = new BrokenGlassServerInfo();
            cServerInfo.m_nElement = MessageUnPackInt();
            m_aBrokenGlassesOnServer.addElement( cServerInfo );
        }
        MessageEnd();
        
        for ( int i=0; i<m_aBrokenGlassesOnServer.size(); i++ )
        {
            BrokenGlassServerInfo cServerInfo = (BrokenGlassServerInfo)m_aBrokenGlassesOnServer.get( i );
            // zbijamy po cichu odpowiednia szybe
            FXBrokenGlass_BreakGlassAtElementQuietly( cServerInfo.m_nElement );
        }
    }
   
                                                
    
    void MsgSendClientBrokenGlass( BrokenGlassServerInfo cInfo )
    {
        LogD("\n");
        MessageNew( _MSG_CLIENT_BROKEN_GLASS );
        MessagePackInt( cInfo.m_nElement );
        MessagePackBoolean( cInfo.m_bBullet );
        MessagePackVector( cInfo.m_vPosition );
        MessagePackVector( cInfo.m_vDirection );
        MessagePackFloat( cInfo.m_fVelocity );
        MessagePackFloat( cInfo.m_fRadius );
        MessageSend();
    }
    
    void MsgRecvClientBrokenGlass()
    {
        BrokenGlassServerInfo cServerInfo = new BrokenGlassServerInfo();
        cServerInfo.m_nElement    = MessageUnPackInt();
        cServerInfo.m_bBullet     = MessageUnPackBoolean();
        cServerInfo.m_vPosition   = MessageUnPackVector();
        cServerInfo.m_vDirection  = MessageUnPackVector();
        cServerInfo.m_fVelocity   = MessageUnPackFloat();
        cServerInfo.m_fRadius     = MessageUnPackFloat();
        MessageEnd();
                
        m_aBrokenGlassesOnServer.addElement( cServerInfo );
        
        for ( int nC = 0; nC < m_aBrokenGlassesOnClient.size() ; nC++ )
        {
            BrokenGlassClientInfo cClientInfo = (BrokenGlassClientInfo)m_aBrokenGlassesOnClient.get( nC );
            
            if ( cServerInfo.m_nElement == cClientInfo.GetElement() )
            {
                //znalazl sie na liscie klienta, czyli szyba juz zbita
                m_aBrokenGlassesOnClient.removeElement( cClientInfo );        
                //wychodzimy stad
                return;
            }
        }
        
        //jesli nie ma tej szyby na liscie m_aBrokenGlassesClient
        //to oznacza ze na kliencie ta szyba sie z jakis przyczyn nie
        //zbila, zatem musimy to zrobic recznie
        ClientBreakGlass( cServerInfo );
    }
    
    
    void MsgSendClientRespawn()
    {
        LogD("\n");
        MessageNew( _MSG_CLIENT_RESPAWN );
        MessageSend();
    }
    
    void MsgRecvClientRespawn()
    {
        MessageEnd();
        
        Respawn();
    }
    
}

/**
 * Klasa przechowujaca informacje o zbitych szybach na kliencie
 */
class BrokenGlassClientInfo
{
    final static float _CONFIRM_TIME_OUT = 3.0f;
    
    int     m_nElement = -1;
    float   m_fTime    = 0.0f;
    
    BrokenGlassClientInfo(int nElement, float fTime )
    {
        m_nElement = nElement;
        m_fTime    = fTime;
    }
    
    int GetElement() { return m_nElement; }
    
    boolean IsTimeOut(float fTimeNow)
    {
        return (m_fTime + _CONFIRM_TIME_OUT < fTimeNow);
    }
}

/**
 * Klasa przechowujaca informacje o zbitych szybach na kliencie
 */
class BrokenGlassServerInfo
{
    int     m_nElement   = -1;
    boolean m_bBullet    = false;
    Vector  m_vPosition  = null;
    Vector  m_vDirection = null;
    float   m_fVelocity  = 0.0f;
    float   m_fRadius    = 0.0f;
           
    BrokenGlassServerInfo( int nElement, boolean bBullet, 
                           Vector vPosition, Vector vDirection, 
                           float fVelocity, float fRadius )
    {
        m_nElement   = nElement;
        m_bBullet    = bBullet;
        m_vPosition  = new Vector( vPosition );
        m_vDirection = new Vector( vDirection );
        m_fVelocity  = fVelocity;
        m_fRadius    = fRadius;
    }
    
    BrokenGlassServerInfo()
    {
    }
}
