/**
 * Klasa przechowujaca obiekty sieciowe, ktore nie sa aktualnie
 * wykorzystywane w grze i mozna bedzie z nich skorzystac
 * w przyszlosci (recycling)
 */
public class NetObjectsManager extends GameObject
{
    SpareNetObjectsList m_cSpareObjectsList = null;
    
    boolean m_bUseSpareObjectsList = true;
    
    ModuleNet m_cModuleNet = null;
    
    void OnCreate()
    {
        m_cSpareObjectsList = new SpareNetObjectsList();
        
        if ( GetModule() instanceof ModuleNet )
            m_cModuleNet = (ModuleNet)GetModule();
        else
            NetLog.Error("Obiekt klasy "+this.getClass().getName()+" powinien dzialac na module sieciowym\n");
    }
    
    void OnDestroy()
    {
        if ( m_cSpareObjectsList != null )
        {
            m_cSpareObjectsList.Destroy();
        }
        m_cModuleNet = null;
    }
        
    
    /**
     * Stworzenie obiektu.
     * Jesli taki obiekt juz istnieje na liscie zapasowych to jest zwracany
     * Jesli nie, to jest stworzony przez cCreatora
     */
    GameObject CreateNewObject( Class cClass, GameObject cCreator )
    {
        return CreateNewObject( cClass.getName(), cCreator );
    }
    GameObject CreateNewObject( String sClassName, GameObject cCreator )
    {
        //szukamy w m_cSpareObjectsList taki obiekt
        GameObject cObj = m_bUseSpareObjectsList ?
                          m_cSpareObjectsList.GetSpareObject( sClassName ) : 
                          null ;
        if ( cObj == null )
        {
            //takiego obiektu nie ma w liscie, wiec go tworzymy
            cObj = cCreator.CreateObject( sClassName );    
            //LogR(" Obiekt "+cObj+" stworzony \n");
        }
        return cObj; 
    }
    
    /**
     * Skasowanie obiektu
     * Jesli obiekt ten nalezy do "reuzywalnych" to zostanie
     * zapamietany w liscie zapasowej
     * Jesli nie bedzie nam juz potrzeby to od razu jest kasowany
     */
    void DeleteObject( GameObject cObj )
    {
        if ( m_bUseSpareObjectsList && ModuleNet.IsGameGoing() &&
             ObjectClassHandledInSpareList( cObj )  )
        {
            if ( CanAddToSpateList(cObj) )
                m_cSpareObjectsList.Add( cObj );
            //LogR(" Obiekt "+cObj+" dodany do listy oczekujacej \n");
            //Obiekt dodany do listy spare musi sam zadbac o to
            //aby sie "wyczyscic" i przelaczyc w odpowiedni stan
        }
        else
            cObj.delete();
    }
    
    /**
     * Metoda decyduje, czy obiekt nadaje sie do listy
     * do ponownego uzycia.
     */
    boolean ObjectClassHandledInSpareList( GameObject cObj )
    {
        //na razie tylko stosujemy inventory objecty
        return cObj instanceof InvObject;
    }
    
    
    boolean CanAddToSpateList( GameObject cObj )
    {
        if ( !( cObj instanceof InvObject) )
            return false;
        
        if ( m_cSpareObjectsList.IsOnList( cObj ) )
            return false;
            
        InvObject cInvObj = (InvObject)cObj;
        if ( cInvObj.nInvState != EInvObjState._ONGROUND )
        {
            NetLog.InvObj("Obiekt "+cInvObj.GetNetInfo()+" nie moze wejsc do listy SPARE\n");
            if ( NetLog.IsEnabled( NetLog._TYPE_INVENTORY_OBJECT ) )
                Thread.dumpStack();
            return false;
        }
        
        if ( !( cObj instanceof Weapon) )
            return true;
        Weapon cWeapon = (Weapon)cObj;
        if ( cWeapon.IsInWalkerMode() )
        {
            NetLog.InvObj("Obiekt "+cInvObj.GetNetInfo()+" nie moze wejsc do listy SPARE (IsInWalkerMode())\n");
            if ( NetLog.IsEnabled( NetLog._TYPE_INVENTORY_OBJECT ) )
                Thread.dumpStack();
            return false;
        }
        
        if ( cWeapon instanceof CannonController )
        {
            NetLog.InvObj("Obiekt "+cInvObj.GetNetInfo()+" nie moze wejsc do listy SPARE ("+cWeapon.getClass().getName()+")\n");
            if ( NetLog.IsEnabled( NetLog._TYPE_INVENTORY_OBJECT ) )
                Thread.dumpStack();
            return false;
        }
        
        return true;
    }
        
    
    void ShowObjects()
    {
        m_cSpareObjectsList.LogList( "" );
    }
    
    
}
