public class PlayerNetBase extends PlayerHUD
{
    PlayerNetBase thisPlayerNetBase = this;

    
    /** Maksymakle ziarno fizyki z jakim powinien byc symulowany gracz
     */
    //uwaga: w chwili obecnej trzeba tu stosowac wielokrotnosc
    //potegi 2, (do czasu az zaimplementowane bedzie zaokraglanie czasu)
    static float sm_fPhysicsGranularity = 1.0f / 16f;

    /**
     * CLIENT:
     * flaga mowiaca o tym, ze gracz zostal juz zainicjalozowany z serwera
     */
    private
    boolean m_bInitializedFromServer     = false;
    boolean IsInitializedFromServer(){ return m_bInitializedFromServer; }

    
    /** Modul sieciowy
     */
    private 
    ModuleNet   m_cModule   = null;
    ModuleNet   GetModuleNet(){ return m_cModule; }
   
    /** Obiekt nadzorujacy gra
     */
    private
    NetGameInfo m_cGameInfo = null;
    NetGameInfo GetGameInfo(){ return m_cGameInfo; }
    
    /**SERVER Skojarzona z graczem struktura przechowujaca dane
     * o osiagnieciach gracza
     */
    private
    NetPlayerInfo m_cPlayerInfo = null;
    NetPlayerInfo GetPlayerInfo(){ return m_cPlayerInfo; }
    void          SetPlayerInfo(NetPlayerInfo cInfo){ m_cPlayerInfo = cInfo; }
    

    /**
     * Ilosc respawnow gracza w grze
     * Zliczana lokalnie
     */        
    private
    int  m_nRespawnCnt = 0;
    int  GetRespawnCnt(){ return m_nRespawnCnt; }
    void SetRespawnCnt(int nVal){ m_nRespawnCnt = nVal; }
    void IncRespawnCnt(){ m_nRespawnCnt++; }

    
    /** SERVER
     * Aktualna kamera gracza z klienta (bedzie tu sie przelaczac 
     * pomiedzy: "zwykla" kamera, deathcamera, ewentualnie jakas free camera
     * TODO: w przyszlosci przystosowac do przechowywania wektora kamer
     */
    public CameraPlayer cCurrentCamera = null;
    
    
    /**SERVER Identyfikator hosta, do ktorego nalezy ten gracz.
     * Sluzy do usuwania gracza gdy sie odlacza klient. 
     * Ustawiane gdy gracz jest dolaczany
     */
    public int nOwnerID;
    
    ///////////////////////////////////////////////////////////////////////////
    // stany playera
    byte   m_nState = 0;
    byte   GetState(){ return m_nState; }
    static final byte _STATE_DIE          =   0; 
    static final byte _STATE_SPAWNED      =   1; 
    static final byte _STATE_IN_VEHICLE   =   2; 
    static final byte _STATE_ON_LIFT      =   3;//TODO: ten stan musi byc niezalezny od pozostalych!!
    static final byte _STATE_BINOCULARS   =   4;
    static final byte _STATE_AIMING       =   5;
    
    //ostatnio ustalany stan z serwera
    private byte  m_nStateFromServer = _STATE_DIE;
    //czas ostatniej zmiany stanu playera
    private float m_fStateFromServerTime = -1.0f;
    
    void SetStateFromServer(byte nState,float fTime)
    {
        m_nStateFromServer     = (nState != _STATE_DIE) ? 
                                 _STATE_SPAWNED : _STATE_DIE;
        m_fStateFromServerTime = fTime;
    }
    
    byte  GetStateFromServer() { return m_nStateFromServer; }
    float GetStateFromServerTime() { return m_fStateFromServerTime; }
    
    void CheckStateFromServer( byte nState,float fTime )
    {
        //daje 2.5 sekundy marginesu, bo akcje mogly sie przeciez rozjechac
        if ( (m_nStateFromServer != nState) && (m_fStateFromServerTime + 2.5f < fTime) )
            MsgSendServerStateRequest();
    }

    /** Aktualne parametry spawn point gracza
     */
    Vector  vSpawnPointPosition = null;
    Vector  vSpawnPointForward  = null;
    boolean bSpawnPointInside   = false;
    float   fSpawnTime          = -1.0f;
    
    /** Zmienna sluzaca do replikacji motion.bAiming
     */
    boolean bAiming = false;
    
    
    /** Aktualny zoom kamery na wlascicielu (interpretowany na serwerze gry sieciowej)
     */
    public float fCurrentViewZoom   = fEyeZoomingFactor;
    public float fLastViewZoom      = fCurrentViewZoom;
    
    public int  nCurrentViewMode   = -1;
    public int  nLastViewMode      = nCurrentViewMode;
    
    
    //czy gracz ma sprawdzac obiekt "na celowniku"
    private boolean m_bCheckTargetInfo = false;
    
    ////////////////////////////////////////////////////////////////////
    // SPAKOWANE WERSJE DANYCH DO PRZESYLANIA W REPLICATION SETACH
    //
    final static byte _STATE_BITS_FIRING_MASK   = (byte)0x03;
    final static byte _STATE_BITS_WALKING       = (byte)0x04;
    final static byte _STATE_BITS_DUCKING       = (byte)0x08;
    final static byte _STATE_BITS_CRAWLING      = (byte)0x10;
    final static byte _STATE_BITS_AIMING        = (byte)0x20;
    final static byte _STATE_BITS_RECOILING     = (byte)0x40;
    final static byte _STATE_BITS_HIT_REACTION  = (byte)0x80;

    /** 'Spakowana' orientacja gracza w poziomie (SetDir) */
    public byte byAngleH;
    /** 'Spakowana' orientacja gracza w pionie (fAimAngleY) */
    public byte byAngleV;
    
    /** 'Spakowana' wersja booleanow i zmiennych kilkubitowych replikowanych w graczu 
     * (bWalking;bDucking;bCrawling;nFiring;bAiming;nHitReaction;bRecoiling)
     */
    public byte byStateBits;
    
    /** 'Spakowana' wersja ilosci zycia gracza (stosunek fHealth/fMaxHealth)
     */
    public byte byHealth;
    
    ////////////////////////////////////////////////////////////////////
    
    /** Wspolczynnik nieistotnosci gracza
     */
    public static float _UNIMPORTANCE_FACTOR = 0.0015f;

    ///////////////////////////////////////////////////////////
    /**
     * Definicje message'y
     *  
     * Nazwa messega zaczyna sie od SERVER gdy wysyla go klient do serwera,
     * Nazwa messega zaczyna sie od CLIENT gdy wysyla go server do klienta,
     * Lub inaczej mowiac nazwa messega zaczyna sie od docelowego hosta.
     * 
     * Podobnie jest z nazwami metod operujacych na messegach.
     * Nazwa metody messega zaczyna sie od
     * - MsgSend, jesli wysylamy wiadomosc
     * - MsgRecv, jelsi odbieramy wiadomosc
     * Nasptepnie w na  zwie message'a wystepuje host docelowy,
     * moze to byc:
     * - Server
     * - Client
     * byc moze w przyszlosci bedzie mozna wyroznic jeszcze
     * - Owner, NotOwner
     * Ogolnie
     * MsgSend<host-docelowy><message name> - wyslanie messega 
     * MsgRecv<host-odbierajacy><message name> - odebranie message'a po stronie odbierajacego
    */
    static final int MSG_SERVER_ACTION_REQ        = 0; // client informuje ze chce wykonujac akcje
    static final int MSG_CLIENT_ACTION            = 1; // server rzada wykonania akcji
    static final int MSG_CLIENT_PLAYER_DIED       = 2; // server rzada usmiercenia gracza
	static final int MSG_CLIENT_RESPAWN           = 3; // server wymusza respawn klienta
    static final int MSG_CLIENT_RESPAWN_REJECT    = 4; // server odmawia respawnu
    static final int MSG_CLIENT_STATE             = 5; // server wysyla stan obiektu
    static final int MSG_CLIENT_STATE_SHORT       = 6; // jak powyzej, ale uproszczona wersja
    static final int MSG_CLIENT_INVENTORY_CONTENT = 7; // server wysyla stan inventory
    static final int MSG_CLIENT_NO_INV_ROOM       = 8; // po akcji podnoszenia inv objecta jak nie ma miejsca
    
    
	static final int MSG_SERVER_MOVE               = 10; // klient wysyla serwerowi swoje "ruchy"
    static final int MSG_SERVER_SWITCH_UNDER_WATER = 11; // klient wysyla serwerowi info ze zaczyna/konczy tonac
    static final int MSG_SERVER_STATE_REQUEST      = 12; // klient wysyla do serwera prosba o swoj faktyczny stan
    static final int MSG_CLIENT_ADJUST_STATE       = 13; // serwer do wlasciciela

	static final int MSG_CLIENT_HEALTH			= 20; // server przesyla stan zdrowia
    static final int MSG_CLIENT_OVERLOAD     	= 21; // server przesyla stan overload
    
    static final int MSG_SERVER_INV_WISH_LIST   = 22; // klient wysyla liste obiektow ktora sobie wybral w inwentory
    
    static final int MSG_SERVER_TEST_DATA       = 23;
    static final int MSG_CLIENT_TEST_DATA       = 24; 
    static final int MSG_SERVER_VEHICLE_COLL      = 25;
    static final int MSG_CLIENT_VEHICLE_COLL_ANSW = 26;    
        
    //UWAGA: komunikaty powyzej 50 sa przydzielane w klasach potomnych!
        

    int METHOD_SayServerIAmReady    = 0;
    int METHOD_ApplyTargetActor     = 0;
    int METHOD_SetTargetActorNull   = 0;
    int METHOD_TryToSetNetGameInfo  = 0;
    int METHOD_TryToSetYourName     = 0;
    
    /** Gdy ktos cos wlasnie powiedzial (voice msg)*/
    boolean m_bJustSpoken = false;
    /** Czas rozpoczecia gadki*/
    float   m_fJustSpokenTime = 0.0f;
    void JustSpoken()
    {
        m_bJustSpoken     = true;
        m_fJustSpokenTime = GetTime();
    }
    
    //////////////////////////////////////////////////////////////////////////////////////////
    
    PlayerNetBase()
    {
        super();
        
        Role = ERole._AUTONOMOUS;
        
        //TODO: pobrac pierwszego lepszego ze skryptu ModelsNet.scr,
        MeshName     = NetPlayerProfile.sm_sModelDir+"HSoldierNet.3DA";
        
        m_cImplantSystemClass = NetImplantSystem.class;
    }

    public void OnCreate()
    {
        METHOD_SayServerIAmReady    = RegisterCallMethod( "SayServerIAmReady" );
        METHOD_ApplyTargetActor     = RegisterCallMethod( "ApplyTargetActor" );
        METHOD_SetTargetActorNull   = RegisterCallMethod( "SetTargetActorNull" );
        METHOD_TryToSetNetGameInfo  = RegisterCallMethod( "TryToSetNetGameInfo" );
        METHOD_TryToSetYourName     = RegisterCallMethod( "TryToSetYourName" );
        
        super.OnCreate();
       
        if ( GetModule() instanceof ModuleNet )
        {
            m_cModule = (ModuleNet)GetModule();
            if ( bNetIsOwner )
            {
                // pozwoli odnalezc tego gracza
                if (this instanceof PlayerNet)
                    GetModuleNet().SetMainPlayerNet( (PlayerNet)this );
            }
            GetModuleNet().AddToBarrierSystem( this );
        }
        else
        {
            CrashLog(" Obiekt "+this+" powinien istniec tylko na module sieciowym!\n");
            return;
        }
        
        cGameInitInfo     = m_cModule.cGameInitInfo;
        TryToSetNetGameInfo();
                
        //SetCollFlags( ECollFlags.OBJCOL_FLAG_AUTOATTACH );
        SetFlag( EFlags._CAST_SHADOWS );
        SetLighting( true );
        SetFacingFlag( false );
        SetMeshCullSizeEnable( false );
        SetAnimMorphTime( 0.25f );
        ReloadPartialSkins();
        SetUnImportanceFactor(_UNIMPORTANCE_FACTOR);
    }

    public void OnDestroy()
    {
        super.OnDestroy();
        
        ImplantTermovision.RemoveFromSystem( this );
        
        thisPlayerNetBase   = null;
        cGameInitInfo       = null;
        m_cModule           = null;
        m_cGameInfo         = null;
        //TODO: zastanowic sie czy nie skasowac
        m_cPlayerInfo       = null;
        cCurrentCamera      = null;
        vSpawnPointPosition = null;
        vSpawnPointForward  = null;
        m_cFloor            = null;
        fNetRelevantLevel   = -1.0f;
        
        UnregisterInInteractionMap();
    }
    
    public void Respawn()
    {
        nLeaning = -1;
        if (Camera != null)
            Camera.ForceStopLeaning();
        
        if (m_cDeathCameraEyeDummy != null)
            m_cDeathCameraEyeDummy.DeactivateOnModuleAlways();
        
        m_nState            = _STATE_SPAWNED;
        bUnableToAct        = false;
        bCheckForLadder     = false;
        bCheckForLadderExit = false;
        //to taki brzydki hack, zeby dobrze
        //gracz deformowal sie przy obrotach
        SetAnimationType();
        //TODO: wylaczyc aktualne animacje (przeladowanie,itp)
        ClearFlag( EFlags._NO_ANIMATION_REFRESH );
        ClearFlag( EFlags._NO_ANIMATION );
    }
               

        
    void TryToSetNetGameInfo()
    {
        if ( m_cModule.GetGameInfo() != null )
            SetNetGameInfo( m_cModule.GetGameInfo() );
        else
            CallMethodIn( METHOD_TryToSetNetGameInfo, 0.2f );
    }
    
    void TryToSetYourName()
    {
        if ( m_cPlayerInfo != null && m_cPlayerInfo.IsNickValid() )
            SetName( m_cPlayerInfo.GetNick() );
        else
            CallMethodIn( METHOD_TryToSetYourName, 0.2f );
    }
    

    void SetNetGameInfo( NetGameInfo cGameInfo )
    {
        m_cGameInfo = cGameInfo;
        m_bCheckTargetInfo = m_cGameInfo instanceof NetGameInfoTeam;
    }

    /** replikacje z servera na klienta
     */
    
    // replikacja stanu gracza z servera do klienta, ktory nie jest wlascicielem gracza
    static int sm_nRepSetRelevantLevel;
    static int sm_nRepSetClientNotOwnerStatePacked;
    static int sm_nRepSetClientNotOwnerHealthWhileDriving;
    static int sm_nRepSetFiring;
    static int sm_nRepSetCamera;
    
    //static int sm_repSetClientNotOwnerInitAlways;
    
    // w tym secie zawarte sa informacje potrzebne do zreplikowania stanu gracza
    // z serwera na clientow, ktorzy nie sa wlascicielami gracza
    //static String sm_strSetClientNotOwnerStatePacked = "nativePosAndSpeedPacked;byAngleH;byAngleV;byStateBits;byHealth";
    static String sm_strSetClientNotOwnerStatePacked = "nativePosAndSpeedPacked;byAngleH;byAngleV;byStateBits;byHealth;m_bWeaponFireOnServer";
    static String sm_strSetClientNotOwnerHealthWhileDriving = "byHealth";
    
    // to leci najpierw od wlasciciala do serwera, 
    // pozniej z serwera na pozostalych klientow
    static String sm_strSetFiring = "nFiring";
        
    // to leci najpierw od wlasciciala do serwera, 
    // pozniej z serwera na pozostalych klientow
    static String sm_strSetCamera = "nCurrentViewMode;fCurrentViewZoom";

    
    /** Definiowanie replikacji
     */
    public void ReplicationInit()
    {
        //czy to nie powinno pojsc kanalem pewnym?
        //TODO: jak bedziemy replikowac relevantnosc to dorobic nativeRelevant
        sm_nRepSetRelevantLevel            = NewReplicationSet( "fNetRelevantLevel" );
                    
        sm_nRepSetClientNotOwnerStatePacked = NewReplicationSet( sm_strSetClientNotOwnerStatePacked );
        sm_nRepSetClientNotOwnerHealthWhileDriving = NewReplicationSet( sm_strSetClientNotOwnerHealthWhileDriving );

        sm_nRepSetFiring                   = NewReplicationSet( sm_strSetFiring );
        
        //na razie jeszcze nie wykorzystane
        sm_nRepSetCamera                   = NewReplicationSet( sm_strSetCamera );
        
        SetNetPriority( 3.0f );
        SetMinRelevantLevel( (float)(1.0/java.lang.Math.pow( 2, 16 )) );
    }
  
    static boolean sm_bReplicatePos = true;
    
    boolean bDebugClientSendState = false;
    
    /** Replikacja odpowiednich zestawow dla odpowiednich warunkow
     */
    public void OnReplicateServer()
    {
        if ( bNetInit ) 
        {
            MsgSendClientState();

            if ( bNetIsOwner )
                MsgSendClientInventoryContent();
        }
           
        if ( ! bNetIsOwner &&  (m_nState != _STATE_DIE) )
        {
            PrepareDataForReplication();
            if ( m_nState != _STATE_IN_VEHICLE )
                Replicate( sm_nRepSetClientNotOwnerStatePacked );   
            else
                Replicate( sm_nRepSetClientNotOwnerHealthWhileDriving );   
            
            //Replicate( sm_nRepSetRelevantLevel );
        }
    }
    
    /** Replikacja danych z klienta na serwer dla odpowiednich zestawow
     */
    public void OnReplicateClient()
    {
        if ( bNetIsOwner )
        {
            //Replicate( repSetCamera );
            if ( m_nState != _STATE_IN_VEHICLE )
                Replicate( sm_nRepSetFiring );
        }
    }
    
    /** Metoda (nie handler!) wolana po otrzymaniu danych, gdy gracz jest na serwerze
     */
    public void OnReceivedDataOnServer()
    {
    }
    
    /** Metoda (nie handler!) wolana po otrzymaniu danych, gdy gracz jest na kliencie
     */
    public void OnReceivedDataOnClient()
    {
        if (!bNetIsOwner)
        {
            nFiring = (byte)(byStateBits & _STATE_BITS_FIRING_MASK);
            bWalking = (byStateBits & _STATE_BITS_WALKING) != 0;
            bDucking = (byStateBits & _STATE_BITS_DUCKING) != 0;
            bCrawling = (byStateBits & _STATE_BITS_CRAWLING) != 0;
            bAiming = (byStateBits & _STATE_BITS_AIMING) != 0;
            bRecoiling = (byStateBits & _STATE_BITS_RECOILING) != 0;
            if ((byStateBits & _STATE_BITS_HIT_REACTION) != 0)
                nHitReaction = BaseAnimations.A_TAKE_HIT_FRONT;
            else
                nHitReaction = 0;

            fHealth = (float)byHealth * fMaxHealth / 127.f;
            
            //jezeli aktualnie bron nie strzela, ale my "wiemy" ze ostatnio
            //strzelala, to wymuszamy na chwilke strzelanie
            m_bForcedFireOnClientNotOwner = ( (nFiring == Weapon._NO_FIRING) && 
                                              (m_bWeaponFireOnServer) &&
                                              (ActiveWeapon != null) &&
                                              (ActiveWeapon.nType != EWeaponType._GRENADE) );
            //jak mamy wymusic to zacznijmy strzelac
            if ( m_bForcedFireOnClientNotOwner )
                nFiring = Weapon._FIRING_PRIMARY;
        }
    }

    /** Metoda wolana na serwerze bezposrednio po zakonczeniu replikacji
     */
    void OnPostReplicateDataOnServer()
    {
        //zapominamy o tym czy ostatnio strzelalismy
        m_bWeaponFireOnServer  = false;
        //zapominamy o tym czy ostatnio dostalismy postrzal
        m_bHitReactionOnServer = false;
    }

    /**
     * Server:
     * Info czy na ramce fizyki byl oddany strzal
     */
    public boolean  m_bWeaponFireOnServer = false;
    
    /**
     * Server
     * Info czy na ramce fizyki byla odpalana animacja postrzalu
     */
    public boolean  m_bHitReactionOnServer = false;
    
    
    /**
     * Client
     * Info mowiace o tym czy wlasnie jestesmy w trakcie strzelania
     * wymuszonego przez serwer
     */
    private boolean m_bForcedFireOnClientNotOwner = false;
    
    /** 
     * Server:
     * Callback z broni mowiacy o tym ze wlasnie strzelila
     * Tu bedzidziemy zliczali strzaly na serwerze
     */
    void WeaponFireOnServer()
    {
        m_bWeaponFireOnServer = true;
    }
    
    /** 
     * Client not owner:
     * Callback z broni mowiacy o tym ze wlasnie strzelila na kliencie nie wlascicielu
     * Tu bedzidziemy detektowali czy jest to strzal wymuszony z serwera i
     * jesli tak, to wylaczamy strzelanie
     */
    void WeaponFireOnClientNotOwner()
    {
        if ( m_bForcedFireOnClientNotOwner )
        {
            m_bForcedFireOnClientNotOwner = false;
            nFiring = Weapon._NO_FIRING;
        }
    }



    
    /**
     */
    public void PrepareDataForReplication()
    {
        byStateBits = (byte)(nFiring & _STATE_BITS_FIRING_MASK);
        if (bWalking)
            byStateBits |= _STATE_BITS_WALKING;
        if (bDucking)
            byStateBits |= _STATE_BITS_DUCKING;
        if (bCrawling)
            byStateBits |= _STATE_BITS_CRAWLING;
        if (bAiming)
            byStateBits |= _STATE_BITS_AIMING;
        if (bRecoiling)
            byStateBits |= _STATE_BITS_RECOILING;
        //if (nHitReaction != 0)
        if ( m_bHitReactionOnServer )
            byStateBits |= _STATE_BITS_HIT_REACTION;

        byHealth = (byte)(127.f * fHealth / fMaxHealth);
    }
    
    
    public boolean IsRelevantConstans(){ return false; }
    
    /** Okreslenie warunku kiedy gracz jest symulowany na cliencie
     * Wolane dla kazdego clienta
     */
    public float GetRelevantLevel()
    {
        if ( fNetForceRelevantLevel >= 0.0f )
            return fNetForceRelevantLevel;
        
        if ( bNetIsOwner )
            return 1.0f;
        
        float fRel = 1.0f-((GetCurrentClientDistance() - GetModule().fMinDistToCalculateRelevant)/
                           (GetModule().fMaxDistToCalculateRelevant-GetModule().fMinDistToCalculateRelevant));
        if ( fRel > 1.0f )
             fRel = 1.0f;
        return fRel;
    }
    
    public boolean DeleteIfNotRelevant()
    {
        return false;
    }
    
	TriggerObject m_LastExecutedVehicle = null;
	float m_fLastExecuteVehicleTime = 0f;

    // aktywny element
    void ExecuteTrigger()
	{
        m_bExecutingTrigger = false;
		TriggerObject cTrigger = GetActiveTrigger();
        if ( cTrigger != null )
		{
			if (cTrigger instanceof Vehicle)
			{
				float fTime = GetTime();
				if (m_LastExecutedVehicle == cTrigger)
				{
					if (fTime - m_fLastExecuteVehicleTime < 2f)
						return;
				}
				m_fLastExecuteVehicleTime = fTime;
				m_LastExecutedVehicle = cTrigger;
			}
	        cTrigger.ActionExecute( this );
		}
        else
            LogD(" GetActiveTrigger() == null, action canceled\n");
	}
	
	boolean CanGetOut()
	{
		if ( (cSeatInfo != null) && (cSeatInfo.GetOccupiedObject() instanceof Vehicle) )
		{
			float fTime = GetTime();
			if (m_LastExecutedVehicle == cSeatInfo.GetOccupiedObject())
			{
				if (fTime - m_fLastExecuteVehicleTime < 2f)
					return false;
			}
			m_fLastExecuteVehicleTime = fTime;
			m_LastExecutedVehicle = cSeatInfo.GetOccupiedObject();
		}
		return true;
	}
	
    
    //-------------------------------------------------------------------------         
    
    void Execute( Actor executor )
    {
    }
    
    public boolean IsActive(ControlObject Activator)
    {
        return false;
    }

    String GetTip()
    {
        if ( m_cPlayerInfo != null && m_cPlayerInfo.IsNickValid() )
            return m_cPlayerInfo.GetNick();
        else
            return "PlayerNet";
    }

    
    protected int m_nVehicleToGetInOut   = -1;
    protected int m_nSeatIndexToGetInOut = -1;
    
    int         m_nFloorToAttach    = -1;
    MeshObject  m_cFloor            = null;
    float       _FLOOR_MARGIN       = 0.0f;
    
    
    /** Metoda wysylajaca na klienta stan obiektu podczas pierwszej replikacji
     */
    void MsgSendClientState()
    {
        MsgSendClientState( true, -1 );
    }
    void MsgSendClientStateShort(int nHostId)
    {
        MsgSendClientState( false, nHostId );
    }
    void MsgSendClientState( boolean bOnInit, int nHostId )
    {
        int[] aSkins = m_cModelInfo.GetCurrentPartialSkins();
        
        MessageNew( bOnInit ? MSG_CLIENT_STATE : MSG_CLIENT_STATE_SHORT );
        MessagePackFloat( GetTime() );
        if ( bOnInit )
        {
            MessagePackString( MeshName );
            for ( int i=0; i<aSkins.length; i++ )
                MessagePackInt( aSkins[ i ] );
            MessagePackInt( GetTeamSkinSet() );
            MessagePackInt( GetNextGlobalChangeSkinCounter() );
        }
        
        MessagePackByte( m_nState );
        MessagePackInt( GetRespawnCnt() );
        if ( m_cFloor != null )
            MessagePackInt( m_cFloor.GetNetId() );
        else
            MessagePackInt( -1 );
        
        // TODO: uwzglednic wszystkie tryby playera
        switch ( m_nState )
        {
            // jest w stanie DieState
        case _STATE_DIE:
            MessagePackDieState();
            MessageSend();
            return;
            //break;
                
            // jest juz wyspawnowany
        
        case _STATE_IN_VEHICLE:
            MessagePackInt( m_nVehicleToGetInOut );
            MessagePackInt( m_nSeatIndexToGetInOut );
            MessagePackInt( PlayerNetBaseDriver.GetNextGlobalDriverCounter() );
            // TODO
            // uwzglednic dodatkowe zmienne w stanie onhevicle
            // uwaga:wyslanie wiadomosci poza switchem!
            break;
        
        case _STATE_BINOCULARS:
            MessagePackInt( (cBinoculars != null) ? 
                            cBinoculars.GetNetId() : -1  );
            // uwaga:wyslanie wiadomosci poza switchem!
            break;
            
       case _STATE_AIMING:
            // uwaga:wyslanie wiadomosci poza switchem!
            break;
        
        case _STATE_SPAWNED:
            MessagePackVector( GetPosition() );
            MessagePackVector( GetAimDirection() );
            MessagePackBoolean( motion.bAiming );
            MessagePackByte( nFiring );
            // TODO
            // trzeba bedzie doslac bWalking;bDucking;bCrawling i od razu gostka przelaczyc
            // uwaga:wyslanie wiadomosci poza switchem!
            break;
            
        default:
            CrashLog( this + " Stan "+m_nState+" nieznany - MsgSendClientState nieudane!\n");
            return;
        }
                
        // aktywne implanty
        for ( int i=0; i<10; i++ )
            MessagePackBoolean( IsImplantActive( i ) ) ;
        
        if ( bOnInit )
            MessageSend();
        else
            MessageSendHost( nHostId );
    }
    
    void MsgRecvClientState(boolean bOnInit)
    {
        //zapamietajmy ze przyszla juz inicjalizacja
        m_bInitializedFromServer     = true;
        
        boolean[] aImplants          = null;
        int    [] aSkins             = new int[ NetPlayerModelInfo.GetSkinsPartialCnt() ];
        String    sMeshName          = null;
        int       nTeamSkinSet       = 0;    
        int       nChangeSkinCounter = ChangeStateCounter.GetInitValue();
        
        //po pierwsze odczytajmy czas
        float     fTime     = MessageUnPackFloat();
        // i sprawdzmy czy to najnowsze wiesci o obiekcie
        boolean   bNewState = (fTime >= GetStateFromServerTime());
        
        
        if ( bOnInit )
        {
            sMeshName           = MessageUnPackString();
            for ( int i=0; i<aSkins.length; i++ )
                aSkins[ i ]     = MessageUnPackInt();
            nTeamSkinSet        = MessageUnPackInt();
            nChangeSkinCounter  = MessageUnPackInt();
        }
        
        byte      nState       = MessageUnPackByte();
        int       nRespawnCnt  = MessageUnPackInt();
        int       nFloor       = MessageUnPackInt();
                
        switch ( nState )
        {
        case _STATE_DIE:
            MessageUnPackDieState();
            MessageEnd();
            break;
        
        case _STATE_IN_VEHICLE:
            m_nVehicleToGetInOut   = MessageUnPackInt();
            m_nSeatIndexToGetInOut = MessageUnPackInt(); 
            int nDriverCounter     = MessageUnPackInt();
            aImplants = MsgRecvClientStateReadImplants();
            MessageEnd();
            if ( ! VerifyDriverCounter( nDriverCounter ) )
                nState = _STATE_SPAWNED;
            break;
       
        case _STATE_BINOCULARS:
            m_nBinocularsToUse = MessageUnPackInt();
            m_cBinocularsToUse = null;
            aImplants = MsgRecvClientStateReadImplants();
            MessageEnd();
            break;
        
        case _STATE_AIMING:
            aImplants = MsgRecvClientStateReadImplants();
            MessageEnd();
            break;
                            
        case _STATE_SPAWNED:
            vSpawnPointPosition = MessageUnPackVector();
            vSpawnPointForward  = MessageUnPackVector();
            
            motion.bAiming = MessageUnPackBoolean();
            nFiring        = MessageUnPackByte();
        
            aImplants = MsgRecvClientStateReadImplants();
            MessageEnd();
            break;
            
        default:
            CrashLog( this + " Odczytano MSG_CLIENT_STATE: "+
                      nState+
                      " (stan nieznany)\n");
            return;
        }
        
        if ( bOnInit )
        {
            m_cModelInfo = new NetPlayerModelInfo( sMeshName, aSkins );
            if ( MeshName.compareToIgnoreCase( sMeshName ) != 0 )
            {
                LogD("Przeladowuje mesha z "+MeshName+" na "+sMeshName+"\n");
                SetMeshName( sMeshName );
                ReloadMesh();
            }
            ReloadPartialSkins();
            ReloadTeamSkins( nTeamSkinSet, nChangeSkinCounter );
        }
        if ( bNewState )
        {
            if (nState != _STATE_DIE)
                MsgRecvClientStateApplyImplants( aImplants );
            SetRespawnCnt( nRespawnCnt );
            ChangeState( nState, nFloor, bOnInit );
            SetStateFromServer( nState, fTime );
        }
        else
        {
            NetLog.Player("Otrzymalismy stary stan "+nState+
                          " bOnInit:"+bOnInit+
                          " "+GetNetInfo()+" - ignore\n");
        }
        
        if ( bNetIsOwner && bOnInit )
        {
            SayServerIAmReady();
        }
    }
        
    //do nadpisania w PlayerNetBaseDriver
    boolean VerifyDriverCounter(int nCounter){ return false; }
    
    void MsgSendServerStateRequest()
    {
        if ( Role == ERole._AUTHORITY )
            return;
        
        MessageNew( MSG_SERVER_STATE_REQUEST );
        MessageSend();
    }
    

    void MsgRecvServerStateRequest()
    {
        if ( Role < ERole._AUTHORITY )
            return;
        int nHostId = MessageGetSenderId();
        MessageEnd();
        
        MsgSendClientStateShort( nHostId );
        SendClientInvObjStateFromInventory( nHostId );
    }

    void SendClientInvObjStateFromInventory(int nHostId)
    {
    }
    
    void SayServerIAmReady()
    {
        DisableCallMethod( METHOD_SayServerIAmReady );
        if ( m_cGameInfo != null )
             m_cGameInfo.MsgSendServerIAmReady();
        else
            CallMethodIn( METHOD_SayServerIAmReady, 0.1f );
    }
    
    void SetProfile( NetPlayerProfile cProfile )
    {
        m_cModelInfo = new NetPlayerModelInfo( cProfile );
        SetMeshName( cProfile.GetMeshName() );
		SetName( cProfile.GetPlayerName() );
    }
    
    /** Ustawienie nazwy mesha i od razu do pary nazwy pliku z animacjami
     */
    void SetMeshName( String sName )
    {
        MeshName = sName;
        /*TODO: TYLKO DO BETY!! wszystkie meshe maja te same skrypty
        int nDot = MeshName.lastIndexOf(".");
        String sFileName = new String( MeshName.substring( 0, nDot ) );
        sAnimIniFile = sFileName + ".scr";*/
    }
    
    void ReloadMesh()
    {
        LoadMesh();
        
        SetupElementIDs();
    }
    
    
    boolean[] MsgRecvClientStateReadImplants()
    {
        boolean aImplants[] = new boolean [ 10 ];
        for ( int i=0; i<10; i++ )
            aImplants[i] = MessageUnPackBoolean();
        return aImplants;
    }
        
    void MsgRecvClientStateApplyImplants( boolean[] aImplants )
    {
        for ( int i=0; i<10; i++ )
        {
            if ( aImplants[ i ] )
                ImplantOn( i );
            else
                ImplantOff( i );
        }
    }
           
    
    /** Przelacza playera w odpowiedni stan na podstawie danych
     * odczytanych w MsgRecvClientState()
     *
     */
    //TODO: zapewnic przejscia miedzy wszystkimi stanami!
    void ChangeState(int nState, int nFloor, boolean bOnInit )
    {
        DisableCallMethod("DelayedSitDownToDrive");
        
        switch ( nState )
        {
        case _STATE_DIE:
            // wymusic stan DIE, jesli jest inny
            if (sCurrentState.compareTo("DieState") != 0)
            {
                if ( bOnInit )
                {
                    //tak naprawde podczas inicjalizacji powinien juz byc
                    //ustawiony DieState, ale...
                    if ( m_nState == _STATE_IN_VEHICLE )
                        ActionGetOutDo();
                    else if ( m_nState == _STATE_BINOCULARS )
                        StopUsingBinocularsDo();
                    else if ( m_nState == _STATE_AIMING )
                        RestoreViewAfterAiming();
                    SetState("DieState");
                }
                else
                {
                    //podczas gry dostalismy prikaz zabicia,
                    //nie mamy innego wyjscie jak to zrobic
                    PlayerDied();
                }
            }
            break;
        
        case _STATE_IN_VEHICLE:
            if (sCurrentState.compareTo("DieState") == 0)
                Respawn();
            else
                if ( m_nState == _STATE_BINOCULARS )
                    StopUsingBinocularsDo();
                else
                    if ( m_nState == _STATE_AIMING )
                        RestoreViewAfterAiming();
            ActionGetInDo();
            CallMethodIn("DelayedSitDownToDrive",0.5f);
            break;
        
        case _STATE_BINOCULARS:
            if (sCurrentState.compareTo("DieState") == 0)
                Respawn();
            else
                if ( m_nState == _STATE_IN_VEHICLE )
                    ActionGetOutDo();
                else
                    if ( m_nState == _STATE_AIMING )
                        RestoreViewAfterAiming();
            
            StartUsingBinocularsDo();
            m_nState = _STATE_BINOCULARS;
            break;

        case _STATE_AIMING:
            if (sCurrentState.compareTo("DieState") == 0)
                Respawn();
            else
                if ( m_nState == _STATE_IN_VEHICLE )
                    ActionGetOutDo();
                else
                    if ( m_nState == _STATE_BINOCULARS )
                        StopUsingBinocularsDo();
            
            InitViewForAiming();
            //m_nState = _STATE_AIMING;
            break;
                            
        case _STATE_SPAWNED:
            // jezeli aktualny stan jest DIE to trzeba wyspawnowac
            if (sCurrentState.compareTo("DieState") == 0)
                Respawn();
            else
            {   
                if ( m_nState == _STATE_IN_VEHICLE )
                    ActionGetOutDo();
                else
                    if ( m_nState == _STATE_AIMING )
                        RestoreViewAfterAiming();
                    else
                        if ( m_nState == _STATE_BINOCULARS )
                        {
                            StopUsingBinocularsDo();
                            m_nState = _STATE_SPAWNED;
                            return;
                        }
                
                SetPosition( vSpawnPointPosition );
                SetAimDirection( vSpawnPointForward );
                UpdateMotion();
            }
            break;
            
        default:
            CrashLog("PlayerNetBase:ChangeState() Stan "+nState+" nieznany!\n");
        }
                
        if ( m_cFloor != null && m_cFloor.GetNetId() != nFloor )
            DetachPlayerFromFloorDo();
        
        if ( m_cFloor == null && nFloor > -1 )
        {
            m_nFloorToAttach = nFloor;
            AttachPlayerToFloorDo();
        }
    }

    float RoundToPhysicsGranularity(float fTime)
    {
        int iMulTime = (int)(fTime / sm_fPhysicsGranularity);
        return ((float)iMulTime * sm_fPhysicsGranularity);
    }

    void DelayedSitDownToDrive()
    {
        SitDownToDrive();
    }
    
    // TODO, "odpytac" NetGameInfo czy to wroc czy przyjaciel, 
    // niech tymczasowo wszyscy beda moimi wrogami ;-)
    int GetTeamRelation(Pawn target)
    {
        return _TEAM_RELATION_HOSTILE;
    }
    
    boolean IsSpawned()  {     return m_nState != _STATE_DIE;   }

    /**
     * update kamery do oceny relewantnosci
     */
    void UpdateCurrentCamera()
    {
        cCurrentCamera.fAimAngleY = -fAimAngleY;
        if (fLastViewZoom != fCurrentViewZoom)
        {
            cCurrentCamera.SetZoom( fCurrentViewZoom );
            fLastViewZoom = fCurrentViewZoom;
        }
        if (nLastViewMode != nCurrentViewMode)
        {
            cCurrentCamera.SetMode((byte)nCurrentViewMode);
            nLastViewMode = nCurrentViewMode;
        }
    }

    /**
     * 
     * Gracz-przeciwnik, ktorego nick bedzie wyswietlany pod celownikiem
     * 
     */
    void SetTargetVisibilityRange(float fRange){_TARGET_VISIBILITY_RANGE = fRange;}
    static float _TARGET_VISIBILITY_RANGE = 10000.0f;
    static float _TARGET_INFO_HIDE_TIME   = 1.5f;
    ChromeActor m_cTargetActor = null;
    
    void CheckTriggers()
    {
        CheckTriggers( false );
    }
    void CheckTriggers(boolean bOnlyTargetInfo)
    {
        if (bDisableTriggerDetection || Camera==null) 
            return;
        
        SetGateFenceObjectFlag(true);
        
        TriggerObject Obj = GetActiveTrigger(Camera.GetPosition(),
                                             Camera.GetForwardVector(),
                                             -fTriggerVisibilityRange);
        
        ChromeActor cTargetActor = null;
        if ( m_bCheckTargetInfo )
        {
            cTargetActor = GetTargetActor( Camera.GetPosition(),
                                           Camera.GetForwardVector(),
                                         -_TARGET_VISIBILITY_RANGE );
        }

        SetGateFenceObjectFlag(false);
        
        // gdy zmienil sie aktywny trigger to go ustawiamy
        if (ActiveTrigger != Obj && !bOnlyTargetInfo) 
            SetActiveTrigger( Obj );
        
        if ( m_bCheckTargetInfo )
        {
            if ( m_cTargetActor != null &&  
                   cTargetActor == null  )
            {
                //byl jakis cel a nie ma,
                //chowamy z opoznieniem
                CallMethodIn( METHOD_SetTargetActorNull, 
                              _TARGET_INFO_HIDE_TIME );
                //a pokazujemy jeszcze tego co poprzednio
                ApplyTargetActor();
            }
            else
            {
                //zmiana, ewentualne odswiezenie
                SetTargetActor( cTargetActor );
                //no i ewentualne chowanie juz nie wchodzi w gre
                DisableCallMethod( METHOD_SetTargetActorNull );
            }
        }
    }
    
    ////////////////////////////////////////////////////////////////
    // Target info displaying on HudCrossHair - end
    ////////////////////////////////////////////////////////////////
    
    void SetTargetActor(ChromeActor cTarget)
    {
        m_cTargetActor = cTarget;
        ApplyTargetActor();
    }
    
    void SetTargetActorNull()
    {
        DisableCallMethod( METHOD_SetTargetActorNull );
        SetTargetActor( null );
    }
    
    ChromeActor GetTargetActor(Vector vStart, Vector vDir, float fDistance)
    {
        ChromeActor cTarget = null;
        
        Vector vEnd = new Vector(vDir);
        vEnd.Mul( fDistance );
        vEnd.Add( vStart );

        // liczymy kolizje
        Collision stCollInfo = TraceForCollision( vStart, vEnd, this );
        
        // czy jest kolizja z graczem lub pojazdem?
        if ( stCollInfo != null )
        {
            if ( stCollInfo.Obstacle instanceof PlayerNet )
                cTarget =  (ChromeActor)stCollInfo.Obstacle;
            else if ( stCollInfo.Obstacle instanceof Vehicle )
                cTarget = (Vehicle)stCollInfo.Obstacle;
        }
        return cTarget;
    }
        
        
    void ApplyTargetActor()
    {
        DisableCallMethod( METHOD_ApplyTargetActor );
        
        if ( HUD == null || HUD.GetHUDCrossHair() == null )
            return;

        NetTargetInfo cTargetInfo = null;
        if ( m_cTargetActor != null )
        {
            cTargetInfo = GetTargetInfo( m_cTargetActor );
            
            if ( cTargetInfo == null )
            {
                // hide info about player becouse someone else
                // is in target
                
                CallMethodIn( METHOD_SetTargetActorNull, 0.0f );
                return;
            }
        }
        
        if (HUD.GetHUDCrossHair()!=null)
            HUD.GetHUDCrossHair().SetTargetInfo( cTargetInfo );
    }
    
    NetTargetInfo GetTargetInfo(ChromeActor cTarget)
    {
        NetTargetInfo cTargetInfo = null;
                
        if ( m_cGameInfo != null )
		{
		    cTargetInfo = m_cGameInfo.GetTargetInfo( cTarget, m_cPlayerInfo );    
		}
        return cTargetInfo;
    }

    ////////////////////////////////////////////////////////////////
    // Target info displaying on HudCrossHair - end
    ////////////////////////////////////////////////////////////////
    
            
    void ShowHUDNetOnJoin()
    {
        if ( m_cGameInfo != null )
            m_cGameInfo.ActivateStatisticsOnJoin();
    }
    
    void ShowHUDNetOnPlaeseWait()
    {
        if ( m_cGameInfo != null )
            m_cGameInfo.ActivateStatisticsOnPleaseWait();
    }
    
    void ShowHUDNetOnRespawnRequest()
    {
        if ( m_cGameInfo != null )
            m_cGameInfo.ActivateStatisticsOnRespawnRequest();
    }
    
    void HideHUDNetOnRespawnRequest()
    {
        if ( m_cGameInfo != null )
            m_cGameInfo.DeactivateStatisticsOnRespawnRequest();
    }
    
    void ShowHUDNetOnRespawnReject(int nReason)
    {
        if ( m_cGameInfo != null )
            m_cGameInfo.ActivateStatisticsOnRespawnReject( nReason );
    }
    
    void HideHUDNetOnRespawn()
    {
        if ( m_cGameInfo != null )
            m_cGameInfo.DeactivateStatisticsOnRespawn();
    }
    
    void LockHUDNet(boolean bLock)
    {
        if ( m_cGameInfo != null )
            m_cGameInfo.LockHUDNet( bLock );
    }

    void AttachPlayerToFloor(MeshObject cFloor)
    {
        LogD("\n");
        m_cFloor = cFloor;
        AttachPlayerToFloorDo();
        
        _FLOOR_MARGIN = GetPosition().fY - m_cFloor.GetPosition().fY + 2.0f;
    }
    
    
    void AttachPlayerToFloorDo()
    {
        if ( m_cFloor == null )
        {
            GameObject cObj = GetObjectByNetId( m_nFloorToAttach );
        
            if (!( cObj instanceof MeshObject ))
            {
                CallMethodIn("AttachPlayerToFloorDo",0.1f);
                return;
            }
            
            m_cFloor = (MeshObject)cObj;
        }
        
        
        if ( m_cFloor instanceof LiftPlatformBase )
            ((LiftPlatformBase)m_cFloor).AttachPlayerDo( this );
        //TODO: jak bedzie zrobiony FlyingTransporter do sieci, to odremowac to
        /*if ( m_cFloor instanceof AirBusNet )
            ((AirBusNet)m_cFloor).AttachPlayerDo( this );
        */
        //TODO: przewidziec pozostale platformy!!!! 
     }   
        
    void SwitchParentTo( ControlObject cParent )
    {
        if ( GetParent() != cParent )
        {
            if ( GetParent() != null )
                DetachFromParentAndKeepWorldPos();
            if ( cParent != null )
                AttachAsChildToAndKeepWorldPos( cParent );
        }
    }
        
    void DetachPlayerFromFloor()
    {
        DetachPlayerFromFloorDo();
    }
        
    void DetachPlayerFromFloorDo()
    {
        if ( m_cFloor != null )
        {
            if ( m_cFloor instanceof LiftPlatformBase )
                ((LiftPlatformBase)m_cFloor).DetachPlayerDo( this );
            //TODO: jak bedzie zrobiony FlyingTransporter do sieci, to odremowac to
            /*if ( m_cFloor instanceof AirBusNet )
                ((AirBusNet)m_cFloor).DetachPlayerDo( this );
            */
        }
/* TODO DEL (nie powinno byc juz potrzebne)
        if ( Role == ERole._AUTHORITY )
        {
            HACKStopPosReplication( 1.5f );
            //HACKMovePlayer(0.1f);
            //CallMethodIn("HACKMovePlayer",0.5f);
        }
*/        
    }
    
    
/* TODO DEL    
    boolean bReplicationPosStopped = false;
    
    Vector vReplicationPosStopedPos = null;
    
    void HACKStopPosReplication(float fTime)
    {
        bReplicationPosStopped = true;
        vReplicationPosStopedPos = GetPosition();
        
        CallMethodEvery("CheckStartPosReplication",0.1f);
        CallMethodIn("StartPosReplication",fTime);
    }
    
    void CheckStartPosReplication()
    {
        if ( !vReplicationPosStopedPos.equals( GetPosition() ) )
            StartPosReplication();
    }
    
    void StartPosReplication()
    {
        DisableCallMethod("CheckStartPosReplication");
        bReplicationPosStopped = false;
    }
*/
    
    ImplantSystem GetImplantSystem()
	{
		return cImplantSystem;
	}

    void InitImplants()
    {    
		cImplantSystem.InitImplants( GetModule() , this );
		
        // pobieramy modyfikatory implantow
        java.util.Vector implantMods = cGameInitInfo.cLevels.GetCurrent().aImplantModifiers;
        
		// modyfikujemy implanty poleceniami ze skryptu
		cImplantSystem.ApplyImplantChanges( implantMods );
        // zaczynamy update'owac implanty
        cImplantSystem.StartUpdate();
    }
    
    void FinishImplants()
    {
        cImplantSystem.StopUpdate();
    }
    
    
    
    void HideHud()
    {
        bDisableTriggerDetection = true;
        SetTargetActorNull();
        if ( HUD != null )
        {
             HUD.Hide();
             HUD.ActivateHudMap( false );
        }
    }
        
    void ShowHud()
    {
        bDisableTriggerDetection = false;
        if ( HUD != null )
             HUD.Show();
    }
    
    /**
     * Called when game settings changes
     */
    void OnUpdateGameSettings()
    {
        //should refresh HudMap to correct
        //display players, vehicles, etc.
        if (( HUD             != null )&&
            ( HUD.GetHUDMap() != null ))
        {
            HUD.GetHUDMap().OnUpdateGameSettings();
        }
    }
    /*
    void ActivatePlayerDeathCamera(int nMode)
    {
        //TODO: przenisc kamera do playera
        m_cModule.ActivatePlayerDeathCamera( thisPlayerNetBase, nMode );
    }*/

    boolean bDebug = false;
    void StartDebuggingSpeed()
    {
        ReplicationManagerMsg( 3 );
        bDebug = true;
        bDebugJump = true;
    }
            
    void DebugShowPos()
    {
        if (bDebug)
        {
            LogD(" vPos "+GetPosition()+"\n");
            Thread.dumpStack();
        }

    }

    /**
     * Globalny licznik zmian stanow skinow teamowych obiektow
     */
    private static 
    ChangeStateCounter sm_cGlobalChangeSkinCounter = new ChangeStateCounter();
    static void CleanGlobalChangeSkinCounter(){ sm_cGlobalChangeSkinCounter.Clean(); }
    static int  GetNextGlobalChangeSkinCounter(){ return sm_cGlobalChangeSkinCounter.GetNext(); }

    
    /**
     * Indywidualny licznik zapamietujacy ostatnio
     * odebrana i wykonana akcje zmiany skina teamowego na tym obiekcie
     */
    private 
    ChangeStateCounter m_cChangeSkinCounter = new ChangeStateCounter(this);
    boolean VerifyChangeSkinCounter( int nCounter ){ return m_cChangeSkinCounter.Verify( nCounter ); }
    int     GetChangeSkinCounter(){ return m_cChangeSkinCounter.Get(); }

    /**
     * Przeladowanie skinow graczy kontrolowane licznikiem
     * Jesli okaze sie ze podany licznik jest nowszy (przejdzie weryfikacje)
     * to ladujemy skiny takie jak podano,
     * jesli jest stary to nie odrzucamy przeladowania tylko przeladowujemy
     * skiny takie jakie mielismy poprzednio (po to, bowiem wczesniej moglo
     * byc zawolane ReloadPartialSkins i w danej chwili nie ma zadnych
     * skinow teamowych zaladowanych)
     */
    void ReloadTeamSkins(int nTeamSkinSet,int nChangeSkinCounter )
    {
        int nSkinSetToReload =  VerifyChangeSkinCounter( nChangeSkinCounter ) ?
                                nTeamSkinSet : GetTeamSkinSet();
        ReloadTeamSkins( nSkinSetToReload );
    }
        
    //tylko do testow
    /*
    public class FreehandState extends Player.FreehandState
    {
        public void OnEnter()
        {
            super.OnEnter();
            //RegisterKeyRequest();
        }
        void OnInputKey(int nKeyCode, boolean bKeyState, char nKeyAscii)
        {
            super.OnInputKey( nKeyCode, bKeyState, nKeyAscii );

            // CLIENT, WLASCICIEL
            if( bNetIsOwner )//&& (Role < ERole._AUTHORITY) )
            {
                if (bKeyState)
                {
                    switch (nKeyCode)
                    {
                    case EKey._RETURN :
                        //TO tylko do testow, FIXIT
                        StartSlowDownOnRecoil();
                        break;
                    }
                }
            }
        }
    }
    */
    
    /**
     * Jakie raczki (jakies broni) pokazac gdy ActiveWeapon == null ?
     */
    /*Class GetWeaponClassIfActiveWeaponNull()
    {
        return null;
    }*/
    
    boolean IsBot(){ return false; }
    boolean IsAutoPlayer(){ return false; }
        
    
    boolean IsLocalAutoControl() { return bNetIsOwner; }
  
    float GetTimeForLadder()
    {
        return GetPhysicsLastUpdateTime();
    }
  
    /** metody do nadpisania
     */
    void ActionRespawnCancel(){}
    void MsgSendClientHealth(){}
    void MsgSendClientOverload(float fTime,float fOverload){}
    void MsgSendClientInventoryContent(){};
    void CleanPlayer(){}
    void CleanPlayerMotion(){}
    
    void MsgSendServerActionRequest( int nAction ){}
    void MsgSendServerActionRequest( int nAction, int nParam1 ){}
    void MsgSendServerActionRequest( int nAction, int nParam1, int nParam2 ){}
    void MsgSendClientAction( int nAction ){}
    void MsgSendClientAction( int nAction, int nParam ){}
    void MsgSendClientAction( int nAction, int nParam1, int nParam2 ){}
    void MsgSendClientAction( int nAction, int nParam1, int nParam2, int nParam3 ){}
    void MsgSendClientAction( int nAction, int nParam1, int nParam2, int nMode, float fTimeStamp ){}
    void MsgSendClientAction( int nAction, int nParam1, int nParam2, int nParam3, int nMode, float fTimeStamp ){}
    void MsgSendClientActionMode( int nAction, int nParam1, int nParam2, int nMode ){}
    void MessagePackDieState(){}
    void MessageUnPackDieState(){}
    void PlayerDied(){}
    
    int m_nBinocularsToUse = -1;
    Binoculars m_cBinocularsToUse = null;
    void StartUsingBinocularsDo(){}
    void StopUsingBinocularsDo(){}
    
    void CheckInventoryContent(){}
    void InventoryContentChecked(){}
    
    DeadBody GetDeadBody(){ return null; } //do nadpisania w PlayerNetBaseMortal
    
    String GetNetInfo()
    { 
        return super.GetNetInfo()+
               " "+GetName()+
               ((GetPlayerInfo()!=null)?(" HostId:"+GetPlayerInfo().GetClientOwnerId()):"")+
               " "+(bDead?"DEAD":"ALIVE"); 
    }
}


