/**
 * System wykrywania niezgodnosci stanu gracza puppeta (notownera). 
 * Bedziemy analizowali czy przychodza zmiany stanu gracza
 * (ktore powinny tylko miec miejsce gdy gracz jest "zywy")
 * i czy w tym czasie gracz u nas jest widoczny. Jezeli bedzie
 * jakas niezgodnosc trzeba bedzie ja usunac
 * Mozemy miec generalnie 2 konfliktowe sytuacje.
 * 1. Gracz jest u nas wyspalnowany, ale z jakiejs
 * przyczyny ktos mu zawolal EnableRendering( false ), albo
 * DeactivateOnModule(). Wtedy po stwierdzeniu takiej sytuacji
 * sami podejmujemy decyzje i go pokazujemy
 * 2. Gracz z jakiejs przyczyny nie zostal u nas wyspawnowany,
 * wtedy prosimy serwer, aby przyslal do nas prawdziwy stan gracza.
 * 
 * Uwaga. Sytuacja 1. (a w zasadzie jej sposob rozwiazania)
 * jest o tyle ryrykowna, ze istnieje niebezpieczenstwo 
 * dotarcie do nas "starych" pakietow z ruchem gracza
 * w czasie pomiedzy wyspawnowaniem trupka (schowaniem gracza) 
 * a zmiana stanu gracza na "DieState". Wtedy nieslusznie "ozywimy"
 * gracza!
 */
public class PuppetsVisibilityManager
{
    static boolean sm_bUseManager = true;
    
    PlayerNetBaseMover m_cOwner = null;
        
    PuppetsVisibilityManager(PlayerNetBaseMover cOwner)
    {
        m_cOwner = cOwner;
        Reset();
    }
    
    void Destroy()
    {
        m_cOwner = null;
    }
    
    /**
     * Zmienne przydatne do analizy
     */
    float m_fFirstInReplPosTime = -1.0f;
    float m_fLastInReplPosTime  = -1.0f;
    int   m_nInReplPosCnt       = 0;
    
    float m_fStartCheckingTime  = -1.0f;
    
    boolean m_bStopped = false;
    float   m_fRestartTime = -1.0f;
    
    void Reset()
    {
        m_fFirstInReplPosTime = -1.0f;
        m_fLastInReplPosTime  = -1.0f;
        m_nInReplPosCnt       = 0;
        m_fStartCheckingTime  = -1.0f;
        m_bStopped            = false;
    }
    
    final static float _STOP_TIME = 20.0f;
    
    void StopForAWhile()
    {
        if ( m_cOwner == null )
            return;
        
        Reset();
        
        m_bStopped = true;
        m_fRestartTime = m_cOwner.GetTime() + _STOP_TIME ;
    }
    
    void Check(float fLastReplPosTime)
    {
        if ( ! sm_bUseManager )
            return;
        
        if ( m_bStopped )
        {
            if ( m_fRestartTime > m_cOwner.GetTime() )
                return;
            m_bStopped = false;
        }
        
        if ( m_cOwner == null )
        {
            NetLog.Warning("PuppetsVisibilityManager m_cOwner == null\n");
            return;
        }
        
        if ( m_cOwner.IsActiveOnModule() && m_cOwner.IsRenderable() )
        {
            //NetLog.PlayerMortal("CheckLocalInvisible() Visible ");
            Reset();
            return;
        }
        
        NetLog.PlayerMortal("CheckLocalInvisible() Invisible!!!! ");
        
        //nie jest widoczny, poczekajmy - sprawdzmy od jakiego czasu
        
        if ( m_fLastInReplPosTime < fLastReplPosTime )
        {
            if ( m_fFirstInReplPosTime < 0.0f )
            {
                 m_fFirstInReplPosTime = fLastReplPosTime;
                 //nie mam czasu wnikac czy czas z replikacji to ten sam
                 m_fStartCheckingTime  = m_cOwner.GetTime();
                 
                 NetLog.PlayerMortal("start"+
                                     " FirstReplTime:"+m_fFirstInReplPosTime+
                                     " StartTime:"+m_fStartCheckingTime );
            }
            
            m_fLastInReplPosTime = fLastReplPosTime;
            m_nInReplPosCnt++;
            
            NetLog.PlayerMortal(" LastReplTime:"+m_fLastInReplPosTime+
                                " ReplCnt:"+m_nInReplPosCnt );
                        
            //jezeli odebralismy 10 stanow w ktorym gracz byl niewidoczny
            if ( m_nInReplPosCnt > 4 )
            {
                NetLog.PlayerMortal(" > 4 "+
                                    " LastReplTime:"+m_fLastInReplPosTime+
                                    " FirstReplTime:"+m_fFirstInReplPosTime );
                //i to sie dzialo w ciagu ostatnich 5 sekund to znaczy ze cos nie tak
                if ( ( m_fLastInReplPosTime - m_fFirstInReplPosTime ) < 9.0f )
                {
                    
                    //albo prosimy serwer o prawdziwy stan albo sami robimy porzadek
                    if ( m_cOwner.m_nState == PlayerNetBase._STATE_DIE )
                    {
                        NetLog.Test("!! Gracz "+m_cOwner.GetNetInfo()+" nie jest widoczny! "+
                                    " IsActiveOnModule() "+m_cOwner.IsActiveOnModule()+
                                    " IsRenderable() "+m_cOwner.IsRenderable()+
                                    " m_nState "+m_cOwner.m_nState+
                                    " -> send server state request ");
                        m_cOwner.MsgSendServerStateRequest();
                    }
                    else
                    {
                        NetLog.Warning("!! Gracz "+m_cOwner.GetNetInfo()+" nie jest widoczny! "+
                                       " IsActiveOnModule() "+m_cOwner.IsActiveOnModule()+
                                       " IsRenderable() "+m_cOwner.IsRenderable()+
                                       " m_nState "+m_cOwner.m_nState+
                                       " -> force visible ");
                        m_cOwner.ActivateOnModule(); 
                        m_cOwner.EnableRendering( true );
                    }
                    StopForAWhile();
                    return;
                }
                Reset();
            }
        }
        else
        {
            //jesli minelo 5 sekund od rozpoczecia cyklu to sprawdzamy od nowa
            if ( ( m_cOwner.GetTime() - m_fStartCheckingTime ) > 9.0f )
            {
                Reset();
                NetLog.PlayerMortal(" ResetCheckLocalInvisible() minelo 5 sekund ");
            }
        }
    }
}
