/**
 * Obiekt ktory bedzie nadzorowal poruszanie sie (obroty,kolysanie)
 * innych obiektow
 */
public class NetObjectPositionController extends GameObject
{
    /** nadzorowany obiekt */
    private ControlObject m_cControlledObj = null;
    void SetControlledObject(ControlObject cObj){ m_cControlledObj = cObj; }
    
    /** aktualny tryb poruszania */
    private int m_nMovingMode = _MOVING_MODE_NONE;
    final static int _MOVING_MODE_NONE   = 0x0000;
    final static int _MOVING_MODE_ROTATE = 0x0001;
    final static int _MOVING_MODE_SWING  = 0x0002;
    
    /** Parametry
     */
    /** okres obrotu */
    private float m_fRotatePeriod      = 20.0f;
    void SetRotatePeriod(float fPeriod)
    { 
        m_fRotatePeriod = fPeriod; 
        if (m_bRotateChangeDirTimeByPeriod)
            SetRotateChangeDirTimeByPeriod();
    }
    /** minimalny czas po ktorym nastapi zmiana kierunku obrotu*/
    private float m_fRotateChangeDirTimeMin = 10.0f;
    /** maksymalny czas po ktorym nastapi zmiana kierunku obrotu*/
    private float m_fRotateChangeDirTimeMax = 30.0f;
    void SetRotateChangeDirTime(float fTime){ SetRotateChangeDirTime(fTime,fTime); }
    void SetRotateChangeDirTime(float fMin,float fMax)
    {
        m_fRotateChangeDirTimeMin = fMin;
        m_fRotateChangeDirTimeMax = fMax;
    }
    /** Czy czas zmiany kierunku obrotu zalezy sztywno od okresu?*/
    private boolean m_bRotateChangeDirTimeByPeriod = true;
    void SetRotateChangeDirTimeByPeriod()
    {
        SetRotateChangeDirTime( m_fRotatePeriod * 0.5f, 
                                m_fRotatePeriod * 1.5f );
    }
    
    
    final static float _SWING_PERIOD_SLOW = 1.5f;
    final static float _SWING_PERIOD_FAST = 1.2f;
    
    /** okres kolysania*/
    private float m_fSwingPeriod       = _SWING_PERIOD_SLOW;
    void SetSwingPeriod(float fPeriod){ m_fSwingPeriod = fPeriod; }
    
    final static float _SWING_AMPLITUDE_SLOW =  2.5f;
    final static float _SWING_AMPLITUDE_FAST = 20.0f;
    
    /** amplituda kolysania*/
    private float m_fSwingAmplitude    = _SWING_AMPLITUDE_SLOW;
    void SetSwingAmplitude(float fAmplitude){ m_fSwingAmplitude = fAmplitude; }

    
    NetObjectPositionController()
    {
        if (m_bRotateChangeDirTimeByPeriod)
            SetRotateChangeDirTimeByPeriod();
    }
            
    
    int METHOD_RotateChangeDir = 0;
    void OnCreate()
    {
        METHOD_RotateChangeDir = RegisterCallMethod( "RotateChangeDir" );
    }
    
    /*void StartRotate()
    {
        StartMoving( _MOVING_MODE_ROTATE );
    }*/
    
    void StartRotateAndSwingSlow()
    {
        m_fSwingPeriod       = _SWING_PERIOD_SLOW;
        m_fSwingAmplitude    = _SWING_AMPLITUDE_SLOW;
        StartRotateAndSwing();
    }
    
    void StartRotateAndSwingFast()
    {
        m_fSwingPeriod       = _SWING_PERIOD_FAST;
        m_fSwingAmplitude    = _SWING_AMPLITUDE_FAST;
        StartRotateAndSwing();
    }
    
    void StartRotateAndSwing()
    {
        StartMoving( _MOVING_MODE_ROTATE | _MOVING_MODE_SWING );
    }
    
    void StartMoving(int nMode)
    {
        m_nMovingMode |= nMode;
        //TMP
        m_nMovingMode  = nMode;
        
        m_fLastTime    = GetTime();
        
        if ( (m_nMovingMode & _MOVING_MODE_ROTATE) != 0 )
            RotateChangeDir();
        CheckActivateOnModule();
    }
    
    void StopMoving()
    {
        m_nMovingMode = _MOVING_MODE_NONE;
        DisableCallMethod( METHOD_RotateChangeDir );
        CheckActivateOnModule();
    }
    
    void CheckActivateOnModule()
    {
        if ( m_nMovingMode != _MOVING_MODE_NONE )
            ActivateOnModule();
        else
            DeactivateOnModuleAlways();
    }
    
    void RotateChangeDir()
    {
        DisableCallMethod( METHOD_RotateChangeDir );
        
        m_nRotateDir *= -1;
        
        if ( (m_nMovingMode & _MOVING_MODE_ROTATE) != 0 )
        {
            float fTime = m_fRotateChangeDirTimeMin  +
                          RandomFloat( m_fRotateChangeDirTimeMax-m_fRotateChangeDirTimeMin );
            CallMethodIn( METHOD_RotateChangeDir, fTime );
        }
    }
    
    
    void OnFrame()
    {
        if ( m_cControlledObj != null && m_nMovingMode != _MOVING_MODE_NONE )
            Moving();
    }
    
    
    
    private int   m_nRotateDir         = 1;
    private float m_fLastSwing         = 0.0f;
    private float m_fLastTime          = 0.0f;
    
    void Moving()
    {
        float fTime = GetTime();
        
        if ( (m_nMovingMode & _MOVING_MODE_ROTATE) != 0 )
        {
            Vector vDir = m_cControlledObj.GetForwardVector();
            float fDeltaTime = fTime - m_fLastTime;
            float fRotate    = 360.0f * m_nRotateDir * fDeltaTime / m_fRotatePeriod;
            vDir = RotateVector( vDir, fRotate, Vector.Up );
            m_cControlledObj.SetDir( vDir );    
        }
    
        if ( (m_nMovingMode & _MOVING_MODE_SWING) != 0 )
        {
            Vector vPos = m_cControlledObj.GetPosition();
            
            float fFrac  = (float)(((fTime/m_fSwingPeriod)) * (float)java.lang.Math.PI);
            float fSwing = m_fSwingAmplitude * (float)Math.abs((float)java.lang.Math.sin(fFrac));
            
            vPos.fY += fSwing - m_fLastSwing ;
            m_fLastSwing = fSwing;
            
            m_cControlledObj.SetPositionWorld( vPos );    
        }
        
        m_fLastTime = fTime;
    }
}
