import java.lang.reflect.*;
import java.util.*;

public abstract class AIController extends GameObject
    implements IFreezable
{
    /////////////////////////////////////////////////////////////////////
    // PRIORYTET PRZECIWNIKA
    /** dodatkowa waga, gdy przeciwnik prawie niezywy */
    static float _LOW_HEALTH_WEIGHT = 0.5f;
    /** waga zwiazana z odlegloscia */
    static float _DISTANCE_WEIGHT = 5f;
    /** waga zwiazana z importance celu */
    static float _TARGET_IMPORTANCE_WEIGHT = 5f;
    /** waga zwiazana z tym, ze ten przeciwnik mnie atakuje */
    static float _ENEMY_ATTACK_WEIGHT = 2f;
    /** waga zwiazana z widocznoscia (prawdziwa) celu */
    static float _VISIBILITY_WEIGHT = 4f;
    /** waga zwiazana z biezacym przeciwnikiem */
    static float _ACTIVE_ENEMY_WEIGHT = 0.5f;
    /** kiedy uznajemy, ze kat do przeciwnika jest niewielki */
    static float _SHARP_ANGLE_TO_TARGET = 10f;
    /** cosinus kata pomiedzy wektorem patrzenia a wektorem do przeciwnika */
    static float _SHARP_ANGLE_TO_TARGET_COS = 0;
    /** kiedy uznajemy, ze kat do przeciwnika jest niewielki */
    static float _SHARP_ANGLE_TO_TARGET_WEIGHT = 1f;
    /** mnoznik max health, kiedy uznajemy ze przeciwnik jest prawie niezywy */
    static float _LOW_HEALTH_COEFF = 0.1f; 
    /** waga przeciwnika, ktory jest celem edytorowym */
    static float _EDITOR_ENEMY_WEIGHT = 10f;
    
    /** czas przypisywany do zmiennych w czasie inicjalizacji */
    final static float _INIT_TIME = -100f;
    
    /** krotki okres czasu od od ostatniego kontaktu (widzenia, slyszenia) wroga */
    static float _ENEMY_NOTICED_SHORT_TIME_MIN = 0.5f;
    static float _ENEMY_NOTICED_SHORT_TIME_MAX = 5f;
    static float _ENEMY_NOTICED_SHORT_TIME_DEFAULT = (_ENEMY_NOTICED_SHORT_TIME_MAX - _ENEMY_NOTICED_SHORT_TIME_MIN) / 2f;
    /** dlugi okres czasu od ostatniego kontaktu (widzenia, slyszenia) wroga     */    
    static float _ENEMY_NOTICED_LONG_TIME = 60f;    
    /** czas od momentu, kiedy ai nie widzi zadnego wroga do czasu, kiedy moze mu spasc poziom
     * zaalarmowania na LOW */
    static float _TIME_TO_CALM_DOWN_AFTER_ENEMY_LOST = 10f;    
    /** czas, po jakim AI stwierdza, ze przestaje slyszec odglos pochodzacy od przeciwnika */
    static float _TIME_TO_INVALIDATE_NOISE = 20f;    
    
    /** stopnie zaalarmowania ai */
    static final public int _ALARM_LEVEL_LOW = 0;
    static final public int _ALARM_LEVEL_MEDIUM = 1;
    static final public int _ALARM_LEVEL_HIGH = 2;
    
    static final public int _FAR_AWAY_FROM_ENEMY_DIST = 4000;
    static final public int _CLOSE_TO_ENEMY_DIST = 1500;		
    static final public int _VERY_CLOSE_TO_ENEMY_DIST = 800;		

    /** mesh kontrolowanego obiektu */
    NewAI m_MeshAI = null;
    
    /** obiekty do pominiecia przy traceowaniu */
    ControlObject[] m_aTraceExcludeObjects = new ControlObject[3];

    /** kolejka rozkazow wewnetrznych dla AI */
    TasksList m_TaskList = new TasksList();
    /** biezacy task AI */
    Task m_CurrentTask = null;
    
    /** Znacznik, ze po zawolaniu Unfreeze nalezy wykonac nastepny task */
    boolean m_bPerformNextTaskOnUnfreeze = false;
    
    /** zmysly AI */
    IAIHearingSense m_HearingSense = null;
    IAIPainSense m_PainSense = null;
    ObjectsSense m_ObjectsSense = null;
    
    /** tablica scenariuszy AI */
    static Scenario m_aScenarios[] = null;
    
    /** id aktualnie wykonywanego scenariusza	 */
    int m_nCurrScenario = -1;

    /** id ostatniego scenariusza, dla ktorego zawolalo sie prepare */
    int m_nLastPreparedScenario = -1;
    
    /** wartosc pomocnicza dla aktualnie wykonywanego scenariusza (scenariusze nie moga pamietac stanu)     */
    int m_nHelperScenariosInt = 0;
    
    /** ostatnie zdarzenie, ktore spowodowalo zmiane scenriusza	 */
    int m_nLastEvent = -1;
    
    /** tablica generatorow */
    Generator m_aGenerators[] = null;
    
#if AI_DEBUG    
    /** Filtr wybierajacy scenariusze przy wypisywaniu */
    BitVector m_DebugScenariosFilter = new BitVector();
#endif        

    /** informacja o obiektach otaczajacych AI */
    java.util.HashMap m_aAllies = new java.util.HashMap(7);
    java.util.ArrayList m_aAlliesList = new java.util.ArrayList();
    java.util.HashMap m_aEnemies = new java.util.HashMap(3);
    java.util.ArrayList m_aPriorityEnemies = new java.util.ArrayList();
    java.util.ArrayList m_RecognitionListNonPlayer = new java.util.ArrayList();
    java.util.ArrayList m_RecognitionListPlayer = new java.util.ArrayList();

    /** wlaczenie strzelania */
    boolean m_bCanFire = false;
    /** Znacznik, ze ai moze przycelowac przed strzelaniem  */
    boolean m_bCanAim = false;    

    /** Aktualna celnosc */
    float m_fCurrAccuracy = 1.0f;
    
    /** aktywny wrog (ktorego aktualnie atakuje	 */
    ChromeActor m_cActiveEnemy = null;
    
    /** Informacja o rozpoznawanym obiekcie (w zasiegu wzroku, ale jeszcze nie rozpoznany */
    EnemyInfo m_cSuspectInfo = null;
    
    final static float _SUSPECT_HOLDING_TIME = 5.f;
    
    /** Czas potrzebny na rozpoznanie przeciwnika gdy na skraju zasiegu widzenia ai */
    float fMaxEnemyRecognitionTime = 3.f;
    /** Czas potrzebny na rozpoznanie przeciwnika gdy na skraju zasiegu widzenia ai (dla zaalarmowanego AI) */
    float fMaxEnemyRecognitionTimeAlarmed = 0.f;

    
    /** Czas po jakim zapomina, ze uslyszal strzal */
    final static float _TIME_TO_INVALIDATE_HEARD_SHOT = 30;	

    /** po jakim czasie ai ma ponownie zareagowac na zdarzenie */
    final static float _HEARD_SHOOT_ACCEPT_TIME = 3;
    final static float _HEARD_BULLET_ACCEPT_TIME = 3;
    final static float _HEARD_EXPLOSION_ACCEPT_TIME = 3;
    final static float _HEARD_STEPS_ACCEPT_TIME = 3;
    final static float _HEARD_SCREAM_ACCEPT_TIME = 3;
    final static float _HEARD_DANGEROUS_ACCEPT_TIME = 3;
    /** czasy ostatniej reakcji na zdarzenia (stosowane do obslugi zdarzenia) */
    float m_fLastHeardEnemyShotAcceptTime = _INIT_TIME;
    float m_fLastHeardFriendShotAcceptTime = _INIT_TIME;
    float m_fLastHeardBulletAcceptTime = _INIT_TIME;
    float m_fLastHeardExplosionAcceptTime = _INIT_TIME;
    float m_fLastHeardEnemyStepsAcceptTime = _INIT_TIME;
    float m_fLastHeardScreamAcceptTime = _INIT_TIME;
    float m_fLastHeardDangerousAcceptTime = _INIT_TIME;
    
    /////////////////////////////////////////////////////////////////////
    // WIDZENIE TRUPOW
    /** ostatni czas sprawdzenia widzenia trupow */
    float m_fLastDyingCheckTime = _INIT_TIME;
    /** ostatnio widziany trup */
    ChromeActor m_LastBodySpotted = null;
    
    final static float _TIME_BTW_DYING_CHECK = 1f;
    final static float _MAX_DIST_TO_SEE_DEAD_BODY = 3000f;
    final static float _MAX_DIST_TO_SEE_DEAD_BODY2 = _MAX_DIST_TO_SEE_DEAD_BODY * _MAX_DIST_TO_SEE_DEAD_BODY;
    
    /////////////////////////////////////////////////////////////////////
    // SLYSZENIE
    /** ostatni czas uslyszenia wystrzalu wroga */
    float m_fLastHeardEnemyShotTime = _INIT_TIME;
    /** ostatni czas uslyszenia wystrzalu przyjaciela */
    float m_fLastHeardFriendShotTime = _INIT_TIME;
    /** ostatni czas uslyszenia rykoszetu */
    float m_fLastHeardRicochetTime = _INIT_TIME;
    /** ostatni czas uslyszenia wybuchu */
    float m_fLastHeardExplosionTime = _INIT_TIME;
    /** ostatni czas uslyszenia niebezpiecznego obiektu */
    float m_fLastHeardDObjectTime = _INIT_TIME;
    /** ostatni czas krokow */
    float m_fLastHeardEnemyStepsTime = _INIT_TIME;
    /** Zapamietana pozycja krokow wroga */
    Vector m_vLastHeardEnemyStepsPos = null;
    /** ostatni czas krzyku */
    float m_fLastHeardScreamTime = _INIT_TIME;
    /** ostatni czas postrzelenia przez nieprzyjaciela */
    float m_fLastGotShotTime = _INIT_TIME;
    /** dlugi czas waznosci dzwieku */
    static float _LONG_SOUND_IMPORTANCE_TIME = 5f;
    /** kiedy stwierdza ze dlugo nie slyszal odglosu strzalu */
    static float _VERY_LONG_SOUND_IMPORTANCE_TIME = 10.f;
    /** krotki czas waznosci dzwieku */
    static float _SHORT_SOUND_IMPORTANCE_TIME = 2f;

    /** miejsce, skad ostatnio doszedl dzwiek strzalu przeciwnika */
    Vector m_vLastHeardEnemyShotPos = null;
    float m_fLastEnemyShotDist = 0f;

    /** miejsce, skad ostatnio doszedl dzwiek strzalu przyjaciela */
    Vector m_vLastHeardFriendShotPos = null;
    /** miejsce, gdzie byl ostatnio przeciwnik strzelajacego przyjaciela */
    Vector m_vLastFriendsTargetPos = null;

    /** manager optymalizujacy "slyszenie" */
    static NoiseManager m_NoiseManager = null;
    
    /** maks odleglosc, na ktora AI podejdzie by sprawdzic cialo */
    static final float _MAX_DIST_TO_CHECK_BODY = 2000f;
    /** ostatni umierajacy, ktorego uslyszalo to AI */
    ChromeActor m_LastDyingAISpotted = null;
    
    ChromeActor m_LastDangerousObject = null;
    
    /** stozek make noise pojazdu (ai ma reagowac na noise dangerous object gdy jest w tym stozku) */
    static float m_fVehicleNoiseConeBack = 800f;
    static float m_fVehicleNoiseConeAngle = 5f;
    static float m_fVehicleNoiseConeDot = 0f;

    /** biezace morale AI */
    float m_fMorale = 1f;
    /** biezacy stopien zaalarmowania */
    int m_iCurrAlarmLevel = _ALARM_LEVEL_LOW;
    // czas ostatniego przejscia AI do LOW_ALARM_LEVEL
    float m_fLastAlarmedTime = _INIT_TIME;
    // czas, przez jaki AI po uspokojeniu nie wolno odpalac animacji "wyluzowania"
    static float _TIME_TO_PLAY_CALM_IDLE_ANIM = 60f;
    /** czasy reakcji indeksowane m_iCurrAlarmLevel */
    float _REACTION_TIME[] = { 0.75f, 1f, 1.5f };
    /** biezacy obiekt za ktorym nalezy podazac */
    ChromeActor m_ObjectToFollow = null;
    static float _ENEMY_ATTACK_IMPORTANCE_TIME = 5f;
    static float _TIME_BTW_MANY_ENEMIES_CHECK = 3f;
    float m_fAttackByManyEnemiesCheckTime = 0;
    int m_iAttackingEnemies = 0;
    boolean m_bIsFrozen = false;
    static float _TIME_BTW_ANGLE_CHECK = 0.1f;
    static float _MIN_TIME_BTW_TRACE_CHECK = 0.05f;
    static float _MAX_TIME_BTW_TRACE_CHECK = 0.4f;
    static float _MIN_TIME_BTW_GUN_TRACE_CHECK = 0.05f;
    static float _MAX_TIME_BTW_GUN_TRACE_CHECK = 1f;
    static float _MAX_TIME_BTW_RECOGNIZE_TRACE_CHECK = 2f; // maksymalny czas pomiedzy traceami do rozpoznawania przeciwnikow
    static float _MIN_DIST_TO_PLAYER_RECOGNIZE2 = 200f * 200f; // dolny prog kwadratu odleglosci do wyliczania czasu pomiedzy rozpoznawaniem wroga
    static float _MAX_DIST_TO_PLAYER_RECOGNIZE2 = 2000f * 2000f; // gorny prog kwadratu odleglosci do wyliczania czasu pomiedzy rozpoznawaniem wroga
    static float _MIN_TIME_BTW_NPC_ENEMY_RECOGNITION_CALL = 0.1f;
    static float _MAX_TIME_BTW_NPC_ENEMY_RECOGNITION_CALL = 4f;
    static float _MIN_TIME_BTW_PLAYER_ENEMY_RECOGNITION_CALL = 0.1f;
    static float _MAX_TIME_BTW_PLAYER_ENEMY_RECOGNITION_CALL = 1f;
    RecognitionListInfo m_NonPlayerListInfo = new RecognitionListInfo(m_RecognitionListNonPlayer);
    RecognitionListInfo m_PlayerListInfo = new RecognitionListInfo(m_RecognitionListPlayer);
    Player m_Player = null;

    boolean m_bEnemyPresenceStated = false; // czy ai zauwazylo jakiegos wroga (czy zostalo zaalarmowane)
    
    float m_fLastTraceFromGunTimeCheck = 0;
    float m_fLastFromGunVisibilityChangeTime = 0;
    boolean m_bActiveEnemyTraceFromGun = false;
    Vector m_vLastHeardRicochetMakerPos = null; // ostatnia pozycja
    Vector m_vLastHeardExplosionMakerPos = null;
    float m_fLastRicochetDist = 0f;
    float m_fLastExplosionDist = 0f;
    int _MANY_ATTACKING_ENEMIES = 2;

    /** czas, kiedy stracilismy przeciwnika */
    float m_fEnemyLostTime = _INIT_TIME;
    
    float m_fLastImpulseTime = _INIT_TIME;
    static float _MAX_IMPULSE_TIME = 60f;
    
    
    /////////////////////////////////////////////////////////////////////
    // Obsluga halasow w 'przejsciach' (drzwi, windy, etc.)
    /** Minimalna przerwa miedzy kolejnymi sprawdzeniami halasow w przejsciach */
    public float _PASSAGE_NOISE_UPDATE_TIME = 0.5f;
    /** Minimalny czas, co jaki AI powinno reagowac na pojawienie sie przeciwnika w przejsciu */
    public float _PASSAGE_ENEMY_ACCEPT_TIME = 1.f;
    
    /** czas, kiedy ostatnio sprawdzany byl halas z przejscia  */
    float m_fLastPassageNoiseTime = _INIT_TIME;
    /** czas, kiedy ostatnio zostalo obsluzone zdarzenie pojawienia sie przeciwika w przejsciu  */
    float m_fLastPassageEnemyNoiseTime = _INIT_TIME;
    /** ostatnia znana pozycja przejscia, w ktorym pojawil sie przeciwnik */
    Vector m_vLastPassageEnemyNoisePos = null;



    /////////////////////////////////////////////////////////////////////
    // OBSLUGA WAYPOINTOW - ROZKAZY EDYTOROWE
    
    /** biezaca aktywnosc */
    IAIActivity m_CurrActivity = null;
    /** biezacy condition dla m_CurrActivity */
    Condition m_CurrCondition = null;
    /** numer biezaco wykonywanego behaviora w waypoincie */
    int m_iCurrWaypointBehavior = 0;
    /** czy mozna wykonywac biezaca aktywnosc - m_CurrCondition zostal spelniony */
    boolean m_bPerformCurrActivity = false;
    /** rodzaj wykonywanej aktywnosci - motiontype czy behavior */
    boolean m_bActivityInWaypoint = false;
    
    /////////////////////////////////////////////////////////////////////////////
    
    /** biezacy fightpoint */
    public FightPoint m_CurrFightPoint = null;
    /** czas - moment w ktorym AI powinno juz wyjsc z FightPointa */
    public float m_fFightPointLeaveTime = 0;
    
    /////////////////////////////////////////////////////////////////////////////
    /** Czas ostatniego zauwazenia przeciwnika (kasowany, gdy zawolane JustSpottedEnemy */
    protected float m_fLastEnemySpottedTime = _INIT_TIME;
    protected float m_fLastEnemySpottedReactionTime = _INIT_TIME;
    protected float m_fLastEnemySpottedScreamHeardTime = _INIT_TIME;
    protected float m_fLastHearNoiseTraceCheckTime = _INIT_TIME;
    
    /** Co jaki czas mozna sprawdzac Trace'a po uslyszeniu dzwieku (jesli wymaga on trace'a) */
    public final static float _TIME_BTW_HEAR_NOISE_TRACE_CHECK = 3;

    public final static float _SHORT_OBJECT_SPOTTED_TIME = 1.f;
    public final static float _OBJECT_SPOTTED_REACTION_TIME = 60.f;
    public final static float _MIN_TIME_BTW_SUBSEQUENT_SCREAMS = 60.f;
    
    /////////////////////////////////////////////////////////////////////////////

    // ostatnie obrazenia jakich doznal ai
    public float m_fLastDamageTaken = 0;
    
    
    int METHOD_CheckCurrWaypointCnd = 0;
    int METHOD_WaitingForAppear = 0;
    int METHOD_OnTaskUpdate = 0;
    int METHOD_OnTaskTimePassed = 0;

    #if HELPER_TUNE
    MeshObject cHelperTune = null;
    #endif
    
    boolean m_bSitsInVehicleOnSeat = false; // czy AI rzeczywiscie siedzi w pojezdzie (po fazie wsiadania i przed faza wysiadania)
    
    public void OnCreate()
    {
        bSG = false;
        METHOD_CheckCurrWaypointCnd = RegisterCallMethod("CheckCurrWaypointCnd");
        METHOD_WaitingForAppear = RegisterCallMethod("WaitingForAppear");
        
        METHOD_OnTaskUpdate = RegisterCallMethod("OnTaskUpdate");
        METHOD_OnTaskTimePassed = RegisterCallMethod("OnTaskTimePassed");
    
        if ( (m_NoiseManager == null) || (m_NoiseManager.GetModule() != GetModule()) )
        {
            m_NoiseManager = (NoiseManager) CreateObject("NoiseManager");
            m_NoiseManager.ActivateOnModule();
            m_NoiseManager.SetTimePeriod(1f);
            //CrashLog("NoiseManager created.\n");
        }
        
        CallMethodEveryMinMax("UpdateNpcEnemiesRecognition",
                              _MIN_TIME_BTW_NPC_ENEMY_RECOGNITION_CALL,
                              _MAX_TIME_BTW_NPC_ENEMY_RECOGNITION_CALL);
        CallMethodEveryMinMax("UpdatePlayerEnemiesRecognition",
                              _MIN_TIME_BTW_PLAYER_ENEMY_RECOGNITION_CALL,
                              _MAX_TIME_BTW_PLAYER_ENEMY_RECOGNITION_CALL);
    }
    
    void StartEvent()
    {
        m_Player = ((ChromeModuleSingle) GetModule()).MainPlayer;

        // sprawdzenie, bo WaitingForAppear moglo juz ustawic Waypoint
        if (GetCurrWaypoint() == null)
        {
            // pobranie pierwszego waypointa i obsluga
            ServeNextWaypointActivity();
        }
        
        RequestEvent(AIEvents._EV_START);
    }
    
    /** inicjacja kontrolera - metode nalezy pokryc (by np. stworzyc w niej zmysly) */
    public void InitController(NewAI ai)
    {
        m_MeshAI = ai;
            
        SetName("Controller of " + ai.GetName());
        AddGenerators();
        InitScenarios();		
        CallMethodIn("StartEvent", 1f);
        
        _SHARP_ANGLE_TO_TARGET_COS = (float) Math.cos(_SHARP_ANGLE_TO_TARGET * Math.PI / 180f);

        // liczymy dot stozka do make noise pojazdu
        if (m_MeshAI.m_nInstanceNumber == 0)
            m_fVehicleNoiseConeDot = (float) Math.cos(m_fVehicleNoiseConeAngle * Math.PI / 180f);
        #if HELPER_TUNE
        if (cHelperTune == null)
        {
        cHelperTune = Tools.AddHelperMesh(GetMesh(), Tools._HM_MESH_EXCLAMATION);
        cHelperTune.AttachAsChildTo(GetMesh());
        cHelperTune.SetPosition(new Vector(0, 200, 0));
        cHelperTune.EnableRendering(false);
        }
        #endif		
    }
    
    /** w tej metodzie dodaja sie (automatycznie) scenariusze */
    public void InitScenarios()
    {
        if (/* m_aScenarios == null || */ m_MeshAI.m_nInstanceNumber == 0)
        {
            LogLine(" I will create Scenarios "+Scenario._NUM_OF_SCENARIOS);

            Class[] aScenariosClasses = GetClassesFromClass("Scenario");
            Field fieldId;
            int id;
            
            Scenario scenario;
            m_aScenarios = new Scenario[Scenario._NUM_OF_SCENARIOS];
            for (int i=0; i<aScenariosClasses.length; i++)
            {
                if (aScenariosClasses[i] != Scenario.class)
                {
                    try {
                        fieldId = aScenariosClasses[i].getDeclaredField("_ID");
                        id = fieldId.getInt(null);
                        if (m_aScenarios[id] == null)
                        {
                            m_aScenarios[id] = (Scenario)aScenariosClasses[i].newInstance();
                            m_aScenarios[id].Initialize();
                        }
                        else
                            CrashLog("ERROR: Duplicated _ID field value in: " + aScenariosClasses[i].getName() +
                                     " and " + m_aScenarios[id].getClass().getName() + "!!!\n");
                        
                    }
                    catch (Exception e)
                    {
                        CrashLog("ERROR accessing field _ID in scenario " + aScenariosClasses[i].getName() + "\n");
                    }
                }
            }
        }
    }
    
    /** metode nalezy pokryc, by dodac generatory */
    public void AddGenerators()
    {
#if AI_DEBUG        
        m_DebugScenariosFilter.SetAllBits();
#endif
    }
    
    public final NewAI GetMesh()
    {
        return m_MeshAI;
    }
    
    public AIMotionState GetMotionState()
    {
        return m_MeshAI.m_Motion;
    }
    
    /** pojednycze zawolanie metody przez call method */
    void CallMethodOnceIn(int iMethodID, float fTime)
    {
        DisableCallMethod(iMethodID);
        CallMethodIn(iMethodID, fTime);
    }
    
    /** czy minal czas fTimeInterval od fStartTime do fCurrTime */
    public static final boolean TimePassed(float fStartTime, float fTimeInterval, float fCurrTime)
    {
        return fCurrTime - fStartTime > fTimeInterval;
    }
    
    
    public void NotifyMeshActivation(boolean bActivate)
    {
        if (bActivate)
            DisableCallMethod(METHOD_WaitingForAppear);
        else
            StartWaitingForAppear();
    }
    
    
    public void Freeze()
    {
        if (!IsFrozen())
        {
            // freezujemy zmysly
            if (m_HearingSense != null)
                ((Sense)m_HearingSense).Freeze();
            if (m_PainSense != null)
                ((Sense)m_PainSense).Freeze();
            if (m_ObjectsSense != null)
                ((Sense)m_ObjectsSense).Freeze();
            FreezeCallMethods();
            UnfreezeCallMethod(METHOD_WaitingForAppear);
            m_bIsFrozen = true; 
        }
    }
    
    public void Unfreeze()
    {
        if (IsFrozen())
        {
            // odfreezujemy zmysly
            if (m_HearingSense != null)
                ((Sense)m_HearingSense).Unfreeze();
            if (m_PainSense != null)
                ((Sense)m_PainSense).Unfreeze();
            if (m_ObjectsSense != null)
                ((Sense)m_ObjectsSense).Unfreeze();
            UnfreezeCallMethods();
            m_bIsFrozen = false;
            
            if (m_bPerformNextTaskOnUnfreeze)
                PerformNextTask();
        }
    }
    
    public boolean IsFrozen()
    {
        return m_bIsFrozen;
    }
    
    private boolean m_bDead = false;
    
    public void Die()
    {
        if (!IsDead())
        {
            // zmysly umieraja
            if (m_HearingSense != null)
                ((Sense)m_HearingSense).Die();
            if (m_PainSense != null)
                ((Sense)m_PainSense).Die();
            if (m_ObjectsSense != null)
                ((Sense)m_ObjectsSense).Die();
            DisableCallMethod("StartEvent");
            DisableCallMethod(METHOD_WaitingForAppear);
            DisableCallMethod(METHOD_CheckCurrWaypointCnd);
            DisableCallMethod("ServeNextWaypointActivity");
            DisableCallMethod("OnGlobalTimePassed");
            DisableCallMethod(METHOD_OnTaskTimePassed);
            DisableCallMethod(METHOD_OnTaskUpdate);
            DisableCallMethod("SetLowAlarmLevel");
            DisableCallMethod("CheckForAnyDying");
            DisableCallMethod("UpdateNpcEnemiesRecognition");
            DisableCallMethod("UpdatePlayerEnemiesRecognition");
            m_bDead = true;
            #if HELPER_TUNE
            if (cHelperTune != null)
            cHelperTune.EnableRendering(false);
            #endif			
        }
    }
    
    public void Respawn()
    {
        m_bDead = false;
        
        if (m_HearingSense != null)
            ((Sense)m_HearingSense).Respawn();
        if (m_PainSense != null)
            ((Sense)m_PainSense).Respawn();
        if (m_ObjectsSense != null)
            ((Sense)m_ObjectsSense).Respawn();
    }
    
    public boolean IsDead()
    {
        return m_bDead;
    }
    
    /////////////////////////////////////////////////////////////////////////////
    
    void StartWaitingForAppear()
    {
        CallMethodEvery(METHOD_WaitingForAppear, 0.1f, 0.1f * GetMesh().GetTimeDeviation());
        UnfreezeCallMethod(METHOD_WaitingForAppear);
    }
    
    void WaitingForAppear()
    {
        if (IsFrozen())
        {
            MotionType mt;
            if (m_Player == null && // jesli jeszcze nie byl zainicjowany (bDeactivateOnStart w meshu)
                GetCurrWaypoint() == null &&
                GetMesh().cFirstWaypoint != null &&
                ((mt = GetMesh().cFirstWaypoint.cMotion) instanceof mt_Appear) &&
                ((mt_Appear)mt).bAppear)
            {
                ServeNextWaypointActivity();
            }
            else
            {
                if (GetCurrWaypoint() != null &&
                    ((mt = GetCurrWaypoint().cMotion) instanceof mt_Appear) &&
                    ((mt_Appear)mt).bAppear)
                {
                    CheckCurrWaypointCnd();
                }
                else 
                    return; // czekamy, az bedzie appear
            }
            if (!m_bPerformCurrActivity)
                return;
        }
        DisableCallMethod(METHOD_WaitingForAppear);
    }
    
    /////////////////////////////////////////////////////////////////////////////
    // OBSLUGA WAYPOINTOW - ROZKAZY EDYTOROWE
    public void ServeNextWaypointActivity()
    {
        if (m_CurrActivity != null)
            m_CurrActivity.StopActivity(this);
        
        m_bPerformCurrActivity = false;
        if (GetCurrWaypoint() == null)
        {
            // pierwszy waypoint
            GetMesh().SetWaypoint(GetMesh().cFirstWaypoint);
            m_bActivityInWaypoint = false;
            m_iCurrWaypointBehavior = 0;
            if (GetCurrWaypoint() != null)
            {
                if (GetCurrWaypoint().cMotion == null)
                    GetCurrWaypoint().cMotion = new mt_Move();
                m_CurrActivity = (IAIActivity) GetCurrWaypoint().cMotion;
                m_CurrCondition = GetCurrWaypoint().cEnterCondition;
            }
        }
        else
        {
            boolean bHasBehaviors = (GetCurrWaypoint().aBehaviors != null) && (GetCurrWaypoint().aBehaviors.length > 0);
            if ( ((!m_bActivityInWaypoint) && bHasBehaviors) ||
                 ((m_bActivityInWaypoint) && (m_iCurrWaypointBehavior > 0) &&
                  (m_iCurrWaypointBehavior < GetCurrWaypoint().aBehaviors.length)) )
            {
                // jezeli wykonalismy wlasnie motiontype i w punkcie jest behavior lub
                // wykonujemy juz behavior i chcemy wykonac nastepny
                m_bActivityInWaypoint = true;
                m_CurrActivity = (IAIActivity) GetCurrWaypoint().aBehaviors[m_iCurrWaypointBehavior];
                if (m_CurrActivity != null)
                    m_CurrCondition = GetCurrWaypoint().aBehaviors[m_iCurrWaypointBehavior].cCondition;
                if (m_iCurrWaypointBehavior == 0)
                {
                    // wykonujemy akcje w waypoincie
                    InvokeActionChain(GetCurrWaypoint().aActions);
                }
                m_iCurrWaypointBehavior++;
            }
            else
            {
                if (!bHasBehaviors)
                {
                    Waypoint actionChainWaypoint = GetCurrWaypoint();
                    // wykonujemy akcje w waypoincie
                    InvokeActionChain(actionChainWaypoint.aActions);
                    // sprawdz, czy nie zmienil sie Waypoint (np. act_ChangeWaypoint)
                    if (actionChainWaypoint != GetCurrWaypoint())
                        return;
                }
                GetMesh().SetWaypoint(GetMesh().GetNextWaypoint());
                if (GetCurrWaypoint() != null)
                {
                    m_bActivityInWaypoint = false;
                    m_iCurrWaypointBehavior = 0;
                    if (GetCurrWaypoint().cMotion == null)
                        GetCurrWaypoint().cMotion = new mt_Move();
                    m_CurrActivity = (IAIActivity) GetCurrWaypoint().cMotion;
                    m_CurrCondition = GetCurrWaypoint().cEnterCondition;
                }
                else
                {
					m_bPerformCurrActivity = false;
                    m_CurrActivity = null;
                    m_CurrCondition = null;
                }
            }
        }

        CallMethodEvery(METHOD_CheckCurrWaypointCnd, 0.1f, 0.1f * GetMesh().GetTimeDeviation());
        CheckCurrWaypointCnd();
        //CallMethodIn(METHOD_CheckCurrWaypointCnd, 0); //TST - bylo bez CallMethod
    }
    
    public Behavior GetCurrBehavior()
    {
        if ((GetCurrWaypoint() != null) && (m_bActivityInWaypoint) &&
            (GetCurrWaypoint().aBehaviors.length > 0))
            return GetCurrWaypoint().aBehaviors[m_iCurrWaypointBehavior - 1];
        else
            return null;
    }
    
    public void CheckCurrWaypointCnd()
    {
        if (GetCurrWaypoint() == null)
        {
#if AI_DEBUG            
            DebugUtils.CrashLog(m_MeshAI, "CheckCurrWaypointCnd: GetCurrWaypoint() == null !!!\n");
#endif            
            DisableCallMethod(METHOD_CheckCurrWaypointCnd);
            return;
        }
        if (!m_bPerformCurrActivity)
        {
            if (m_CurrCondition == null ||
                m_CurrCondition.Evaluate())
            {
                m_bPerformCurrActivity = true;

                SetInCannon(false); // niestety to jest w tym miejscu
                
                m_CurrActivity.StartActivity(this);
                
                // przegenerowanie scenariuszy
                RequestEvent(AIEvents._EV_EDITOR_ACTIVITY);
                DisableCallMethod(METHOD_CheckCurrWaypointCnd);
            }
        }
    }
    
    public void EditorActivityEnded()
    {
        ServeNextWaypointActivity();
    }
    
    public boolean IsEditorEnemy(EnemyInfo info)
    {
        if (info == null)
            return false;
        return info.m_bEditorEnemy;
    }
    
    public boolean ShouldPerformEditorActivity()
    {
        return ( (GetCurrWaypoint() != null) && (m_bPerformCurrActivity) &&
                 ((m_CurrActivity.IsCritical()) ||
                  ((GetActiveEnemy() == null) && TimeFromImpulsePassed())) );
    }

    public boolean TimeFromImpulsePassed()
    {
        return (GetTime() - m_fLastImpulseTime) > _MAX_IMPULSE_TIME;
    }
    
    void C()
    {
        CrashLog("GetCurrWaypoint():" + GetCurrWaypoint() +
                 ", m_bPerformCurrActivity:" + m_bPerformCurrActivity +
                 ", m_CurrActivity:" + m_CurrActivity +
                 ((m_CurrActivity == null)?"":", m_CurrActivity.IsCritical():" + m_CurrActivity.IsCritical()) +
                 ", GetActiveEnemy():" + GetActiveEnemy() + "\n");
    }

    public Waypoint GetCurrWaypoint()
    {
        return m_MeshAI.GetWaypoint();
    } 

    public IAIActivity GetCurrActivity()
    {
        return m_CurrActivity;
    }

    abstract void SetObjectToGetIn(ISeatVehicle cObjectToGetIn, EnumGetInType cGetInAs);
    abstract SeatInfo GetSeat();
    abstract public void GotIn();
    abstract public void GotOut();
    abstract boolean InsideVehicle();
    abstract void SetObjectToDrive(IDrivenVehicle cObjectToDrive);
    abstract IDrivenVehicle GetDrivenVehicle();
    abstract void Drive();
    abstract public void Checkpoint();
    abstract public void MovingEnd();

    abstract boolean InsideCannon();
    abstract void SetInCannon(boolean bInCannon);
    
    public void VehicleDestroyed(Vector vCollPoint, Vector vNormal)
    {
    }
    
    private BitVector m_bv = new BitVector();
    
    /////////////////////////////////////////////////////////////////////
    // WYKONYWANIE SCENARIUSZY, OBSLUGA TASKOW
    public BitVector RecalcGenerators()
    {
#if AI_DEBUG        
        DebugUtils.ClearLog();
        DebugUtils.AddLog(m_MeshAI, "\nRecalcGenerators in AI:" + m_MeshAI.GetName() + "\n");
#endif
        m_bv.SetAllBits();
        
        for (int i = 0; i < m_aGenerators.length; i++)
        {
            m_aGenerators[i].UpdateScenarios();
            m_bv.CalcIntersectionWith(m_aGenerators[i].m_CurrScenarios);
        }

#if AI_DEBUG        
        for (int i = 0; i < m_aGenerators.length; i++)
        {
            LogScenarioVector(m_aGenerators[i].getClass().getName(),
            m_aGenerators[i].m_CurrScenarios);
        }
        DebugUtils.AddLog(m_MeshAI, "-----------------------------------------------------\n");
        LogScenarioVector("ChosenScenarios:", m_bv);
        DebugUtils.WriteOutLog(m_MeshAI);
#endif
        
        return m_bv;
    }
    
    public int GetTheBestScenario(BitVector bv)
    {
        int n = bv.GetFirstBitSet();
        
        if (n == -1)
            return -1; // ups, nie ma zadnego scenariusza
        
        float fMaxPriority = -1f;
        BitVector bestScenarios = new BitVector();
        int iMaxPriorityScenarios = 0;
        float fProbabSum = 0.0f; // suma prawdopodobienstw dla zbioru scenariuszy 
        // puli losowania
        
        while (n != -1)
        {
            if (fMaxPriority < m_aScenarios[n].m_fPriority)
            {
                fMaxPriority = m_aScenarios[n].m_fPriority;
                bestScenarios.ClearAllBits();
                iMaxPriorityScenarios = 0;
                fProbabSum = 0.0f;
            }
            
            if (m_aScenarios[n].m_fPriority == fMaxPriority)
            {
                bestScenarios.SetBit(n);
                iMaxPriorityScenarios++;
                fProbabSum += GetScenario(n).GetProbability();;
            }
            
            n = bv.GetNextBitSet(n + 1);
        }

        int nBestScenario = -1;
        float fChoice = RandomFloat(fProbabSum);
        int nSize = iMaxPriorityScenarios;
        float fChoiceSum = 0.0f;		
        n = bestScenarios.GetFirstBitSet();		
        fChoiceSum += GetScenario(n).GetProbability();
        
        if (fChoice < fChoiceSum)
        {
#if AI_DEBUG            
            DebugUtils.CrashLog(m_MeshAI, " Selected n " + n + " \n");
#endif
            
            return n;
        }
        
        for(int i=0; i<nSize;i++)  // suma skutecznosci wszystkich celow
        {
            n = bestScenarios.GetNextBitSet(n + 1);
            fChoiceSum += GetScenario(n).GetProbability();
            if(fChoice < fChoiceSum)
            {
#if AI_DEBUG            
            DebugUtils.CrashLog(m_MeshAI, " Selected n " + n + " \n");
#endif
                return n;
            }
        }		
#if AI_DEBUG            
            DebugUtils.CrashLog(m_MeshAI, " Selected -1 \n");
#endif
        return -1;
    }

#if AI_DEBUG
private BitVector m_LSVFilter = new BitVector();    
    void LogScenarioVector(String sHeader, BitVector scenarios)
    {
        DebugUtils.AddLog(m_MeshAI, StringUtils.FillSpacesAtEnd(sHeader, 27));
        
        m_LSVFilter.Set(m_DebugScenariosFilter);
        if (!m_MeshAI.bLogEditorScenarios)
            m_LSVFilter.CalcIntersectionWithComplement(Scenario._EDITOR_SCENARIOS);
        
        for (int j = 0; j < Scenario._NUM_OF_SCENARIOS; j++)
        {
            
            if (m_LSVFilter.IsBitSet(j))
            {
                if (scenarios.IsBitSet(j))
                {
                    DebugUtils.AddLog(m_MeshAI, StringUtils.FillSpacesAtEnd(m_aScenarios[j].toString(), 5));
                }
                else
                    DebugUtils.AddLog(m_MeshAI, StringUtils.FillSpacesAtEnd("----", 5));
            }
        }
        DebugUtils.AddLog(m_MeshAI, "\n");
    }
#endif
    
    TasksList GetTaskList()
    {
        return m_TaskList;
    }

    /** Wolana przez m_MeshAI po zakonczeniu odgrywania animacji */
    public void AnimationEnd(int nID)
    {
        Notify(AIEvents._NOTICE_ANIM_END);
    }
    

    /** wolana przez mesh po przerwaniu animacji */
    public void AnimationStopped(int nID)
    {
        Notify(AIEvents._NOTICE_ANIM_STOPPED);
    }

    
    public void CallRequestEvent()
    {
        RequestEvent(AIEvents._EV_UNDEFINED, false);
    }
    
    /** Zawolanie RequestEvent bez wymuszonego przegenerowania scenariuszy
     */
    void RequestEvent(int nEvent)
    {
        RequestEvent(nEvent, false);
    }
    
    /** obsluga zdarzen powodujacych przeliczanie generatorow
     *  nEvent - zdarzenie
     *  cObject - dowolne dane dla obslugi zdarzenia 
     *  bForceRegenerate - znacznik, ze scenariusze musza zostac przegenerowane, nawet, gdy wygeneruje sie taki sam
     * */
    
    void RequestEvent(int nEvent, boolean bForceRegenerate)
    {
        GetMesh().ProfileBegin(NewAI.PROFILE_RequestEvent);
#if AI_DEBUG        
        DebugUtils.CrashLog(m_MeshAI, "DBG: RequestEvent: " + AIEvents.GetEventName(nEvent) + "\n");
#endif        
        if (IsDead())
            return;
        
        if (!bForceRegenerate && FilterEvent(nEvent))
            return;
        
        // przelicz stan ai - warunki sprawdzane w generatorach
        RecalcState();
        
        // przeliczamy scenariusze
        BitVector bv = RecalcGenerators();
        
        ServiceEvent(nEvent); // obsluga zdarzen (rejestrowanie czasow itd)

        // sprawdzenie czy (nieskonczony) biezacy scenariusz jest w liscie wybranych scenariuszy
        if (!bForceRegenerate &&
            (nEvent != AIEvents._EV_SCENARIO_END) && (m_nCurrScenario != -1) &&
            (bv.IsBitSet(m_nCurrScenario)) )
            return;

        int nSelectedScenario = GetTheBestScenario(bv);
        if (nSelectedScenario < 0) // jesli zaden scenariusz nie jest przepuszczony
        {
#if AI_DEBUG
            DebugUtils.CrashLog(m_MeshAI, "SELECTED SCENARIOS N o N e" + "\n");
#endif
            nSelectedScenario = WaitForEnemyAction._ID;
        }

        BreakScenario(false);

        m_nLastEvent = nEvent;
        m_nCurrScenario = nSelectedScenario;
        
        Scenario cScenario = GetScenario(nSelectedScenario);
#if AI_DEBUG
        DebugUtils.CrashLog(m_MeshAI, " SELECTED SCENARIO " + nSelectedScenario + " " + cScenario + " \n");        
#endif
        // przygotowujemy zadania dla wybranego scenariusza
        cScenario.PrepareScenario(this);
        m_nLastPreparedScenario = nSelectedScenario;
        if (m_CurrentTask == null)
            PerformNextTask();
        
        GetMesh().ProfileEnd(NewAI.PROFILE_RequestEvent);
    }
    
    /** filtrowanie zdarzen
     * @return true, gdy zdarzenie zostaje odfiltrowane
     */
    boolean FilterEvent(int nEvent)
    {
        return false;
    }
    
    /** obsluga zaakceptowanych zdarzen */
    void ServiceEvent(int nEvent)
    {
    }	
    
    /** obsluga komunikatow dla biezacych Task'ow */
    void Notify(int nNotice)
    {
        if (nNotice == AIEvents._NOTICE_END_TASK)
        {
            PerformNextTask();
            return;
        }
        
        if (m_CurrentTask != null)
            m_CurrentTask.ServeNotice(nNotice);
    }
    
    /** zwraca aktualnie wykonywany scenariusz */
    Scenario GetCurrScenario()
    {
        if (m_nCurrScenario >= 0 && m_nCurrScenario < Scenario._NUM_OF_SCENARIOS)
            return m_aScenarios[m_nCurrScenario];
        else
            return null;
    }
    
    void OutputCurrScenario()
    {
        if (m_nCurrScenario >= 0 && m_nCurrScenario < Scenario._NUM_OF_SCENARIOS)
            CrashLog("Current Scenario: " + m_aScenarios[m_nCurrScenario] + "\n");
    }
    
    /** zwraca aktualnie wykonywane zadanie	 */
    Task GetCurrTask()
    {
        return m_CurrentTask;
    }
    
    /** zwraca scenariusz o danym indeksie */
    Scenario GetScenario(int nId)
    {
        if(nId < 0 || nId > Scenario._NUM_OF_SCENARIOS)
            return null;
        return m_aScenarios[nId];
    }
    
    /** ustawienie odliczania czasu dla calego scenaiusza */
    void StartGlobalTime(float fTime)
    {
        CallMethodIn("OnGlobalTimePassed", fTime);		
    }
    
    void ClearGlobalTime()
    {
        DisableCallMethod("OnGlobalTimePassed");	
    }
    
    public void OnGlobalTimePassed()
    {
        Notify(AIEvents._NOTICE_GLOBAL_TIME_PASSED);
    }   
    
    /** ustawienie odliczania czasu dla biezacego */
    void StartTaskTime(float fTime)
    {
        CallMethodIn(METHOD_OnTaskTimePassed, fTime);		
    }
    
    void ClearTaskTime() 
    {
        DisableCallMethod(METHOD_OnTaskTimePassed);	
    }
    
    void StartTaskUpdate(float fTime)
    {
        CallMethodEvery(METHOD_OnTaskUpdate, fTime, fTime * GetMesh().GetTimeDeviation());
    }
    
    void ClearTaskUpdate()	
    {
        DisableCallMethod(METHOD_OnTaskUpdate);				
    }
    
    void OnTaskUpdate()
    {
        m_CurrentTask.Update();	
    }
    
    public void OnTaskTimePassed()
    {
        Notify(AIEvents._NOTICE_TASK_TIME_PASSED);
    }

    void AddTask(Task cTask)
    {
        cTask.SetAIController(this);
        m_TaskList.AddTask(cTask, false);
    }
    
    void AddCleaningTask(Task cTask)
    {
        cTask.SetAIController(this);
        m_TaskList.AddTask(cTask, true);
    }
    
    public int _MAX_ALLOWED_TASKS_PER_FRAME = 20;
    protected float m_fLastPerformTaskTime = 0;
    protected int m_nTasksPerCurrFrame = 0;
    /** wykonuje nastepne zadanie z listy   */
    boolean PerformNextTask()
    {
        if (IsFrozen())
        {
            m_bPerformNextTaskOnUnfreeze = true;
            return false;
        }
        else
            m_bPerformNextTaskOnUnfreeze = false;

        
        float fTime = GetTime();
        if (Math.abs(m_fLastPerformTaskTime - fTime) > 0.001f) // zmienil sie czas - nowa ramka
        {
            m_fLastPerformTaskTime = fTime;
            m_nTasksPerCurrFrame = 0;
        }
        else
        {
            if (m_nTasksPerCurrFrame++ == _MAX_ALLOWED_TASKS_PER_FRAME)
                LogR("WARNING: Max allowed tasks per frame reached in " + GetName() + "\n");
        }
        
        if (m_CurrentTask != null)
            m_CurrentTask.OnEndTask();
        
        Task cNextTask = null;        
        cNextTask = GetTaskList().GetFirstTask();
        m_CurrentTask = cNextTask;

        // jesli nie ma wiecej zadan to czynnosci koncowe i przerwanie scenariusza
        if (cNextTask == null) 
            return false;

        m_CurrentTask.Perform();
        return true;
    }
    
    /** przerywa scenariusz	 */
    public void BreakScenario(boolean bFast)
    {
        if(m_CurrentTask!=null && 
           !m_CurrentTask.IsScenarioCleaning())
        {
            m_CurrentTask.Cancel(bFast);
            m_CurrentTask = null;
        }
        
        int nId = 0;
        while( nId < m_TaskList.GetTasksNum() )
        {
            Task cTask = m_TaskList.GetTask(nId);
            if(cTask instanceof TaskTerminator)
            {
                m_TaskList.RemoveTask(nId);
                break;
            }
            if(cTask.IsScenarioCleaning())
                nId++;
            else
                m_TaskList.RemoveTask(nId);
        }
    }

    /////////////////////////////////////////////////////////////////////
    /** Przelicznie warunkow sprawdzanych przez generatory
     */
    void RecalcState()
    {
        //TODO: przelicz widocznosc gracza z guna
    }

    /////////////////////////////////////////////////////////////////////
    /** Dostepne pozycje, z ktorych ai moze strzelac
     */
    /////////////////////////////////////////////////////////////////////
    public final static int _FIRINGPOS_STAND        = 1;
    public final static int _FIRINGPOS_STAND_AIMING = 2;
    public final static int _FIRINGPOS_DUCK         = 4;
    public final static int _FIRINGPOS_DUCK_AIMING  = 8;

    
    public int GetValidFiringPositions()
    {
        return 0;
    }
    
    public boolean IsValidFiringPosition(int nFiringPos)
    {
        return false;
    }
        
    /////////////////////////////////////////////////////////////////////
    // ZMYSLY I POBIERANIE OBIEKTOW Z OTOCZENIA AI
    private static Vector m_vMinNoiseExt = new Vector();
    private static Vector m_vMaxNoiseExt = new Vector();
    
    public float _NOISE_EXT_ADD = 200.f;
    
    /** notyfikacja o uslyszeniu dzwieku */
    public void NoiseMsg(Actor Noisy, float fNoiseLevel, int nNoiseType)
    {
        // nie reagujemy na "swoje" dzwieki
        if (Noisy == GetMesh())
            return;

        // AI uslyszalo rykoszet w tym momencie
        if (nNoiseType == Pawn._NOISE_BULLET_HIT)
        {
            m_fLastHeardRicochetTime = GetTime();
            m_fLastImpulseTime = m_fLastHeardRicochetTime;
            
            if (Noisy != null)
                m_vLastHeardRicochetMakerPos = Noisy.GetPosition();
            else
                m_vLastHeardRicochetMakerPos = GetMesh().GetPosition();

            m_fLastRicochetDist = fNoiseLevel;
            
            if (GetAlarmLevel() < _ALARM_LEVEL_MEDIUM)
                SetAlarmLevel(_ALARM_LEVEL_MEDIUM);
            if ( TimePassed(m_fLastHeardBulletAcceptTime, _HEARD_BULLET_ACCEPT_TIME, GetTime()) )
            {
                m_NoiseManager.Request(this, AIEvents._EV_NOISE_BULLET_HIT);
                m_fLastHeardBulletAcceptTime = GetTime();
            }
            return;
        }

        // AI uslyszalo wybuch w tym momencie
        if (nNoiseType == Pawn._NOISE_EXPLOSION)
        {
            m_fLastHeardExplosionTime = GetTime();
            m_fLastImpulseTime = m_fLastHeardExplosionTime;
            
            if (Noisy != null)
                m_vLastHeardExplosionMakerPos = Noisy.GetPosition();
            else
                m_vLastHeardExplosionMakerPos = GetMesh().GetPosition();
            m_fLastExplosionDist = fNoiseLevel;
            if (GetAlarmLevel() < _ALARM_LEVEL_MEDIUM)
                SetAlarmLevel(_ALARM_LEVEL_MEDIUM);
            if ( TimePassed(m_fLastHeardExplosionAcceptTime, _HEARD_EXPLOSION_ACCEPT_TIME, GetTime()) )
            {
                m_NoiseManager.Request(this, AIEvents._EV_NOISE_EXPLOSION);
                m_fLastHeardExplosionAcceptTime = GetTime();
            }

            if (Noisy instanceof ChromeActor)
            {
                if ( GetMesh().DetectEnemyOnNoise() &&
                     GetMesh().IsEnemy((ChromeActor) Noisy) )
                EnemyPresenceStated();
            }
            
            return;
        }
        

        if (nNoiseType == Pawn._NOISE_PASSAGE && Noisy != null) 
        {
            float fTime = GetTime();
            if (TimePassed(m_fLastPassageNoiseTime, _PASSAGE_NOISE_UPDATE_TIME, fTime) &&
                TimePassed(m_fLastPassageEnemyNoiseTime, _PASSAGE_ENEMY_ACCEPT_TIME, fTime))
            {
                m_fLastPassageNoiseTime = fTime;
                Noisy.GetMeshExtentsInWorld(m_vMinNoiseExt, m_vMaxNoiseExt);
                m_vMinNoiseExt.fX -= _NOISE_EXT_ADD;
                m_vMinNoiseExt.fY -= _NOISE_EXT_ADD;
                m_vMinNoiseExt.fZ -= _NOISE_EXT_ADD;
                m_vMaxNoiseExt.fX += _NOISE_EXT_ADD;
                m_vMaxNoiseExt.fY += _NOISE_EXT_ADD;
                m_vMaxNoiseExt.fZ += _NOISE_EXT_ADD;
                ControlObject[] aObjs = GetObjectsInExtents(m_vMinNoiseExt, m_vMaxNoiseExt);
                if (aObjs != null)
                {
                    for (int i = 0; i < aObjs.length; i++)
                    {
                        if (aObjs[i] instanceof ChromeActor)
                        {
                            ChromeActor actorObj = (ChromeActor)aObjs[i];
                            if (!actorObj.IsDead() &&
                                m_MeshAI.IsEnemy(actorObj))
                            {
                                if (GetAlarmLevel() < _ALARM_LEVEL_MEDIUM)
                                    SetAlarmLevel(_ALARM_LEVEL_MEDIUM);

                                // probujemy dodac wroga do listy
                                EnemyInfo info = AddEnemy(actorObj);
                                if (info != null)
                                {
                                    info.m_fLastKnownPosTime = fTime;
                                    info.m_vLastKnownPos = info.m_Reference.GetPosition();
                                }

                                m_fLastPassageEnemyNoiseTime = m_fLastImpulseTime = fTime;
                                m_vLastPassageEnemyNoisePos = Noisy.GetPosition();
                                m_NoiseManager.Request(this, AIEvents._EV_NOISE_PASSAGE);

                                return;
                            }
                        }
                    }
                }
            }
            return;
        }


        if (Noisy instanceof ChromeActor)
        {
            ChromeActor actor = (ChromeActor) Noisy;
            boolean reactOnScream = false;
            // AI uslyszalo krzyk w tym momencie
            if ((nNoiseType == Pawn._NOISE_SCREAM ||
                 nNoiseType == Pawn._NOISE_BODY_FALL) && 
                Noisy instanceof NewAI)
            {
                NewAI ai = (NewAI)Noisy;
                int screamType = ai.GetLastScreamType();
                if (screamType == NewAI._SCREAMTYPE_DEATH ||
                    nNoiseType == Pawn._NOISE_BODY_FALL)
                {
                    ReactOnDeath(actor, false);
                    reactOnScream = true;   
                }
                else if (screamType == NewAI._SCREAMTYPE_PAIN_ENEMY)
                {
                    reactOnScream = true;
                    // TODO: jakas fajna reakcja
                }
                else if (screamType == NewAI._SCREAMTYPE_ENEMY_SPOTTED)
                {
                    reactOnScream = true;
                    m_fLastEnemySpottedScreamHeardTime = GetTime();
                    if (GetMesh().IsFriend(ai))
                    {
                        ChromeActor spottedEnemy = ai.GetController().GetActiveEnemy();
                        if (spottedEnemy != null && GetMesh().IsEnemy(spottedEnemy))
                        {
                            EnemyInfo info = AddEnemy(spottedEnemy);
                            if (info != null)
                            {
                                if (!info.m_bRecognized)
                                {
                                    // sprawdz, czy krzyczacy jest widoczny
                                    float fTime = GetTime();
                                    if (TimePassed(m_fLastHearNoiseTraceCheckTime, _TIME_BTW_HEAR_NOISE_TRACE_CHECK, fTime))
                                    {
                                        m_fLastHearNoiseTraceCheckTime = fTime;
                                        
                                        if (AICheckTraceAbsolute(ai,
                                                                 ai.GetPointToLookAt(0),
                                                                 GetMesh().GetLookFromPoint()))
                                        {
                                            RecognizeEnemy(info); // widzi krzyczacego - rozpoznaj jego wroga
                                        }
                                    }

                                }
                                else
                                    RecognizeEnemy(info); // update pozycji
                            }
                        }
                    }
                }

                if (reactOnScream)
                {
                    m_fLastHeardScreamTime = GetTime();
                    m_fLastImpulseTime = m_fLastHeardScreamTime;

                    if (GetAlarmLevel() < _ALARM_LEVEL_MEDIUM)
                        SetAlarmLevel(_ALARM_LEVEL_MEDIUM);
                    if ( TimePassed(m_fLastHeardScreamAcceptTime, _HEARD_SCREAM_ACCEPT_TIME, GetTime()) )
                    {
                        m_NoiseManager.Request(this, AIEvents._EV_NOISE_SCREAM);
                        m_fLastHeardScreamAcceptTime = GetTime();
                    }
                    return;
                }
            }
            
            boolean bMadeByEnemy = GetMesh().IsEnemy(actor);
            
            if ( (bMadeByEnemy) &&
                 ((Noisy instanceof NewAI) || (Noisy instanceof Vehicle) ||
                  (Noisy instanceof Player) || (Noisy instanceof StationaryCannon)) )
            {
                // probujemy dodac wroga do listy
                EnemyInfo info = AddEnemy(actor);
                if (info != null)
                {
                    //                RecognizeEnemy(info); // zapisujemy informacje, ze nieprzyjaciel "spowodowal" dzwiek
                    info.m_fLastKnownPosTime = GetTime();
                    info.m_vLastKnownPos = info.m_Reference.GetPosition();
                }
            }
            
            if ( (bMadeByEnemy) && (nNoiseType == Pawn._NOISE_MOTION) )
            {
                m_fLastHeardEnemyStepsTime = GetTime();
                m_fLastImpulseTime = m_fLastHeardEnemyStepsTime;
                m_vLastHeardEnemyStepsPos = Noisy.GetPosition();
                
                if (GetAlarmLevel() < _ALARM_LEVEL_MEDIUM)
                    SetAlarmLevel(_ALARM_LEVEL_MEDIUM);
                if ( TimePassed(m_fLastHeardEnemyStepsAcceptTime, _HEARD_STEPS_ACCEPT_TIME, GetTime()) )
                {
                    m_NoiseManager.Request(this, AIEvents._EV_NOISE_STEPS);
                    m_fLastHeardEnemyStepsAcceptTime = GetTime();
                }
                return;
            }
            
            // AI uslyszalo strzal w tym momencie
            if (nNoiseType == Pawn._NOISE_WEAPON_FIRING)
            {
                if (bMadeByEnemy)
                {
                    m_fLastHeardEnemyShotTime = GetTime();
                    m_fLastImpulseTime = m_fLastHeardEnemyShotTime;
                    
                    m_vLastHeardEnemyShotPos = Noisy.GetPosition();
                    m_fLastEnemyShotDist = fNoiseLevel;
                    
                    if (GetAlarmLevel() < _ALARM_LEVEL_MEDIUM)
                        SetAlarmLevel(_ALARM_LEVEL_MEDIUM);
                    if ( TimePassed(m_fLastHeardEnemyShotAcceptTime, _HEARD_SHOOT_ACCEPT_TIME, GetTime()) )
                    {
                        m_NoiseManager.Request(this, AIEvents._EV_NOISE_ENEMY_SHOT);
                        m_fLastHeardEnemyShotAcceptTime = GetTime();
                    }
                    
                    if (GetMesh().DetectEnemyOnNoise())
                        EnemyPresenceStated();
                    return;
                }
                else
                {
                    m_fLastHeardFriendShotTime = GetTime();
                    m_vLastHeardFriendShotPos = Noisy.GetPosition();
                    m_vLastFriendsTargetPos = null;
                    if (Noisy instanceof NewAI)
                    {
                        ChromeActor friendsEnemy = ((NewAI)Noisy).GetController().GetActiveEnemy();
                        if (friendsEnemy != null)
                            m_vLastFriendsTargetPos = friendsEnemy.GetPosition();
                    }
                    
                    m_fLastImpulseTime = m_fLastHeardFriendShotTime;
                    
                    if (GetAlarmLevel() < _ALARM_LEVEL_MEDIUM)
                        SetAlarmLevel(_ALARM_LEVEL_MEDIUM);
                    if ( TimePassed(m_fLastHeardFriendShotAcceptTime, _HEARD_SHOOT_ACCEPT_TIME, GetTime()) )
                    {
                        m_NoiseManager.Request(this, AIEvents._EV_NOISE_FRIEND_SHOT);
                        m_fLastHeardFriendShotAcceptTime = GetTime();
                    }
                    return;
                }
            }
            
            
            if (nNoiseType == Pawn._NOISE_DANGEROUS_OBJECT_NEARBY)
            {
                if (Noisy instanceof Grenade)
                {
                    float fGrenOverAI = Noisy.GetPositionVectorVolatile().fY - GetMesh().GetPositionVectorVolatile().fY;
                    if (!bMadeByEnemy || fGrenOverAI > 400 || fGrenOverAI < 100)
                        return; // jesli przyjaciel rzuca granatem albo granat nizej niz AI, albo bardzo wysoko - ignoruj
                    if (m_MeshAI.bCanUseRadioCommunication)
                        m_MeshAI.SendRadioMessage(RadioCommunicationSystem._RADIO_MSG_GRENADE);
                }
                m_fLastImpulseTime = GetTime();
                m_fLastHeardDObjectTime = m_fLastImpulseTime;

                m_fLastHeardEnemyStepsTime = m_fLastImpulseTime;
                m_vLastHeardEnemyStepsPos = Noisy.GetPosition();

                if (GetAlarmLevel() < _ALARM_LEVEL_MEDIUM)
                    SetAlarmLevel(_ALARM_LEVEL_MEDIUM);
                
                if (Noisy instanceof Vehicle)
                {
                    Vehicle vehicle = (Vehicle) Noisy;
                    
                    // sprawdzamy jeszcze, czy nie znajdujemy sie daleko od samochodu - wtedy nie uciekamy
                    Vector vFromJeepToAI = vehicle.GetForwardVector();
                    vFromJeepToAI.Mul(-500f);
                    vFromJeepToAI.Add(m_vLastHeardEnemyStepsPos);
                    vFromJeepToAI.Negate();
                    vFromJeepToAI.Add(GetMesh().GetPositionVectorVolatile());
                    vFromJeepToAI.fY = 0f;
                    vFromJeepToAI.Normalize();
                    
                    Vector vVehicleForward = vehicle.GetForwardVectorVolatile();
                    vVehicleForward.fY = 0f;
                    vVehicleForward.Normalize();
                    
                    if (vFromJeepToAI.Dot(vVehicleForward) < m_fVehicleNoiseConeDot)
                        return;
                }
                
                if ( TimePassed(m_fLastHeardDangerousAcceptTime, _HEARD_DANGEROUS_ACCEPT_TIME, m_fLastImpulseTime) )
                {
                    m_NoiseManager.Request(this, AIEvents._EV_DANGEROUS_OBJECT);
                    m_fLastHeardDangerousAcceptTime = m_fLastImpulseTime;
                    m_LastDangerousObject = (ChromeActor) Noisy;
                }
                
                return;
            }
        }
    }
    
    /** notyfikacja o zmianie wektora obiektow */
    public void ObjectsListMsg(ControlObject [] aObjs)
    {
        // wypelniamy odpowiednie listy obiektami
        RedistributeObjects(aObjs);
    }
    
    public void UpdateObjectsInformation()
    {
        // sprawdzamy, czy nie zauwazylismy umierajacego przyjaciela lub neutrala
        CheckForAnyDying();
        // sortujemy wektor przeciwnikow po priorytecie
        RecalculateEnemiesVector();
        // notyfikujemy kontroler o prawdopodobnej zmianie wektorow obiektow
        CheckObjectSenseEvent();
    }
    
    /** notyfikacja o otrzymaniu postrzalu */
    public void PainMsg(float fDamage, float fPhSimDamage, GameObject Assassin,Vector vHitLocation,
                        Vector vSpeed, int nMeshElementorBoneID, int nDamageType)
    {
        m_fLastDamageTaken = fDamage;
        if (!IsDead() && Assassin instanceof ChromeActor)
        {
            ChromeActor actor = (ChromeActor) Assassin;
            
            if (!m_MeshAI.IsEnemy(actor))
            {
                Log("Przestan do mnie strzelac, idioto !!!\n");
            }
            else
            {
                EnemyInfo info = AddEnemy(actor);
                if (info == null)
                    return;
                
                MarkAsRecognized(info);
                GotShotFromEnemy(info, fDamage);
            }
        }
    }
    
    
    /** Notyfikacja o kolizji z obiektem */
    public void CollisionMsg(ControlObject obstacle, Vector point, Vector normal)
    {
    }

    
    /** wrzucenie przekazanych przez zmysl ObjectsSense obiektow */
    void RedistributeObjects(ControlObject [] aObjs)
    {
    }
    
    EnemyInfo AddEnemy(ChromeActor actor)
    {
        if (actor == null)
            return null;
        if (actor instanceof Grenade)
            return null;
        
        EnemyInfo info = (EnemyInfo) m_aEnemies.get(actor);
        if (info == null)
        {
            info = new EnemyInfo(actor);
            m_aEnemies.put(actor, info); // dodajemy obiekt do wrogow
            if (actor.m_bRecognizedAsPlayer)
                m_RecognitionListPlayer.add(actor);
            else
                m_RecognitionListNonPlayer.add(actor);
        }
        return info;
    }
    
    EnemyInfo AddEnemy(ChromeActor actor, boolean bEditorEnemy)
    {
        EnemyInfo info = AddEnemy(actor);
        
        if (info != null)
            info.m_bEditorEnemy = bEditorEnemy;
        return info;
    }
    

    void MarkAsRecognized(EnemyInfo info)
    {
        info.m_bRecognized = true;
        
        //Zaznacz, ze rozpoznawnie zakonczone
        info.m_fRecognitionStartTime = -1; 
        info.m_fRecognitionStopTime = GetTime();
        
        if (info == m_cSuspectInfo) // rozpoznany nie jest juz podejrzany ;-)
            m_cSuspectInfo = null;
    }
    
    void RecognizeEnemy(EnemyInfo info)
    {
        MarkAsRecognized(info);
        
        if (info.m_Reference.IsActiveOnModule())
        {
            info.m_fLastKnownPosTime = GetTime();
            info.m_vLastKnownPos = info.m_Reference.GetPosition();
        }
    }
    
    void RemoveEnemy(ChromeActor actor)
    {
        m_aEnemies.remove(actor);
        int iInd = m_RecognitionListPlayer.indexOf(actor);
        if (iInd != -1)
            m_RecognitionListPlayer.remove(iInd);
        else
        {
            iInd = m_RecognitionListNonPlayer.indexOf(actor);
            if (iInd != -1)
                m_RecognitionListNonPlayer.remove(iInd);
        }
    }
    
    void RemoveAllEditorEnemies(boolean bReallyRemove)
    {
        Set enemiesSet = m_aEnemies.keySet();
        Iterator enemiesIter = enemiesSet.iterator();
        while (enemiesIter.hasNext())
        {
            ChromeActor enemy = (ChromeActor)enemiesIter.next();
            EnemyInfo info = GetEnemy(enemy);
            if (info != null && 
                info.m_bEditorEnemy)
            {
                if (bReallyRemove)
                    RemoveEnemy(enemy);
                else
                    info.m_bEditorEnemy = false;
            }
        }
    }
    
    EnemyInfo GetEnemy(ChromeActor actor)
    {
        EnemyInfo info = (EnemyInfo) m_aEnemies.get(actor);
        return info;
    }
    
    /** przesianie obiektow - dodanie ich do odpowiedniej
     * listy (wrogowie, przyjaciele, inne obiekty) */
    void AddToProperList(ChromeActor actor, float fTime)
    {
        if ( m_MeshAI.IsFriend(actor) || m_MeshAI.IsNeutral(actor) )
        {
            // jezeli nie ma przyjaciela wsrod przyjaciol
            FriendInfo info = (FriendInfo) m_aAllies.get(actor);
            if (info == null)
            {
                info = new FriendInfo(actor);
                m_aAllies.put(actor, info); // dodajemy obiekt do przyjaciol
            }
            RemoveEnemy(actor);
        }
        else
        {
            AddEnemy(actor);
            m_aAllies.remove(actor); // usuwamy wroga sposrod przyjaciol
        }
    }

    public void UpdateNpcEnemiesRecognition()
    {
        UpdateEnemiesRecognition(m_NonPlayerListInfo);
    }

    public void UpdatePlayerEnemiesRecognition()
    {
        UpdateEnemiesRecognition(m_PlayerListInfo);
    }
    
    void UpdateEnemiesRecognition(RecognitionListInfo listInfo)
    {
        GetMesh().ProfileBegin(NewAI.PROFILE_Recognition);

        ObjectsSense senseObjects = (ObjectsSense)m_ObjectsSense;
        if (senseObjects == null)
            return;
        
        int iListSize = listInfo.m_List.size();
        if (iListSize == 0)
            return;

        float fTime = GetTime();
        EnemyInfo info;
        float fTimeBtwTraceCheck = GetTimeBtwTraceCheck();

        //normalizacja, bo mogla sie zmienic dlugosc listy
        if (listInfo.m_iListNdx >= iListSize)
            listInfo.m_iListNdx = 0;
        int iLastNdx = listInfo.m_iListNdx;
        
        do
        {
            info = GetEnemy((ChromeActor) listInfo.m_List.get(listInfo.m_iListNdx));
            
            if ( (!info.m_Reference.bDead) && 
                 info.m_Reference.IsActiveOnModule() &&
                 (fTime - info.m_fLastTraceTimeCheck > fTimeBtwTraceCheck) )
            {
                if (!info.m_bRecognized)
                {
                    boolean bRecognized = false;
                    if (info.m_Reference == m_Player) // playera traktujemy specjalnie
                    {
                        if (CanRecognize(info, senseObjects._AI_RECOGNIZE_RANGE))
                            bRecognized = true;
                        else // daj sobie czas na rozpoznanie ;-)
                        {
                            if (CanRecognize(info, senseObjects._AI_SIGHT_RANGE))
                            {
                                m_cSuspectInfo = info;
                                // widzi go dobrze, zobacz, co z czasem
                                if (info.m_fRecognitionStartTime > 0) // juz zaczal rozpoznawac
                                {
                                    float fMaxTime = (GetAlarmLevel() <= _ALARM_LEVEL_LOW) ? 
                                                     fMaxEnemyRecognitionTime :
                                                     fMaxEnemyRecognitionTimeAlarmed;
                                    float fRecognitionTime = fMaxTime *
                                                             Tools.InterpLinear(senseObjects._AI_RECOGNIZE_RANGE2, 
                                                                                senseObjects._AI_SIGHT_RANGE2,
                                                                                info.m_Reference.GetDistance2To(GetMesh()));

                                    if (TimePassed(info.m_fRecognitionStartTime, fRecognitionTime, fTime))
                                        bRecognized = true;
                                }
                                else
                                {
                                    info.m_fRecognitionStartTime = fTime; // zaznacz czas rozpoczecia rozpoznawania
                                    info.m_fRecognitionStopTime = fTime; // 'wykasuj' czas zakonczenia rozpoznawania
                                    CallMethodIn("CallRequestEvent", 0); // przegererowanie scenariuszy
                                }
                            }
                            else if (info.m_fRecognitionStartTime > 0)
                            {
                                info.m_fRecognitionStartTime = -1; // zaznacz, ze rozpoznawanie przerwane
                                info.m_fRecognitionStopTime = fTime;
                                CallMethodIn("CallRequestEvent", 0); // przegererowanie scenariuszy
                            }
                        }
                    }
                    else // zwykle obiekty rozpoznajemy od razu
                    {
                        if (CanRecognize(info, senseObjects._AI_SIGHT_RANGE))
                            bRecognized = true;
                    }
                    
                    if (bRecognized)
                    {
                        MarkAsRecognized(info);
                        m_fLastEnemySpottedTime = fTime;
                        EnemyPresenceStated();
                    }
                    listInfo.m_iListNdx++;
                    break;
                }
            }
            
            //przejscie na nastepny element do sprawdzenia
            if (++listInfo.m_iListNdx >= iListSize)
                listInfo.m_iListNdx = 0;
        } while (listInfo.m_iListNdx != iLastNdx);
        GetMesh().ProfileEnd(NewAI.PROFILE_Recognition);
    }
    
    
    boolean IsSuspectValid()
    {
        return m_cSuspectInfo != null &&
            (m_cSuspectInfo.m_fRecognitionStartTime > 0 || // w trakcie rozpoznawania
             m_cSuspectInfo.m_fRecognitionStopTime < 0 ||
             !TimePassed(m_cSuspectInfo.m_fRecognitionStopTime, 
                         _SUSPECT_HOLDING_TIME, 
                         GetTime()));
    }
    
    /** Oblicza zakres widocznosci podanego obiektu
     */
    protected float CalcVisibilityRangeForObject(float fBaseSightRange, ChromeActor cTarget)
    {
        ChromeModuleSingle module = (ChromeModuleSingle)GetModule();
        float fMaxDistanceLvl = Math.min(fBaseSightRange * 
                                         module.GetVisibilityRangeFactor(),
                                         module.GetVisibilityRangeThreshold()),
            fMaxDistanceTgt = Math.min(fBaseSightRange * 
                                       cTarget.GetVisibilityRangeFactor(GetMesh()),
                                       cTarget.GetVisibilityRangeThreshold(GetMesh())),
            fMaxDistance = Math.min(fMaxDistanceLvl, fMaxDistanceTgt);
        return fMaxDistance;
    }
    
    
    boolean CanRecognize(EnemyInfo info, float fRecognitionDistance)
    {
        if (info.m_bRecognized)
            return true;
        
        ChromeActor target = (ChromeActor)info.m_Reference;
        
        float fDistance2 = m_MeshAI.GetDistance2To(target);
        float fVisRange = CalcVisibilityRangeForObject(fRecognitionDistance, target);
        if (fDistance2 < fVisRange * fVisRange)
        {
            // sprawdzamy, czy widzimy przeciwnika
            if (EnemyVisible(info) && EnemyCanBeTracedToRecognize(info))
                return true;
        }
        return false;
    }
    
    void CheckForAnyDying()
    {
        ObjectsSense senseObjects = (ObjectsSense) m_ObjectsSense;
        if (senseObjects == null)
            return;
        
        float fTime = GetTime();
        if (!TimePassed(m_fLastDyingCheckTime, _TIME_BTW_DYING_CHECK, fTime))
            return;
        
        m_fLastDyingCheckTime = fTime;
        
        m_aAlliesList.clear();
        m_aAlliesList.addAll(m_aAllies.values());
        for (int i = 0; i < m_aAlliesList.size(); i++)
        {
            FriendInfo friendInfo = (FriendInfo) m_aAlliesList.get(i);
            
            if (!friendInfo.m_Reference.IsActiveOnModule())
                continue;
            
            if (!friendInfo.m_Reference.bDead)
                continue;
            
            if (friendInfo.WasBodySpotted())
                continue;
            
            Vector vFriendPos = friendInfo.m_Reference.GetPositionVectorVolatile();
            Vector vThisPos = m_MeshAI.GetPositionVectorVolatile();
            if (vFriendPos.Distance2(vThisPos) > _MAX_DIST_TO_SEE_DEAD_BODY2)
                continue;
            
            vFriendPos.Sub(vThisPos);
            vFriendPos.Normalize();
            Vector vLookDir = GetMesh().GetLookDir();
            if (vLookDir.Dot(vFriendPos) < senseObjects._AI_SIGHT_ANGLE_COS)
                continue;
            
            if (TimePassed(friendInfo.m_fLastDeadBodyTraceTimeCheck, _TIME_BTW_DYING_CHECK, fTime))
            {
                friendInfo.m_fLastDeadBodyTraceTimeCheck = fTime;
                if (AICheckTraceAbsoluteVehicleCheck(friendInfo.m_Reference,
                                                     friendInfo.m_Reference.GetPointToLookAt(0),
                                                     GetMesh().GetLookFromPoint()))
                {
                    // zauwazylismy nowego trupa
                    friendInfo.m_fDeadBodySeeTime = fTime;
                    if (m_LastBodySpotted == null)
                        m_LastBodySpotted = friendInfo.m_Reference;
                    else
                        if ( (m_LastBodySpotted.IsActiveOnModule()) && (friendInfo.m_Reference.IsActiveOnModule()) &&
                             (GetMesh().GetDistance2To(m_LastBodySpotted) > GetMesh().GetDistance2To(friendInfo.m_Reference)) )
                            m_LastBodySpotted = friendInfo.m_Reference;
                    
                    if (GetAlarmLevel() < _ALARM_LEVEL_MEDIUM)
                        SetAlarmLevel(_ALARM_LEVEL_MEDIUM);
                    
                    ReactOnDeath(friendInfo.m_Reference, false);
                    
                    EnemyPresenceStated(); // gdzies jest wrog - zobaczylismy trupa
                }
            }
        }
    }
    
    public void ReactOnDeath(ChromeActor body, boolean bDestroyEnemy)
    {
        m_fLastHeardScreamTime = GetTime();
        m_fLastImpulseTime = m_fLastHeardScreamTime;
        
        m_LastDyingAISpotted = body;
        if (m_LastDyingAISpotted instanceof NewAI)
        {
            NewAI killed = (NewAI) m_LastDyingAISpotted;
            if ( (killed.m_Killer != null) && (GetMesh().IsEnemy(killed.m_Killer)) )
            {
                EnemyInfo info = AddEnemy(killed.m_Killer);
                if ( (info != null) && (killed.m_Killer.IsActiveOnModule() && bDestroyEnemy) )
                {
                    MarkAsRecognized(info);
                    info.m_vLastKnownPos = info.m_Reference.GetPosition();
                    info.m_fLastKnownPosTime = m_fLastHeardScreamTime;
                }
            }
        }
    }
    
    /** metoda update'ujaca tablice wrogow */
    void RecalculateEnemiesVector()
    {
        m_aPriorityEnemies.clear();
        m_aPriorityEnemies.addAll(m_aEnemies.values());
        RecalcEnemiesPriority();
        java.util.Collections.sort(m_aPriorityEnemies);
    }
    
    /** metoda wyliczajaca priorytet wrogow */
    void RecalcEnemiesPriority()
    {
        float fTime = GetTime();

        EnemyInfo enemyInfo = null;
        for (int i = 0; i < m_aPriorityEnemies.size(); i++)
        {
            enemyInfo = (EnemyInfo) m_aPriorityEnemies.get(i);
            enemyInfo.m_fPriority = 0f;
            
            if (!enemyInfo.m_Reference.IsActiveOnModule())
                continue;
            
            if (!GetMesh().IsEnemy(enemyInfo.m_Reference))
                continue;
            
            // jezeli nie zyje, to nie jest istotny
            if (enemyInfo.m_Reference.bDead)
                continue;
            
            // jezeli przeciwnik jest wrogiem edytorowym
            if (IsEditorEnemy(enemyInfo))
            {
                // ma duzy priorytet
                enemyInfo.m_fPriority += _EDITOR_ENEMY_WEIGHT;
                // znamy pozycje wroga, ktorego chcemy atakowac
                enemyInfo.m_vLastKnownPos = enemyInfo.m_Reference.GetPosition();
                enemyInfo.m_fLastKnownPosTime = fTime;
            }
            else
            {
                if (!enemyInfo.m_bRecognized)
                    continue;
            }
            
            // jezeli ma malo zycia, to go chcemy dobic
            if (enemyInfo.m_Reference.fHealth < enemyInfo.m_Reference.fMaxHealth *
                                                _LOW_HEALTH_COEFF)
                enemyInfo.m_fPriority += _LOW_HEALTH_WEIGHT;
            
            // staramy sie trzymac obecnego przeciwnika
            if (enemyInfo.m_Reference == m_cActiveEnemy)
                enemyInfo.m_fPriority += _ACTIVE_ENEMY_WEIGHT;
            
            ObjectsSense senseObjects = (ObjectsSense) m_ObjectsSense;
            if (senseObjects != null)
            {
                // priorytet w zaleznosci od odleglosci
                float fDist = enemyInfo.m_Reference.GetDistanceTo(m_MeshAI);
                float fDistanceCoeff = fDist / senseObjects._AI_SIGHT_RANGE;
                if (fDistanceCoeff < 0f)
                    fDistanceCoeff = 0f;
                if (fDistanceCoeff > 1f)
                    fDistanceCoeff = 1f;
                enemyInfo.m_fPriority += (1 - fDistanceCoeff) * _DISTANCE_WEIGHT;

                if (enemyInfo.m_Reference == GetActiveEnemy())
                {
                    // biezacego wroga trace'ujemy nawet, gdy go nie ma w stozku
                    if (EnemyCanBeTraced(enemyInfo))
                    {
                        // jezeli jest w stozku widzenia, to stwierdzamy, ze zauwazylismy wroga
                        if (EnemyVisible(enemyInfo))
                            EnemyPresenceStated();
                        
                        enemyInfo.m_fPriority += _VISIBILITY_WEIGHT;
                        enemyInfo.m_fLastKnownPosTime = fTime;
                        enemyInfo.m_vLastKnownPos = enemyInfo.m_Reference.GetPosition();
                    }
                }
                else
                {
                    if (EnemyReallyVisible(enemyInfo, fTime))
                    {
                        enemyInfo.m_fPriority += _VISIBILITY_WEIGHT;
                        EnemyPresenceStated();
                    }
                }
                
                if (TimePassed(enemyInfo.m_fLastKnownPosTime, _ENEMY_NOTICED_LONG_TIME, fTime))
                {
                    enemyInfo.m_fPriority = 0;
                    continue;
                }
            }
            
            // czy wystarczy maly kat, by sie obrocic w strone przeciwnika 
            if (enemyInfo.m_fLastAngleCos > _SHARP_ANGLE_TO_TARGET_COS)
                enemyInfo.m_fPriority += _SHARP_ANGLE_TO_TARGET_WEIGHT;
            
            // waznosc celu
            enemyInfo.m_fPriority += m_MeshAI.fTargetImportance /
                                     m_MeshAI.fMaxTargetImportance *
                                     _TARGET_IMPORTANCE_WEIGHT;
            
            // modyfikujemy w zaleznosci, czy nas atakuje
            enemyInfo.m_fPriority += enemyInfo.m_fDamageFromThis / GetMaxHealth() *
                                     _ENEMY_ATTACK_WEIGHT;
        }
        if ( (GetCurrWaypoint() != null) && (m_bPerformCurrActivity) &&
             (m_CurrActivity instanceof mt_Destroy) )
        {
            mt_Destroy destroy = (mt_Destroy) m_CurrActivity;
            boolean bAllDead = true;
            // jezeli mamy rozkaz destroy i zabilismy wszystkich w grupie
            for (int i = 0; i < destroy.allTargets.size(); i++)
            {
                if ( !((IDestroyTarget) destroy.allTargets.get(i)).IsDead() )
                {
                    bAllDead = false;
                    break;
                }
            }
            if (bAllDead)
                EditorActivityEnded();
        }
    }
    
    void CheckObjectSenseEvent()
    {
        // probujemy zmienic przeciwnika
        CheckEnemiesEvent();		

        // sprawdzamy, czy mamy sie uspokoic
        TryToReduceAlarmLevel();
    }
    
    /** sprawdza zdarzenia od wrogow */
    void CheckEnemiesEvent()
    {
        if ( (m_cActiveEnemy != null) && (GetMesh().IsEnemy(m_cActiveEnemy)) )
        {
            if (!m_cActiveEnemy.IsActiveOnModule())
            {
                TryToChangeEnemy();
                return;
            }
            
            ActiveEnemyVisibleFromGun();
            if (m_fLastFromGunVisibilityChangeTime == GetTime())
                RequestEvent(AIEvents._EV_ACTIVE_ENEMY_VISIBLE_FROM_GUN_CHANGE);
            
            // jezeli wrog jest niezywy, probujemy zmienic
            if (m_cActiveEnemy.bDead)
            {
                TryToChangeEnemy();
                return;
            }
            
            // sprawdzamy, czy nie mamy wroga o wiekszym priorytecie
            EnemyInfo cInfoMaxPriority = (EnemyInfo) m_aPriorityEnemies.get(0);
            EnemyInfo cInfoCurrent = (EnemyInfo) m_aEnemies.get(m_cActiveEnemy);
            
            if (cInfoCurrent.m_fPriority == 0)
            {
                TryToChangeEnemy();
                return;
            }
            
            if (cInfoCurrent.m_Reference != cInfoMaxPriority.m_Reference)
            {
                if (cInfoCurrent.m_fPriority < cInfoMaxPriority.m_fPriority)
                {
                    TryToChangeEnemy();
                    return;
                }
            }
        }
        else
            TryToChangeEnemy();
    }

    /** sprawdzamy, czy mozemy zmniejszyc poziom zaalarmowania */
    void TryToReduceAlarmLevel()
    {
        if (GetAlarmLevel() != _ALARM_LEVEL_MEDIUM)
            return;
        
        float fTime = GetTime();
        // jezeli nie mamy przeciwnika - zgubilismy go jakis czas temu
        if ( (GetActiveEnemy() == null) &&
             (TimePassed(m_fEnemyLostTime, _TIME_TO_CALM_DOWN_AFTER_ENEMY_LOST, fTime)) &&
             (TimePassed(m_fLastHeardEnemyShotTime, _TIME_TO_INVALIDATE_NOISE, fTime)) &&
             (TimePassed(m_fLastHeardFriendShotTime, _TIME_TO_INVALIDATE_NOISE, fTime)) &&
             (TimePassed(m_fLastHeardRicochetTime, _TIME_TO_INVALIDATE_NOISE, fTime)) &&
             (TimePassed(m_fLastHeardExplosionTime, _TIME_TO_INVALIDATE_NOISE, fTime)) &&
             (TimePassed(m_fLastHeardDObjectTime, _TIME_TO_INVALIDATE_NOISE, fTime)) &&
             (TimePassed(m_fLastHeardEnemyStepsTime, _TIME_TO_INVALIDATE_NOISE, fTime)) &&
             (TimePassed(m_fLastHeardScreamTime, _TIME_TO_INVALIDATE_NOISE, fTime))
            )
        {
            SetAlarmLevel(_ALARM_LEVEL_LOW);
        }
    }
    
    /** probujemy zmienic przeciwnika */
    void TryToChangeEnemy()
    {
        EnemyInfo cInfo = GetBestEnemy();
        if ( (cInfo != null) && (m_cActiveEnemy != cInfo.m_Reference) )
        {
            SetActiveEnemy(cInfo.m_Reference);
            RequestEvent(AIEvents._EV_ACTIVE_ENEMY_CHANGED);
            return;
        }

        if ( (cInfo == null) && (m_cActiveEnemy != null) &&
             GetMesh().bCanForgetAboutEnemy)
        {
            SetActiveEnemy(null);
            RequestEvent(AIEvents._EV_FREEDOM);
            return;
        }
    }
    
    /** zwraca najlepszego wroga (o najwyzszym priorytecie) */
    EnemyInfo GetBestEnemy()
    {
        if (m_aPriorityEnemies.size() > 0)
        {
            EnemyInfo enemy = (EnemyInfo) m_aPriorityEnemies.get(0);
            if (enemy.m_fPriority == 0)
                return null;
            else
                return enemy;
        }
        return null;
    }

    /** czy przeciwnik jest (byl przez jakis czas) w stozku widzenia */
    boolean EnemyVisible(EnemyInfo info)
    {
        if (info == null)
            return false;
        else
        {
            ObjectsSense objectSense = (ObjectsSense) m_ObjectsSense;
            if (objectSense == null)
                return false;
            
            float fTime = GetTime();
            if (fTime - info.m_fLastVisibleCheckTime > _TIME_BTW_ANGLE_CHECK)
            {
                info.m_fLastVisibleCheckTime = fTime;
                
                info.m_fLastAngleCos = AngleCosToObj(info.m_Reference);
                if (info.m_fLastAngleCos > objectSense._AI_SIGHT_ANGLE_COS)
                {
                    info.m_fLastVisibleTime = fTime;
                    return true;
                }
            }
            
            if (info.m_fLastAngleCos > objectSense._AI_SIGHT_ANGLE_COS)
                return true;
            else
                return false;
        }
    }

    void EnemyPresenceStated()
    {
        if (!m_bEnemyPresenceStated)
        {
            CallMethodIn("CallOnEnemyDetectedActions",
                         GetMesh().fDetectionDelay * ChromeGame.GetAIDetectionDelayDiffLevelMult());
            m_bEnemyPresenceStated = true;
        }
    }
    
    public void CallOnEnemyDetectedActions()
    {
        DisableCallMethod("CallOnEnemyDetectedActions");
        if (!GetMesh().bDead)
            GetMesh().InvokeActionChain(GetMesh().aOnEnemyDetected);
    }
    
    boolean ActiveEnemyReallyVisible()
    {
        if (GetActiveEnemy() != null)
            return EnemyReallyVisible(GetActiveEnemy(), GetTime());
        return false;
    }
    
    /** czy widac przeciwnika */
    boolean EnemyReallyVisible(ChromeActor actor, float fTime)
    {
        EnemyInfo info = (EnemyInfo) m_aEnemies.get(actor);
        if (info == null)
            return false;
        else 
            return EnemyReallyVisible(info, fTime);
    }
    
    /** czy widac przeciwnika */
    boolean EnemyReallyVisible(EnemyInfo info, float fTime)
    {
        if (info == null)
            return false;
        
        if (!info.m_Reference.IsActiveOnModule())
            return false;
        
        if (!EnemyVisible(info))
            return false;
        
        if (EnemyCanBeTraced(info))
        {
            info.m_fLastReallyVisibleTime = fTime;
            info.m_fLastKnownPosTime = fTime;
            info.m_vLastKnownPos = info.m_Reference.GetPosition();
            return true;
        }
        return false;
    }
    
    /** czy widac biezacego przeciwnika z broni */
    boolean ActiveEnemyVisibleFromGun()
    {
        if (GetActiveEnemy() == null)
            return false;
        
        float fTime = GetTime();
        if (TimePassed(m_fLastTraceFromGunTimeCheck, GetTimeBtwGunTraceCheck(), fTime))
        {
            m_fLastTraceFromGunTimeCheck = fTime;
            
            boolean bLastVisibility = m_bActiveEnemyTraceFromGun;
            m_bActiveEnemyTraceFromGun = AICheckTraceAbsoluteVehicleCheck(GetActiveEnemy(),
                                                                          GetActiveEnemy().GetPointToLookAt(0),
                                                                          GetMesh().GetVisibilityCheckFireOrigin());
            // jezeli podczas tego sprawdzenia zmienila sie widzialnosc wroga z guna
            if (m_bActiveEnemyTraceFromGun != bLastVisibility)
                m_fLastFromGunVisibilityChangeTime = fTime;
        }
        return m_bActiveEnemyTraceFromGun;
    }
    
    boolean ActiveEnemyCanBeTraced()
    {
        if (GetActiveEnemy() == null)
            return false;
        return EnemyCanBeTraced((EnemyInfo) m_aEnemies.get(GetActiveEnemy()));
    }
    
    boolean EnemyCanBeTraced(EnemyInfo info)
    {
        float fTime = GetTime();
        if (TimePassed(info.m_fLastTraceTimeCheck, GetTimeBtwTraceCheck(), fTime))
        {
            info.m_fLastTraceTimeCheck = fTime;
            if (AICheckTraceAbsoluteVehicleCheck(info.m_Reference,
                                                 info.m_Reference.GetPointToLookAt(0),
                                                 GetMesh().GetLookFromPoint()))
            {
                info.m_fLastTraceVisibilityTime = fTime;
                info.m_bTraceVisible = true;
            }
            else
                info.m_bTraceVisible = false;
        }
        return info.m_bTraceVisible;
    }
    
    boolean EnemyCanBeTracedToRecognize(EnemyInfo info)
    {
        float fTime = GetTime();
        if (TimePassed(info.m_fLastTraceTimeCheck, GetTimeBtwRecognize(), fTime))
        {
            info.m_fLastTraceTimeCheck = fTime;
            
            if (AICheckTraceAbsoluteVehicleCheck(info.m_Reference,
                                                 info.m_Reference.GetPointToLookAt(0),
                                                 GetMesh().GetLookFromPoint()))
            {
                info.m_fLastTraceVisibilityTime = fTime;
                info.m_bTraceVisible = true;
            }
            else
                info.m_bTraceVisible = false;
        }
        return info.m_bTraceVisible;
    }
    
    float GetTimeBtwRecognize()
    {
        if (m_Player == null)
        {
            CrashLog("GetTimeBtwRecognize() : Nie ma playera !!!\n");
            return _MAX_TIME_BTW_RECOGNIZE_TRACE_CHECK;
        }
        return _MAX_TIME_BTW_RECOGNIZE_TRACE_CHECK *
               Tools.InterpLinear(_MIN_DIST_TO_PLAYER_RECOGNIZE2, 
                                  _MAX_DIST_TO_PLAYER_RECOGNIZE2,
                                  GetMesh().GetDistance2To(m_Player));
    }
    
    /** cosinus kata pomiedzy wektorem patrzenia tego ai a wektorem
     * kierunku do obiektu obj */
    float AngleCosToObj(ControlObject obj)
    {
        Vector vObjDir = obj.GetPosition();
        vObjDir.Sub(GetMesh().GetPosition());
        //vObjDir.Normalize(); 
        Vector vLookDir = m_MeshAI.GetLookDir();
        //vLookDir.Normalize();
        //INFO: dot2D normalizuje 
        return MathUtils.dot2D(vLookDir, vObjDir);
    }
    
    /** czy obiekt jest w zasiegu wzroku (ale nie musi byc naprawde widoczny -
     * o tym decyduje dopiero trace) */
    boolean ObjInSightAngle(ControlObject obj)
    {
        ObjectsSense objectSense = (ObjectsSense) m_ObjectsSense;
        if (objectSense == null)
            return false;
        
        return AngleCosToObj(obj) > objectSense._AI_SIGHT_ANGLE_COS;
    }
    
    /** czas ostatniego udanego trace'a wroga     */
    float GetLastTraceVisibilityTime()
    {
        if (GetActiveEnemy() == null)
            return _INIT_TIME;
        
        EnemyInfo info = (EnemyInfo) m_aEnemies.get(GetActiveEnemy());
        return info.m_fLastTraceVisibilityTime;
    }

    /** czy minal "krotki czas" odkad stracilismy kontakt z wrogiem  */
    boolean ActiveEnemyLostMoreThanShort()
    {
        if (GetActiveEnemy() == null)
            return false;
        EnemyInfo enemyInfo = (EnemyInfo) m_aEnemies.get(GetActiveEnemy());
        if (TimePassed(enemyInfo.m_fLastKnownPosTime, GetEnemyNoticedShortTime(), GetTime()))
            return true;
        else
            return false;
    }
    
    /** czy minal "dlugi czas" odkad stracilismy kontakt z wrogiem  */
    boolean ActiveEnemyLostMoreThanLong()
    {
        if (GetActiveEnemy() == null)
            return false;
        EnemyInfo enemyInfo = (EnemyInfo) m_aEnemies.get(GetActiveEnemy());
        if (TimePassed(enemyInfo.m_fLastKnownPosTime, _ENEMY_NOTICED_LONG_TIME, GetTime()))
            return true;
        else
            return false;
    }	
    
    float GetEnemyNoticedShortTime()
    {
        if (GetActiveEnemy() != null)
        {
            ObjectsSense senseObjects = (ObjectsSense) m_ObjectsSense;
            
            return Tools.InterpLinear(0, senseObjects._AI_SIGHT_RANGE2, GetActiveEnemyDistance2()) *
                   (_ENEMY_NOTICED_SHORT_TIME_MAX - _ENEMY_NOTICED_SHORT_TIME_MIN) +
                   _ENEMY_NOTICED_SHORT_TIME_MIN;
        }
        else
            return _ENEMY_NOTICED_SHORT_TIME_DEFAULT;
    }

    /** czy minal "krotki czas" odkad stracilismy kontakt wzrokowy z biezacym wrogiem  */
    boolean ActiveEnemyVisibilityLostMoreThanLong()
    {
        if (GetActiveEnemy() == null)
            return false;
        EnemyInfo enemyInfo = (EnemyInfo) m_aEnemies.get(GetActiveEnemy());
        return TimePassed(enemyInfo.m_fLastTraceVisibilityTime,
                          _ENEMY_NOTICED_LONG_TIME, GetTime());
    }
    
    boolean ActiveEnemyVisibilityLostMoreThanShort()
    {
        if (GetActiveEnemy() == null)
            return false;
        EnemyInfo enemyInfo = (EnemyInfo) m_aEnemies.get(GetActiveEnemy());
        return TimePassed(enemyInfo.m_fLastTraceVisibilityTime,
                          GetEnemyNoticedShortTime(), GetTime());
    }
    
    boolean ActiveEnemyGunTraceLostMoreThanShort()
    {
        if (GetActiveEnemy() == null)
            return false;
        return (!m_bActiveEnemyTraceFromGun) &&
               TimePassed(m_fLastFromGunVisibilityChangeTime, GetEnemyNoticedShortTime(), GetTime());
    }
    
    /////////////////////////////////////////////////////////////////////
    // METODY ZWIAZANE Z BRONIA I STRZELANIEM
    Weapon GetActiveWeapon()
    {
        return m_MeshAI.ActiveWeapon;
    }
    
    int GetActiveWeaponAmmo()
    {
        if (GetActiveWeapon() == null)
            return 0;
        else
            return GetActiveWeapon().nAmmo;
    }
    
    int GetAmmoLeftForActiveWeapon()
    {
        return GetMesh().m_nAmmoForActiveWeapon;
    }
    
    
    /** wlaczenie strzelania */
    void SetCanFire(boolean bCanAim)
    {
        m_bCanFire = true;
        if (bCanAim)
            SetCanAim();
        else
            ResetCanAim();
    }
    
    /** wylaczenie strzelania */
    void ResetCanFire()
    {
        m_bCanFire = false;
        ResetCanAim();
    }	
    
    
    /** wlaczenie przycelowania */
    void SetCanAim()
    {
        m_bCanAim = true;
    }
    
    /** wylaczenie przycelowania */
    void ResetCanAim()
    {
        m_bCanAim = false;
    }
    
    void MessageWeapon(int nState)
    {
    }

    void TargetHitNotification(ChromeActor victim, int damageType)
    {
    }
    
    /** Handler wolany po zmianie broni przez AI  */
    public void ActiveWeaponChanged()
    {
        ComputeAccuracy(); 
    }
    
    /** Handler wolany po zmianie MotionState
     */
    public void MotionStateChanged()
    {
        ComputeAccuracy();
    }
    
    public void ComputeAccuracy()
    {
    }

    /** Funkcja zwracajaca aktualny rozrzut broni AI */
    public float GetAimingDeviation()
    {
        return 1.f - m_fCurrAccuracy;
    }
    
    /////////////////////////////////////////////////////////////////////
    // WYLICZANIE I POBIERANIE PARAMETROW ZWIAZANYCH ZE STANEM AI
    void CalculateMorale()
    {
        m_fMorale = 1f;
        if (GetHealth() < 0.3f * GetMaxHealth())
            m_fMorale = Math.max(m_fMorale, 0.5f);
        if (GetActiveWeaponAmmo() < 10)
            m_fMorale *= 0.8f;
        // TODO:
        // wrogowie dookola (kupa)
        // silni wrogowie
        // przyjaciele dookola
        // bron
        // niebezpieczne obiekty
    }
    
    int GetAlarmLevel()
    {
        return m_iCurrAlarmLevel;
    }
    
    void SetAlarmLevel(int iAlarmLevel)
    {
        if ( (m_iCurrAlarmLevel != _ALARM_LEVEL_LOW) && 
             (iAlarmLevel == _ALARM_LEVEL_LOW) )
        {
            m_fLastAlarmedTime =  GetTime();
        }
        
        if (iAlarmLevel == _ALARM_LEVEL_LOW)
            m_bEnemyPresenceStated = false;
        
        if ( (iAlarmLevel >= _ALARM_LEVEL_LOW) && (iAlarmLevel <= _ALARM_LEVEL_HIGH) )
            m_iCurrAlarmLevel = iAlarmLevel;
        #if HELPER_TUNE
        if (cHelperTune != null)
        cHelperTune.EnableRendering(m_iCurrAlarmLevel > _ALARM_LEVEL_LOW);
        #endif
    }
    
    boolean CanPlayCalmIdleAnims()
    {
        if (m_fLastAlarmedTime == _INIT_TIME)
            return true;
        else
            return (GetTime() - m_fLastAlarmedTime) > _TIME_TO_PLAY_CALM_IDLE_ANIM;
    }
    
    void ChangeAlarmLevel(int iValueToAdd)
    {
        //TODO: zmniejszac gdzies alarmlevel
        m_iCurrAlarmLevel += iValueToAdd;
        if (m_iCurrAlarmLevel < _ALARM_LEVEL_LOW)
            m_iCurrAlarmLevel = _ALARM_LEVEL_LOW;
        
        if (m_iCurrAlarmLevel > _ALARM_LEVEL_HIGH)
            m_iCurrAlarmLevel = _ALARM_LEVEL_HIGH;
    }

    /** czasy reakcji AI zalezne od stopnia zaalarmowania */	
    float GetReactTime()
    {
        return _REACTION_TIME[m_iCurrAlarmLevel];
    }
    
    /** ustalenie obiektu, za ktorym nalezy podazac */
    void SetObjectToFollow(ChromeActor object)
    {
        m_ObjectToFollow = object;
    }
    
    ChromeActor GetObjectToFollow()
    {
        return m_ObjectToFollow;
    }

    /** czy przed krotkim momentem slytszal odglos strzalu	 */
    public boolean JustHeardEnemyShot()
    {
        return (GetTime() - m_fLastHeardEnemyShotTime < _LONG_SOUND_IMPORTANCE_TIME);
    }
    
    public Vector GetLastHeardEnemyShotPos()
    {
        return m_vLastHeardEnemyShotPos;
    }
    
    /** czy ostatnio slyszal odglos stzralu */
    public boolean LastHeardEnemyShot()
    {
        return (GetTime() - m_fLastHeardEnemyShotTime < _VERY_LONG_SOUND_IMPORTANCE_TIME);
    }
    
    
    public boolean IsValidLastHeardEnemyShotPos()
    {
        return (GetTime() - m_fLastHeardEnemyShotTime < _TIME_TO_INVALIDATE_HEARD_SHOT);
    }
    
    public boolean JustHeardFriendShot()
    {
        return (GetTime() - m_fLastHeardFriendShotTime < _LONG_SOUND_IMPORTANCE_TIME);
    }

    public boolean IsValidLastHeardFriendShotPos()
    {
        return (GetTime() - m_fLastHeardFriendShotTime < _TIME_TO_INVALIDATE_HEARD_SHOT);
    }
    
    public boolean JustHeardRicochet()
    {
        return (!TimePassed(m_fLastHeardRicochetTime, _LONG_SOUND_IMPORTANCE_TIME, GetTime()));
    }
    
    public boolean JustHeardExplosion()
    {
        return (!TimePassed(m_fLastHeardExplosionTime, _LONG_SOUND_IMPORTANCE_TIME, GetTime()));
    }
    
    public boolean JustHeardDangerousObject()
    {
        return (GetTime() - m_fLastHeardDObjectTime < _SHORT_SOUND_IMPORTANCE_TIME);
    }
    
    public boolean JustHeardEnemySteps()
    {
        return (GetTime() - m_fLastHeardEnemyStepsTime < _LONG_SOUND_IMPORTANCE_TIME);
    }
    
    public boolean JustHeardScream()
    {
        return (GetTime() - m_fLastHeardScreamTime < _LONG_SOUND_IMPORTANCE_TIME);
    }
    
    public Vector GetLastDyingScreamPos()
    {
        if (m_LastDyingAISpotted == null)
            return null;
        
        return m_LastDyingAISpotted.GetPosition();
    }
    
    public ChromeActor GetLastDyingAISpotted()
    {
        return m_LastDyingAISpotted;
    }
    
    public boolean JustGotDamageFromEnemy()
    {
        return (GetTime() - m_fLastGotShotTime < _LONG_SOUND_IMPORTANCE_TIME);
    }
    
    public float GetLastRicochetMakerDist()
    {
        return m_fLastRicochetDist;
    }
    
    public Vector GetLastRicochetMakerPos()
    {
        return m_vLastHeardRicochetMakerPos;
    }
    
    public float GetLastExplosionMakerDist()
    {
        return m_fLastExplosionDist;
    }
    
    public Vector GetLastExplosionMakerPos()
    {
        return m_vLastHeardExplosionMakerPos;
    }
    
    public float GetLastEnemyShotMakerDist()
    {
        return m_fLastEnemyShotDist;
    }
    
    public boolean JustSpottedEnemyOnce()
    {
        float fTime = GetTime();
        boolean bRet = false;
        if (m_fLastEnemySpottedTime > _INIT_TIME)
        {
            bRet = fTime - m_fLastEnemySpottedTime < _SHORT_OBJECT_SPOTTED_TIME &&
                   fTime - m_fLastEnemySpottedReactionTime > _OBJECT_SPOTTED_REACTION_TIME;
            if (bRet)
                m_fLastEnemySpottedReactionTime = fTime;  
        }
        return bRet;
    }
    
    public boolean HeardScreamLately()
    {
        if (m_fLastEnemySpottedScreamHeardTime > _INIT_TIME)
            return !TimePassed(m_fLastEnemySpottedScreamHeardTime, _MIN_TIME_BTW_SUBSEQUENT_SCREAMS, GetTime());
        else
            return false;
    }
    
    void GotShotFromEnemy(EnemyInfo info, float fDamage)
    {
        info.m_fDamageFromThis += fDamage;
        info.m_fLastDamageFromThisTime = GetTime();
        m_fLastGotShotTime = info.m_fLastDamageFromThisTime;
        
        info.m_fLastKnownPosTime = info.m_fLastDamageFromThisTime;
        info.m_vLastKnownPos = info.m_Reference.GetPosition();

        SetAlarmLevel(_ALARM_LEVEL_HIGH);
        EnemyPresenceStated();
        RequestEvent(AIEvents._EV_PAIN);
    }
    
    /** czy jest daleko od aktywnego wroga 	 */
    boolean IsFarAwayFromActiveEnemy()
    {
        return GetMesh().GetDistanceTo(m_cActiveEnemy) > _FAR_AWAY_FROM_ENEMY_DIST;
    }
    /** czy jest blisko aktywnego wroga 	 */
    boolean IsNearToActiveEnemy()
    {
        return GetMesh().GetDistanceTo(m_cActiveEnemy) < _CLOSE_TO_ENEMY_DIST;		
    }	
    
    boolean IsVeryCloseToActiveEnemy()
    {
        return GetMesh().GetDistanceTo(m_cActiveEnemy) < _VERY_CLOSE_TO_ENEMY_DIST;
    }

    /** czy atakuje mnie wlasnie kilku wrogow */
    public boolean AttackedByManyEnemies()
    {
        float fTime = GetTime();
        if (!TimePassed(m_fAttackByManyEnemiesCheckTime, _TIME_BTW_MANY_ENEMIES_CHECK, fTime))
            return m_iAttackingEnemies >= _MANY_ATTACKING_ENEMIES;

        m_fAttackByManyEnemiesCheckTime = fTime;
        m_iAttackingEnemies = 0;
        EnemyInfo enemy = null;
        for (int i = 0; i < m_aPriorityEnemies.size(); i++)
        {
            enemy = (EnemyInfo) m_aPriorityEnemies.get(i);

            if (!TimePassed(enemy.m_fLastDamageFromThisTime, _ENEMY_ATTACK_IMPORTANCE_TIME, fTime))
                m_iAttackingEnemies++;
        }
        return m_iAttackingEnemies >= _MANY_ATTACKING_ENEMIES;
    }
    
    public boolean IsFighting()
    {
        //TODO
        return (GetActiveEnemy() != null);
    }
    
    public boolean IsMoraleLow()
    {
        //TODO
        return m_fMorale < 0.4f;
    }
    
    void SetActiveEnemy(ChromeActor cEnemy)
    {
        DisableCallMethod("SetLowAlarmLevel");
        if (cEnemy == null)
        {
            m_fEnemyLostTime = GetTime();
            if (GetAlarmLevel() >= _ALARM_LEVEL_HIGH)
                SetAlarmLevel(_ALARM_LEVEL_MEDIUM);
        }
        else
            SetAlarmLevel(_ALARM_LEVEL_HIGH);
        
        m_cActiveEnemy = cEnemy;
    }
    
    ChromeActor GetActiveEnemy()
    {
        return m_cActiveEnemy;
    }
    
    
    float GetActiveEnemyDistance2()
    {
        if (m_cActiveEnemy != null)
            return GetMesh().GetDistance2To(m_cActiveEnemy);
        else
            return -1;
    }
    
    float GetTimeBtwTraceCheck()
    {
        if (m_cActiveEnemy != null)
        {
            ObjectsSense senseObjects = (ObjectsSense) m_ObjectsSense;
            return Tools.InterpLinear(0, senseObjects._AI_SIGHT_RANGE2, GetActiveEnemyDistance2()) *
                   (_MAX_TIME_BTW_TRACE_CHECK - _MIN_TIME_BTW_TRACE_CHECK) +
                   _MIN_TIME_BTW_TRACE_CHECK;
        }
        else
            return _MAX_TIME_BTW_TRACE_CHECK;
    }

    float GetTimeBtwGunTraceCheck()
    {
        if (m_cActiveEnemy != null)
        {
            ObjectsSense senseObjects = (ObjectsSense) m_ObjectsSense;
            return Tools.InterpLinear(0, senseObjects._AI_SIGHT_RANGE2, GetActiveEnemyDistance2()) *
                   (_MAX_TIME_BTW_GUN_TRACE_CHECK - _MIN_TIME_BTW_GUN_TRACE_CHECK) +
                   _MIN_TIME_BTW_GUN_TRACE_CHECK;
        }
        else
            return _MAX_TIME_BTW_GUN_TRACE_CHECK;
    }
    
    /** czy biezacy przeciwnik jest silniejszy */
    public boolean EnemyStronger()
    {
        if (GetActiveEnemy() == null)
            return false;

        if ((GetHealth() < GetMaxHealth() * 0.2f) && 
            (m_fLastDamageTaken < GetMaxHealth() - 
                                  GetHealth())) // zeby nie uciekal gdy dostanie jeden silny postrzal
            return true;
        return false;
    }
    
    public Vector GetLastEnemyKnownPos()
    {
        if (GetActiveEnemy() == null)
            return null;
        
        EnemyInfo info = (EnemyInfo) m_aEnemies.get(GetActiveEnemy());
        return info.m_vLastKnownPos;
    }
    
    public ChromeActor GetLastDeadBodySpotted()
    {
        return m_LastBodySpotted;
    }
    
    public FriendInfo GetDeadBodyFriendInfo()
    {
        if (m_LastBodySpotted == null)
            return null;
        return (FriendInfo) m_aAllies.get(m_LastBodySpotted);
    }
    
    public Vector GetLastDeadBodySpottedPos()
    {
        if (m_LastBodySpotted == null)
            return null;
        return m_LastBodySpotted.GetPosition();
    }
    
    public int Grenades()
    {
        if (GetMesh().m_nGrenadesLeft >= 0)
            return GetMesh().m_nGrenadesLeft;
        else
            return Integer.MAX_VALUE;
    }
    
    public boolean CanThrowGrenade()
    {
        return false;
    }

    float GetHealth()
    {
        return m_MeshAI.fHealth;
    }
    
    float GetMaxHealth()
    {
        return m_MeshAI.fMaxHealth;
    }
    
    /** zwraca jednostkowy wektor w kierunku przecinym do zadanego obiektu */
    Vector GetOppositeDir(ControlObject cObject)
    {
        Vector vDir = GetMesh().GetPosition();
        vDir.Sub(cObject.GetPositionVectorVolatile());
        vDir.Normalize();
        return vDir;
    }
    
    boolean HasAmmo()
    {   
        return (GetActiveWeaponAmmo() > 0);
    }
    
    boolean CanShoot()
    {
        return ( (GetActiveWeapon() != null) && (HasAmmo()) );   
    }
    
    boolean CanReload()
    {
        return (GetMesh().m_nAmmoForActiveWeapon > 0);
    }
    
    void ReloadWeapon()
    {
        GetMesh().WeaponReload();
    }
    
    //pt:. -------------   TESTY --------------------------------------------------
    void ShowScenarios()
    {
        for(int i = 0; i < Scenario._NUM_OF_SCENARIOS; i++)
        {
            GameObject.LogLine(" Sc: - "+ m_aScenarios[i].GetID());           
        }
    }    
    
    /** pokazuje przeciwnikow w poblizu	*/
    void ShowEnemies()
    {
        if(m_aPriorityEnemies==null)
            return;
        LogLine(" Show enemies: ");	
        for(int i=0; i<m_aPriorityEnemies.size(); i++)
        {
            EnemyInfo cInfo = (EnemyInfo)m_aPriorityEnemies.get(i);
            LogLine("   enem " + cInfo.m_Reference + " vis "+ObjInSightAngle(cInfo.m_Reference));	
        }
    }
    
    /** pokazuje liste taskow */
    String TaskListtoString()
    {
        return "<curr: "+m_CurrentTask+"> "+m_TaskList;
    }
    
    void Debug(boolean bEnable)
    {
        DebugUtils.m_bDebug = bEnable;
    }

    /////////////////////////////////////////////////////////////////////
    // Obsluga TriggerObjectow
    //
    protected TriggerObject m_CurrentTriggerObject = null;
    protected ExecutionInfo m_CurrentExecutionInfo = null;
    
    public void TriggerObjectNotification(TriggerObject trigger, ExecutionInfo currentExecutionInfo)
    {
        m_CurrentTriggerObject = trigger;
        m_CurrentExecutionInfo = currentExecutionInfo;
        Notify(AIEvents._NOTICE_TRIGGER_OBJECT_NOTIFICATION);        
    }
    
    public final TriggerObject GetCurrentTrigger()
    {
        return m_CurrentTriggerObject;
    }
    
    public final ExecutionInfo GetCurrentExecutionInfo()
    {
        return m_CurrentExecutionInfo;
    }
    
    
    static Vector l_vEnd = new Vector();
    boolean AICheckTraceAbsolute(ControlObject cTarget, Vector vStart, Vector vEnd)
    {
        l_vEnd.Set(vEnd);
        m_aTraceExcludeObjects[0] = cTarget;
        m_aTraceExcludeObjects[1] = m_MeshAI;
        return m_MeshAI.AITrace(vStart, l_vEnd, NewAI._AITRACE_IGNORE_GLASS, m_aTraceExcludeObjects) == null;
    }
    
    boolean AICheckTraceAbsoluteVehicleCheck(ControlObject cTarget, Vector vStart, Vector vEnd)
    {
        l_vEnd.Set(vEnd);
        m_aTraceExcludeObjects[0] = cTarget;
        m_aTraceExcludeObjects[1] = m_MeshAI;
        
        if (cTarget instanceof Vehicle)
        {
            Vehicle vehicle = (Vehicle) cTarget;
            vehicle.TurnOffPassengersCollisions();
            boolean bTraceReturn = (m_MeshAI.AITrace(vStart, l_vEnd, NewAI._AITRACE_IGNORE_GLASS, m_aTraceExcludeObjects) == null);
            vehicle.TurnOnPassengersCollisions();
            return bTraceReturn;
        }
        else
            return (m_MeshAI.AITrace(vStart, l_vEnd, NewAI._AITRACE_IGNORE_GLASS, m_aTraceExcludeObjects) == null);
    }

    /////////////////////////////////////////////////////////////////////
    /* INTERFEJS DO HIDEOUTOW */
    public abstract boolean CanAttackFromCurrentHideout();
    public abstract ChromeActor HideoutsNear();
    public abstract Vector GetDestHideoutPos();
    public abstract boolean IsHidingBehindHideout();
    public abstract boolean IsBehindHideout();
    public abstract boolean CanHitActiveEnemyFromHideout();
    
    //*********************************************************
    //					SaveGame
    //*********************************************************
    
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[AIController.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_AICONTROLLER)
        {
            super.SGLoadChunk(cFC);
            m_TaskList = (TasksList)cFC.LoadObjectRef();
            m_CurrentTask = (Task)cFC.LoadObjectRef();
            m_bPerformNextTaskOnUnfreeze = cFC.LoadBool();
            
            m_HearingSense = (IAIHearingSense)cFC.LoadObject((ISaveGame)m_HearingSense, this);
            m_PainSense = (IAIPainSense)cFC.LoadObject((ISaveGame)m_PainSense, this);
            m_ObjectsSense = (ObjectsSense)cFC.LoadObject((ISaveGame)m_ObjectsSense, this);

            m_nCurrScenario = cFC.LoadInt();
            m_nLastPreparedScenario = cFC.LoadInt();
            m_nHelperScenariosInt = cFC.LoadInt();
            m_nLastEvent = cFC.LoadInt();
            
            m_aAllies.clear();
            int size = cFC.LoadInt();
            FriendInfo friendInfo;
            for (int i = 0; i < size; i++)
            {
                friendInfo = (FriendInfo)cFC.LoadObjectRef();
                m_aAllies.put(friendInfo.m_Reference,
                              friendInfo);
            }
            
            m_aEnemies.clear();
            size = cFC.LoadInt();
            EnemyInfo enemyInfo;
            for (int i = 0; i < size; i++)
            {
                enemyInfo = (EnemyInfo)cFC.LoadObjectRef();
                m_aEnemies.put(enemyInfo.m_Reference,
                               enemyInfo);
            }

            m_RecognitionListNonPlayer.clear();
            size = cFC.LoadInt();
            for (int i = 0; i < size; i++)
            {
                ChromeActor actor = (ChromeActor)cFC.LoadObjectRef();
                m_RecognitionListNonPlayer.add(actor);
            }

            m_RecognitionListPlayer.clear();
            size = cFC.LoadInt();
            for (int i = 0; i < size; i++)
            {
                ChromeActor actor = (ChromeActor)cFC.LoadObjectRef();
                m_RecognitionListPlayer.add(actor);
            }

            
            m_bCanFire = cFC.LoadBool();
            m_bCanAim = cFC.LoadBool();
            m_fCurrAccuracy = cFC.LoadFloat();
            
            m_cActiveEnemy = (ChromeActor)cFC.LoadObjectRef();
            m_cSuspectInfo = (EnemyInfo)cFC.LoadObjectRef();
            
            m_fLastHeardEnemyShotAcceptTime = cFC.LoadFloat();
            m_fLastHeardFriendShotAcceptTime = cFC.LoadFloat();
            m_fLastHeardBulletAcceptTime = cFC.LoadFloat();
            m_fLastHeardExplosionAcceptTime = cFC.LoadFloat();
            m_fLastHeardEnemyStepsAcceptTime = cFC.LoadFloat();
            m_fLastHeardScreamAcceptTime = cFC.LoadFloat();
            m_fLastHeardDangerousAcceptTime = cFC.LoadFloat();
            m_fLastDyingCheckTime = cFC.LoadFloat();

            m_LastBodySpotted = (ChromeActor)cFC.LoadObjectRef();

            m_fLastHeardEnemyShotTime = cFC.LoadFloat();
            m_fLastHeardFriendShotTime = cFC.LoadFloat();
            m_fLastHeardRicochetTime = cFC.LoadFloat();
            m_fLastHeardExplosionTime = cFC.LoadFloat();
            m_fLastHeardDObjectTime = cFC.LoadFloat();
            m_fLastHeardEnemyStepsTime = cFC.LoadFloat();
            m_vLastHeardEnemyStepsPos = cFC.LoadVector();
            m_fLastHeardScreamTime = cFC.LoadFloat();
            m_fLastGotShotTime = cFC.LoadFloat();
            m_vLastHeardEnemyShotPos = cFC.LoadVector();
            m_fLastEnemyShotDist = cFC.LoadFloat();
            m_vLastHeardFriendShotPos = cFC.LoadVector();
            m_vLastFriendsTargetPos = cFC.LoadVector();
            
            m_LastDyingAISpotted = (ChromeActor)cFC.LoadObjectRef();
            m_LastDangerousObject = (ChromeActor)cFC.LoadObjectRef();

            m_fMorale = cFC.LoadFloat(); 
            m_iCurrAlarmLevel = cFC.LoadInt();
            m_fLastAlarmedTime = cFC.LoadFloat(); 
            m_ObjectToFollow = (ChromeActor)cFC.LoadObjectRef();
            m_fAttackByManyEnemiesCheckTime = cFC.LoadFloat(); 
            m_iAttackingEnemies = cFC.LoadInt();
            m_bIsFrozen = cFC.LoadBool();

            // z RecognitionListInfo serializujemy tylko indeksy
            m_NonPlayerListInfo.m_iListNdx = cFC.LoadInt();
            m_PlayerListInfo.m_iListNdx = cFC.LoadInt();

            m_bEnemyPresenceStated = cFC.LoadBool();
            m_fLastTraceFromGunTimeCheck = cFC.LoadFloat(); 
            m_fLastFromGunVisibilityChangeTime = cFC.LoadFloat(); 
            m_bActiveEnemyTraceFromGun = cFC.LoadBool();
            m_vLastHeardRicochetMakerPos = cFC.LoadVector();
            m_vLastHeardExplosionMakerPos = cFC.LoadVector();

            m_fLastRicochetDist = cFC.LoadFloat(); 
            m_fLastExplosionDist = cFC.LoadFloat(); 
            m_fEnemyLostTime = cFC.LoadFloat(); 
            m_fLastImpulseTime = cFC.LoadFloat();
            
            m_fLastPassageNoiseTime = cFC.LoadFloat();
            m_fLastPassageEnemyNoiseTime = cFC.LoadFloat();
            m_vLastPassageEnemyNoisePos = cFC.LoadVector();


            m_CurrActivity = (IAIActivity)cFC.LoadObjectRef();
            m_CurrCondition = (Condition)cFC.LoadObjectRef();
            m_iCurrWaypointBehavior = cFC.LoadInt();
            m_bPerformCurrActivity = cFC.LoadBool();
            m_bActivityInWaypoint = cFC.LoadBool();

            m_CurrFightPoint = (FightPoint)cFC.LoadObjectRef();
            m_fFightPointLeaveTime = cFC.LoadFloat(); 

            m_fLastEnemySpottedTime = cFC.LoadFloat(); 
            m_fLastEnemySpottedReactionTime = cFC.LoadFloat(); 
            m_fLastEnemySpottedScreamHeardTime = cFC.LoadFloat(); 
            m_fLastHearNoiseTraceCheckTime = cFC.LoadFloat();
            m_bDead = cFC.LoadBool();

            m_CurrentTriggerObject = (TriggerObject)cFC.LoadObjectRef();
            m_CurrentExecutionInfo = (ExecutionInfo)cFC.LoadObjectRef();
            
            m_fLastDamageTaken = cFC.LoadFloat();
            
            m_bSitsInVehicleOnSeat = cFC.LoadBool();
        }
        
        cFC.delete();
    }
    

    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[AIController.SGSaveChunk]\n");
        
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_AICONTROLLER, cFCParent);
        
        super.SGSaveChunk(cFC);
        cFC.SaveObjectRef(m_TaskList);
        cFC.SaveObjectRef(m_CurrentTask);
        cFC.SaveBool(m_bPerformNextTaskOnUnfreeze);
        
        cFC.SaveObject((ISaveGame)m_HearingSense);
        cFC.SaveObject((ISaveGame)m_PainSense);
        cFC.SaveObject((ISaveGame)m_ObjectsSense);

        cFC.SaveInt(m_nCurrScenario);
        cFC.SaveInt(m_nLastPreparedScenario);
        cFC.SaveInt(m_nHelperScenariosInt);
        cFC.SaveInt(m_nLastEvent);
        
        cFC.SaveInt(m_aAllies.size());
        Collection alliesCollection = m_aAllies.values();
        Iterator alliesIterator = alliesCollection.iterator();
        while (alliesIterator.hasNext())
            cFC.SaveObjectRef((FriendInfo)alliesIterator.next());

        cFC.SaveInt(m_aEnemies.size());
        Collection enemiesCollection = m_aEnemies.values();
        Iterator enemiesIterator = enemiesCollection.iterator();
        while (enemiesIterator.hasNext())
            cFC.SaveObjectRef((EnemyInfo)enemiesIterator.next());
		
		if (m_RecognitionListNonPlayer == null)
			GameObject.CrashLog("[AIController.SGSaveChunk] SG-ERROR!!! m_RecognitionListNonPlayer = null\n");        
        cFC.SaveInt(m_RecognitionListNonPlayer.size());
        for (int i = 0; i < m_RecognitionListNonPlayer.size(); i++)
		{
			if (m_RecognitionListNonPlayer == null)
				GameObject.CrashLog("[AIController.SGSaveChunk] SG-ERROR!!! m_RecognitionListNonPlayer = null\n");			
            cFC.SaveObjectRef((ChromeActor)m_RecognitionListNonPlayer.get(i));
		}

		if (m_RecognitionListPlayer == null)
			GameObject.CrashLog("[AIController.SGSaveChunk] SG-ERROR!!! m_RecognitionListPlayer = null\n");						
		
        cFC.SaveInt(m_RecognitionListPlayer.size());
        for (int i = 0; i < m_RecognitionListPlayer.size(); i++)
		{
			if (m_RecognitionListPlayer == null)
				GameObject.CrashLog("[AIController.SGSaveChunk] SG-ERROR!!! m_RecognitionListPlayer = null\n");						
            cFC.SaveObjectRef((ChromeActor)m_RecognitionListPlayer.get(i));
		}
        
        cFC.SaveBool(m_bCanFire);
        cFC.SaveBool(m_bCanAim);
        cFC.SaveFloat(m_fCurrAccuracy);
        
        cFC.SaveObjectRef(m_cActiveEnemy);
        cFC.SaveObjectRef(m_cSuspectInfo);

        cFC.SaveFloat(m_fLastHeardEnemyShotAcceptTime);
        cFC.SaveFloat(m_fLastHeardFriendShotAcceptTime);
        cFC.SaveFloat(m_fLastHeardBulletAcceptTime);
        cFC.SaveFloat(m_fLastHeardExplosionAcceptTime);
        cFC.SaveFloat(m_fLastHeardEnemyStepsAcceptTime);
        cFC.SaveFloat(m_fLastHeardScreamAcceptTime);
        cFC.SaveFloat(m_fLastHeardDangerousAcceptTime);
        cFC.SaveFloat(m_fLastDyingCheckTime);

        cFC.SaveObjectRef(m_LastBodySpotted);

        cFC.SaveFloat(m_fLastHeardEnemyShotTime);
        cFC.SaveFloat(m_fLastHeardFriendShotTime);
        cFC.SaveFloat(m_fLastHeardRicochetTime);
        cFC.SaveFloat(m_fLastHeardExplosionTime);
        cFC.SaveFloat(m_fLastHeardDObjectTime);
        cFC.SaveFloat(m_fLastHeardEnemyStepsTime);
        cFC.SaveVector(m_vLastHeardEnemyStepsPos);
        cFC.SaveFloat(m_fLastHeardScreamTime);
        cFC.SaveFloat(m_fLastGotShotTime);
        cFC.SaveVector(m_vLastHeardEnemyShotPos);
        cFC.SaveFloat(m_fLastEnemyShotDist);
        cFC.SaveVector(m_vLastHeardFriendShotPos);
        cFC.SaveVector(m_vLastFriendsTargetPos);
        
        cFC.SaveObjectRef(m_LastDyingAISpotted);
        cFC.SaveObjectRef(m_LastDangerousObject);

        cFC.SaveFloat(m_fMorale); 
        cFC.SaveInt(m_iCurrAlarmLevel);
        cFC.SaveFloat(m_fLastAlarmedTime); 
        cFC.SaveObjectRef(m_ObjectToFollow);
        cFC.SaveFloat(m_fAttackByManyEnemiesCheckTime); 
        cFC.SaveInt(m_iAttackingEnemies);
        cFC.SaveBool(m_bIsFrozen);

        // z RecognitionListInfo serializujemy tylko indeksy
        cFC.SaveInt(m_NonPlayerListInfo.m_iListNdx);
        cFC.SaveInt(m_PlayerListInfo.m_iListNdx);

        cFC.SaveBool(m_bEnemyPresenceStated);
        cFC.SaveFloat(m_fLastTraceFromGunTimeCheck); 
        cFC.SaveFloat(m_fLastFromGunVisibilityChangeTime); 
        cFC.SaveBool(m_bActiveEnemyTraceFromGun);
        cFC.SaveVector(m_vLastHeardRicochetMakerPos);
        cFC.SaveVector(m_vLastHeardExplosionMakerPos);

        cFC.SaveFloat(m_fLastRicochetDist); 
        cFC.SaveFloat(m_fLastExplosionDist); 
        cFC.SaveFloat(m_fEnemyLostTime); 
        cFC.SaveFloat(m_fLastImpulseTime); 
        cFC.SaveFloat(m_fLastPassageNoiseTime);
        cFC.SaveFloat(m_fLastPassageEnemyNoiseTime);
        cFC.SaveVector(m_vLastPassageEnemyNoisePos);

        
        
        cFC.SaveObjectRef((ISaveGame)m_CurrActivity);
        cFC.SaveObjectRef(m_CurrCondition);
        cFC.SaveInt(m_iCurrWaypointBehavior);
        cFC.SaveBool(m_bPerformCurrActivity);
        cFC.SaveBool(m_bActivityInWaypoint);

        cFC.SaveObjectRef(m_CurrFightPoint);
        cFC.SaveFloat(m_fFightPointLeaveTime); 

        cFC.SaveFloat(m_fLastEnemySpottedTime); 
        cFC.SaveFloat(m_fLastEnemySpottedReactionTime); 
        cFC.SaveFloat(m_fLastEnemySpottedScreamHeardTime);
        cFC.SaveFloat(m_fLastHearNoiseTraceCheckTime);
        
        cFC.SaveBool(m_bDead);

        cFC.SaveObjectRef(m_CurrentTriggerObject);
        cFC.SaveObjectRef(m_CurrentExecutionInfo);
        
        cFC.SaveFloat(m_fLastDamageTaken);

        cFC.SaveBool(m_bSitsInVehicleOnSeat);
        
        cFC.delete(); 
    }				
    
    public void SGUpdate()
    {
        FileChunk.Log("[AIController.SGUpdate]\n");
        super.SGUpdate();
        
        if (m_HearingSense != null)
        {
            m_HearingSense.Init(this);
            ((ISaveGame)m_HearingSense).SGUpdate();
        }
        if (m_PainSense != null)
        {
            m_PainSense.Init(this);
            ((ISaveGame)m_PainSense).SGUpdate();
        }
        if (m_ObjectsSense != null)
        {
            m_ObjectsSense.Init(this);
            ((ISaveGame)m_ObjectsSense).SGUpdate();
        }

        if (m_TaskList != null)
            m_TaskList.SetAIControllerInTasks(this);
        if (m_CurrentTask != null)
            m_CurrentTask.SetAIController(this);

        // pobierz playera, bo Start event wolane z opoznieniem...
        m_Player = ((ChromeModuleSingle) GetModule()).MainPlayer;
        
        if (m_aScenarios != null)
            for (int i = 0; i < m_aScenarios.length; i++)
                if (m_aScenarios[i] != null)
                    m_aScenarios[i].Initialize();

        RecalculateEnemiesVector();
        m_fLastPerformTaskTime = 0;
        m_nTasksPerCurrFrame = 0;
    }
}

/** informacja o przeciwniku */
class EnemyInfo implements Comparable, ISaveGame
{
    /** referencja do tego przeciwnika */
    ChromeActor m_Reference = null;
    /** priorytet tego wroga - im wieksza liczba tym priorytet wiekszy */
    float m_fPriority = 1f;
    
    /** czas, kiedy ostatnio znajdowal sie w okregu widzenia tego AI */
    float m_fLastInSightRangeTime = AIController._INIT_TIME;
    
    /** ostatni cos kata pomiedzy kierunkiem patrzenia ai a kierunkiem do wroga */
    float m_fLastAngleCos = 0;
    /** czas, kiedy ostatnio ten wrog byl w stozku widzenia */
    float m_fLastVisibleTime = AIController._INIT_TIME;
    /** kiedy ostatnio byl sprawdzany kat do wroga */
    float m_fLastVisibleCheckTime = AIController._INIT_TIME;
    
    /** czas, kiedy ostatnio byl widoczny przez trace */
    float m_fLastTraceVisibilityTime = AIController._INIT_TIME;
    /** czy ostatnio byl widziany (podczas ostatniego trace'a) */
    boolean m_bTraceVisible = false;
    /** czas, kiedy ostatnio byl trace'owany */
    float m_fLastTraceTimeCheck = 0f;
    
    /** czas, kiedy przeciwnik byl naprawde widziany */
    float m_fLastReallyVisibleTime = AIController._INIT_TIME;
    
    /** ostatnia znana pozycja */
    Vector m_vLastKnownPos = null;
    /** czas, kiedy zostala ustalona ostatnia znana pozycja wroga */
    float m_fLastKnownPosTime = AIController._INIT_TIME;
    
    /** obrazenia, jakie AI otrzymalo od tego przeciwnika */
    float m_fDamageFromThis = 0;
    /** czas ostatniego trafienia od tego przeciwnika */
    float m_fLastDamageFromThisTime = AIController._INIT_TIME;
    
    /** czy chromeactor zostal rozpoznany jako wrog */
    boolean m_bRecognized = false;

    boolean m_bEditorEnemy = false;

    /** Czas, kiedy zaczelo sie rozpoznawanie przeciwnika */
    float m_fRecognitionStartTime = -1;
    /** Czas, kiedy rozpoznawanie przeciwnika zostalo przerwane */
    float m_fRecognitionStopTime = -1; 
    
    EnemyInfo()
    {
        //SG
    }
    
    EnemyInfo(ChromeActor ai)
    {
        m_Reference = ai;
    }
    
    public int compareTo(Object o)
    {
        float fObjPriority = ((EnemyInfo) o).m_fPriority;
        return (m_fPriority < fObjPriority ? 1 : (m_fPriority == fObjPriority ? 0 : -1));
    }
    
    public String toString()
    {
        return("m_Reference:" + m_Reference + "\n" +
               "m_fPriority:" + m_fPriority + "\n" +
               "m_fLastInSightRangeTime:" + m_fLastInSightRangeTime + "\n" +
               "m_fLastAngleCos:" + m_fLastAngleCos + "\n" +
               "m_fLastVisibleTime:" + m_fLastVisibleTime + "\n" +
               "m_fLastVisibleCheckTime:" + m_fLastVisibleCheckTime + "\n" +
               "m_fLastTraceVisibilityTime:" + m_fLastTraceVisibilityTime + "\n" +
               "m_bTraceVisible:" + m_bTraceVisible + "\n" +
               "m_fLastTraceTimeCheck:" + m_fLastTraceTimeCheck + "\n" +
               "m_fLastReallyVisibleTime:" + m_fLastReallyVisibleTime + "\n" +
               "m_vLastKnownPos:" + m_vLastKnownPos + "\n" +
               "m_fLastKnownPosTime:" + m_fLastKnownPosTime + "\n" +
               "m_fDamageFromThis:" + m_fDamageFromThis + "\n" +
               "m_fLastDamageFromThisTime:" + m_fLastDamageFromThisTime + "\n");
    }
    
    //*********************************************************
    //					SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[EnemyInfo.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_ENEMY_INFO)
        {
            m_Reference = (ChromeActor)cFC.LoadObjectRef();
            m_fPriority = cFC.LoadFloat();
            m_fLastInSightRangeTime = cFC.LoadFloat();
            m_fLastAngleCos = cFC.LoadFloat();
            m_fLastVisibleTime = cFC.LoadFloat();
            m_fLastVisibleCheckTime = cFC.LoadFloat();
            m_fLastTraceVisibilityTime = cFC.LoadFloat();
            m_bTraceVisible = cFC.LoadBool();
            m_fLastTraceTimeCheck = cFC.LoadFloat();
            m_fLastReallyVisibleTime = cFC.LoadFloat();
            m_vLastKnownPos = cFC.LoadVector();
            m_fLastKnownPosTime = cFC.LoadFloat();
            m_fDamageFromThis = cFC.LoadFloat();
            m_fLastDamageFromThisTime = cFC.LoadFloat();
            m_bRecognized = cFC.LoadBool();
            m_bEditorEnemy = cFC.LoadBool();
            
            m_fRecognitionStartTime = cFC.LoadFloat();
            m_fRecognitionStopTime = cFC.LoadFloat();
            
            //GameObject.CrashLog("Loading done for enemy info (" + hashCode() + ")\n\n" + this + "\n");
        }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[EnemyInfo.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_ENEMY_INFO, cFCParent);

        cFC.SaveObjectRef(m_Reference);
        cFC.SaveFloat(m_fPriority);
        cFC.SaveFloat(m_fLastInSightRangeTime);
        cFC.SaveFloat(m_fLastAngleCos);
        cFC.SaveFloat(m_fLastVisibleTime);
        cFC.SaveFloat(m_fLastVisibleCheckTime);
        cFC.SaveFloat(m_fLastTraceVisibilityTime);
        cFC.SaveBool(m_bTraceVisible);
        cFC.SaveFloat(m_fLastTraceTimeCheck);
        cFC.SaveFloat(m_fLastReallyVisibleTime);
        cFC.SaveVector(m_vLastKnownPos);
        cFC.SaveFloat(m_fLastKnownPosTime);
        cFC.SaveFloat(m_fDamageFromThis);
        cFC.SaveFloat(m_fLastDamageFromThisTime);
        cFC.SaveBool(m_bRecognized);
        cFC.SaveBool(m_bEditorEnemy);

        cFC.SaveFloat(m_fRecognitionStartTime);
        cFC.SaveFloat(m_fRecognitionStopTime);

        //GameObject.CrashLog("Saving done for enemy info (" + hashCode() + ")\n\n" + this + "\n");
        cFC.delete(); 
    }				
    
    public void SGUpdate()
    {
        FileChunk.Log("[EnemyInfo.SGUpdate]\n");
        //		super.SGUpdate();
        //
    }

}

/** informacja o przyjacielu */
class FriendInfo implements ISaveGame
{
    /** referencja do tego przyjaciela */
    ChromeActor m_Reference = null;
    
    /** czas, kiedy ostatnio byla proba trace trupa tego przyjaciela */
    float m_fLastDeadBodyTraceTimeCheck = AIController._INIT_TIME;

    /** czas, kiedy ten trup zostal zauwazony przez ai */
    float m_fDeadBodySeeTime = AIController._INIT_TIME;
    
    FriendInfo()
    {
        //SG
    }
    
    FriendInfo(ChromeActor ai)
    {
        m_Reference = ai;
    }
    
    boolean WasBodySpotted()
    {
        return (m_fDeadBodySeeTime != AIController._INIT_TIME);
    }
    
    //*********************************************************
    //					SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[FriendInfo.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_FRIEND_INFO)
        {
            m_Reference = (ChromeActor)cFC.LoadObjectRef();
            m_fLastDeadBodyTraceTimeCheck = cFC.LoadFloat();
            m_fDeadBodySeeTime = cFC.LoadFloat();
        }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[FriendInfo.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_FRIEND_INFO, cFCParent);

        cFC.SaveObjectRef(m_Reference);
        cFC.SaveFloat(m_fLastDeadBodyTraceTimeCheck);
        cFC.SaveFloat(m_fDeadBodySeeTime);

        cFC.delete(); 
    }				
    
    public void SGUpdate()
    {
        FileChunk.Log("[FriendInfo.SGUpdate]\n");
        //		super.SGUpdate();
        //
    }

}

class RecognitionListInfo
{
    int m_iListNdx = 0;
    java.util.ArrayList m_List = null;
    RecognitionListInfo(java.util.ArrayList list)
    {
        m_List = list;
    }
}