import java.io.*;

class RadioCommunicationSystem extends GameObject
{
    //////////////////////////////////////////////////////
    // SYSTEM KOMUNIKACJI RADIOWEJ STEROWANEJ PRZEZ AI  //
    //////////////////////////////////////////////////////
    

    /////////////////////////////////////////////////
    // Identyfikatory typow komunikatow radiowych
    //
    public static final int _RADIO_MSG_FIRE_IN_THE_HOLE           = 0;
    public static final int _RADIO_MSG_GRENADE                    = 1;
    public static final int _RADIO_MSG_COVER_ME                   = 2;
    public static final int _RADIO_MSG_COVER_ME_ANSWER            = 3;

    public static final int _RADIO_MSG_ENEMY_HIT                  = 4;
    public static final int _RADIO_MSG_WOUNDED                    = 5;
    public static final int _RADIO_MSG_CLEAR                      = 6;
    
    public static final int _RADIO_MSG_ENEMY_SPOTTED              = 7;
    public static final int _RADIO_MSG_INTRUDER                   = 8;
    public static final int _RADIO_MSG_SUSPECT                    = 9;
    public static final int _RADIO_MSG_WATCHOUT                   = 10;
    public static final int _RADIO_MSG_SNIPER                     = 11;
    
    public static final int _RADIO_MSG_BACKUP_CALL                = 12;

    public static final int _RADIO_MSG_ROGER                      = 13;
    

    public static final int _NOF_RADIO_MSG_TYPES                  = 14;
    
    /////////////////////////////////////////////////
    // Parametry komunikatow radiowych
    //
    
    /** Minimalna przerwa miedzy kolejnymi komunikatami [s] */
    public static float _MIN_TIME_BTW_MESSAGES                    = 2;
    
    /** Nazwa katalogu, w ktorym znajduja sie dzwieki komunikatow */
    public static String _MESSAGES_SOUNDS_DIRECTORY = "Data/Sounds/AI/Radio";
    
    
    /** Czas, kiedy moze byc najwczesniej odegrany nastepny komunikat radiowy */
    float m_fNextRadioMessageTime = 0;
    
    /** Nazwa katalogu, w ktorym sa dzwieki komunikatow, z doklejonym "/" na koncu (do budowania sciezki) */
    String m_sSoundsPathPrefix;
    
    
    RadioSoundsGroup m_aSoundsGroups[] = new RadioSoundsGroup[] {
                                            new RadioSoundsGroup("fire", 0, 0), //_RADIO_MSG_FIRE_IN_THE_HOLE
                                            new RadioSoundsGroup("grenade", 5, 10), //_RADIO_MSG_GRENADE
                                            new RadioSoundsGroup("coverme", 7, 30, _RADIO_MSG_COVER_ME_ANSWER, 0.5f),//_RADIO_MSG_COVER_ME
                                            new RadioSoundsGroup("covered", 1, 1),//_RADIO_MSG_COVER_ME_ANSWER

                                            new RadioSoundsGroup("enemyhit", 10, 60),//_RADIO_MSG_ENEMY_HIT
                                            new RadioSoundsGroup("wounded", 10, 60),//_RADIO_MSG_WOUNDED
                                            new RadioSoundsGroup("clear", 10, 60),//_RADIO_MSG_CLEAR

                                            new RadioSoundsGroup("attack", 10, 60, _RADIO_MSG_ROGER, 0.2f),//_RADIO_MSG_ENEMY_SPOTTED
                                            new RadioSoundsGroup("intruder", 10, 60, _RADIO_MSG_ROGER, 0.2f),//_RADIO_MSG_INTRUDER
                                            new RadioSoundsGroup("suspect", 3, 10),//_RADIO_MSG_SUSPECT
                                            new RadioSoundsGroup("watchout", 10, 60),//_RADIO_MSG_WATCHOUT
                                            new RadioSoundsGroup("sniper", 1, 40),//_RADIO_MSG_SNIPER
                                            new RadioSoundsGroup("basereport", 10, 60),//_RADIO_MSG_BACKUP_CALL
                                            new RadioSoundsGroup("roger", 10, 60),//_RADIO_MSG_ROGER
                                       };

    float m_aNextMessageOfTypeTime[] = new float[_NOF_RADIO_MSG_TYPES];
    
    
    /////////////////////////////////////////////////
    // Parametry aktualnej komunikacji radiowej
    //
    NewAI m_VoiceOwner_1;
    String m_sVoiceWave_1;
    
    NewAI m_VoiceOwner_2;
    String m_sVoiceWave_2;
    float m_fWaveDuration_2;
    
    
    int m_nCurrentWaveID = 0;
    int m_nCurrentVoiceNo;
    
    
    /////////////////////////////////////////////////
    public void InitMessages()
    {
        File fullPath = new File(_MESSAGES_SOUNDS_DIRECTORY);
        if (fullPath != null && fullPath.exists())
            m_sSoundsPathPrefix = fullPath.getPath() + "/";
        else
            m_sSoundsPathPrefix = "";

        if (m_aSoundsGroups != null)
            for (int i = 0; i < m_aSoundsGroups.length; i++)
                if (m_aSoundsGroups[i] != null)
                    m_aSoundsGroups[i].Init(_MESSAGES_SOUNDS_DIRECTORY);
        InitializeTimers();
    }
 
    void InitializeTimers()
    {
        m_fNextRadioMessageTime = 0;
        if (m_aNextMessageOfTypeTime != null)
            for (int i = 0; i < m_aNextMessageOfTypeTime.length; i++)
                m_aNextMessageOfTypeTime[i] = 0;
    }
    
    public boolean RadioMessage(int nMsgType, NewAI voiceOwner1, NewAI voiceOwner2)
    {
        float time = GetTime();

        if (time >= m_fNextRadioMessageTime &&
            m_aSoundsGroups != null &&
            m_aSoundsGroups[nMsgType] != null &&
            time >= m_aNextMessageOfTypeTime[nMsgType])
        {
            m_nCurrentVoiceNo = 0;
            String fileName = m_aSoundsGroups[nMsgType].GetSoundName();
            if (fileName != null) // jesli jest nazwa pliku
            {
                m_sVoiceWave_1 = m_sSoundsPathPrefix + fileName;
                m_sVoiceWave_2 = null;

                float duration1, duration2 = 0;
                
                int answerMsgType = m_aSoundsGroups[nMsgType].GetAnswerMsgType();
                if (answerMsgType != RadioSoundsGroup._NO_ANSWER) // jesli wymagana jest odpowiedz
                {
                    if (voiceOwner2 != null && 
                        m_aSoundsGroups[answerMsgType] != null &&
                        (fileName = m_aSoundsGroups[answerMsgType].GetSoundName()) != null)
                    {
                        m_sVoiceWave_2 = m_sSoundsPathPrefix + fileName;
                        duration2 = GetWaveDuration(m_sVoiceWave_2);
                    }
                    else
                        return false;
                }

                m_VoiceOwner_1 = voiceOwner1;
                m_VoiceOwner_2 = voiceOwner2;
                
                m_nCurrentVoiceNo = 1;
                m_nCurrentWaveID = PlayWave(m_sVoiceWave_1);
                duration1 = GetWaveDuration(m_nCurrentWaveID);

                m_fNextRadioMessageTime = time + duration1 + duration2 + _MIN_TIME_BTW_MESSAGES;
                m_aNextMessageOfTypeTime[nMsgType] = m_fNextRadioMessageTime + m_aSoundsGroups[nMsgType].GetRandomDelay();

                voiceOwner1.SetNextScreamTime(time + duration1);
                if (m_sVoiceWave_2 != null) // oznacza, ze jest 2. rozmowca i jego gadka
                {
                    voiceOwner2.SetNextScreamTime(time + duration1 + duration2);
                    m_fWaveDuration_2 = duration2;
                    duration1 += m_aSoundsGroups[nMsgType].GetAnswerDelay();
                }
                DisableCallMethod("RadioWaveEnd");
                CallMethodIn("RadioWaveEnd", duration1);
                //LogR("RadioMessage: " + nMsgType + " (" + m_aSoundsGroups[nMsgType].sNamePrefix + ")\n");
                return true;
            }
        }
        return false;
    }
    
    public void BreakRadioMessage(NewAI voiceOwner)
    {
        if ((voiceOwner == m_VoiceOwner_1 && m_nCurrentVoiceNo == 1) ||
            (voiceOwner == m_VoiceOwner_2 && m_nCurrentVoiceNo == 2))
        {
            StopWave(m_nCurrentWaveID);
			m_nCurrentWaveID = 0;
            DisableCallMethod("RadioWaveEnd");
            m_nCurrentVoiceNo = 0;
        }
    }
    
    public void Reinitialize()
    {
        DisableCallMethod("RadioWaveEnd");
		if (m_nCurrentWaveID != 0)
			StopWave(m_nCurrentWaveID);
		m_nCurrentWaveID = 0;
        m_nCurrentVoiceNo = 0;
        InitializeTimers();
    }
    
    public float GetNextMessageTime(int nMsgType)
    {
        if (m_aSoundsGroups != null &&
            m_aSoundsGroups[nMsgType] != null)
            return Math.max(m_aNextMessageOfTypeTime[nMsgType], m_fNextRadioMessageTime);
        else
            return -1;

    }
    
    public boolean CheckIfMessageNeedsBuddy(int nMsgType)
    {
        return m_aSoundsGroups != null &&
               m_aSoundsGroups[nMsgType] != null &&
               m_aSoundsGroups[nMsgType].GetAnswerMsgType() != RadioSoundsGroup._NO_ANSWER;

    }
    
    /** Metoda wolana po zakonczeniu odtwarzania aktualnego wave'a
     */
    public void RadioWaveEnd()
    {
        DisableCallMethod("RadioWaveEnd");
        if (m_nCurrentVoiceNo == 1) // skoncyzl sie pierwszy wave
        {
            if (m_sVoiceWave_2 != null) // ma odegrac drugi
            {
                m_nCurrentVoiceNo = 2;
                m_nCurrentWaveID = PlayWave(m_sVoiceWave_2);
                CallMethodIn("RadioWaveEnd", m_fWaveDuration_2);
                return;
            }
        } 
        m_nCurrentVoiceNo = 0;
		m_nCurrentWaveID = 0;
    }
    
    ////////////////////////////////////////////////////////////////
    // SYSTEM KOMUNIKACJI RADIOWEJ STEROWANEJ AKCJAMI EDYTOROWYMI //
    ////////////////////////////////////////////////////////////////

    ChromeModuleSingle m_Module = null;
	boolean m_bCommunicatePlaying = false;
	
	RadioCommunicationSystem()
	{
	}
	
	void Init(ChromeModuleSingle module)
	{
		m_Module = module;
        InitMessages();
	}
	
	public void PlayRadioCommunicate(FileName cFileName)
	{
		if (cFileName == null)
			return;
		if ( (cFileName.sName == null) || (cFileName.sName.length() == 0) )
			return;
		if (!cFileName.Exists())
			return;
		
		if (!m_bCommunicatePlaying)
		{
			float fTime = GetWaveDuration(cFileName.sName);
			if (fTime <= 0)
				return;
			m_Module.MainPlayer.AttachOneShotSound(cFileName.sName);
			m_bCommunicatePlaying = true;
			CallMethodIn("CommunicatePlayed", fTime);
		}
	}
	
	public void CommunicatePlayed()
	{
		DisableCallMethod("CommunicatePlayed");
		m_bCommunicatePlaying = false;
	}
    
    long l_nWavePos = -1;
    public void OnAudioPreReinitialize()
    {
        if (m_nCurrentWaveID != 0 && 
            IsPlayingWave(m_nCurrentWaveID))
        {
            l_nWavePos = GetPlayingWavePos(m_nCurrentWaveID);
        }
    }
    
    public void OnAudioReinitialize()
    {
        
        if (l_nWavePos != -1)
        {
            if (m_nCurrentVoiceNo == 1)
                m_nCurrentWaveID = PlayWaveFromPos(m_sVoiceWave_1, l_nWavePos);
            else
                if (m_nCurrentVoiceNo == 2)
                    m_nCurrentWaveID = PlayWaveFromPos(m_sVoiceWave_2, l_nWavePos);
        }
        
        l_nWavePos = -1;
    }
}


class RadioSoundsGroup extends SoundsGroup
{
    public static final int _NO_ANSWER = -1;
    
    protected int m_nAnswerMsgType;
    protected float m_fBeforeAnswerDelay;

    public RadioSoundsGroup(String namePrefix, float minDelayTime, float maxDelayTime)
    {
        this(namePrefix, minDelayTime, maxDelayTime, _NO_ANSWER, 0);
    }
    
    public RadioSoundsGroup(String namePrefix, 
                            float minDelayTime, float maxDelayTime, 
                            int answerMsgType, float beforeAnswerDelay)
    {
        super(namePrefix, 0, minDelayTime, maxDelayTime);
        m_nAnswerMsgType = answerMsgType;
        m_fBeforeAnswerDelay = beforeAnswerDelay;
    }
    
    public int GetAnswerMsgType()
    {
        return m_nAnswerMsgType;
    }
    
    public float GetAnswerDelay()
    {
        return m_fBeforeAnswerDelay;
    }
}
