public class GridManager extends GameObject
{
    int m_nMaxRequests = 50;
    float m_fTimePeriod = 1f;
    
    NewAI[] m_aRequests = null;
    int m_nPutNdx = 0;
    int m_nGetNdx = 0;
    
    float m_fCurrTime = 0;
    float m_fStartPeriodTime = 0f;
    int m_nRequestsPerPeriod = 0;
    int m_nRequestsRealized = 0;
    float m_fRequestsRealized = 0f;
    float m_fRequestsToHandlePerFrame = 1f;
    int m_nFramesPerPeriod = 0;
    int m_nRequestsInList = 0;

    
    public void OnCreate()
    {
        m_aRequests = new NewAI[m_nMaxRequests];
        m_fStartPeriodTime = GetTime();
    }
    
    public void SetTimePeriod(float fPeriod)
    {
        m_fTimePeriod = fPeriod;
    }
    
    public void Request(ObjectsSense obj)
    {
        m_nRequestsPerPeriod++;
        m_nRequestsInList++;
        
        if (m_nRequestsInList == m_nMaxRequests + 1)
            Resize();

        m_aRequests[m_nPutNdx] = obj.m_Controller.GetMesh();
        m_nPutNdx++;
        if (m_nPutNdx >= m_nMaxRequests)
            m_nPutNdx = 0;
    }

    //String s = "";
    
    public void OnFrame()
    {
        //Tools.ShowInfoTop("coming:" + s + ", per frame:" + m_fRequestsToHandlePerFrame + ",calls awaiting: " + m_nRequestsInList, GetModule());
        float fTime = GetTime();
        m_nFramesPerPeriod++;
        
        if (fTime - m_fStartPeriodTime > m_fTimePeriod)
        {
            //s = "" + m_nRequestsPerPeriod;
            m_fRequestsToHandlePerFrame = (float) (Math.max(m_nRequestsPerPeriod, m_nRequestsInList)) / (float) m_nFramesPerPeriod;

            m_fStartPeriodTime = fTime;
            m_nRequestsPerPeriod = 0;
            m_nRequestsRealized = 0;
            m_fRequestsRealized = 0f;
            m_nFramesPerPeriod = 0;
        }
        
        m_fRequestsRealized += m_fRequestsToHandlePerFrame;
        
        if (m_nRequestsInList > 0)
        {
            int nRequestToRealize = (int) (java.lang.Math.ceil(m_fRequestsRealized) - m_nRequestsRealized);
            m_nRequestsRealized += nRequestToRealize;
            for (int i = 0; i < nRequestToRealize; i++)
                CheckRequest();
        }
    }
    
    public void DumpTable()
    {
        CrashLog("TABDUMP\n");
        for (int i = 0; i < m_nMaxRequests; i++)
            CrashLog(m_aRequests[i] + " ");
        CrashLog("\nGET:" + m_nGetNdx + "\n");
        CrashLog("PUT:" + m_nPutNdx + "\n");
        CrashLog("REQUESTSINLIST:" + m_nRequestsInList + "\n");
    }
    
    private void Resize()
    {
        NewAI[] temp = new NewAI[m_nMaxRequests * 2];
        int j = 0;
        for (int i = m_nGetNdx; i < m_nMaxRequests; i++, j++)
            temp[j] = m_aRequests[i];
        for (int i = 0; i < m_nGetNdx; i++, j++)
            temp[j] = m_aRequests[i];
        
        m_aRequests = temp;
        m_nGetNdx = 0;
        m_nPutNdx = m_nMaxRequests;
        m_nMaxRequests *= 2;
    }
    
    private void CheckRequest()
    {
        if (m_nRequestsInList > 0)
        {
            m_nRequestsInList--;
            if (m_aRequests[m_nGetNdx] != null)
                ((NewAI)m_aRequests[m_nGetNdx]).GetController().m_ObjectsSense.DoUpdateTargetsInRadius();
            m_nGetNdx++;
            if (m_nGetNdx >= m_nMaxRequests)
                m_nGetNdx = 0;
        }
    }
    
    public void Remove(ObjectsSense obj)
    {
        NewAI toRemove = obj.m_Controller.GetMesh();
        for (int i = 0; i < m_aRequests.length; i++)
        {
            if (m_aRequests[i] == toRemove)
                m_aRequests[i] = null;
        }
    }
    
    
    //*********************************************************
    //                      SaveGame
    //*********************************************************
    
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[GridManager.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_GRID_MANAGER)
        {
            super.SGLoadChunk(cFC);
            
            m_nMaxRequests = cFC.LoadInt();
            m_fTimePeriod = cFC.LoadFloat();

            int elemz = cFC.LoadInt();
            if (elemz < 0)
                m_aRequests = null;
            else
            {
                m_aRequests = new NewAI[elemz];
                for (int i = 0; i < elemz; i++)
                    m_aRequests[i] = (NewAI)cFC.LoadObjectRef();
            }

            m_nPutNdx = cFC.LoadInt();
            m_nGetNdx = cFC.LoadInt();
            
            m_fCurrTime = cFC.LoadFloat();
            m_fStartPeriodTime = cFC.LoadFloat();
            m_nRequestsPerPeriod = cFC.LoadInt();
            m_nRequestsRealized = cFC.LoadInt();
            m_fRequestsRealized = cFC.LoadFloat();
            m_fRequestsToHandlePerFrame = cFC.LoadFloat();
            m_nFramesPerPeriod = cFC.LoadInt();
            m_nRequestsInList = cFC.LoadInt();
        }
        
        cFC.delete();
    }
    

    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[GridManager.SGSaveChunk]\n");
        
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_GRID_MANAGER, cFCParent);
        
        super.SGSaveChunk(cFC);

        cFC.SaveInt(m_nMaxRequests);
        cFC.SaveFloat(m_fTimePeriod);

        if (m_aRequests != null)
        {
            cFC.SaveInt(m_aRequests.length);
            for (int i = 0; i < m_aRequests.length; i++)
                cFC.SaveObjectRef(m_aRequests[i]);
        }
        else
            cFC.SaveInt(-1);

        cFC.SaveInt(m_nPutNdx);
        cFC.SaveInt(m_nGetNdx);
        
        cFC.SaveFloat(m_fCurrTime);
        cFC.SaveFloat(m_fStartPeriodTime);
        cFC.SaveInt(m_nRequestsPerPeriod);
        cFC.SaveInt(m_nRequestsRealized);
        cFC.SaveFloat(m_fRequestsRealized);
        cFC.SaveFloat(m_fRequestsToHandlePerFrame);
        cFC.SaveInt(m_nFramesPerPeriod);
        cFC.SaveInt(m_nRequestsInList);

        cFC.delete(); 
    }
    
    public void SGUpdate()
    {
        FileChunk.Log("[GridManager.SGUpdate]\n");
        super.SGUpdate();
    }
    
}