public class NoiseManager extends GameObject
{
    int m_nMaxRequests = 50;
    float m_fTimePeriod = 1f;
    
    RequestInfo[] m_aRequests = null;
    int m_nPutNdx = 0;
    int m_nGetNdx = 0;
    
    float m_fCurrTime = 0;
    float m_fSummaryTime = 0f;
    int m_nRequestsPerPeriod = 0;
    int m_nRequestsToHandlePerFrame = 1;
    int m_nFramesPerPeriod = 0;
    int m_nRequestsInList = 0;
    
    public void OnCreate()
    {
        m_aRequests = new RequestInfo[m_nMaxRequests];
        for (int i = 0; i < m_aRequests.length; i++)
            m_aRequests[i] = new RequestInfo();
        m_fCurrTime = GetTime();
    }
    
    public void SetTimePeriod(float fPeriod)
    {
        m_fTimePeriod = fPeriod;
    }
    
    public void Request(AIController ai, int nEventType)
    {
        m_nRequestsPerPeriod++;
        m_nRequestsInList++;
        
        if (m_nRequestsInList == m_nMaxRequests + 1)
            Resize();

        m_aRequests[m_nPutNdx].Set(ai, nEventType);
        
        m_nPutNdx++;
        if (m_nPutNdx >= m_nMaxRequests)
            m_nPutNdx = 0;
    }
    
    public void OnFrame()
    {
        float fTime = GetTime();
        m_fSummaryTime += (fTime - m_fCurrTime);
        m_nFramesPerPeriod++;
        
        if (m_fSummaryTime > m_fTimePeriod)
        {
            m_nRequestsToHandlePerFrame = (int) java.lang.Math.ceil( (float) m_nRequestsPerPeriod / m_nFramesPerPeriod );

            m_fSummaryTime = 0;
            m_nRequestsPerPeriod = 0;
            m_nFramesPerPeriod = 0;
        }
        m_fCurrTime = fTime;
        
        if (m_nRequestsInList > 0)
        {
            for (int i = 0; i < m_nRequestsToHandlePerFrame; i++)
                CheckRequest();
        }
    }
    
    public void DumpTable()
    {
        CrashLog("TABDUMP\n");
        for (int i = 0; i < m_nMaxRequests; i++)
            CrashLog(m_aRequests[i] + " ");
        CrashLog("\nGET:" + m_nGetNdx + "\n");
        CrashLog("PUT:" + m_nPutNdx + "\n");
        CrashLog("REQUESTSINLIST:" + m_nRequestsInList + "\n");
    }
    
    public void Resize()
    {
        RequestInfo [] temp = new RequestInfo[m_nMaxRequests * 2];
        for (int i = 0; i < temp.length; i++)
            temp[i] = new RequestInfo();
        
        int j = 0;
        for (int i = m_nGetNdx; i < m_nMaxRequests; i++, j++)
            ((RequestInfo) temp[j]).Set(m_aRequests[i]);
        for (int i = 0; i < m_nGetNdx; i++, j++)
            ((RequestInfo) temp[j]).Set(m_aRequests[i]);
        
        m_aRequests = temp;
        m_nGetNdx = 0;
        m_nPutNdx = m_nMaxRequests;
        m_nMaxRequests *= 2;
    }
    
    public void CheckRequest()
    {
        if (m_nRequestsInList > 0)
        {
            m_nRequestsInList--;
            m_aRequests[m_nGetNdx].m_ObjToCall.GetController().RequestEvent(m_aRequests[m_nGetNdx].m_nEventType);
            m_nGetNdx++;
            if (m_nGetNdx >= m_nMaxRequests)
                m_nGetNdx = 0;
        }
    }
    

    //*********************************************************
    //                      SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {
        FileChunk.Log("[NoiseManager.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_NOISE_MANAGER)
        {
            super.SGLoadChunk(cFC);
            
            m_nMaxRequests = cFC.LoadInt();
            m_fTimePeriod = cFC.LoadFloat();
            
            int elemz = cFC.LoadInt();
            if (elemz < 0)
                m_aRequests = null;
            else
            {
                m_aRequests = new RequestInfo[elemz];
                for (int i = 0; i < elemz; i++)
                    m_aRequests[i] = (RequestInfo)cFC.LoadObjectRef();
            }

            m_nPutNdx = cFC.LoadInt();
            m_nGetNdx = cFC.LoadInt();
            
            m_fCurrTime = cFC.LoadFloat();
            m_fSummaryTime = cFC.LoadFloat();
            m_nRequestsPerPeriod = cFC.LoadInt();
            m_nRequestsToHandlePerFrame = cFC.LoadInt();
            m_nFramesPerPeriod = cFC.LoadInt();
            m_nRequestsInList = cFC.LoadInt();
        }
        
        cFC.delete();
    }
    

    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[NoiseManager.SGSaveChunk]\n");
        
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_NOISE_MANAGER, cFCParent);
        
        super.SGSaveChunk(cFC);

        cFC.SaveInt(m_nMaxRequests);
        cFC.SaveFloat(m_fTimePeriod);

        if (m_aRequests != null)
        {
            cFC.SaveInt(m_aRequests.length);
            for (int i = 0; i < m_aRequests.length; i++)
                cFC.SaveObjectRef(m_aRequests[i]);
        }
        else
            cFC.SaveInt(-1);

        cFC.SaveInt(m_nPutNdx);
        cFC.SaveInt(m_nGetNdx);
        cFC.SaveFloat(m_fCurrTime);
        cFC.SaveFloat(m_fSummaryTime);
        cFC.SaveInt(m_nRequestsPerPeriod);
        cFC.SaveInt(m_nRequestsToHandlePerFrame);
        cFC.SaveInt(m_nFramesPerPeriod);
        cFC.SaveInt(m_nRequestsInList);
        
        cFC.delete(); 
    }
    
    public void SGUpdate()
    {
        FileChunk.Log("[NoiseManager.SGUpdate]\n");
        super.SGUpdate();
    }
}


class RequestInfo implements ISaveGame
{
    public NewAI m_ObjToCall = null;
    public int m_nEventType = 0;
    
    void Set(RequestInfo info)
    {
        m_ObjToCall = info.m_ObjToCall;
        m_nEventType = info.m_nEventType;
    }
    
    void Set(AIController ai, int nEventType)
    {
        m_ObjToCall = ai.GetMesh();
        m_nEventType = nEventType;
    }
    
    //*********************************************************
    //                     SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {
        FileChunk.Log("[RequestInfo.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_REQUEST_INFO)
        {
            m_ObjToCall = (NewAI)cFC.LoadObjectRef();
            m_nEventType = cFC.LoadInt();
        }
        
        cFC.delete();
    }
    

    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[RequestInfo.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_REQUEST_INFO, cFCParent);
        
        cFC.SaveObjectRef(m_ObjToCall);
        cFC.SaveInt(m_nEventType);
        
        cFC.delete(); 
    }
    
    public void SGUpdate()
    {
        FileChunk.Log("[RequestInfo.SGUpdate]\n");
    }
}
