public class TaskMotionStateChange extends Task
{
    byte m_iMotionSpeed;
    byte m_iOldMotionSpeed;
    boolean m_bUpdateMotion;
    boolean m_bForceUpdate;
    
    
    
    public TaskMotionStateChange()
    {
        //SG
    }
    
	public TaskMotionStateChange(byte iMotionSpeed)
	{
        this(iMotionSpeed, true);
	}

    
    public TaskMotionStateChange(byte iMotionSpeed, boolean bUpdateMotion)
    {
        this(iMotionSpeed, true, false);
    }

    
    public TaskMotionStateChange(byte iMotionSpeed, boolean bUpdateMotion, boolean bForceUpdate)
	{
        m_iMotionSpeed = iMotionSpeed;
        m_bUpdateMotion = bUpdateMotion;
        m_bForceUpdate = bForceUpdate;
	}
    
    /** wykonuje zadanie */
    public void Perform()
    {
		super.Perform();
        
        m_iOldMotionSpeed = m_cAI.GetMesh().m_Motion.GetSpeed();
        m_cAI.GetMesh().m_Motion.SetSpeed(m_iMotionSpeed);

        if (m_bUpdateMotion && 
            (m_bForceUpdate || (m_iOldMotionSpeed != m_iMotionSpeed)))
        {
            // Zrob UpdateMotion w AI i zawolaj handler czasowy po czasie potrzebnym na zmiane stanu
            m_cAI.StartTaskTime(m_cAI.GetMesh().UpdateMotion());
        }
        else
        {
            m_cAI.GetMesh().m_OldMotion.SetSpeed(m_iMotionSpeed);
            m_cAI.Notify(AIEvents._NOTICE_END_TASK); // to juz jest koniec
        }
    }	
    
    /** Odwolanie akcji */
    public void Cancel(boolean bQuick)
    {
        super.Cancel(bQuick);

        //DEL, bo moze powodowac 'zajaczki' np. jesli zmieni sie widocznosc przeciwnika
        // poza tym to nieuzasadnione, zeby wracac do poprzedniego...
        // Przywroc poprzedni MotionState
        //m_cAI.GetMesh().m_Motion.SetSpeed(m_iOldMotionSpeed);
		//m_cAI.GetMesh().UpdateMotion();
    }
    
    /** obsluguje komunikat wyslany z AIControllera     */
    public void ServeNotice(int nNotice)
    {
        super.ServeNotice(nNotice);
        switch(nNotice)
		{
		case AIEvents._NOTICE_TASK_TIME_PASSED:
            m_cAI.Notify(AIEvents._NOTICE_END_TASK);
			break;            
		}
    }	    
	
	public String toString()
	{
        String sSpeedName;
        if (m_iMotionSpeed >= 0 && m_iMotionSpeed < MotionState.SPEED_COUNT)
            sSpeedName = MotionState.aSpeedNames[m_iMotionSpeed];
        else
            sSpeedName = "incorrect value: " + m_iMotionSpeed;
		return 	"TaskMotionStateChange(" + sSpeedName + 
                (m_bUpdateMotion ? ")" : ", NoUpdateMotion)");
	}
    
    //*********************************************************
    //					SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[TaskMotionStateChange.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_TASK_MOTION_STATE_CHANGE)
        {
            super.SGLoadChunk(cFC);
            m_iMotionSpeed = (byte)cFC.LoadInt();
            m_iOldMotionSpeed = (byte)cFC.LoadInt();
            m_bUpdateMotion = cFC.LoadBool();
            m_bForceUpdate = cFC.LoadBool();
        }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[TaskMotionStateChange.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_TASK_MOTION_STATE_CHANGE, cFCParent);
        
        super.SGSaveChunk(cFC);
        cFC.SaveInt((int)m_iMotionSpeed);
        cFC.SaveInt((int)m_iOldMotionSpeed);
        cFC.SaveBool(m_bUpdateMotion);
        cFC.SaveBool(m_bForceUpdate);

        cFC.delete(); 
    }				
    
    public void SGUpdate()
    {
        FileChunk.Log("[TaskMotionStateChange.SGUpdate]\n");
        super.SGUpdate();
        //
    }

}
