public class TaskWait extends Task
{
    /////////////////////////////////////////////////
    // Bity oznaczajace warunki przerwania waita
    // Musza byc potegami 2 (do sumy bitowej)
    //
    public static int _STOP_CND_NONE        = 0;
    public static int _STOP_CND_TAKE_DAMAGE = 1;
    public static int _STOP_CND_RELOAD      = 2;
    //TODO: inne warunki
    
    public static float _DEFAULT_CHECK_INTERVAL = 0.2f;
    
    float m_fDuration;
    int m_nStopConditions;
    float m_fCheckInterval;
    
    
    public TaskWait()
    {
        //SG
    }
    
    public TaskWait(float fDuration)
    {
        this(fDuration, _STOP_CND_NONE);
    }
    
    public TaskWait(float fDuration, int nStopConditions)
    {
        this(fDuration, nStopConditions, _DEFAULT_CHECK_INTERVAL);
    }
    
    public TaskWait(float fDuration, int nStopConditions, float fCheckInterval)
	{
        m_fDuration = fDuration;
        m_nStopConditions = nStopConditions;
        m_fCheckInterval = fCheckInterval;
	}
    
    /** wykonuje zadanie */
    public void Perform()
    {
        if (m_nStopConditions != _STOP_CND_NONE)
            SetUpdating(m_fCheckInterval);
        
		super.Perform();
        
        m_cAI.StartTaskTime(m_fDuration);
    }	

    
	public void Update()
	{
        if ((m_nStopConditions & _STOP_CND_TAKE_DAMAGE) != 0)
            if (m_cAI.JustGotDamageFromEnemy())
            {
                m_cAI.Notify(AIEvents._NOTICE_END_TASK);
            }
        
        if ((m_nStopConditions & _STOP_CND_RELOAD) != 0)
            if ((m_cAI.GetActiveWeapon() != null) &&
                (m_cAI.GetActiveWeaponAmmo() == 0) &&
                (m_cAI.GetMesh().m_nAmmoForActiveWeapon > 0) )
            {
                m_cAI.Notify(AIEvents._NOTICE_END_TASK);
            }
    }

    /** obsluguje komunikat wyslany z AIControllera     */
    public void ServeNotice(int nNotice)
    {
        super.ServeNotice(nNotice);
        switch(nNotice)
		{
		case AIEvents._NOTICE_TASK_TIME_PASSED:
            m_cAI.Notify(AIEvents._NOTICE_END_TASK);
			break;            
		}
    }	
   
    public String toString()
    {
        return 	"TaskWait(" + m_fDuration + ")";
    }
    
    //*********************************************************
    //					SaveGame
    //*********************************************************
    
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        FileChunk.Log("[TaskWait.SGLoadChunk]\n");
        
        FileChunk cFC = GameObject.LoadChunk(cParentFC);   
        
        if (cFC.GetID() == ESGChunksChrome._CHK_TASK_WAIT)
        {
            super.SGLoadChunk(cFC);
            m_fDuration = cFC.LoadFloat();
            m_nStopConditions = cFC.LoadInt();
            m_fCheckInterval = cFC.LoadFloat();
      }
        
        cFC.delete();
    }
    
    public void SGSaveChunk(FileChunk cFCParent)
    {
        FileChunk.Log("[TaskWait.SGSaveChunk]\n");
        
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_TASK_WAIT, cFCParent);
        
        super.SGSaveChunk(cFC);
	    cFC.SaveFloat(m_fDuration);
        cFC.SaveInt(m_nStopConditions);
        cFC.SaveFloat(m_fCheckInterval);

        cFC.delete(); 
    }				
    
    public void SGUpdate()
    {
        FileChunk.Log("[TaskWait.SGUpdate]\n");
        super.SGUpdate();
        //
    }
    
}
