public class BitVector
{
	long m_iFlag = 0;
	int m_iFlagSize = 64;
	
    /** Ustaw bit o podanym indeksie */
	public final int SetBit(int iNo)
	{
		#if DEBUG
		if ((iNo < 0) || (iNo >= m_iFlagSize))
		{
			GameObject.CrashLog("BitVector.SetBit Bit number exceeded:" + iNo + "\n");
			return iNo;
		}
		#endif
		m_iFlag |= ((long)1 << iNo);
        return iNo;
	}
	
    /** Skasuj bit o podanym indeksie */
	public final void ClearBit(int iNo)
	{
		#if DEBUG
		if ((iNo < 0) || (iNo >= m_iFlagSize))
		{
			GameObject.CrashLog("BitVector.ClearBit Bit number exceeded:" + iNo + "\n");
			return;
		}
		#endif
		m_iFlag &= ~((long)1 << iNo);
	}
	
    /** Ustaw zawartosc wektora bitow na podana */
	public final void Set(BitVector bitVector)
	{
		m_iFlag = bitVector.m_iFlag;
	}
    
    /** Ustaw wszystkie bity w wektorze */
	public final void SetAllBits()
	{
		m_iFlag = ~((long)0);
	}
	
    /** Wyczysc wszystkie bity w wektorze*/
	public final void ClearAllBits()
	{
		m_iFlag = ((long)0);
	}
    
    /** Sprawdzenie czy bit o podanym indeksie jest ustawiony */
    public final boolean IsBitSet(int iNo)
    {
        return (m_iFlag & ((long)1 << iNo)) != 0;
    }
    
    /** zwraca indeks pierwszego ustawionego bitu
     * (jesli zaden nie ustawiony to zwraca -1 */
	public final int GetFirstBitSet()
	{
        return GetNextBitSet(0);
	}

    /** zwraca indeks pierwszego ustawionego bitu liczac od podanej pozycji (wlacznie)
     *  @param nStartIdx pozycja od ktorej zaczyna sie sprawdzanie
     *  @return pozycja, na ktorej jest ustawiony bit albo -1 gdy do konca sa same zera */
	public final int GetNextBitSet(int nStartIdx)
	{
        for (int i = nStartIdx; i < m_iFlagSize; i++)
            if (IsBitSet(i))
                return i;
		return -1;
	}
    
    /** zwraca ilosc bitow w BitVectorze */
    public final int GetBitsNum()
    {
        return m_iFlagSize;
    }
    
    /** Znalezienie czesci wspolnej z podanym wektorem bitow */
    public final void CalcIntersectionWith(BitVector bitVector)
    {
		m_iFlag &= bitVector.m_iFlag;
    }
    
    /** Znalezienie czesci wspolnej z zanegowanym wektorem bitow */
    public final void CalcIntersectionWithComplement(BitVector bitVector)
    {
		m_iFlag &= ~bitVector.m_iFlag;
    }
    
    public final String toString()
    {
        String ret = "";
        for (int i=0; i<m_iFlagSize; i++)
            ret += ((IsBitSet(i)) ? "1" : "0");
        return ret;
    }
}
