/** klasa podstawowa objective, obiekt tej klasy moze byc objective glownym
 * albo objective grupy (tez glownym), zawierajacym inne objectivy
 */
public class SingleObjective
{
	/** numer objective */
	int m_iObjectiveNo = 0;
	
    /** numer objective w grupie - po tym identyfikujemy objective z kodu */
    //uzywane tylko w ObjectiveGroup
	int m_iObjectiveGroupNo = 0;
    
	/** opis objective */
	String m_ObjectiveInfo = "";
	
	/** czy objective zostal wykonany */
	boolean m_bAccomplished = false;

	/** sprite bedacy wyswietlanym tekstem objective */
	Sprite m_SpriteText = null;

	/** czy jest w tej chwili wlaczony (wyswietlany na hudzie) */
	private boolean m_bEnabled = true;
    /** czy jest w tej chwili wlaczony (wyswietlany na hudzie) na danym kompie sieciowym*/
	private boolean m_bEnabledNet = true;

	/** czy objective jest objective glownym (nadobjectivem) dla grupy objectives */
	boolean m_bIsSuperObjective = false;

	/** czy objective jest podobjective jakiegos nadobjective */
	boolean m_bIsSubObjective = false;

	/** statyczny manager objectivow */
	static ObjectivesManager m_Manager = null;

	/** czy objective powinien byc aktywny na starcie levelu */
	boolean m_bEnableOnStart = false;
	
    SingleObjective()
    {
        //SG
    }
    
	SingleObjective(int iObjectiveNo, String info, boolean bIsGroup)
	{
		m_iObjectiveNo = iObjectiveNo;
		m_ObjectiveInfo = new String(info);
		m_bIsSuperObjective = bIsGroup;
		
		CreateTextSprite();
	}
	
	static void SetObjectivesManager(ObjectivesManager manager)
	{
		m_Manager = manager;
	}
    
    void SetAsGroupObjective(boolean bGroup)
    {
		m_bIsSuperObjective = bGroup;
    }

	void CreateTextSprite()
	{
		if (m_SpriteText == null)
		{
			m_SpriteText = (Sprite) ObjectivesManager.m_Module.CreateObject("Sprite");
			m_SpriteText.SetNoTexture();
			m_SpriteText.ProcessMouse(false);
			m_SpriteText.MoveTop();
			m_SpriteText.SetFont(ChromeFonts.FONT_HINT);
			m_SpriteText.SetTextPosFlags(ESpriteText.MULTILINE);
			m_SpriteText.DisplayText(m_ObjectiveInfo); // wyswietlamy byc moze pusty string
			m_SpriteText.Hide(); // na poczatek ukrywamy
		}
	}

	void AttachTextToParent(Sprite parent)
	{
		m_SpriteText.DetachFromParent();
		parent.AddChild(m_SpriteText);
	}
	
	float RecalculatePosition(float fPositionY, Vector vWindowSize)
	{
		if (!CanShow())
			return 0f;
		if ( (m_SpriteText != null) && (m_ObjectiveInfo.length() > 0) )
		{
            m_SpriteText.SetRelativePos(new Vector(ObjectivesManager._TEXT_INDENT_X,
												   fPositionY,
												   0));
            Vector vNewSize = new Vector(vWindowSize);
            if (vNewSize.fX + m_SpriteText.GetPos().fX + 10 > m_SpriteText.GetScreenWidth())
                vNewSize.fX = m_SpriteText.GetScreenWidth() - m_SpriteText.GetPos().fX - 10;
			m_SpriteText.SetSize(vNewSize); // na sztywno ustalamy wielkosc okna
			
			fPositionY = m_SpriteText.GetTextExtent(m_ObjectiveInfo).fY;
			fPositionY += ObjectivesManager._TEXT_INDENT_Y;
			m_SpriteText.DisplayText(m_ObjectiveInfo);
			return fPositionY; // zwracamy szerokosc Y tego sprite'a z tekstem
		}
		return 0f;
	}

    void Enable()
    {
        Enable( false );
    }
    
	void Enable(boolean bNet)
	{
		if (m_bIsSubObjective)
		{
			// musimy sprawdzic, czy jego rodzic jest widoczny, tylko wtedy go pokazujemy
			SingleObjective obj = m_Manager.FindObjective(m_iObjectiveNo, 0);
			if ( (obj != null) && (obj.IsEnabled( bNet )) )
			{
                EnableDo( bNet );
				
				// jezeli jest przypisana location do tego objective, pokazujemy ja tez
				m_Manager.EnableLocation( m_iObjectiveNo, 
                                          ((ObjectiveGroup) this).m_iObjectiveGroupNo,
                                          bNet );
			}
			return;
		}
		else
		{
			EnableDo( bNet );

			m_Manager.EnableLocation(m_iObjectiveNo, 0, bNet);
		}
		
		if (m_bIsSuperObjective)
		{
			// pokazujemy wszystkie jego dzieci
			java.util.Vector subObjectives = m_Manager.FindSubObjectives(m_iObjectiveNo);
			for (int i = 0; i < subObjectives.size(); i++)
			{
				((SingleObjective) subObjectives.elementAt(i)).Enable( bNet );
				((SingleObjective) subObjectives.elementAt(i)).UpdateObjectiveText();
			}
		}
		
		UpdateObjectiveText();
	}
    
    void EnableDo( boolean bNet )
    {
        if ( bNet )
		    m_bEnabledNet = true;
        else
            m_bEnabled = true;
                
		if (CanShow() && m_SpriteText != null)
			m_SpriteText.Show();
    }
    
    void Disable()
    {
        Disable( false );
    }
	
	void Disable(boolean bNet)
	{
        if ( bNet )
		    m_bEnabledNet = false;
        else
            m_bEnabled = false;
		
		if (m_SpriteText != null)
			m_SpriteText.Hide();
		
		if (m_bIsSubObjective)
			m_Manager.DisableLocation(m_iObjectiveNo, ((ObjectiveGroup) this).m_iObjectiveGroupNo, bNet );
		else
			m_Manager.DisableLocation(m_iObjectiveNo, 0, bNet);
		
		if (m_bIsSuperObjective)
		{
			// chowamy wszystkie jego dzieci
			java.util.Vector subObjectives = m_Manager.FindSubObjectives(m_iObjectiveNo);
			for (int i = 0; i < subObjectives.size(); i++)
			{
				((SingleObjective) subObjectives.elementAt(i)).Disable( bNet );
				((SingleObjective) subObjectives.elementAt(i)).UpdateObjectiveText();
			}
		}
		UpdateObjectiveText();
	}


    boolean IsEnabled()
    {
        return IsEnabled( false );
    }
    
    boolean IsEnabled(boolean bNet)
    {
        return bNet ? m_bEnabledNet : m_bEnabled;
    }
    
    boolean CanShow()
    {
        return m_bEnabled && m_bEnabledNet;
    }
     
    
	void HideThisOnly()
	{
		// ukrywamy sprite'a
		if (m_SpriteText != null)
			m_SpriteText.Hide();
		// ukrywamy location
		if (m_bIsSubObjective)
			m_Manager.DisableLocation(m_iObjectiveNo, ((ObjectiveGroup) this).m_iObjectiveGroupNo,false);
		else
			m_Manager.DisableLocation(m_iObjectiveNo, 0,false);
	}
	
	void ShowThisOnly()
	{
		// pokazujemy sprite'a
		if (m_SpriteText != null)
			m_SpriteText.Show();
		// pokazujemy location
		if (m_bIsSubObjective)
			m_Manager.EnableLocation(m_iObjectiveNo, ((ObjectiveGroup) this).m_iObjectiveGroupNo,false);
		else
			m_Manager.EnableLocation(m_iObjectiveNo, 0, false);
	}
	
	void UpdateObjectiveText()
	{
		if (m_bAccomplished)
			m_SpriteText.SetTextColor(0.5f, 0.5f, 0.5f, 1.0f);
		else
			m_SpriteText.SetTextColor(1.0f, 1.0f, 1.0f, 1.0f);
		m_SpriteText.DisplayText(m_ObjectiveInfo);
	}
	
	void SetAccomplished()
	{
		m_bAccomplished = true;
		UpdateObjectiveText();
			
		if (m_bIsSubObjective)
			m_Manager.AccomplishedLocation(m_iObjectiveNo, ((ObjectiveGroup) this).m_iObjectiveGroupNo);
		else
			m_Manager.AccomplishedLocation(m_iObjectiveNo, 0);
        
        if (m_bIsSuperObjective)
        {
			// accomplishujemy wszystkie jego dzieci
			java.util.Vector subObjectives = m_Manager.FindSubObjectives(m_iObjectiveNo);
			for (int i = 0; i < subObjectives.size(); i++)
				((SingleObjective) subObjectives.elementAt(i)).SetAccomplished();
        }
	}
    
    void SetUnaccomplished()
    {
		m_bAccomplished = false;
		UpdateObjectiveText();
			
		if (m_bIsSubObjective)
			m_Manager.UnaccomplishedLocation(m_iObjectiveNo, ((ObjectiveGroup) this).m_iObjectiveGroupNo);
		else
			m_Manager.UnaccomplishedLocation(m_iObjectiveNo, 0);
        
        if (m_bIsSuperObjective)
        {
			// accomplishujemy wszystkie jego dzieci
			java.util.Vector subObjectives = m_Manager.FindSubObjectives(m_iObjectiveNo);
			for (int i = 0; i < subObjectives.size(); i++)
				((SingleObjective) subObjectives.elementAt(i)).SetUnaccomplished();
        }
    }

    
    void Destroy()
    {
        if (m_SpriteText != null)
            m_SpriteText.RemoveSprite(m_SpriteText);
    }
    
	boolean IsAccomplished()
	{
		return m_bAccomplished;
	}
}
