 import java.io.*;

/** Klasa podstawowych animacji postaci Chrome'a.
 */
public class BaseAnimations
{
    /** MeshObject dla ktorego animacje definiuje ten obiekt */
    MeshObject cMesh = null;

    /** Klasa obiektu uzywajacego zestawu animacji, uzywane przy przelaczaniu broni FPP */
    Class cOwnerClass = null;
    
    /**Nazwa kosci bazowej.
     * <br>Dla niej uruchamiane sa animacje dla nog i calego ciala
     */
    String ROOT_BONE                 = "";
    /**Nazwa kosci zaczynajacej hierarchie torsu.
     * <br>Dla niej uruchamiane sa animacje dla gory ciala.
     */
    String TORSO_BONE                = "Spine";

    /**
     */
    String UPPERTORSO_BONE           = "Spine1";

    /** Indeksy do tablicy animacji */

    //dla kazdego rodzaju broni oddzielnie cale cialo:
    static byte A_NONE            = 0;
    static byte A_STAND           = 1;
    static byte A_FORWARD         = 2;
    static byte A_BACKWARD        = 3;
    static byte A_STRAFE_LEFT     = 4;
    static byte A_STRAFE_RIGHT    = 5;
    static byte A_JUMP            = 6;
    static byte A_JUMP_FORWARD    = 7;
    static byte A_BUTT_HIT        = 8;

    static byte A_CRAWL2DUCK        = 9;
    static byte A_CRAWL2STAND       = 10;
    static byte A_DUCK2CRAWL        = 11;
    static byte A_STAND2CRAWL       = 12;

    static byte A_DIE               = 13; //TEMP

    //tylko bez broni, cale cialo:
    static byte A_KICK              = 14;
    static byte A_PUNCH             = 15;

    //kazda bron, tylko gora:
    static byte A_WEA_FIRE          = 16;
    static byte A_WEA_RELOAD        = 17;
    static byte A_WEA_GET           = 18;
    static byte A_WEA_PUTAWAY       = 19;
    
    static byte A_SWIM2DUCK        = 20;
    static byte A_SWIM2STAND       = 21;
    static byte A_DUCK2SWIM        = 22;
    static byte A_STAND2SWIM       = 23;

    static byte A_DRIVE			   = 24;
    static byte A_BINOCULARS_ON    = 25;
    static byte A_DRIVE_SPEEDER    = 26;
    static byte A_RIDE             = 27;//w samochodzie jako pasazer
    
    static byte A_TAKE_HIT_FRONT    = 28;
    static byte A_TAKE_HIT_BACK     = 29;
    static byte A_TAKE_HIT_LEFT     = 30;
    static byte A_TAKE_HIT_RIGHT    = 31;
    
    static byte A_DEATH_FRONT       = 32;
    static byte A_DEATH_BACK        = 33;
    static byte A_DEATH_LEFT        = 34;
    static byte A_DEATH_RIGHT       = 35;
    static byte A_DEATH_DUCK        = 36;
    static byte A_DEATH_CRAWL       = 37;
    
    static byte A_GLADDEN_1         = 38;
    static byte A_GLADDEN_2         = 39;
    static byte A_GLADDEN_12        = 40;

    static byte A_IDLE              = 41;
    
    static byte A_WEA_FIRE2         = 42;
    
    static byte A_DRIVE_WALKER      = 43;//walker

    static byte A_CNT               = 44;
    
    /**Tablica zawierajaca identyfikatory animacji wykorzystywane przy
     * wywolaniu {@link #MeshObject.PlayAnim(int iAnimID, int iCount)}
     */
    public int Anims[] = new int[ A_CNT ];

    public float fAnimMorphTime = 0.15f;

        
    BaseAnimations( MeshObject mesh, String sRootBone,
                    String sTorsoBone, String sUpperTorsoBone, Class ownerClass )
    {
        cMesh = mesh;
        cOwnerClass = ownerClass;
        ROOT_BONE = sRootBone;
        TORSO_BONE = sTorsoBone;
        UPPERTORSO_BONE = sUpperTorsoBone;

        //chcemy sie animowac
        //cMesh.ClearFlag(EFlags._NO_ANIMATION);

        cMesh.SetAnimMorphTime(fAnimMorphTime);

        //te animacje sa niezalezne od rodzaju ruchu i broni
        Anims[A_NONE]               = cMesh.GetAnimID(ROOT_BONE, "none");
        //GameObject.CrashLog("["+mesh.getClass().getName()+"] Anims[A_NONE]="+Anims[A_NONE]+"\n");
        
        Anims[A_KICK]               = cMesh.GetAnimID(ROOT_BONE, "kick");
        Anims[A_PUNCH]              = cMesh.GetAnimID(ROOT_BONE, "punch");
        if ( mesh.GetModule() instanceof ModuleNet )
        {
            Anims[A_DIE]          = cMesh.GetAnimID(ROOT_BONE, "pad");
        
            Anims[A_DEATH_FRONT]  = cMesh.GetAnimID(ROOT_BONE, "death_front");
            Anims[A_DEATH_BACK]   = cMesh.GetAnimID(ROOT_BONE, "death_back" );
            Anims[A_DEATH_LEFT]   = cMesh.GetAnimID(ROOT_BONE, "death_left" );
            Anims[A_DEATH_RIGHT]  = cMesh.GetAnimID(ROOT_BONE, "death_right");
            Anims[A_DEATH_DUCK]   = cMesh.GetAnimID(ROOT_BONE, "death_duck" );
            Anims[A_DEATH_CRAWL]  = cMesh.GetAnimID(ROOT_BONE, "death_crawl");
            
            Anims[A_GLADDEN_1]    = cMesh.GetAnimID(ROOT_BONE, "radosc1_loop");
            Anims[A_GLADDEN_2]    = cMesh.GetAnimID(ROOT_BONE, "radosc2_loop");
            Anims[A_GLADDEN_12]   = cMesh.GetAnimID(ROOT_BONE, "radosc12_loop");  
            
            Anims[A_IDLE]         = cMesh.GetAnimID(ROOT_BONE, "rozglada_sie");  
            
        }
        else                        
            Anims[A_DIE]                = cMesh.GetAnimID(ROOT_BONE, "death"); //TEMP
        //WriteAnimIniTemplate("anim_template.ini");

        //GameObject.CrashLog("["+mesh.getClass().getName()+"] Anims[A_PUNCH]="+Anims[A_PUNCH]+"\n");
        
        // dodatkowy warunek dla granatu
        if(mesh instanceof FPPHandsGrenade)
        {
            FPPHandsGrenade cGrenade = (FPPHandsGrenade)mesh;
            cGrenade.nFireID = Anims[A_WEA_FIRE];
        }
        
        //PlayNoMorph(A_NONE);
    }

/*
    public void UpdateMotion(MotionState motion, Weapon ActiveWeapon)
    {
        SetupAnims(motion, ActiveWeapon);
    }
*/
    /** Ustawia animacje zaleznie od rodzaju broni i motionState gracza
     */
    void SetupAnims(MotionState motion, Weapon ActiveWeapon)
    {
        byte weaponType;
        boolean bPumpAction = false;

        if (ActiveWeapon != null)
        {
            weaponType = ActiveWeapon.nType;
            bPumpAction = ActiveWeapon.bPumpAction;
        }
        else
            weaponType = EWeaponType._HAND;

        String prefix = "";
        //choose weapon type
        switch (weaponType)
        {
        case EWeaponType._BINOCULARS: prefix = "L"; break;
        case EWeaponType._HAND      : 
        //TODO: dorobic animacja dla chodzenia bez broni !!!! prefix = "N"; break;
        case EWeaponType._KNIFE     : prefix = "K"; break;
        case EWeaponType._PISTOL    : prefix = "P"; break;
        case EWeaponType._RIFLE     : prefix = "R"; break;
        case EWeaponType._HEAVY     : prefix = "H"; break;
        case EWeaponType._GRENADE   : prefix = "G"; break;
        default: prefix = "U"; //unknown
        }
        

        //animacje, ktore nie zaleza od stanu ruchu (tylko od uzywanej broni)
        Anims[A_BUTT_HIT]           = cMesh.GetAnimID(ROOT_BONE, prefix+"b");

        Anims[A_CRAWL2DUCK]         = cMesh.GetAnimID(ROOT_BONE, prefix+"c2d");
        Anims[A_CRAWL2STAND]        = cMesh.GetAnimID(ROOT_BONE, prefix+"c2s");
        Anims[A_DUCK2CRAWL]         = cMesh.GetAnimID(ROOT_BONE, prefix+"d2c");
        Anims[A_STAND2CRAWL]        = cMesh.GetAnimID(ROOT_BONE, prefix+"s2c");
        
        //TODO: osobne animacje do plywania
        Anims[A_SWIM2DUCK]         = cMesh.GetAnimID(ROOT_BONE, prefix+"c2d");
        Anims[A_SWIM2STAND]        = cMesh.GetAnimID(ROOT_BONE, prefix+"c2s");
        Anims[A_DUCK2SWIM]         = cMesh.GetAnimID(ROOT_BONE, prefix+"d2c");
        Anims[A_STAND2SWIM]        = cMesh.GetAnimID(ROOT_BONE, prefix+"s2c");

        //choose motion mode
        switch (motion.iSpeed)
        {
        case MotionState.SPEED_RUN : prefix += "r"; break;
        case MotionState.SPEED_WALK : prefix += "w"; break;
        case MotionState.SPEED_DUCK : prefix += "d"; break;
        case MotionState.SPEED_CRAWL : prefix += "c"; break;
        //TODO: osobna animacja do plywania
        case MotionState.SPEED_SWIM : prefix += "c"; break;
        default: ;
        }
                        
        Anims[A_JUMP]               = cMesh.GetAnimID(ROOT_BONE, prefix+"j");
        Anims[A_JUMP_FORWARD]       = cMesh.GetAnimID(ROOT_BONE, prefix+"jf");

        //animacje broni (oprocz fire)
        Anims[A_WEA_RELOAD]         = cMesh.GetAnimID(TORSO_BONE, prefix+"re");
        Anims[A_WEA_GET]            = cMesh.GetAnimID(TORSO_BONE, prefix+"g");
        Anims[A_WEA_PUTAWAY]        = cMesh.GetAnimID(TORSO_BONE, prefix+"p");

        if (motion.bAiming == true)
            prefix += "a";
        else 
            prefix += "n";

        // fire dla shotguna wyglada troche inaczej niz dla rifle
        // fire jest inne dla przycelowywania
        //FIXIT
        if ( bPumpAction )
            Anims[A_WEA_FIRE]         = cMesh.GetAnimID(TORSO_BONE, prefix+"fS");
        else
            Anims[A_WEA_FIRE]         = cMesh.GetAnimID(UPPERTORSO_BONE, prefix+"f");
            
        Anims[A_WEA_FIRE2]         = cMesh.GetAnimID(UPPERTORSO_BONE, prefix+"f2");

        //inne animacje
        Anims[A_STAND]              = cMesh.GetAnimID(ROOT_BONE, prefix+"s");
        Anims[A_FORWARD]            = cMesh.GetAnimID(ROOT_BONE, prefix+"");
        Anims[A_BACKWARD]           = cMesh.GetAnimID(ROOT_BONE, prefix+"b");
        Anims[A_STRAFE_LEFT]        = cMesh.GetAnimID(ROOT_BONE, prefix+"sl");
        Anims[A_STRAFE_RIGHT]       = cMesh.GetAnimID(ROOT_BONE, prefix+"sr");
        
        Anims[A_BINOCULARS_ON]      = cMesh.GetAnimID(ROOT_BONE, "lornetka");
            
        Anims[A_DRIVE]            = cMesh.GetAnimID( ROOT_BONE, "drive" );
        if ( cMesh.GetModule() instanceof ModuleNet )
        {
            Anims[ A_TAKE_HIT_FRONT ] = cMesh.GetAnimID( ROOT_BONE, prefix+"sh_f" );
            Anims[ A_TAKE_HIT_BACK ]  = cMesh.GetAnimID( ROOT_BONE, prefix+"sh_b" );
            Anims[ A_TAKE_HIT_LEFT ]  = cMesh.GetAnimID( ROOT_BONE, prefix+"sh_l" );
            Anims[ A_TAKE_HIT_RIGHT ] = cMesh.GetAnimID( ROOT_BONE, prefix+"sh_r" ); 
        }
        
        Anims[A_RIDE]              = cMesh.GetAnimID(ROOT_BONE, "ride_gun_operate");
        Anims[A_DRIVE_SPEEDER]     = cMesh.GetAnimID(ROOT_BONE, "fly");
        Anims[A_DRIVE_WALKER]      = cMesh.GetAnimID(ROOT_BONE, "walker");
    }

    boolean bLock = false;
    
    void Play(int nID)
    {
        if(bLock) return;
        
        //GameObject.LogR("[BaseAnims] Play "+nID+"\n");
        
        //cMesh.LogLine("Play:"+Anims[nID]+" nID="+nID);
        if(Anims[nID]>-1)
            cMesh.PlayAnim(Anims[nID], 1);
    }

    //odegranie animacji bez morphingu z poprzedniej
    void PlayNoMorph(int nID)
    {
        if(bLock) return;
        
        //GameObject.LogR("[BaseAnims] PlayNoMorph "+nID+"\n");
        if(Anims[nID]>-1)
        {
            cMesh.SetAnimMorphTime( 0 );
            cMesh.PlayAnim(Anims[nID], 1);
            cMesh.SetAnimMorphTime(fAnimMorphTime);
        }
    }
    
    void PlayLoop(int nID)
    {
        if(bLock) return;
        
        //GameObject.LogR("[BaseAnims] PlayLoop "+nID+"\n");
        //cMesh.LogLine("PlayLoop:"+Anims[nID]+" nID="+nID);
        if(Anims[nID]>-1)
            cMesh.PlayAnim(Anims[nID], -1);
    }

    float GetLength(int nID)
    {
        return cMesh.GetAnimLength(Anims[nID]);
    }
    
    void Stop(String bone)
    {
        if(bLock) return;
        
        //GameObject.LogR("[BaseAnims] Stop "+bone+"\n");
        cMesh.StopAnim(bone);
    }
    
    void StopAll()
    {
        if(bLock) return;
        
        //GameObject.LogR("Stop all");        
        cMesh.StopAnim("");
    }

    void Lock()
    {
        bLock = true;
    }
    
    void Unlock()
    {
        bLock = false;
    }
    
    /** Ustawienie ostatniej klatki animacji chowania guna, zeby
     * nie bylo przeskoku jak gun sie wysuwa.
     */
    void SetLastChangeWeaponFrame()
    {
        cMesh.SetFrame(Anims[A_WEA_PUTAWAY], cMesh.GetLastFrame(Anims[A_WEA_PUTAWAY]), -1);
    }

    /** Pobranie pozycji w swiecie dumiesa ktory okresla gdzie jest koniec lufy.
     */
    Vector GetBarrelDummyPos()
    {
        try {
            FPPHands cHands = (FPPHands)cMesh;
            Vector vPos = cHands.GetBarrelPos();
            return vPos;
        }
        catch (ClassCastException e)
        {
            return null;
        }
    }

    /** Pobranie nazwy dummiesa ktory okresla gdzie jest koniec lufy.
     */
    String GetBarrelDummyName()
    {
        try {
            FPPHands cHands = (FPPHands)cMesh;
            return cHands.sBarrelDummyName;
        }
        catch (ClassCastException e)
        {
            return null;
        }
    }
    
    static String[] aPrefixWeapon = {"N", "P", "R", "H", "G",
                                     "None", "Pistol", "Rifle", "Heavy Rifle", "Grenade"};
    static String[] aPrefixMotion = {"r", "w", "d", "c",
                                    "run", "walk", "duck", "crawl"};
    static String[] aPrefixMode   = {"n", "a",
                                    "normal", "aimed"};


    /**[HELP] Tworzy plik template do konfiguracji animacji.
     */
    public void WriteAnimIniTemplate(String filename)
    {
        try {
            java.io.BufferedWriter out = new java.io.BufferedWriter(new FileWriter(filename, false));
            String prefix, prefix2, prefix3;

            //dla kazdej broni
            for (int i=0; i<aPrefixWeapon.length/2; i++)
            {
                out.write("\n");
                out.write("#======== WEAPON TYPE: "+aPrefixWeapon[i+aPrefixWeapon.length/2]+" ========\n");

                out.write(aPrefixWeapon[i]+"b\t(0 0)\n");
                out.write(aPrefixWeapon[i]+"c2d\t(0 0)\n");
                out.write(aPrefixWeapon[i]+"c2s\t(0 0)\n");
                out.write(aPrefixWeapon[i]+"d2c\t(0 0)\n");
                out.write(aPrefixWeapon[i]+"s2c\t(0 0)\n");
                
                //dla kazdego rodzaju ruchu
                for (int j=0; j<aPrefixMotion.length/2; j++) 
                {
                    out.write("#-------- MOTION TYPE: "+aPrefixMotion[j+aPrefixMotion.length/2]+" --------\n");

                    //skakanie - nie w czasie crawlingu
                    if (aPrefixMotion[j].compareTo("c") != 0)
                    {
                        out.write(aPrefixWeapon[i]+aPrefixMotion[j]+"j\t(0 0)\n");
                        out.write(aPrefixWeapon[i]+aPrefixMotion[j]+"jf\t(0 0)\n");
                    }

                    //animacje broni (oprocz fire) - tylko dla trybow z bronia
                    if (aPrefixWeapon[i].compareTo("N") != 0)
                    {
                        out.write(aPrefixWeapon[i]+aPrefixMotion[j]+"re\t(0 0)\n");
                        out.write(aPrefixWeapon[i]+aPrefixMotion[j]+"g\t(0 0)\n");
                        out.write(aPrefixWeapon[i]+aPrefixMotion[j]+"p\t(0 0)\n");
                    }

                    //dla kazdego trybu ruchu (normalny/przycelowanie)
                    for (int k=0; k<aPrefixMode.length/2; k++)
                    {
                        //nie mozna przycelowywac w czasie crawl, run ani bez broni
                        if ( ( (aPrefixMotion[j].compareTo("c") == 0) ||
                               (aPrefixMotion[j].compareTo("r") == 0) ||
                               (aPrefixWeapon[i].compareTo("N") == 0) )&&
                             (aPrefixMode[k].compareTo("a") == 0) )
                            continue;
                        else {
                            out.write("#. . . .  MOTION MODE: "+aPrefixMode[k+aPrefixMode.length/2]+" . . . . \n");

                            //animacje z bronia - tylko dla trybow z bronia
                            if (aPrefixWeapon[i].compareTo("N") != 0)
                            {

                                //dodatkowe przycelowanie - pump action - tylko dla rifle'ow
                                if (aPrefixWeapon[i].compareTo("R") == 0)
                                    out.write(aPrefixWeapon[i]+aPrefixMotion[j]+aPrefixMode[k]+"fS\t(0 0)\n");
                                
                                out.write(aPrefixWeapon[i]+aPrefixMotion[j]+aPrefixMode[k]+"f\t(0 0)\n");
                            }

                            //inne animacje
                            out.write(aPrefixWeapon[i]+aPrefixMotion[j]+aPrefixMode[k]+"s\t(0 0)\n");
                            out.write(aPrefixWeapon[i]+aPrefixMotion[j]+aPrefixMode[k]+"\t(0 0)\n");
                            out.write(aPrefixWeapon[i]+aPrefixMotion[j]+aPrefixMode[k]+"b\t(0 0)\n");
                            out.write(aPrefixWeapon[i]+aPrefixMotion[j]+aPrefixMode[k]+"sl\t(0 0)\n");
                            out.write(aPrefixWeapon[i]+aPrefixMotion[j]+aPrefixMode[k]+"sr\t(0 0)\n");
                        }
                    }
                }
            }
            out.close();
        } catch (IOException e) 
        {
            cMesh.CrashLog("Error creating/writing to file"+filename+"\n");
            e.printStackTrace();
        }
    }    
}
