/** Atrybuty dotyczace poruszania sie i stanow
 */
public class MotionState implements ISaveGame
{
    final static byte SPEED_STILL   = 0;
    final static byte SPEED_SWIM    = 1;
    final static byte SPEED_CRAWL   = 2;
    final static byte SPEED_DUCK    = 3;
    final static byte SPEED_WALK    = 4;
    final static byte SPEED_RUN     = 5;
    
    // Uzywane w graczu TYLKO do pobierania predkosci wchodzenia.
    // Gracz chodzac po drabinie NIE MA iSpeed ustawione na ta wartosc!
    final static byte SPEED_LADDER  = 6;
    
    final static byte SPEED_COUNT   = 7;
    
    // tablica z nazwami ruchow, do debugowania
    static String[] aSpeedNames = { "SPEED_STILL",
                                    "SPEED_SWIM",
                                    "SPEED_CRAWL",
                                    "SPEED_DUCK",
                                    "SPEED_WALK",
                                    "SPEED_RUN",
                                    "SPEED_LADDER" };
    
  
    final static byte   _CLIMBING_FALSE     = 0,
                        _CLIMBING_TRUE      = 1,
                        _CLIMBING_LEAVING   = 2;
    
    final static byte   _JUMPING_FALSE        = 0,
                        _JUMPING_TRUE         = 1,
                        _JUMPING_OFF_LADDER   = 2;
    
    //
    // poruszanie sie
    //
    public byte     iMoveX  = 0;     // strafe lewo/prawo
    public byte     iMoveY  = 0;     // gora/dol (wchodzenie na drabiny)
    public byte     iMoveZ  = 0;     // przod/tyl
    public byte     iTurn   = 0;     // obrot prawo/lewo
    
    /** Tryb poruszania. Rowny SPEED_...
     * Reprezentuje stan klawiszy modyfikacji trybu poruszania. */
    public byte     iSpeed  = 0;

    //
    //flagi przelaczajace stany w graczu
    //
    
    /** Przycelowujemy? */
    public boolean bAiming      = false;

    /** Strzelamy? */
    public boolean bFiring      = false;
    
    /** Przeladowujemy bron? */
    public boolean bReloading   = false;
    
    /** Wspinamy sie? */
    public byte nClimbing       = _CLIMBING_FALSE;
    
    boolean IsClimbing() { return nClimbing > _CLIMBING_FALSE; }
        
    /** Skaczemy? */
    public byte nJumping        = _JUMPING_FALSE;
    
    boolean IsJumping()  { return nJumping  > _JUMPING_FALSE;  }
    
    MotionState()
    {
        Reset();
    }

    void Reset()
    {
        Stop();
        bAiming     = false;
        bFiring     = false;
        bReloading  = false;
        nClimbing   = _CLIMBING_FALSE;
        nJumping    = _JUMPING_FALSE;
    }

    

    MotionState(MotionState motion)
    {
        Set(motion);
    }

    void Set(MotionState motion)
    {
        iMoveX      = motion.iMoveX;
        iMoveY      = motion.iMoveY;
        iMoveZ      = motion.iMoveZ;
        iTurn       = motion.iTurn;
        iSpeed      = motion.iSpeed;
        bFiring     = motion.bFiring;
        bAiming     = motion.bAiming;
        nClimbing   = motion.nClimbing;
        bReloading  = motion.bReloading;
        nJumping    = motion.nJumping;

        iMoveX_copy = motion.iMoveX_copy;
        iMoveY_copy = motion.iMoveY_copy;
        iMoveZ_copy = motion.iMoveZ_copy;
        iTurn_copy  = motion.iTurn_copy;
        iSpeed_copy = motion.iSpeed_copy;
    }
    
    void SetMove(MotionState motion)
    {
        iMoveX      = motion.iMoveX;
        iMoveY      = motion.iMoveY;
        iMoveZ      = motion.iMoveZ;
        
        iMoveX_copy = motion.iMoveX_copy;
        iMoveY_copy = motion.iMoveY_copy;
        iMoveZ_copy = motion.iMoveZ_copy;
    }
    
    boolean IsEqualMoving(MotionState motion)
    {
        return 
          ( (motion.iMoveX       == iMoveX) &&
            (motion.iMoveY       == iMoveY) &&
            (motion.iMoveZ       == iMoveZ) &&
            (motion.iSpeed       == iSpeed) &&
            (motion.iTurn        == iTurn) );
    }
    
    boolean Changed(MotionState old)
    {
        if ( IsEqual(old) )
            return false;
        
        boolean bJumpingOff = false;
        // jezeli roznia sie tylko flaga jumping ( true -> false )
        if ( old.IsJumping() && !IsJumping() )
        {
            bJumpingOff = true;
            
            MotionState test = new MotionState( old );    
            test.nJumping = _JUMPING_FALSE;
            if ( IsEqual(test) )
                return false;
        }
                       
        return true;
    }
        
       
        
    boolean IsEqual(MotionState motion)
    {
        return 
          ( (motion.iMoveX       == iMoveX) &&
            (motion.iMoveY       == iMoveY) &&
            (motion.iMoveZ       == iMoveZ) &&
            (motion.iSpeed       == iSpeed) &&
            (motion.iTurn        == iTurn) &&
            (motion.bFiring      == bFiring) &&
            (motion.bAiming      == bAiming) &&
            (motion.bReloading   == bReloading) &&
            (motion.nClimbing    == nClimbing) &&
            (motion.nJumping     == nJumping) 
/*
            (motion.iMoveX_copy == iMoveX_copy) &&
            (motion.iMoveY_copy == iMoveY_copy) &&
            (motion.iMoveZ_copy == iMoveZ_copy) &&
            (motion.iTurn_copy == iTurn_copy) &&
            (motion.iSpeed_copy == iSpeed_copy)*/
            );
    }
 
    /** Procka do testow, jesli porownanie nie da wyniku pozytywnego
     * to liogowane sa roznice...
     */
    boolean IsEqualShow(MotionState motion)
    {
        if (IsEqual( motion ))
            return true;
        
        GameObject.Log("IsEqual failed: ");
        
        if (motion.iMoveX       != iMoveX)
            GameObject.Log("iMoveX org:"+iMoveX+" old:"+motion.iMoveX+",");
        if (motion.iMoveY       != iMoveY) 
            GameObject.Log("iMoveY org:"+iMoveY+" old:"+motion.iMoveY+",");
        if (motion.iMoveZ       != iMoveZ) 
            GameObject.Log("iMoveZ org:"+iMoveZ+" old:"+motion.iMoveZ+",");
        if (motion.iSpeed       != iSpeed) 
            GameObject.Log("iSpeed org:"+iSpeed+" old:"+motion.iSpeed+",");
        if (motion.iTurn        != iTurn) 
            GameObject.Log("iTurn org:"+iTurn+" old:"+motion.iTurn+",");
        if (motion.bFiring      != bFiring) 
            GameObject.Log("bFiring org:"+bFiring+" old:"+motion.bFiring+",");
        if (motion.bAiming      != bAiming) 
            GameObject.Log("bAiming org:"+bAiming+" old:"+motion.bAiming+",");
        if (motion.bReloading   != bReloading) 
            GameObject.Log("bReloading org:"+bReloading+" old:"+motion.bReloading+",");
        if (motion.nClimbing    != nClimbing) 
            GameObject.Log("nClimbing org:"+nClimbing+" old:"+motion.nClimbing+",");
        if (motion.nJumping     != nJumping) 
            GameObject.Log("nJumping org:"+nJumping+" old:"+motion.nJumping+",");    
        
        GameObject.Log("\n");
         
         
        return false;
    }
        
    /** Przywrocic te zmienne z MotionState'a ktore nie maja
     * przychodzic w MS z sieci z klienta na serwera
     */
    void Recover( MotionState motion )
    {
        bReloading  = motion.bReloading;
        nClimbing   = motion.nClimbing;
               
        // TODO: tu umiescic odtwarzanie wszystkich flag, ktore
        // nie beda replikowane w MS z klienta na serwera, a
        // klient bedzie prosil o ich ustawienie na serwerze
        // poprzez event.
    }
    
    void Stop()
    {
        iMoveX = iMoveY = iMoveZ = iTurn = 0;
        iSpeed = SPEED_RUN;          //domyslnie chcemy biegac
    }

    void StopMove()
    {
        iMoveX = iMoveY = iMoveZ = iTurn = 0;
    }
    
    boolean IsMoving()
    {
        return ((iTurn !=0 ) || (iMoveX != 0) || (iMoveY != 0) || (iMoveZ != 0));
    }
    
    boolean IsMovingLinear()
    {
        return ((iMoveX != 0) || (iMoveY != 0) || (iMoveZ != 0));
    }

    /** Efektywna predkosc poruszania sie.
     */
    //TODO wyrzucic to
    int GetMotionSpeed()
    {
        if ( iMoveZ < 0 )
            if ( iSpeed > SPEED_WALK )
                return SPEED_WALK;
            else 
                return iSpeed;
        else
            if ((iMoveZ > 0) || (iMoveX != 0))
                return iSpeed;
            else 
                return 0;
    }
    
    Vector GetMotionVector()
    {
        return new Vector(iMoveX, iMoveY, iMoveZ);
    }

    boolean CanRun()
    {
        return ((iSpeed == SPEED_RUN) && (iMoveZ > -1));
    }
///////////////////////////////////////////////////////////////////////////////    
    
    //modyfikujemy element lezacy najnizej na stosie
    void IncMoveZ()
    {
        if (nStackDepth == 0)
        {
            if (iMoveZ < 1)
              iMoveZ++;
        }
        else
            if (iMoveZ_copy < 1)
                iMoveZ_copy++;
    }

    void DecMoveZ()
    {
        if (nStackDepth == 0)
        {
            if (iMoveZ > -1)
                iMoveZ--;
        }
        else
            if (iMoveZ_copy > -1)
                iMoveZ_copy--;
    }

    void IncMoveY()
    {
        if (nStackDepth == 0) 
        {
            if (iMoveY < 1)
              iMoveY++;
        }
        else
            if (iMoveY_copy < 1)
                iMoveY_copy++;
    }

    void DecMoveY()
    {
        if (nStackDepth == 0)
        {
            if (iMoveY > -1)
                iMoveY--;
        }
        else
            if (iMoveY_copy > -1)
                iMoveY_copy--;
    }

    void IncMoveX()
    {
        if (nStackDepth == 0)
        {
            if (iMoveX < 1)
                iMoveX++;
        }
        else
            if (iMoveX_copy < 1)
                iMoveX_copy++;
    }

    void DecMoveX()
    {
        if (nStackDepth == 0) 
        {
            if (iMoveX > -1)
                iMoveX--;
        }
        else
            if (iMoveX_copy > -1)
                iMoveX_copy--; 
    }

    void IncTurn()
    {
        if (nStackDepth == 0)
        {
            if (iTurn < 1)
                iTurn++;
        }
        else
            if (iTurn_copy < 1)
                iTurn_copy++;
    }
    
    void DecTurn()
    {
        if (nStackDepth == 0)
        {
            if (iTurn > -1)
                iTurn--;
        }
        else
            if (iTurn_copy > -1)
                iTurn_copy--;
    }
    
    
    void SetMoveFromKeysState(boolean bForward,
                              boolean bBackward,
                              boolean bRight,
                              boolean bLeft)
    {
        StopMove();
        if ( bForward )  IncMoveZ();
        if ( bBackward ) DecMoveZ();
        if ( bRight )    DecMoveX();
        if ( bLeft )     IncMoveX();
    }
    
///////////////////////////////////////////////////////////////////////////////    
//// stack
    
    /**Atrybuty poruszania sie.
     * Uzywane w stosie motion state.
     */

    byte iMoveX_copy;
    byte iMoveY_copy;     
    byte iMoveZ_copy;
    byte iTurn_copy;
    byte iSpeed_copy;
    
    int nStackDepth = 0;

    public void Push()
    {
        if (nStackDepth > 0)        //DEBUG
            return;
            //Log("stack overflow!\n");
        else
        {
            nStackDepth++;
            iMoveX_copy = iMoveX;
            iMoveY_copy = iMoveY;
            iMoveZ_copy = iMoveZ;
            iTurn_copy = iTurn;
            iSpeed_copy = iSpeed;
        }
    }

    public void Pop()
    {
        if (nStackDepth == 0)        //DEBUG
            return;
            //Log("stack underflow!\n");
        else
        {
            nStackDepth--;
            iTurn  = iTurn_copy;
            iMoveX = iMoveX_copy;
            iMoveY = iMoveY_copy;
            iMoveZ = iMoveZ_copy;
            iSpeed = iSpeed_copy;
        }
    }
    
///////////////////////////////////////////////////////////////////////////////
    public String toString()
    {
        String result;
        result = "X:"+iMoveX+" Z:"+iMoveZ+" speed:"+iSpeed;
        if (nStackDepth>0)
            result += "\n stack(X:"+iMoveX_copy+" Z:"+iMoveZ_copy+" speed:"+iSpeed_copy+")";
        return result; 
    }
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
		FileChunk.Log("[MotionState.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_MOTION_STATE)
        {
			// poruszanie sie
			iMoveX = (byte)cFC.LoadInt();	// strafe lewo/prawo
			iMoveY = (byte)cFC.LoadInt();	// gora/dol (wchodzenie na drabiny)
			iMoveZ = (byte)cFC.LoadInt();	// przod/tyl
			iTurn = (byte)cFC.LoadInt();	// obrot prawo/lewo
						    
			// Tryb poruszania. Rowny SPEED_...
			// Reprezentuje stan klawiszy modyfikacji trybu poruszania.
			iSpeed = (byte)cFC.LoadInt();

			//flagi przelaczajace stany w graczu
		
			// Przycelowujemy? 
			bAiming = cFC.LoadBool();	
			// Strzelamy? 
			bFiring = cFC.LoadBool();	
			// Przeladowujemy bron?
			bReloading = cFC.LoadBool();	
			// Wspinamy sie? 
			nClimbing = (byte)cFC.LoadInt();	
			// Skaczemy? 
			nJumping = (byte)cFC.LoadInt();				
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[MotionState.SGSaveChunk]\n");
		
        FileChunk cFC = GameObject.NewChunk(ESGChunksChrome._CHK_MOTION_STATE, cFCParent);
		
		// poruszanie sie
		cFC.SaveInt((int)iMoveX);	// strafe lewo/prawo
		cFC.SaveInt((int)iMoveY);	// gora/dol (wchodzenie na drabiny)
		cFC.SaveInt((int)iMoveZ);	// przod/tyl
		cFC.SaveInt((int)iTurn);	// obrot prawo/lewo
					    
		// Tryb poruszania. Rowny SPEED_...
		// Reprezentuje stan klawiszy modyfikacji trybu poruszania.
		cFC.SaveInt((int)iSpeed);

		//flagi przelaczajace stany w graczu
		
		// Przycelowujemy? 
		cFC.SaveBool(bAiming);	
		// Strzelamy? 
		cFC.SaveBool(bFiring);	
		// Przeladowujemy bron?
		cFC.SaveBool(bReloading);	
		// Wspinamy sie? 
		cFC.SaveInt((int)nClimbing);	
		// Skaczemy? 
		cFC.SaveInt((int)nJumping);	
			
        cFC.delete();
    }	    					
	
	public void SGUpdate() {}
}
