/** Player w samochodzie i przy drzwiach.
 */
public class PlayerTriggered extends Player
{
    /** Aktualnie aktywny trigger */	/*Nie zapisywa*/
    TriggerObject ActiveTrigger = null;
    
    /**Maksymalna odleglosc, z ktorej mozna uzyc triggera. */
    public float fTriggerVisibilityRange  = 250f;

    public boolean bDisableTriggerDetection = false;

    ///////////////////////////////////////////////////////////////////////////////
    //inicjalizacja
    
    public void OnCreate()
    {
        super.OnCreate();
        
        StartCheckDoorable();
        fPushedDamageThreshold = fDamageThreshold;
    }

    void OnDestroy()
    {
        super.OnDestroy();

        StopCheckDoorable();    
    }

    void StartCheckDoorable()
    {
        CallMethodEvery("CheckDoorableObjects", 0.1f);
    }
    
    void StopCheckDoorable()
    {
        DisableCallMethod("CheckDoorableObjects");
    }
    
    ///////////////////////////////////////////////////////////////////////////////
    // obsluga triggerow

    /** Podaje aktywny trigger.
     */
    TriggerObject GetActiveTrigger()
    {
        return ActiveTrigger;
    }
    
    public void SetActiveTrigger(TriggerObject trigger)
    {
        ActiveTrigger = trigger;
    }
    
    void ExecuteTrigger()
    {
        if( ActiveTrigger != null )
        {
            if (ActiveTrigger instanceof ChromeActor &&
                (!((ChromeActor)ActiveTrigger).IsAvailableInDemo()) && 
                ChromeModuleSingle._bEnglishDemo1)
            {
                HUDUniversal cHud = ((PlayerHUD)this).HUD.GetHudUniversal(); 
                cHud.ShowDemoOKDialog(((ChromeActor)ActiveTrigger).GetDemoInfoText());
                return;
            }
            else
            {
                ActiveTrigger.Execute( this );
                //TODO zrobic tak i przetestowac dla wszystkich TriggerObjectow
                if (ActiveTrigger instanceof LiftPlatformBase)
                    SetActiveTrigger(null);
            }
        }
        m_bExecutingTrigger = false;
    }
    
        ControlObject[] aGateFenceObjects = null;
    
    /** Wlaczanie i wylaczanie kolizji obiektom klasy Gate z ustawiona flaga bFenceGateType
     */
    void SetGateFenceObjectFlag(boolean bToggle)
    {
        //String aGateClasses[] = {"GateFence", "GateKey"};
        String sGateClass = "Gate";
        
        if(aGateFenceObjects == null)
        {

            java.util.Vector vGateFenceObjects = new java.util.Vector();

            GameObject[] aObjs =  GetObjectsFromClass(sGateClass);
            for(int j=0; j<aObjs.length; j++)
                {
                    //sprawdzam czy Brama ma ustawiona flage (przepuszczanie pociskow)
                    if (((Gate)aObjs[j]).bFenceGateType)
                        vGateFenceObjects.addElement(aObjs[j]);
                }
           
           
            aGateFenceObjects = new Gate[vGateFenceObjects.size()];
            vGateFenceObjects.copyInto(aGateFenceObjects);
            
        }

        for(int i=0; i<aGateFenceObjects.length; i++)
        {
            if(bToggle)
                aGateFenceObjects[i].SetFlag(EFlags._BLOCK_TRACE);
            else
                aGateFenceObjects[i].ClearFlag(EFlags._BLOCK_TRACE);
        }
    }

    
    /** Sprawdza aktywne triggery i zaznacza je na HUDzie
     */
    void CheckTriggers()
    {
        if(bDisableTriggerDetection) return;
        
        SetGateFenceObjectFlag(true);
        
        TriggerObject Obj = GetActiveTrigger(Camera.GetPosition(),
                                             Camera.GetForwardVector(),
                                             -fTriggerVisibilityRange);

        SetGateFenceObjectFlag(false);
        
        // gdy zmienil sie aktywny trigger to go ustawiamy
        if(ActiveTrigger != Obj) 
			SetActiveTrigger( Obj );
    }

    
	///////////////////////////////////////////////////////////////////////////////
	// POJAZDY
	///////////////////////////////////////////////////////////////////////////////

    void GetIn(SeatInfo seatInfo)
    {
		super.GetIn(seatInfo);
		
		if ( (cSeatInfo != null) &&
			 (cSeatInfo.GetOccupiedObject() == seatInfo.GetOccupiedObject()) )
		{
			motion.Stop();
			
			if (seatInfo.GetOccupiedObject() instanceof Vehicle)
			{
				Vehicle vehicle = (Vehicle) seatInfo.GetOccupiedObject();
				
				if (vehicle.nVehicleType == EVehicleType._MOTORCYCLE)
					SetState("MotorDriveState");
				else
				if (vehicle.nVehicleType == EVehicleType._CAR)
					SetState("CarDriveState");
				else
				if (vehicle.nVehicleType == EVehicleType._BOAT)
					SetState("BoatDriveState");
                
                if (cSeatInfo.GetRole() == EPassengerRole._VEHICLE_DRIVER)
		            vehicle.SetMotionMoveFromKeysState( this );
			}
			else
			if (seatInfo.GetOccupiedObject() instanceof StationaryCannon)
				SetState("HeavyGunState");
		}
    }
	
    boolean GetOut()
    {
		if (!super.GetOut())
			return false;
        SetMotionMoveFromKeyState();
		SetState("FreehandState");
        return true;
    }    
    
    /** ustawiamy stan ruchu zgodnie z aktualnymi klawiszami
     */
    void SetMotionMoveFromKeyState()
    {
        if ( ! bNetIsOwner || motion == null )
            return;
        motion.SetMoveFromKeysState( GetKeyState( EGameControl.FORWARD ),
                                     GetKeyState( EGameControl.BACKWARD ),
                                     GetKeyState( EGameControl.STRAFE_RIGHT),
                                     GetKeyState( EGameControl.STRAFE_LEFT) );
        m_cMotionMoveToApply = null;
    }
    
    void ProcessVehicleMotionKeys(int nKeyCode, boolean bKeyState, int iTurn, int iAcc)
    {
        if (bKeyState)
        {
			// jezeli jestesmy ownerem, sterujemy samochodem
			if (cSeatInfo.GetRole() == EPassengerRole._VEHICLE_DRIVER)
			{
				Vehicle vehicle = (Vehicle) cSeatInfo.GetOccupiedObject();
				if(nKeyCode==EGameControl.STRAFE_LEFT)
				{
				    vehicle.motion.TurnLeft();
				}
				else if (nKeyCode==EGameControl.STRAFE_RIGHT)
				{
				    vehicle.motion.TurnRight();
				}
				else if (nKeyCode==EGameControl.FORWARD)
				{
				    vehicle.motion.Accelerate();
				}
				else if (nKeyCode==EGameControl.BACKWARD)
				{
				    vehicle.motion.Decelerate();
				}
			}
			
            if (nKeyCode == EGameControl.ACTION)
            {
                if ( (cSeatInfo != null) && (CanGetOut()) )
                    ActionGetOut();
				else
                    LogLine("EGameControl.ACTION failed");
            }
            else if (nKeyCode==EGameControl.JUMP)
            {
            }
            
        }
        else {
            if(nKeyCode==EGameControl.STRAFE_LEFT)
            {
				((Vehicle) cSeatInfo.GetOccupiedObject()).motion.TurnRight();
            }
            else if (nKeyCode==EGameControl.STRAFE_RIGHT)
            {
                ((Vehicle) cSeatInfo.GetOccupiedObject()).motion.TurnLeft();
            }
            else if (nKeyCode==EGameControl.FORWARD)
            {
                ((Vehicle) cSeatInfo.GetOccupiedObject()).motion.Decelerate();
            }
            else if (nKeyCode==EGameControl.BACKWARD)
            {
                ((Vehicle) cSeatInfo.GetOccupiedObject()).motion.Accelerate();
            }
        }
    }
    
	boolean CanGetOut()
	{
		return true;
	}
	
    void GameActivatedFromMenu()
    {
        super.GameActivatedFromMenu();
        
        if ( sCurrentState.endsWith("DriveState") || sCurrentState.endsWith("HeavyGunState") )
        {
            if ( (cSeatInfo != null) && (cSeatInfo.GetOccupiedObject() != null) )
            {
                if (cSeatInfo.GetOccupiedObject() instanceof Vehicle)
                {
                    Vehicle vehicle = (Vehicle) cSeatInfo.GetOccupiedObject();
                    vehicle.motion.SetMoveFromKeysState( GetKeyState( EGameControl.FORWARD ),
                                                         GetKeyState( EGameControl.BACKWARD ),
                                                         GetKeyState( EGameControl.STRAFE_RIGHT),
                                                         GetKeyState( EGameControl.STRAFE_LEFT) );
                }
            }
        }
    }
    
    
    ///////////////////////////////////////////////////////////////////////////////
    // MOTOR DRIVE STATE    
    ///////////////////////////////////////////////////////////////////////////////

    public class MotorDriveState extends PlayerState
    {
        public void OnEnter()
        {
            super.OnEnter();
            Log("MotorDriveState\n");
           
            // wylaczamy kolizje graczowi az do momentu jak dostanie input z impulsem
			if (GetPhysicsType() == EPhysics._ODE_GRAVITYWALK)
                ODE_DisableODEGWUntilInputImpulse();
        }		
    }

    ///////////////////////////////////////////////////////////////////////////////
    // CAR DRIVE STATE    
    ///////////////////////////////////////////////////////////////////////////////

    int iAcc = 0;
    int iTurn = 0;

    
    public class CarDriveState extends PlayerState
    {
        final int   _VEHICLE_ATTACKER = 1,
                    _VEHICLE_WALKER = 2, 
                    _VEHICLE_APC    = 3;
        
        int nVehicle = 0;
		
		boolean bLoading = false;
		
        public void OnEnter()
        {
			if (bLoading == false)
				super.OnEnter();
			
            // wylaczamy kolizje graczowi az do momentu jak dostanie input z impulsem
			if (GetPhysicsType() == EPhysics._ODE_GRAVITYWALK)
                ODE_DisableODEGWUntilInputImpulse();

            if (cSeatInfo.GetOccupiedObject() instanceof VehicleAttacker)
				nVehicle = _VEHICLE_ATTACKER;
            else
                if (cSeatInfo.GetOccupiedObject() instanceof Walker)
                    nVehicle = _VEHICLE_WALKER;
                else
                    if (cSeatInfo.GetOccupiedObject() instanceof APC)
                        nVehicle = _VEHICLE_APC;
                
			AimCompute();
        }
		
		public void SGUpdate()
		{
			FileChunk.Log("[PlayerTrigger$CarDriveState] : SGUpdate\n");
			
			super.SGUpdate();
			bLoading = true;
			OnEnter();
			bLoading = false;			
		}

        void OnFrame()
        {
            FrameFinished();
            UpdateAccuracy();
        }

        int nFrame = 0;
        void OnPostUpdate()
        {
            nFrame++;
            if ((nVehicle == _VEHICLE_WALKER || nVehicle == _VEHICLE_APC) &&
                nFrame % 3 == 0)
                CheckTriggers( true );
        }
        
	    void StartAiming()
		{
            //w walkerze nie ma przycelowania
			if (nVehicle == _VEHICLE_WALKER)
				return;
	        motion.bAiming = true;
			AimCompute();
		}
		
	    void StopAiming()
		{
			if (nVehicle == _VEHICLE_WALKER)
				return;
	        motion.bAiming = false;
			AimCompute();
		}
		
        void OnInputKey(int nKeyCode, boolean bKeyState, char nKeyAscii)
        {
            ProcessVehicleMotionKeys(nKeyCode, bKeyState, iTurn, iAcc);
            ProcessZoomKeys(nKeyCode, bKeyState, nKeyAscii);
            
			if (nVehicle == _VEHICLE_WALKER)
				return;
			
            if(ProcessAimToggle(nKeyCode, bKeyState, nKeyAscii)) return;
            if (bKeyState)
            {
                if (nKeyCode == EGameControl.AIM)
                    StartAiming();
			}
			else
			{
                if (nKeyCode == EGameControl.AIM)
                    StopAiming();
			}
        }

        public void OnInputMouseButton(int nButton, int nState)
        {
            ProcessMouseButtons(nButton, nState);
        }

        public void ComputeAccuracy()
        {
            AimCompute();
        }
        
		private void AimCompute()
		{
			switch (nVehicle)
            {
            case _VEHICLE_ATTACKER:
            case _VEHICLE_APC:
                VehicleAttacker vehicle = (VehicleAttacker) cSeatInfo.GetOccupiedObject();
                if (vehicle.cCannon == null)
			    	return;
			    
                if (motion.bAiming)
			        fDestAccuracy = vehicle.cCannon.GetAimAccChange();
                else
                    fDestAccuracy = vehicle.cCannon.GetBaseAccuracy();
			    
                //implant nie moze wplywac na celnosc dzialek
                /*if ( (IsImplantActive(GetImplantSystem()._IMPLANT_TARGETING_ASSISTANCE)) )
			    	fDestAccuracy = vehicle.cCannon.GetAimAccChange();*/
			
                //TODO tego ma nie byc!
			    vehicle.cCannon.SetAimMultiplier(fDestAccuracy);
                break;
                
			case _VEHICLE_WALKER:
                Walker walker = (Walker)cSeatInfo.GetOccupiedObject();
                fDestAccuracy = walker.GetMainGunAccuracy();
                break;
            }
		}
    }
	
    public class HeavyGunState extends PlayerState
    {
        public void OnEnter()
        {
            
            super.OnEnter();
            // wylaczamy kolizje graczowi az do momentu jak dostanie input z impulsem
			if (GetPhysicsType() == EPhysics._ODE_GRAVITYWALK)
                ODE_DisableODEGWUntilInputImpulse();
        }

        void OnFrame()
        {
            FrameFinished();
            UpdateAccuracy();
        }

        int nFrame = 0;
        void OnPostUpdate()
        {
            nFrame++;
 
            if (nFrame % 3 == 0)
                CheckTriggers( true );
        }
        
	    void StartAiming()
		{
	        motion.bAiming = true;
			AimCompute();
		}
		
	    void StopAiming()
		{
	        motion.bAiming = false;
			AimCompute();
		}
		
        void OnInputKey(int nKeyCode, boolean bKeyState, char nKeyAscii)
        {
            if(ProcessAimToggle(nKeyCode, bKeyState, nKeyAscii)) return;
            
            if (bKeyState)
            {
                if (nKeyCode == EGameControl.AIM)
                    StartAiming();
			}
			else
			{
                if (nKeyCode == EGameControl.AIM)
                    StopAiming();
			}
        }

        public void OnInputMouseButton(int nButton, int nState)
        {
            ProcessMouseButtons(nButton, nState);
        }

        public void ComputeAccuracy()
        {
            AimCompute();
        }

		private void AimCompute()
		{
			StationaryCannon cannon = (StationaryCannon) cSeatInfo.GetOccupiedObject();
			ICannonController controller = cannon.m_Cannon;
	
			if (motion.bAiming)
			    fDestAccuracy = controller.GetAimAccChange();
            else
                fDestAccuracy = controller.GetBaseAccuracy();
            
			if ( (IsImplantActive(GetImplantSystem()._IMPLANT_TARGETING_ASSISTANCE)) )
				fDestAccuracy = controller.GetAimAccChange();

            //TODO tego ma nie byc!
			controller.SetAimMultiplier(fDestAccuracy);
		}
		
		//*********************************************************
		//					SaveGame
		//*********************************************************
						
		public void SGUpdate()
		{
			FileChunk.Log("[PlayerTriggered$HeavyGunState] : SGUpdate\n");
			
			super.SGUpdate();
			AimCompute();
		}		
	}

    /** Sprawdza, czy gracz powinien moze byc wyprostowany przed filmem.
     * Gdy tak to zwraca -1, jak nie to zwraca poze maksymalna poze.
     */
    byte MovieHomoErectus()
    {
        byte iSpeed;

        iSpeed = MotionState.SPEED_WALK;
        
        //CrashLog("HomoErectus motion.iSpeed="+motion.iSpeed+"\n");
        
        //jesli jestesmy schyleni a ze stanu klawiszy wynika, ze nie
        // powinnismy byc..
        if(motion.iSpeed < MotionState.SPEED_WALK)
        {
            //..to sprobujmy wstac
            //sprawdzamy, ile miejsca mamy nad glowa
            Vector vPos = GetPosition();
            vPos.fY += 10.f;
            Vector vUp = new Vector(vPos);
            vUp.fY += fEllipsoidHeight[iSpeed];
            
            Vector vRightRadius = GetRightVector().Mul2(fEllipsoidRadius[iSpeed]);
            Vector vForwardRadius = GetForwardVector().Mul2(fEllipsoidRadius[iSpeed]);
            Vector aTracesPos[] = { vPos, vPos.Add2(vRightRadius), vPos.Sub2(vRightRadius),
                                          vPos.Add2(vForwardRadius), vPos.Sub2(vForwardRadius) };
            
            // puszczamy 5 promieni do sprawdzenia czy mozna wstac
            Collision stCollObjects = null;
            boolean bCanErectus = true;
            for (int i = 0; (i < 5) && bCanErectus; i++)
            {
                stCollObjects = TraceObjectsExclude(aTracesPos[i], vUp, this);
                bCanErectus = ((stCollObjects == null) || (stCollObjects.Obstacle instanceof NewAI));
            }

            if (!bCanErectus)
            {
                //CrashLog("HomoErectus collision with <"+stCollObjects.Obstacle.GetName()+":"+stCollObjects.Obstacle.GetUniqueID()+">\n");
                //kolizja nad glowa - nie wstajemy, ale moze mozna wstac tylko troche?
                //dla kazdej posredniej pozycji, zaczynajac od najwyzszej (dla logana bedzie tylko 1 - duck)
                for (int i = iSpeed - 1; i>motion.iSpeed; i--)
                {
                    //czy sie miescimy?
                    if ( (vUp.fY - vPos.fY) >= fEllipsoidHeight[i] )
                    {
                        //mozna sie wyprostowac
                        return (byte)i;
                    }
                }
                
                return motion.iSpeed;
            }
            //CrashLog("HomoErectus no collision\n");
        }
        
        // zwracamy poze bez zmian
        return -1;
    }
    
	boolean bMovieKeyRequested = false;
	boolean bMovieMouseButtonRequested = false;
	boolean bMovieMouseAxisRequested = false;
	
    float fPushedDamageThreshold = 0.0f;
    byte nPushedCameraMode = -1;
    
    public void InGameMovieStart()
    {
        super.InGameMovieStart();
        
		bMovieKeyRequested = IsRegisteredKeyRequest();
		bMovieMouseButtonRequested = IsRegisteredMouseButtonRequest();
		bMovieMouseAxisRequested = IsRegisteredMouseAxisRequest();
		
        UnregisterKeyRequest();
        UnregisterMouseButtonRequest();
        UnregisterMouseAxisRequest();

        // wymuszamy zakonczenie przeladowania
        BreakWeaponReload();
        
        motion.bAiming = false;
        motion.bFiring = false;
        
        boolean bInVehicle = false;
        
        if ( (cSeatInfo != null) && (cSeatInfo.GetOccupiedObject() instanceof Vehicle) &&
             ((Vehicle) cSeatInfo.GetOccupiedObject()).motion != null )
        {
            CrashLog("Motion from vehicle\n");
            ((Vehicle) cSeatInfo.GetOccupiedObject()).motion.Stop();
            bInVehicle = true;
        }
        else
        {
            // ustawiamy graczowi, ze stoi i biega :)
            byte iSpeed = MovieHomoErectus();
            
            CrashLog("Motion from player iSpeed="+iSpeed+"\n");

            motion.Stop();

            bWalking = false;
            bDucking = false;
            bCrawling = false;
            bSwimming = false;
            
            if(iSpeed != -1)
            {
                if(iSpeed == MotionState.SPEED_SWIM) 
                {
                    bSwimming = true;
                    motion.iSpeed = iSpeed;
                }
                if(iSpeed == MotionState.SPEED_CRAWL) 
                {
                    bCrawling = true;
                    motion.iSpeed = iSpeed;
                }
                if(iSpeed == MotionState.SPEED_DUCK) 
                {
                    bDucking = true;
                    motion.iSpeed = iSpeed;
                }
            }
            
            UpdateMotion();
            UpdateState();
        }

        if (ActiveWeapon != null && !(ActiveWeapon instanceof WeaponGrenade))
			ActiveWeapon.StopFire();
        
        BodyDeformAimAtVReset();
        BodyDeformAimAtHReset();
        
        StopCheckDoorable();
        
        bLimitMoves = true;
        SetAnimMorphTime(.02f);
		
		if ((bLoadingSG) && (!GetModule().bIsMoviePlayingSG))
			return;
		        
        // dajemy duzy damageThreshold zeby nikt gracza nie zabil
        /*
        fPushedDamageThreshold = fDamageThreshold;
        fDamageThreshold = 10000000f;
        */
        
        ChangeConflictSide(EnumConflictSide._NEUTRAL);
        
        //widzimy gracza z TPP
        if(!bInVehicle)
        {
            if(!bInMovieCameraLeave)
            {
                nPushedCameraMode = Camera.GetMode();
                if( (nPushedCameraMode == CameraPlayer.CMODE_FPP) ||
                    (nPushedCameraMode == CameraPlayer.CMODE_SNIPER) )
                    CameraModeChanged(CameraPlayer.CMODE_TPP);
            }
        }
        else
            nPushedCameraMode = -1;
    }

    public void InGameMovieEnd()
    {
        super.InGameMovieEnd();
        
		if (bMovieKeyRequested)
	        RegisterKeyRequest();
        
        if ( (cSeatInfo != null) && (cSeatInfo.GetOccupiedObject() instanceof Vehicle) &&
             ((Vehicle) cSeatInfo.GetOccupiedObject()).motion != null )
        {
        }
        else
        {
			if (bMovieMouseButtonRequested)
	            RegisterMouseButtonRequest();
			if (bMovieMouseAxisRequested)
		        RegisterMouseAxisRequest();
        }
        
        SetupMouseAcquire();

        StartCheckDoorable();
        
        bLimitMoves = false;
        SetAnimMorphTime(Anims.fAnimMorphTime);
		
		if (bLoadingSG)
			return;		


        // przywracamy fDamageThreshold
        /*
        fDamageThreshold = fPushedDamageThreshold;
        */
        
        ChangeConflictSide(EnumConflictSide._PRO_PLAYER);
        
        //kamera playera
        if(nPushedCameraMode != -1)
            CameraModeChanged(nPushedCameraMode);
    }
    
    ///////////////////////////////////////////////////////////////////////////////
    // BOAT DRIVE STATE    
    ///////////////////////////////////////////////////////////////////////////////
    
    public class BoatDriveState extends PlayerState
    {
        public void OnEnter()
        {
            super.OnEnter();
            Log("BoatDriveState\n");

            // wylaczamy kolizje graczowi az do momentu jak dostanie input z impulsem
			if (GetPhysicsType() == EPhysics._ODE_GRAVITYWALK)
                ODE_DisableODEGWUntilInputImpulse();
        }
    }

    //////////////////////////////////////////////////////
    // System zdarzen gdy gracz wchodzi w gorace punkty //
    //////////////////////////////////////////////////////
    
    void EnteredHotSpot(int nHotSpot)
    {
        //ActionPerformed( EPlayerAction.HOTSPOT_ENTER, nHotSpot );
    }
    
    void LeftHotSpot(int nHotSpot)
    {
        //ActionPerformed( EPlayerAction.HOTSPOT_LEAVE, nHotSpot );
    }
    
    //////////////////////////////////////////////////////
    // Obsluga otwierania/zamykania drzwi               //
    //////////////////////////////////////////////////////

    public void CheckDoorableObjects()
    {
        ((ChromeModule)GetModule()).GetDoorControler().IsNearDoorable(this);
    }


    
    //////////////////////////////////////////////////////
    // Obsluga sciszania i zglasniania dzwiekow wskutek //
    // wchodzenia do budynkow                           //
    //////////////////////////////////////////////////////
    //??
	
    //*********************************************************
	//					SaveGame
    //*********************************************************
	
    /** Metoda wczytujaca zapisane dane w Save gracza
     */	
    public void SGLoadChunk(FileChunk cParentFC)
    {	
        StopAiming();
        
		FileChunk.Log("[PlayerTriggered.SGLoadChunk]\n");
		
		FileChunk cFC = GameObject.LoadChunk(cParentFC);   
		
        if (cFC.GetID() == ESGChunksChrome._CHK_PLAYER_TRIGGERED)
        {	
			super.SGLoadChunk(cFC);
			
			bMovieKeyRequested = IsRegisteredKeyRequest();
			bMovieMouseButtonRequested = IsRegisteredMouseButtonRequest();
			bMovieMouseAxisRequested = IsRegisteredMouseAxisRequest();
						
			ActiveTrigger = (TriggerObject)cFC.LoadObjectRef();

			// Aktualnie aktywny trigger
			bDisableTriggerDetection = cFC.LoadBool();	
			
			nPushedCameraMode = cFC.LoadByte();			
        }
		
		cFC.delete();
    }
    
    /** Metoda zapisujaca dane do Save gracza
     */	
    public void SGSaveChunk(FileChunk cFCParent)
    {
		FileChunk.Log("[PlayerTriggered.SGSaveChunk]\n");
		
        FileChunk cFC = NewChunk(ESGChunksChrome._CHK_PLAYER_TRIGGERED, cFCParent);
		
		super.SGSaveChunk(cFC);		
		
		cFC.SaveObjectRef(ActiveTrigger);

		// Aktualnie aktywny trigger
		cFC.SaveBool(bDisableTriggerDetection);
		
		cFC.SaveByte(nPushedCameraMode);
							        
        cFC.delete();
    }	    	    	
	
	public void SGUpdate()
	{
		super.SGUpdate(); 
		InGameMovieEnd();
	}
   
}
